 MODULE biolodyncellsed

#if defined key_substance && defined key_biolo
   !!======================================================================
   !!                   ***  MODULE  biolodyn  ***
   !! Bilogic dynamics: 
   !!======================================================================
   !! * Modules used
   USE parameters,   ONLY : rsh
#ifdef key_sedim_anc
   USE sedimento,     ONLY : poro
   USE comsubstance, ONLY : nv_state,dtbio
#endif

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC biolo_dyn_cellsed ! routine called by biolo_sksc_sed.F90

   !! * Interface
   INTERFACE biolo_dyn_cellsed
!     MODULE PROCEDURE incellsed_dinodiatzoo
     MODULE PROCEDURE incellsed_nut4phy3zoo2
   END INTERFACE

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE incellsed_nut4phy3zoo2(km,im,jm,c,dc)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE incellsed_nut4phy3zoo2  ***
   !&E
   !&E ** Purpose : Calcul des variations dues aux phenomenes biologiques
   !&E              ayant lieu a l interieur d une maille dans le sediment
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  (M. Sourisseau) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvarp2d, ONLY  : temp
   USE bioloinit, ONLY : iv_nutr_NO3,iv_nutr_SiOH,iv_nutr_PO4,iv_nutr_Pads, &
                         iv_phyto_diat_N,iv_detr_N,iv_detr_Si,iv_detr_P,     &
                         iv_nutr_NH4,iv_spim

   !! * Declaration subroutine

   !! * Arguments
   INTEGER, INTENT( in ) :: im,jm,km    ! cell indexes
   REAL(KIND=rsh),DIMENSION(:),INTENT(in) :: c
   REAL(KIND=rsh),DIMENSION(:),INTENT(inout) :: dc

   !! * Local declarations
   INTEGER               :: iv          ! loop index
                     ! variables relatives a la temperature
   REAL(KIND=rsh)        :: temper,effetchaleur
                     ! variable diatomees
   REAL(KIND=rsh)        :: diatmortsed
                     ! variables sels nutritifs
   REAL(KIND=rsh)        :: rapsiaz,rappaz
                     ! processus mineralisation, nitrification, dissolution
   REAL(KIND=rsh)        :: facto2remin,facto2nitrif,reminazdetsed,     &
                            xnitrifsed,reminpdetsed,dissolsilicesed
                     ! processus adsorption/desorption
   REAL(KIND=rsh)        :: varads,satursedp,adsorpsed,cmaxadsorp,      &
                            desorpsed,cmaxdesorp
                     ! cycle de l oxygene
   REAL(KIND=rsh)        :: oreminsed,onitrifsed

#ifdef key_sedim_anc

   !!----------------------------------------------------------------------
   !! * Executable part

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! saisir le code apres cette limite
   ! =================================
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   temper=temp(km,im,jm)
   IF (temper.gt.30.) THEN
     write(*,*)'tempsed:',km,im,jm,temper
     temper=15.
   END IF
   effetchaleur=exp(p_T_effect*temper)

   ! mortalite des diatomees
   ! -----------------------
   diatmortsed=10.*p_diat_mort*effetchaleur

   ! remineralisation dans le sediment
   ! ---------------------------------
   !facto2remin=c(iv_nutr_NH4)/(c(iv_nutr_NH4)+(p_micz_mort/2))
   !facto2nitrif=c(iv_nutr_NH4)/(c(iv_nutr_NH4)+p_micz_mort)
   facto2remin=1.
   facto2nitrif=1.
   reminazdetsed=facto2remin*p_Nbent_remin*effetchaleur
   xnitrifsed=facto2nitrif*p_nitrifsed*effetchaleur
   reminpdetsed=facto2remin*p_Pbent_remin*effetchaleur
   dissolsilicesed=p_Sibent_dissol*effetchaleur

   rapsiaz=p_diat_iksmith
   rappaz=1/p_phyto_SiNratio

   ! Adsorption et desorption du phosphore dans le sediment
   ! ------------------------------------------------------

   ! adsorption
   ! ----------
   IF (c(iv_spim).ne.0.0) THEN
     varads=min(1.0,c(iv_nutr_Pads)/(p_P_adsormaxspim*c(iv_spim)/1000.))
   ELSE
     varads=0.0
   END IF
   satursedp=max(0.,(p_P_adsormaxspim*c(iv_spim)/1000.-c(iv_nutr_Pads)))
   adsorpsed=p_P_adsor*satursedp               
   cmaxadsorp=adsorpsed*c(iv_nutr_PO4)*dtbio/86400.
   IF (cmaxadsorp.ge.c(iv_nutr_PO4))  adsorpsed=0.9*86400./dtbio                   
   ! desorption
   ! ----------
   desorpsed=p_P_desor*varads
   cmaxdesorp=desorpsed*c(iv_nutr_Pads)*dtbio/86400.
   IF (cmaxdesorp.ge.c(iv_nutr_Pads)) desorpsed=0.9*86400./dtbio

   ! production et consommation d oxygene dans le sediment (inemploye)
   ! -----------------------------------------------------------------

   ! remineralisation
   !oreminsed=reminazdetsed*c(iv_detr_N)*(0.212+0.064)      
   !onitrifsed=xnitrifsed*c(iv_zoo_meso_N)*0.064

   ! Evolution du nitrate
   ! -------------------------------
   dc(iv_nutr_NO3)=xnitrifsed*c(iv_nutr_NH4)

   ! Evolution de la silice dissoute
   ! -------------------------------
   dc(iv_nutr_SiOH)=dissolsilicesed*c(iv_detr_Si)/poro(km,im,jm)

   ! Evolution du phosphate dissous
   ! -------------------------------
   dc(iv_nutr_PO4)=-adsorpsed*c(iv_nutr_PO4)+desorpsed*c(iv_nutr_Pads)/poro(km,im,jm)             &
             +reminpdetsed*c(iv_detr_P)/poro(km,im,jm)

   ! Evolution du phosphate adsorbe
   ! -------------------------------
   dc(iv_nutr_Pads)=adsorpsed*c(iv_nutr_PO4)*poro(km,im,jm)-desorpsed*c(iv_nutr_Pads)

   ! Evolution de l azote des diatomees
   ! -------------------------------
   dc(iv_phyto_diat_N)=-diatmortsed*c(iv_phyto_diat_N)

   ! Evolution de l azote detritique 
   ! -------------------------------
   dc(iv_detr_N)=-reminazdetsed*c(iv_detr_N)+diatmortsed*c(iv_phyto_diat_N)

   ! Evolution de la silice particulaire
   ! -------------------------------
   dc(iv_detr_Si)=-dissolsilicesed*c(iv_detr_Si)+rapsiaz*diatmortsed*c(iv_phyto_diat_N)

   ! Evolution du phosphore detritique
   ! -------------------------------
   dc(iv_detr_P)=rappaz*diatmortsed*c(iv_phyto_diat_N)-reminpdetsed*c(iv_detr_P)

   ! Evolution de l oxygene dissous (inemploye)
   ! ------------------------------------------
   !dc(iv_nutr_NH4)=-oreminsed/poro(km,im,jm)-onitrifsed

   ! Evolution de l ammonium
   ! -------------------------------
   dc(iv_nutr_NH4)=reminazdetsed*c(iv_detr_N)/poro(km,im,jm)-xnitrifsed*c(iv_nutr_NH4)

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
   ! saisir le code avant cette limite
   ! =================================
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#else
        write(*,*)  'incellsed_nut4phy3zoo2 : You should not enter this routine'
#endif

  END SUBROUTINE incellsed_nut4phy3zoo2

   !!======================================================================

  SUBROUTINE incellsed_dinodiatzoo(km,im,jm,c,dc)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE incellsed_dinodiatzoo  ***
   !&E
   !&E ** Purpose : Calcul des variations dues aux phenomenes biologiques
   !&E              ayant lieu a l interieur d une maille dans le sediment
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  (A. Menesguen P. Cugier) Original code
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvarp2d, ONLY  : temp
   USE bioloinit, ONLY : iv_nutr_NO3,iv_nutr_SiOH,iv_nutr_PO4,iv_nutr_Pads, &
                         iv_phyto_diat_N,iv_detr_N,iv_detr_Si,iv_detr_P,     &
                         iv_nutr_NH4,iv_zoo_meso_N,iv_phyto_diat_pp

   !! * Declaration subroutine

   !! * Arguments
   INTEGER, INTENT( in ) :: im,jm,km    ! cell indexes
   REAL(KIND=rsh),DIMENSION(:),INTENT(in) :: c
   REAL(KIND=rsh),DIMENSION(:),INTENT(inout) :: dc

   !! * Local declarations
   INTEGER               :: iv          ! loop index
                     ! variables relative to temperature
   REAL(KIND=rsh)        :: temper,effetchaleur
                     ! variable diatomees
   REAL(KIND=rsh)        :: diatmortsed
                     ! variables sels nutritifs
   REAL(KIND=rsh)        :: rapsiaz,rappaz
                     ! processus mineralisation, nitrification, dissolution
   REAL(KIND=rsh)        :: facto2remin,facto2nitrif,reminazdetsed,     &
                            xnitrifsed,reminpdetsed,dissolsilicesed
                     ! processus adsorption/desorption
   REAL(KIND=rsh)        :: varads,satursedp,adsorpsed,cmaxadsorp,      &
                            desorpsed,cmaxdesorp
                     ! cycle de l oxygene
   REAL(KIND=rsh)        :: oreminsed,onitrifsed

#ifdef key_sedim_anc

   !!----------------------------------------------------------------------
   !! * Executable part

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! saisir le code apres cette limite
   ! =================================
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   temper=temp(km,im,jm)
   IF (temper.gt.30.) THEN
     write(*,*)'tempsed:',km,im,jm,temper
     temper=15.
   END IF
   effetchaleur=exp(p_T_effect*temper)

   ! mortalite des diatomees
   ! -----------------------
   diatmortsed=10.*p_diat_mort*effetchaleur
       
   ! remineralisation dans le sediment
   ! ---------------------------------
   facto2remin=c(iv_nutr_NH4)/(c(iv_nutr_NH4)+(p_micz_mort/2))
   facto2nitrif=c(iv_nutr_NH4)/(c(iv_nutr_NH4)+p_micz_mort)
   reminazdetsed=facto2remin*p_micz_kgraz*effetchaleur
   xnitrifsed=facto2nitrif*bioparam(66)*effetchaleur
   reminpdetsed=facto2remin*p_micz_assim*effetchaleur
   dissolsilicesed=p_micz_excret*effetchaleur
        
   rapsiaz=p_diat_iksmith
   rappaz=1/bioparam(27)

   ! Adsorption et desorption du phosphore dans le sediment
   ! ------------------------------------------------------

   ! adsorption
   ! ----------
   IF (c(iv_phyto_diat_pp).ne.0.0) THEN
     varads=min(1.0,c(iv_nutr_Pads)/(p_P_adsormaxspim*c(iv_phyto_diat_pp)))
   ELSE
     varads=0.0
   END IF
   satursedp=max(0.,(p_P_adsormaxspim*c(iv_phyto_diat_pp)-c(iv_nutr_Pads)))
   adsorpsed=p_P_adsor*satursedp               
   cmaxadsorp=adsorpsed*c(iv_nutr_PO4)*dtbio/86400.
   IF (cmaxadsorp.ge.c(iv_nutr_PO4))  adsorpsed=0.9*86400./dtbio                   
   ! desorption
   ! ----------
   desorpsed=p_P_desor*varads
   cmaxdesorp=desorpsed*c(iv_nutr_Pads)*dtbio/86400.
   IF (cmaxdesorp.ge.c(iv_nutr_Pads)) desorpsed=0.9*86400./dtbio

   ! production et consommation d oxygene dans le sediment
   ! -----------------------------------------------------

   ! remineralisation
   oreminsed=reminazdetsed*c(iv_detr_N)*(0.212+0.064)      
   onitrifsed=xnitrifsed*c(iv_zoo_meso_N)*0.064

   ! Evolution du nitrate
   ! -------------------------------
   dc(iv_nutr_NO3)=xnitrifsed*c(iv_zoo_meso_N)

   ! Evolution de la silice dissoute
   ! -------------------------------
   dc(iv_nutr_SiOH)=dissolsilicesed*c(iv_detr_Si)/poro(km,im,jm)

   ! Evolution du phosphate dissous
   ! -------------------------------
   dc(iv_nutr_PO4)=-adsorpsed*c(iv_nutr_PO4)+desorpsed*c(iv_nutr_Pads)/poro(km,im,jm)             &
             +reminpdetsed*c(iv_detr_P)/poro(km,im,jm)

   ! Evolution du phosphate adsorbe
   ! -------------------------------
   dc(iv_nutr_Pads)=adsorpsed*c(iv_nutr_PO4)*poro(km,im,jm)-desorpsed*c(iv_nutr_Pads)

   ! Evolution de l azote des diatomees
   ! -------------------------------
   dc(iv_phyto_diat_N)=-diatmortsed*c(iv_phyto_diat_N)

   ! Evolution de l azote detritique 
   ! -------------------------------
   dc(iv_detr_N)=-reminazdetsed*c(iv_detr_N)+diatmortsed*c(iv_phyto_diat_N)

   ! Evolution de la silice particulaire
   ! -------------------------------
   dc(iv_detr_Si)=-dissolsilicesed*c(iv_detr_Si)+rapsiaz*diatmortsed*c(iv_phyto_diat_N)

   ! Evolution du phosphore detritique
   ! -------------------------------
   dc(iv_detr_P)=rappaz*diatmortsed*c(iv_phyto_diat_N)-reminpdetsed*c(iv_detr_P)

   ! Evolution de l oxygene dissous
   ! -------------------------------
   dc(iv_nutr_NH4)=-oreminsed/poro(km,im,jm)-onitrifsed

   ! Evolution de l ammonium
   ! -------------------------------
   dc(iv_zoo_meso_N)=reminazdetsed*c(iv_detr_N)/poro(km,im,jm)-xnitrifsed*c(iv_zoo_meso_N)

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
   ! saisir le code avant cette limite
   ! =================================
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#else
        write(*,*)  'incellsed_dinodiatzoo : You should not enter this routine'
#endif

  END SUBROUTINE incellsed_dinodiatzoo

#endif
 END MODULE
