 MODULE biolodyncellwat

#if defined key_substance && defined key_biolo
   !!======================================================================
   !!                   ***  MODULE  biolodyn  ***
   !! Biologic dynamics: Calcul des variations dues aux phenomenes biologiques
   !!                    ayant lieu a l interieur d une maille dans l eau
   !!======================================================================
   !! * Modules used
   USE parameters,   ONLY : rsh,rlg,kmax,epsilon
   USE comvars2d,    ONLY : h0
   USE cometemp,     ONLY   : ventvent
   USE comsubstance, ONLY : irk_mod,dtbio,nv_state,ws3
   USE bioloinit,    ONLY : diag_2d,diag_3d_wat,irk_diag
   USE comvarp2d,    ONLY : temp,sal

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC biolo_dyn_cellwat ! routine called by biolo_sksc_wat.F90

   !! * Interface
   INTERFACE biolo_dyn_cellwat
!     MODULE PROCEDURE incellwat_dinodiatzoo
     MODULE PROCEDURE incellwat_nut4phy3zoo2
   END INTERFACE

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE incellwat_nut4phy3zoo2(km,im,jm,xe,c,dc)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE incellsed_nut4phy3zoo2  ***
   !&E
   !&E ** Purpose : Calcul des variations dues aux phenomenes biologiques
   !&E              ayant lieu a l interieur d une maille dans l eau
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !          (M. Sourisseau A. Menesguen P. Cugier) Original code
   !&E       !  2008-01 (M. Sourisseau) evolutions
   !&E       !  2008-05 (M. Sourisseau) link between microzoo and diatoms
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : attenuation,extinction,alumplafond
   USE bioloinit, ONLY : iv_nutr_NO3,iv_nutr_SiOH,iv_nutr_PO4,iv_nutr_Pads, &
                         iv_phyto_diat_N,iv_phyto_dino_N,iv_detr_N,iv_detr_Si,     &
                         iv_detr_P,iv_nutr_NH4,iv_zoo_meso_N,iv_phyto_nano_N,        &
                         iv_zoo_micr_N,iv_spim,iv_phyto_diat_pp,iv_phyto_dino_pp,         &
                         iv_phyto_nano_pp
   USE bioloinit,    ONLY : p_N_remin,p_P_remin,p_Si_diss,p_T_effect,p_P_adsor,p_P_desor, &
                            p_P_adsormaxspim,p_P_adsormaxsed,p_phyto_ChlNratio,           &
                            p_phyto_SiNratio,p_phyto_NPratio,p_phyto_CNratio,             &
                            p_nano_mumax,p_nano_kNO3,p_nano_kNH4,p_nano_kPO4, &
                            p_nano_mort,p_nano_iksmith,p_diat_iksmith,p_diat_mort, &
                            p_diat_mumax,p_diat_kNO3,p_diat_kNH4,p_diat_kSi,p_diat_kPO4, &
                            p_dino_mumax,p_dino_kNO3,p_dino_kNH4,p_dino_kPO4,p_dino_mort, &
                            p_dino_iksmith,     &
                            p_mesz_thrChl,p_mesz_kivlev,p_mesz_mumax,p_mesz_assim,       &
                            p_mesz_excret,p_mesz_mort1,p_mesz_mort2,       &
                            p_zoo_CDWratio,p_zoo_CNratio,p_micz_mumax,p_micz_kgraz,       &
                            p_micz_assim,p_micz_excret,p_micz_mort,p_micz_thrnano,     &
                            p_phyto_photoratio,p_Nbent_remin,p_Pbent_remin,p_Sibent_dissol, &
                            p_O2_kremin,p_nitrifwat,p_nitrifsed,p_zoo_resp,         &
                            p_extincwat,p_extincspim,p_extincChl1,p_extincChl2,       &
                            p_parradratio,p_erodflux,p_erodcrittau,p_detzoo_wsed,      &
                            p_detphy_wsed,p_mesz_captdiat,p_mesz_captdino,p_mesz_captmicz, &
                            p_micz_captdiat,p_micz_captnano,p_micz_captdet
   USE bioloinit,    ONLY : id_diat_limlight,id_diat_limN,id_diat_limSi, &
                            id_diat_limP,id_dino_limlight,id_dino_limN,  &
                            id_dino_limP,id_nano_limlight,id_nano_limN,  &
                            id_nano_limP,id_extinctioncoeff,           &
                            id_totalchl,id_columnprodtotal,        &
                            id_diatsettling,id_detsettling
   USE toolgeom,     ONLY : f_dzu

   !! * Arguments
   INTEGER, INTENT( in ) ::   km,im,jm      ! ocean spatial-step index
   REAL(KIND=rsh),             INTENT(in) :: xe     !ssh
   REAL(KIND=rsh),DIMENSION(:),INTENT(in) :: c
   REAL(KIND=rsh),DIMENSION(:),INTENT(inout) :: dc

   !! * Local declarations

   ! indices de boucles
   ! ------------------
   INTEGER :: iv,ibbenth,ibsurf,k

   ! variables temperature, salinite
   ! -------------------------------
   REAL(KIND=rsh) :: temper,tempabs,effetchaleur,effetchaleurT,sali

   ! profondeur colonne d eau
   ! ------------------------
   REAL(KIND=rsh) :: d

   ! variables sels nutritifs
   ! ------------------------
   REAL(KIND=rsh) :: rappaz,rapsiaz,fractionno3diat, fractionno3dino,fractionno3nano
   REAL(KIND=rsh) :: fractionnh4diat, fractionnh4dino,fractionnh4nano

   ! variables lumieres
   ! ------------------------
   REAL(KIND=rsh) :: fluxrelatifsurf,fluxrelatiffond

   ! variables diatomees
   ! -------------------
   REAL(KIND=rsh) :: fluxrelatifdiat,effetlumierediat,effetnitdiat,effetamdiat
   REAL(KIND=rsh) :: effetselnut,effetazotediat,effetsilice,effetphosphorediat
   REAL(KIND=rsh) :: pslimdiat,rationdiat,diatmorteau,xintvertdiat,fractionno3,fractionnh4

   ! variables dinoflagelles
   ! -----------------------
   REAL(KIND=rsh) :: fluxrelatifdino,effetnitdino,effetamdino
   REAL(KIND=rsh) :: pslimdino,effetlumieredino,effetazotedino
   REAL(KIND=rsh) :: effetphosphoredino,rationdino,dinomorteau,xintvertdino

   ! variables nano_pico_phytoplancton
   ! ---------------------------------
   REAL(KIND=rsh) :: effetlumierenano,effetnitnano,effetamnano,effetazotenano
   REAL(KIND=rsh) :: effetphosphorenano,pslimnano, rationnano, nanomorteau
   REAL(KIND=rsh) :: xintvertnano

   ! variables mesozooplancton
   ! -------------------------
   REAL(KIND=rsh) :: zdiat,fmesozoo,rationmesozoo,excretionmesozoo
   REAL(KIND=rsh) :: captdiat,captdino,captmicrozoo,captmicrozooN
   REAL(KIND=rsh) :: broumesozoodiat,broumesozoodino,broumesozoomicrozoo
   REAL(KIND=rsh) :: txmortmesozoo,rNC,assimilzoo

   ! variables microzooplancton
   ! --------------------------
   REAL(KIND=rsh) :: fmicrozoo,rationmicrozoo,excretionmicrozoo
   REAL(KIND=rsh) :: txmortmicrozoo,captnano,captdet,broumicrozoonano
   REAL(KIND=rsh) :: broumicrozoodet,broumicrozoodiat,assimilmicrozoo

   ! processus de mineralisation, nitrification, dissolution
   ! -------------------------------------------------------
   REAL(KIND=rsh) :: reminazdeteau,xnitrifeau,reminpdeteau,dissolsiliceeau

   ! processus d adsorption/desorption
   ! ---------------------------------
   REAL(KIND=rsh) :: varads,desorpeau,cmaxdesorp,saturmesp
   REAL(KIND=rsh) :: adsorpeau,cmaxadsorp

   ! cycle de l oxygene
   ! ------------------

   ! vitesses de chute
   ! ------------------

   ! epaisseur des couches et surface libre
   ! --------------------------------------
   REAL(KIND=rsh) :: epn
   REAL(KIND=rsh)   :: dzu

   ! parametrisation broutage par benthos
   ! ------------------------------------


   !!----------------------------------------------------------------------
   !! * Executable part

   d=h0(im,jm)+xe
   dzu=f_dzu(h0(im,jm),xe,km,im,jm)
   epn=dzu

   ! test pour identification des mailles de fond
   ! --------------------------------------------
   ibbenth=0
   IF (km.eq.1) ibbenth=1

   ! test pour identification des mailles de surface
   ! -----------------------------------------------
   ibsurf=0
   IF (km.eq.kmax) ibsurf=1

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! saisir le code apres cette limite
   ! =================================
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   !+++++++++++++++++++++++++++++
   ! Calculs communs
   !+++++++++++++++++++++++++++++

   ! salinite
   sali=sal(km,im,jm)
   ! temperature & effetchaleur
   tempabs=temp(km,im,jm)+273.15_rsh
   temper=temp(km,im,jm)
!cval toseelater Oh la ??? c est quoi ce test ne va pas du tout !!!!!
   IF (temper > 30.0_rsh) THEN
    !write(*,*) 'tempwat',km,im,jm,temper
    temper=15.0_rsh
   ENDIF
   effetchaleur=exp(p_T_effect*temper)
   ! facteurs de conversion
   rapsiaz=p_phyto_SiNratio
   rappaz=1.0_rsh/p_phyto_NPratio

   !++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ! Croissance et mortalite du phytoplancton de base
   !++++++++++++++++++++++++++++++++++++++++++++++++++++++

   IF ((c(iv_nutr_NO3).ne.0.0_rsh).or.(c(iv_nutr_NH4).ne.0.0_rsh)) THEN
     fractionno3=c(iv_nutr_NO3)/(c(iv_nutr_NO3)+c(iv_nutr_NH4))
     fractionnh4=c(iv_nutr_NH4)/(c(iv_nutr_NO3)+c(iv_nutr_NH4))
   ELSE
     fractionno3=0.0_rsh
     fractionnh4=0.0_rsh
   END IF

   ! DIATOMEES
   ! ---------
   ! Effetlumiere formulation de Steele
   ! Formulation de Smith integree sur la profondeur de la boite
   !  avec l hypothese que Ik=Iopt/2.5
   ! ............................................................

   fluxrelatifsurf=p_parradratio*alumplafond(km,im,jm)/(p_diat_iksmith/2.5_rsh)
   fluxrelatiffond=fluxrelatifsurf*attenuation(km,im,jm)
   effetlumierediat=1.0_rsh/epn/extinction(km,im,jm)*                     &
      log((fluxrelatifsurf+sqrt(1.0_rsh+fluxrelatifsurf*                  &
      fluxrelatifsurf))/(fluxrelatiffond+                                 &
      sqrt(1.0_rsh+fluxrelatiffond*fluxrelatiffond)))

   effetnitdiat=c(iv_nutr_NO3)/(c(iv_nutr_NO3)+p_diat_kNO3+                                 &
                   (c(iv_nutr_NH4)*p_diat_kNO3/p_diat_kNH4))
   effetamdiat=c(iv_nutr_NH4)/(c(iv_nutr_NH4)+p_diat_kNH4+                                &
                   (c(iv_nutr_NO3)*p_diat_kNH4/p_diat_kNO3))
   effetazotediat=effetnitdiat+effetamdiat
   effetsilice=c(iv_nutr_SiOH)/(c(iv_nutr_SiOH)+p_diat_kSi)
   effetphosphorediat=c(iv_nutr_PO4)/(c(iv_nutr_PO4)+p_diat_kPO4)

   effetselnut=min(effetazotediat,effetsilice,effetphosphorediat)

   pslimdiat=min(effetlumierediat,effetselnut)
   rationdiat=p_diat_mumax*effetchaleur*pslimdiat

   diatmorteau=p_diat_mort*effetchaleur
   IF(c(iv_phyto_diat_N).le.0.04_rsh) diatmorteau=0.0_rsh

   xintvertdiat=14.e-3*rationdiat*c(iv_phyto_diat_N)*epn

   ! DINOFLAGELLES
   ! -------------
   ! Effetlumiere formulation de Steele

   ! Formulation de Smith integree sur la profondeur de la boite
   !  avec l hypothese que Ik=Iopt/2.5
   ! ............................................................
   fluxrelatifsurf=p_parradratio*alumplafond(km,im,jm)/(p_dino_iksmith/2.5_rsh)
   fluxrelatiffond=fluxrelatifsurf*attenuation(km,im,jm)
   effetlumieredino=1.0_rsh/epn/extinction(km,im,jm)*                     &
      log((fluxrelatifsurf+sqrt(1.0_rsh+fluxrelatifsurf*                  &
      fluxrelatifsurf))/(fluxrelatiffond+                                 &
      sqrt(1.0_rsh+fluxrelatiffond*fluxrelatiffond)))

   effetnitdino=c(iv_nutr_NO3)/(c(iv_nutr_NO3)+p_dino_kNO3+                                 &
                   (c(iv_nutr_NH4)*p_dino_kNO3/p_dino_kNH4))
   effetamdino=c(iv_nutr_NH4)/(c(iv_nutr_NH4)+p_dino_kNH4+                                &
                   (c(iv_nutr_NO3)*p_dino_kNH4/p_dino_kNO3))
   effetazotedino=effetnitdino+effetamdino 
   effetphosphoredino=c(iv_nutr_PO4)/(c(iv_nutr_PO4)+p_dino_kPO4)
   pslimdino=min(effetlumieredino,effetazotedino,effetphosphoredino)
   rationdino=p_dino_mumax*effetchaleur*pslimdino

   dinomorteau=p_dino_mort*effetchaleur
   IF(c(iv_phyto_dino_N).le.0.01_rsh) dinomorteau=0.0_rsh

   xintvertdino=14.e-3*rationdino*c(iv_phyto_dino_N)*epn

   ! NANOPICOPLANCTON
   ! ----------------
   ! Formulation de Smith integree sur la profondeur de la boite
   !  avec l hypothese que Ik=Iopt/2.5
   ! ............................................................
   fluxrelatifsurf=p_parradratio*alumplafond(km,im,jm)/(p_nano_iksmith/2.5_rsh)
   fluxrelatiffond=fluxrelatifsurf*attenuation(km,im,jm)
   effetlumierenano=1.0_rsh/epn/extinction(km,im,jm)*                     &
      log((fluxrelatifsurf+sqrt(1.0_rsh+fluxrelatifsurf*                  &
      fluxrelatifsurf))/(fluxrelatiffond+                                 &
      sqrt(1.0_rsh+fluxrelatiffond*fluxrelatiffond)))


   effetnitnano=c(iv_nutr_NO3)/(c(iv_nutr_NO3)+p_nano_kNO3+                                 &
                     (c(iv_nutr_NH4)*p_nano_kNO3/p_nano_kNH4))
   effetamnano=c(iv_nutr_NH4)/(c(iv_nutr_NH4)+p_nano_kNH4+                                &
                     (c(iv_nutr_NO3)*p_nano_kNH4/p_nano_kNO3))
   effetazotenano=effetnitnano+effetamnano

   effetphosphorenano=c(iv_nutr_PO4)/(c(iv_nutr_PO4)+p_nano_kPO4)
   pslimnano=min(effetlumierenano,effetazotenano,effetphosphorenano)
   rationnano=p_nano_mumax*effetchaleur*pslimnano

   IF ((c(iv_nutr_NO3).gt.1.e-30).or.(c(iv_nutr_NH4).gt.1.e-30)) THEN
           fractionno3diat=effetnitdiat/effetazotediat
           fractionnh4diat=effetamdiat/effetazotediat
           fractionno3dino=effetnitdino/effetazotedino
           fractionnh4dino=effetamdino/effetazotedino
           fractionno3nano=effetnitnano/effetazotenano
           fractionnh4nano=effetamnano/effetazotenano
   ELSE
           fractionno3diat=0.0_rsh
           fractionnh4diat=0.0_rsh
           fractionno3dino=0.0_rsh
           fractionnh4dino=0.0_rsh
           fractionno3nano=0.0_rsh
           fractionnh4nano=0.0_rsh
   ENDIF

   nanomorteau=p_nano_mort*effetchaleur
   IF (c(iv_phyto_nano_N).lt.0.04_rsh) nanomorteau=0.0_rsh

   xintvertnano=14.e-3*rationnano*c(iv_phyto_nano_N)*epn

   !+++++++++++++++++++++++++++++++++++++++++++++
   ! Croissance et mortalite du zooplancton
   !+++++++++++++++++++++++++++++++++++++++++++++

   ! MACROZOOPLANCTON
   ! ----------------
   captdiat=c(iv_phyto_diat_N)*p_mesz_captdiat        !en MolN.l-1
   captdino=c(iv_phyto_dino_N)*p_mesz_captdino        !en MolN.l-1
   captmicrozoo=c(iv_zoo_micr_N)*p_mesz_captmicz   !en gC.l-1
   rNC=1.0_rsh/(12.0_rsh*p_zoo_CNratio)  !en molN.gC-1
   captmicrozooN=captmicrozoo*rNC    ! en MolN.l-1
   IF (captdiat.ne.0.0_rsh .or. captdino.ne.0.0_rsh .or. captmicrozooN.ne.0.0_rsh) THEN
   zdiat=max(0.0_rsh,(captdiat+captdino+captmicrozooN)-p_micz_thrnano) ! en gMolN.l-1
   fmesozoo=1.0_rsh-exp(-p_mesz_kivlev*zdiat)            ! food lim [0-1]
   rationmesozoo=p_mesz_mumax*effetchaleur*fmesozoo     ! specif growth rate d-1
   assimilzoo=0.3_rsh*(3.0_rsh-0.67_rsh*fmesozoo)       ! ass rate s.u.
   broumesozoodiat=(rationmesozoo*captdiat)/(captdiat+captdino+captmicrozooN)   ! d-1
   broumesozoodino=(rationmesozoo*captdino)/(captdiat+captdino+captmicrozooN)   ! d-1
   broumesozoomicrozoo=(rationmesozoo*captmicrozooN)/                    &
                         (captdiat+captdino+captmicrozooN)                      ! d-1
   ELSE
   fmesozoo=0.0_rsh
   assimilzoo=0.0_rsh
   rationmesozoo=0.0_rsh
   broumesozoodiat=0.0_rsh
   broumesozoodino=0.0_rsh
   broumesozoomicrozoo=0.0_rsh
   ENDIF

   excretionmesozoo=p_mesz_excret*effetchaleur*fmesozoo ! specif excret rate d-1
!   txmortmesozoo=max(p_mesz_mort1*effetchaleur,p_mesz_mort2*effetchaleur*c(iv_zoo_meso_N)) ! specif mort rate d-1
   txmortmesozoo=p_mesz_mort1*effetchaleur+p_mesz_mort2*effetchaleur*c(iv_zoo_meso_N) ! specif mort rate d-1

   IF (c(iv_zoo_meso_N).lt.0.01_rsh)  txmortmesozoo=0.0_rsh ! ATTENTION bidouille


   ! MICROZOOPLANCTON
   ! ----------------

   captnano=c(iv_phyto_nano_N)*p_micz_captnano                     !en MolN.l-1
   captdet=min(c(iv_detr_N),c(iv_detr_P)*p_phyto_NPratio)*p_micz_captdet     !en MolN.l-1
   captdiat=c(iv_phyto_diat_N)*p_micz_captdiat                      !en MolN.l-1
   IF (captnano.ne.0.0_rsh .or. captdet.ne.0.0_rsh .or. captdiat.ne.0.0_rsh) THEN
      fmicrozoo=(captnano+captdet+captdiat)/(p_micz_kgraz+captnano+captdet+captdiat)   ! food lim [0-1]
      rationmicrozoo=p_micz_mumax*effetchaleur*fmicrozoo      ! specif growth rate d-1
      assimilmicrozoo=0.3_rsh*(3.0_rsh-0.67_rsh*fmicrozoo)        ! ass rate s.u.
      broumicrozoonano=(rationmicrozoo*captnano)/(captnano+captdet+captdiat)  ! d-1
      broumicrozoodet=(rationmicrozoo*captdet)/(captnano+captdet+captdiat)    ! d-1
      broumicrozoodiat=(rationmicrozoo*captdiat)/(captnano+captdet+captdiat)  ! d-1
   ELSE
      fmicrozoo=0.0_rsh
      assimilmicrozoo=0.0_rsh
      rationmicrozoo=0.0_rsh
      broumicrozoonano=0.0_rsh
      broumicrozoodet=0.0_rsh
      broumicrozoodiat=0.0_rsh
   ENDIF

   excretionmicrozoo=p_micz_excret*effetchaleur*fmicrozoo ! specif excret rate d-1
   txmortmicrozoo=p_micz_mort*effetchaleur+p_mesz_mort2*effetchaleur*c(iv_zoo_micr_N)! specif excret rate d-1

   IF (c(iv_zoo_micr_N).lt.0.01_rsh) txmortmicrozoo=0.0_rsh ! ATTENTION bidouille


   !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ! Remineralisation de la matiere detritique dans l eau
   !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

   reminazdeteau=p_N_remin*effetchaleur
   xnitrifeau=p_nitrifwat*effetchaleur

   reminpdeteau=p_P_remin*effetchaleur

   dissolsiliceeau=p_Si_diss*effetchaleur

   !+++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ! Adsorption et desorption du phosphore dans l eau
   !+++++++++++++++++++++++++++++++++++++++++++++++++++++++

   ! adsorption
   saturmesp=max(0.0_rsh,(p_P_adsormaxspim*c(iv_spim)-c(iv_nutr_Pads)))  !c(iv_spim) en g/l
   adsorpeau=p_P_adsor*saturmesp
   cmaxadsorp=adsorpeau*c(iv_nutr_PO4)*dtbio/86400.0_rlg
   IF (cmaxadsorp.ge.c(iv_nutr_PO4))  adsorpeau=0.9_rlg*86400.0_rlg/dtbio

   ! desorption
   IF(c(iv_spim).ne.0.0_rsh) THEN
     varads=min(1.0_rsh,c(iv_nutr_Pads)/(p_P_adsormaxspim*c(iv_spim)))
   ELSE
     varads=0.0_rsh
   END IF
   desorpeau=p_P_desor*varads
   cmaxdesorp=desorpeau*c(iv_nutr_Pads)*dtbio/86400.0_rlg
   IF (cmaxdesorp.ge.c(iv_nutr_Pads)) desorpeau=0.9_rlg*86400.0_rlg/dtbio

   !++++++++++++++++++++++++++++++++++++++++++++++++
   ! Processus lies a l oxygene dissous (inemploye)
   !++++++++++++++++++++++++++++++++++++++++++++++++

   ! production et consommation d oxygene dans l eau
   ! -----------------------------------------------
   !ophotos=p_phyto_photoratio*0.212*(rationdiat*c(iv_phyto_diat_N)+rationdino*c(iv_phyto_dino_N))
   !oresphyto=(0.01+(1-effetlumierediat)*0.1)*c(iv_phyto_diat_N)*0.212              &
   !         +(0.01+(1-effetlumieredino)*0.1)*c(iv_phyto_dino_N)*0.212
   !orespzoo=p_zoo_resp*c(iv_phyto_nano_N)*rNC
   !oremineau=reminazdeteau*c(iv_detr_N)*0.212
   !onitrifeau=xnitrifeau*c(iv_zoo_meso_N)*0.064

   ! reoxygenation
   ! -------------
   !gst=-173.4292+249.6329*100./tempabs+143.3483*log(tempabs/100.)    &
   !      -21.8492*tempabs/100.+sali*(-0.033096+0.014259*tempabs/100. &
   !      -0.0017*(tempabs/100.)**2)
   !o2sat=1.429*exp(gst)
   !oair=(0.64+0.0256*(ventvent(im,jm)/0.447)**2)/p_diat_iksmith*(o2sat-c(iv_nutr_NH4))
   !oair=oair*ibsurf


   !+++++++++++++++++++++++++++++++++++++++++++++++
   ! EQUATIONS D EVOLUTION  ! concentration en jour-1
   !+++++++++++++++++++++++++++++++++++++++++++++++

   ! Evolution du nitrate
   ! --------------------
   dc(iv_nutr_NO3)=xnitrifeau*c(iv_nutr_NH4)-fractionno3diat*rationdiat*c(iv_phyto_diat_N)            &
     -fractionno3dino*rationdino*c(iv_phyto_dino_N)-fractionno3nano*rationnano*c(iv_phyto_nano_N)

   ! Evolution de la silice dissoute
   ! -------------------------------
   dc(iv_nutr_SiOH)=dissolsiliceeau*c(iv_detr_Si)-rapsiaz*rationdiat*c(iv_phyto_diat_N)

   ! Evolution du phosphate dissous
   ! ------------------------------
   dc(iv_nutr_PO4)=desorpeau*c(iv_nutr_Pads)-adsorpeau*c(iv_nutr_PO4)+reminpdeteau*c(iv_detr_P)            &
         -rappaz*(rationdiat*c(iv_phyto_diat_N)+rationdino*c(iv_phyto_dino_N)+rationnano*c(iv_phyto_nano_N)) &
         +rappaz*((excretionmesozoo*c(iv_zoo_meso_N)+excretionmicrozoo*c(iv_zoo_micr_N))*rNC)
   ! Evolution du phosphate adsorbe
   ! ------------------------------
   dc(iv_nutr_Pads)=adsorpeau*c(iv_nutr_PO4)-desorpeau*c(iv_nutr_Pads)

   ! Evolution de l azote des diatomees
   ! ----------------------------------
   dc(iv_phyto_diat_N)=c(iv_phyto_diat_N)*(rationdiat-diatmorteau)-broumesozoodiat*c(iv_zoo_meso_N)*rNC    &
           -broumicrozoodiat*c(iv_zoo_micr_N)*rNC

   ! Evolution de l azote des dinoflagelles
   ! --------------------------------------
   dc(iv_phyto_dino_N)=(rationdino-dinomorteau)*c(iv_phyto_dino_N)-broumesozoodino*c(iv_zoo_meso_N)*rNC

   ! Evolution de l azote detritique
   ! -------------------------------
   dc(iv_detr_N)=diatmorteau*c(iv_phyto_diat_N)+dinomorteau*c(iv_phyto_dino_N)+nanomorteau*c(iv_phyto_nano_N)         &
         -reminazdeteau*c(iv_detr_N)                                         &
         +((txmortmesozoo+(1.0_rsh-assimilzoo)*rationmesozoo)*c(iv_zoo_meso_N)        &
         +(txmortmicrozoo+(1.0_rsh-assimilmicrozoo)*rationmicrozoo         &
         -broumicrozoodet)*c(iv_zoo_micr_N))*rNC

   ! Evolution de la silice particulaire
   ! -----------------------------------
   dc(iv_detr_Si)=rapsiaz*diatmorteau*c(iv_phyto_diat_N)-dissolsiliceeau*c(iv_detr_Si)               &
         +broumesozoodiat*c(iv_zoo_meso_N)*rNC*rapsiaz                          &
         +broumicrozoodiat*c(iv_zoo_micr_N)*rNC*rapsiaz

   ! Evolution du phosphore detritique
   ! ---------------------------------
   dc(iv_detr_P)=-reminpdeteau*c(iv_detr_P)                                          &
         +(diatmorteau*c(iv_phyto_diat_N)+dinomorteau*c(iv_phyto_dino_N)+nanomorteau*c(iv_phyto_nano_N))*rappaz     &
         +((txmortmesozoo+(1.0_rsh-assimilzoo)*rationmesozoo)*c(iv_zoo_meso_N)        &
         +(txmortmicrozoo+(1.0_rsh-assimilmicrozoo)*rationmicrozoo)*c(iv_zoo_micr_N)) &
          *rNC*rappaz                                                      &
         -broumicrozoodet*c(iv_zoo_micr_N)*rNC*rappaz

   ! Evolution de l oxygene dissous (inemploye)
   ! ------------------------------
   !dc(iv_nutr_NH4)=ophotos+oair-perteo2

   ! Evolution de l ammonium
   ! -----------------------
   dc(iv_nutr_NH4)=reminazdeteau*c(iv_detr_N)                           &
          -fractionnh4diat*rationdiat*c(iv_phyto_diat_N)             &
          -fractionnh4dino*rationdino*c(iv_phyto_dino_N)             &
          -fractionnh4nano*rationnano*c(iv_phyto_nano_N)            &
          -xnitrifeau*c(iv_nutr_NH4)                            &
          +(excretionmesozoo*c(iv_zoo_meso_N)+excretionmicrozoo*c(iv_zoo_micr_N))*rNC

   ! Evolution de l azote du mesozooplancton (microgC/l)
   ! -------------------------------------------------
   dc(iv_zoo_meso_N)=(rationmesozoo*assimilzoo-excretionmesozoo-txmortmesozoo)*c(iv_zoo_meso_N)

   ! Evolution de l''azote du nano_pico_phytoplancton  (micromolN.l-1)
   ! -----------------------------------------------------------------
   dc(iv_phyto_nano_N)=c(iv_phyto_nano_N)*(rationnano-nanomorteau)-broumicrozoonano*c(iv_zoo_micr_N)*rNC

   ! Evolution de la biomasse de microzooplancton  (microgC.l-1)
   ! -----------------------------------------------------------
   dc(iv_zoo_micr_N)=c(iv_zoo_micr_N)*(rationmicrozoo*assimilmicrozoo-excretionmicrozoo)       &
          -txmortmicrozoo*c(iv_zoo_micr_N)                                          &
          -broumesozoomicrozoo*c(iv_zoo_meso_N)

   ! Evolution de la biomasse de MES  (g.l-1)
   ! -----------------------------------------------------------
   dc(iv_spim)=0.0_rsh

   ! Production cumulee des diatomees
   ! --------------------------------
   dc(iv_phyto_diat_pp)=xintvertdiat

   ! Production cumulee des dinoflagelles
   ! ------------------------------------
   dc(iv_phyto_dino_pp)=xintvertdino

   ! Production cumulee des nanoflagelles
   ! ------------------------------------
   dc(iv_phyto_nano_pp)=xintvertnano

  ! ++++++++++++++++++++++++++++++++++++++++++++++++
  !   ENREGISTREMENT VARIABLES DIAGNOSTIQUES
  ! ++++++++++++++++++++++++++++++++++++++++++++++++

   diag_3d_wat(irk_diag(id_diat_limlight),km,im,jm)=effetlumierediat
   diag_3d_wat(irk_diag(id_diat_limN),km,im,jm)=effetazotediat
   diag_3d_wat(irk_diag(id_diat_limSi),km,im,jm)=effetsilice
   diag_3d_wat(irk_diag(id_diat_limP),km,im,jm)=effetphosphorediat
   diag_3d_wat(irk_diag(id_dino_limlight),km,im,jm)=effetlumieredino
   diag_3d_wat(irk_diag(id_dino_limN),km,im,jm)=effetazotedino
   diag_3d_wat(irk_diag(id_dino_limP),km,im,jm)=effetphosphoredino
   diag_3d_wat(irk_diag(id_nano_limlight),km,im,jm)=effetlumierenano
   diag_3d_wat(irk_diag(id_nano_limN),km,im,jm)=effetazotenano
   diag_3d_wat(irk_diag(id_nano_limP),km,im,jm)=effetphosphorenano
   diag_3d_wat(irk_diag(id_extinctioncoeff),km,im,jm)=extinction(km,im,jm)

   !   chlorophylle [mgCChl/m3]
   diag_3d_wat(irk_diag(id_totalchl),km,im,jm)=(c(iv_phyto_diat_N)+c(iv_phyto_dino_N)+c(iv_phyto_nano_N))*p_phyto_ChlNratio

   !   production cumulee dans le temps et sur la verticale de chlorophylle
   diag_2d(irk_diag(id_columnprodtotal),im,jm)=diag_2d(irk_diag(id_columnprodtotal),im,jm)+ epn  &
     *12.e-3*p_phyto_CNratio*(rationdiat*c(iv_phyto_diat_N)+rationdino*c(iv_phyto_dino_N)+rationnano*c(iv_phyto_nano_N))

   !   vitesses de chute des diatomees et du detritique
   diag_3d_wat(irk_diag(id_diatsettling),km,im,jm)=ws3(km,iv_phyto_diat_N,im,jm)*86400.0_rsh
   diag_3d_wat(irk_diag(id_detsettling),km,im,jm)=ws3(km,iv_detr_N,im,jm)*86400.0_rsh

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! saisir le code avant cette limite
  ! =================================
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  END SUBROUTINE incellwat_nut4phy3zoo2

   !!======================================================================

#endif
 END MODULE
