 MODULE biolodynzwat

#if defined key_substance && defined key_biolo
   !!======================================================================
   !!                   ***  MODULE  biolodyn  ***
   !! Biologic dynamics: 
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters,   ONLY : rsh,rlg,nb_var,limin,limax,ljmin,ljmax,kmax,  &
                            epsilon,lchain,valmanq,imin,jmin,limaxp1,ljmaxp1, &
                            ljminp2,ljmaxm1,liminm1,limaxp2,ljminm1,ljmaxp2,  &
                            liminm2,ljminm2
   USE comvars2d,    ONLY : h0fond,id,ig,jb,jh,h0,jjulien,t,cdate,   &
                            iscreenlog,imois,ijour,ian,iheure,iminu,isec
   USE comvars3d,    ONLY : sig
   USE comsubstance, ONLY : irk_mod,dtbio,extinction,   &
                            tdebsubs,nvp,nvpc,nv_adv,ws3,xws,dcdt,cv_wat
   USE sflxatm,      ONLY : rad
   USE bioloinit
   USE toolgeom,     ONLY : f_dzu,f_dzw

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC biolo_dyn_zwat        ! routine called by stepu.F90 and stepv.F90

   !! * Interface
   INTERFACE biolo_dyn_zwat
!     MODULE PROCEDURE verti_dinodiatzoo
     MODULE PROCEDURE verti_nut4phy3zoo2
   END INTERFACE

   !! * Shared module variables
!   REAL(KIND=rsh), PUBLIC      :: txfiltbenth    ! forcage broutage de fond

   !! * Private variables

 CONTAINS

   !!======================================================================

  SUBROUTINE verti_nut4phy3zoo2(xe,cv_wat,sal,temp)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE  ***
   !&E
   !&E ** Purpose : calcul des effets inter-mailles non-hydrauliques
   !&E              (a estimer avant de calculer les variations a l interieur
   !&E               d une cellule donnee)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  (M. Sourisseau A. Menesguen) Original code
   !&E       !  2006-09  (V. Garnier)  module - mise en forme
   !&E       !  2006-09  (T. Odaka)  parallelisation openMP et MPI
   !&E       !  2008-01 (M. Sourisseau) evolutions
   !&E       !  2008-05 (M. Sourisseau) link between microzoo and diatoms
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : obc_cv_ktherm_e,obc_cv_ktherm_w,    &
                            obc_cv_ktherm_n,obc_cv_ktherm_s,    &
                            alumplafond,attenuation
   USE bioloinit,    ONLY : diag_2d,diag_3d_wat,irk_diag,       &
                            id_spim_satused,id_spm_total,       &
                            id_extinctioncoeff,id_diat_max,     &
                            id_diat_datemax,id_diat_datemax,id_diat_depthmax, &
                            id_dino_max,id_dino_datemax,id_dino_depthmax,     &
                            id_nano_max,id_nano_datemax,id_nano_depthmax,     &
                            id_gradsali_max,id_gradsali_depthmax,             &
                            id_gradtemp_max,id_gradtemp_depthmax,             &
                            id_diat_columnprod,id_dino_columnprod,id_nano_columnprod

   USE comsubstance, ONLY : phytomoy,phytocarre
#ifdef key_sedim_anc
   USE sedimento,     ONLY : ksma,dzs
#endif
#ifdef key_messat
   USE comsubstance, ONLY : messat,idateinf,idatesup,idimt_messat, &
                            t_clim_messat,date_s_annee
   USE comsubstance, ONLY : l_messat_clim,l_messat_obs,filemessat_obs,file_mes_obs_path
#endif

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in )   :: xe
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT( in )   :: sal,temp
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT( inout )   :: cv_wat

   !! * Local declarations
   INTEGER                  ::  iv,i,j,k,ivpc         ! loop indexes
   !INTEGER                  ::  ijour,imois,ian,iheure,iminu,isec

   REAL(KIND=rsh),DIMENSION(nb_var,kmax,limin:limax,ljmin:ljmax)      :: cv_wat_pos
   REAL(KIND=rsh)   :: dzu,dzw

                         ! intermediate variable
                            ! --------------------------------------------
   REAL(KIND=rsh)           :: cmes,effetchaleur,rdet

                            ! diagnoses
                            ! ---------
   INTEGER                  :: kmaxdiat,kmaxdino,kmaxnano,kminO2,kdcdz
   REAL(KIND=rsh)           :: xmaxdiat,datemaxdiat,xmaxdino,datemaxdino, &
                               xmaxnano,datemaxnano,xminO2,dateminO2,     &
                               dcdzmax,dcdz,cumuldiat,cumuldino,cumulnano,&
                               xmaxchloro,kmaxchloro,datemaxchloro,       &
                               d,voleau,wchutedet

                            ! variables diatomees
                            ! -------------------
   REAL(KIND=rsh)           :: effetselnut,effetazotediat,effetnitdiat,effetamdiat,    &
                               effetsilice,effetphosphorediat,effetchutedia,diatmorteau

                            ! variables dinoflagelles
                            ! -------------------
   REAL(KIND=rsh)           :: dinomorteau

                            ! variables nanoflagelles
                            ! -------------------
   REAL(KIND=rsh)           :: nanomorteau

                            ! variables zooplancton
                            ! -------------------
   REAL(KIND=rsh)           ::rationmesozoo,assimilzoo,txmortmesozoo,fmesozoo,      &
                              zdiat,captdiat,captdino,captmicrozoo,                 &
                              captmicrozooN,rNC,fmicrozoo,txmortmicrozoo,captnano,  &
                              captdet,rationmicrozoo,assimilmicrozoo

                            ! forcage broutage de fond
                            ! ------------------------
!   REAL(KIND=rsh)           :: foncsinus

                            ! calcul moyenne et ecart-type phyto surface
                            ! ------------------------------------------
   REAL(KIND=rsh)           :: epcalc,cphyto
   REAL(KIND=rlg)           :: duree

#if defined key_messat
                            ! forcage MES par images satellites
                            ! ---------------------------------
   REAL(KIND=rsh), DIMENSION(limin:limax,ljmin:ljmax)   :: forcmes
   CHARACTER(LEN=19)             :: date_start,tool_sectodat
   REAL(KIND=rsh)           :: interp
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_messat
      IF (l_messat_clim .EQV. .TRUE.) THEN

       IF (date_s_annee .NE. jjulien*24*60*60) THEN ! Definit des dates inf et sup pour l interp
        date_s_annee= jjulien*24*60*60              ! de la climato
        IF (idateinf .LT. idatesup) THEN
         IF (date_s_annee .GT. INT(t_clim_messat(idatesup))) THEN
          idateinf=idatesup
          IF (idatesup .NE. idimt_messat) THEN
           idatesup=idatesup+1
          ELSE
           idatesup=1
          END IF
         END IF
        ELSE
         IF ((date_s_annee .GT. INT(t_clim_messat(idatesup))) .AND.   &
               (date_s_annee .LT. INT(t_clim_messat(idateinf)))) THEN
          idateinf=1
          idatesup=idatesup+1
         END IF
        END IF

        WRITE(iscreenlog,*) 'Simul a la date',date_s_annee,jjulien,imois,ijour
        WRITE(iscreenlog,*) 'interpolation entre ',idateinf,'et',idatesup
        WRITE(iscreenlog,*) 'soit, en s, entre ',INT(t_clim_messat(idateinf)), &
                               ' et ',INT(t_clim_messat(idatesup))

       END IF !endif on date_s_annee

       IF (idatesup .GT. idateinf)  THEN
        interp= (date_s_annee -t_clim_messat(idateinf))/        &
                 (t_clim_messat(idatesup)-t_clim_messat(idateinf))
       ELSE
        IF (date_s_annee .GE. INT(t_clim_messat(idateinf))) THEN
        interp= (date_s_annee-t_clim_messat(idateinf))/&
                  ((366*24*60*60+t_clim_messat(idatesup))-t_clim_messat(idateinf))
        ELSE
        interp=(date_s_annee+366*24*60*60-t_clim_messat(idateinf))/ &
                  ((366*24*60*60+t_clim_messat(idatesup))-t_clim_messat(idateinf))
        END IF
       END IF

      END IF  !endif on l_messat_clim

      IF (l_messat_obs .EQV. .TRUE.) THEN
       file_mes_obs_path=TRIM(filemessat_obs)//'/'//CHAR(ian)//'/'//    &
                         CHAR(ian)//CHAR(imois)//CHAR(ijour)//'.nc'
      END IF

#endif

!$OMP PARALLEL DEFAULT(none) SHARED(ig,id,jb,jh,nvp,nvpc,nv_adv,h0,xe,   &
!$OMP& h0fond,ljmin,ljmax,limin,limax,                                   &
!$OMP& iv_spim,iv_nutr_NO3,p_diat_kNO3,iv_nutr_NH4,p_diat_kNH4,          &
!$OMP& iv_nutr_SiOH,p_diat_kSi,iv_nutr_PO4,p_diat_kPO4,iv_phyto_diat_N,  &
!$OMP& iv_detr_N,iv_detr_Si,iv_detr_P,id_spm_total,p_extincspim,         &
!$OMP& p_extincwat,iv_phyto_dino_N,iv_phyto_nano_N,p_phyto_ChlNratio,    &
!$OMP& p_extincChl1,p_extincChl2,id_extinctioncoeff,id_diat_max,         &
!$OMP& id_diat_datemax,id_diat_depthmax,id_dino_datemax,id_dino_depthmax,&
!$OMP& id_dino_max,id_nano_max,id_nano_datemax,id_nano_depthmax,         &
!$OMP& id_gradsali_max,id_gradsali_depthmax,id_gradtemp_max,             &
!$OMP& id_gradtemp_depthmax,iv_phyto_diat_pp,iv_phyto_dino_pp,           &
!$OMP& iv_phyto_nano_pp,id_diat_columnprod,id_dino_columnprod,           &
!$OMP& id_nano_columnprod,                                               &
#ifdef key_messat
!$OMP& messat,forcmes,idateinf,idatesup,interp,l_messat_clim,            &
!$OMP& l_messat_obs,id_spim_satused,                                     &
#endif
!$OMP& cv_wat,irk_mod,xws,diag_3d_wat,irk_diag,cv_wat_pos,               &
!$OMP& extinction,attenuation,alumplafond,rad,ws3,dcdt,t,                &
#ifdef key_sedim_anc
!$OMP& ksma,dzs, &
#endif
!$OMP& obc_cv_ktherm_s,obc_cv_ktherm_n,obc_cv_ktherm_w,obc_cv_ktherm_e,  &
!$OMP& diag_2d,sig,jjulien,sal,temp)                                     &
!$OMP& PRIVATE(i,j,k,iv,d,effetazotediat,effetsilice,effetphosphorediat, &
!$OMP& effetselnut,effetchutedia,cmes,cphyto,dzu,dzw,                    &
!$OMP& diatmorteau,dinomorteau,nanomorteau,                              &
!$OMP& assimilzoo,txmortmesozoo,fmesozoo,rationmesozoo,                  &
!$OMP& zdiat,captdiat,captdino,captmicrozoo,captmicrozooN,               &
!$OMP& rNC,                                                              &
!$OMP& fmicrozoo,captnano,captdet,                                       &
!$OMP& assimilmicrozoo,rationmicrozoo,txmortmicrozoo,                    &
!$OMP& kmaxdiat,xmaxdiat,datemaxdiat,xmaxdino,kmaxdino,datemaxdino,      &
!$OMP& dcdzmax,kdcdz,dcdz,cumuldiat,cumuldino,epcalc,duree,              &
!$OMP& effetnitdiat, effetamdiat,effetchaleur,rdet,wchutedet,            &
!$OMP& xmaxnano,kmaxnano,datemaxnano,cumulnano)

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljmin,ljmax

     cv_wat_pos(:,:,limin:limax,j)=0.5_rsh*(cv_wat(:,:,limin:limax,j)+ABS(cv_wat(:,:,limin:limax,j)))

     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       d=h0(i,j)+xe(i,j)
       IF (d.LE.h0fond) THEN
         ws3(:,1:nv_adv,i,j)=0.0_rsh      ! vitesses de chute ws3 nulles
#if defined key_messat
         forcmes(i,j)=valmanq
#endif
       ELSE

#if defined key_messat
      IF (l_messat_obs .EQV. .TRUE.) THEN

      END IF !endifon l_messat_obs

      IF (l_messat_clim .EQV. .TRUE.) THEN
        forcmes(i,j)=messat(i,j,idateinf)+                                &
               (messat(i,j,idatesup)-messat(i,j,idateinf))*interp

      END IF  !endif on l_messat_clim
#endif

         DO k=1,kmax

    ! calcul de la vitesse de chute des MES minerales en chaque maille
    ! ----------------------------------------------------------------
           ws3(k,iv_spim,i,j)=(xws(1,iv_spim)+xws(2,iv_spim))/2.0_rsh

   ! ==================================================================
   ! recalcul vitesses de chute ws3 en fonction de facteurs biologiques
   ! ==================================================================


   ! calcul de la vitesse de chute des diatomees en chaque maille
   ! ------------------------------------------------------------
           effetnitdiat=cv_wat_pos(iv_nutr_NO3,k,i,j)                        &
                         /(cv_wat_pos(iv_nutr_NO3,k,i,j)+p_diat_kNO3+       &
               cv_wat_pos(iv_nutr_NH4,k,i,j)+p_diat_kNO3/p_diat_kNH4)
           effetamdiat=cv_wat_pos(iv_nutr_NH4,k,i,j)                        &
                         /(cv_wat_pos(iv_nutr_NH4,k,i,j)+p_diat_kNH4+      &
               cv_wat_pos(iv_nutr_NO3,k,i,j)+p_diat_kNH4/p_diat_kNO3)
           effetazotediat=effetnitdiat+effetamdiat

           effetsilice=cv_wat_pos(iv_nutr_SiOH,k,i,j)/                        &
                      (cv_wat_pos(iv_nutr_SiOH,k,i,j)+p_diat_kSi)
           effetphosphorediat=cv_wat_pos(iv_nutr_PO4,k,i,j)                  &
                             /(cv_wat_pos(iv_nutr_PO4,k,i,j)+p_diat_kPO4)
           effetselnut=MIN(effetazotediat,effetsilice,                  &
                                          effetphosphorediat)
           IF (effetselnut.LT.0.0_rsh) THEN
             WRITE(*,*) 'effetselnut:',k,i,j,temp(k,i,j)                &
            ,effetselnut,effetazotediat,effetsilice,effetphosphorediat  &
            ,cv_wat_pos(iv_nutr_NO3,k,i,j),cv_wat_pos(iv_nutr_NH4,k,i,j)         &
            ,cv_wat_pos(iv_nutr_SiOH,k,i,j),cv_wat_pos(iv_nutr_PO4,k,i,j)          &
            ,dcdt(iv_nutr_NO3,k,i,j),dcdt(iv_nutr_NH4,k,i,j)             &
            ,dcdt(iv_nutr_SiOH,k,i,j),dcdt(iv_nutr_PO4,k,i,j)             
           ! pause
           END IF
           effetselnut=MAX(0.0_rsh,effetselnut)
           effetchutedia=effetselnut**0.2_rsh
           ws3(k,iv_phyto_diat_N,i,j)=(xws(1,iv_phyto_diat_N)*effetchutedia+      &
                                  xws(2,iv_phyto_diat_N)*(1.0_rsh-effetchutedia))

   ! calcul de la vitesse de chute du materiel detritique en chaque maille
   ! ---------------------------------------------------------------------
!cmarc           IF (cv_wat_pos(iv_zoo_meso_N,k,i,j).gt.0.0_rsh) THEN
!cmarc               effetchaleur=exp(p_T_effect*temp(k,i,j))
!cmarc               diatmorteau=p_diat_mort*effetchaleur
!cmarc               dinomorteau=p_dino_mort*effetchaleur
!cmarc               nanomorteau=p_nano_mort*effetchaleur
!cmarc               captdiat=cv_wat_pos(iv_phyto_diat_N,k,i,j)
!cmarc               captdino=cv_wat_pos(iv_phyto_dino_N,k,i,j)*0.1_rsh
!cmarc               captmicrozoo=cv_wat_pos(iv_zoo_micr_N,k,i,j)*0.7_rsh
!cmarc               rNC=1.0_rsh/(12.0_rsh*p_zoo_CNratio)
!cmarc               captmicrozooN=captmicrozoo*rNC
!cmarc               rationmesozoo=0.0_rsh
!cmarc               IF (captdiat.ne.0.0_rsh .or. captdino.ne.0.0_rsh .or. captmicrozooN.ne.0.0_rsh) THEN
!cmarc               zdiat=max(0.0_rsh,(captdiat+captdino+captmicrozooN)-p_micz_thrnano)
!cmarc               fmesozoo=1-exp(-p_mesz_kivlev*zdiat)
!cmarc               rationmesozoo=p_mesz_mumax*effetchaleur*fmesozoo
!cmarc               assimilzoo=0.3_rsh*(3.0_rsh-0.67_rsh*fmesozoo)
!cmarc               ELSE
!cmarc               assimilzoo=0.0_rsh
!cmarc               ENDIF
!cmarc               txmortmesozoo=max(p_mesz_mort1*effetchaleur,p_mesz_mort2*effetchaleur*cv_wat_pos(iv_zoo_meso_N,k,i,j))
!cmarc               IF (cv_wat_pos(iv_zoo_meso_N,k,i,j).lt.1.0_rsh)  txmortmesozoo=0.0_rsh
!cmarc               captnano=cv_wat_pos(iv_phyto_nano_N,k,i,j)
!cmarc               captdet=min(cv_wat_pos(iv_detr_N,k,i,j),cv_wat_pos(iv_detr_P,k,i,j)*p_phyto_NPratio)*0.8_rsh
!cmarc               rationmicrozoo=0.0_rsh
!cmarc               IF (captnano.ne.0.0_rsh .or. captdet.ne.0.0_rsh) THEN
!cmarc               fmicrozoo=(captnano+captdet)/(p_micz_kgraz+captnano+captdet)
!cmarc               rationmicrozoo=p_micz_mumax*effetchaleur*fmicrozoo
!cmarc               assimilmicrozoo=0.3_rsh*(3.0_rsh-0.67_rsh*fmicrozoo)
!cmarc               ELSE
!cmarc               assimilmicrozoo=0.0_rsh
!cmarc               ENDIF
!cmarc               txmortmicrozoo=p_micz_mort*effetchaleur

!cmarc               rdet=(diatmorteau*cv_wat_pos(iv_phyto_diat_N,k,i,j)               &
!cmarc                     +dinomorteau*cv_wat_pos(iv_phyto_dino_N,k,i,j)              &
!cmarc                     +nanomorteau*cv_wat_pos(iv_phyto_nano_N,k,i,j))/           &
!cmarc                 (((1.0_rsh-assimilzoo)*rationmesozoo+txmortmesozoo+epsilon)* &
!cmarc                     cv_wat_pos(iv_zoo_meso_N,k,i,j)+(txmortmicrozoo+         &
!cmarc                     rationmicrozoo*(1-assimilmicrozoo)*                &
!cmarc                     cv_wat_pos(iv_zoo_micr_N,k,i,j))*rNC)
   !concernant la MOD : si elle est majoritairement composee de phytopk
!cmarc               IF (rdet.ge.10.0_rsh) THEN
!cmarc               wchutedet=p_detphy_wsed
!cmarc               ENDIF
   !concernant la MOD : si elle est majoritairement composee de zoopk
!cmarc               IF (rdet.lt.0.1_rsh) THEN
!cmarc               wchutedet=p_detzoo_wsed
!cmarc               ENDIF
!cmarc               IF ((rdet.lt.10.0_rsh).and.(rdet.ge.0.1_rsh)) THEN
!cmarc               wchutedet=p_detzoo_wsed*(1.0_rsh/(rdet+1.0_rsh))+          &
!cmarc                         p_detphy_wsed*(1.0_rsh-(1.0_rsh/(rdet+1.0_rsh)))
!cmarc               ENDIF
!cmarc           ELSE
!cmarc               wchutedet=p_detphy_wsed
!cmarc           ENDIF

           wchutedet=(xws(1,iv_detr_N)+xws(2,iv_detr_N))/2.0_rsh

           ws3(k,iv_detr_N,i,j)=wchutedet
           ws3(k,iv_detr_Si,i,j)=wchutedet
           ws3(k,iv_detr_P,i,j)=wchutedet

           DO iv=nvp+1,nv_adv
             ws3(k,iv,i,j)=0.0_rsh
           END DO

#ifdef key_sedim_anc
   ! on ne depose pas le materiel detritique si dzs<0.0005
   ! -----------------------------------------------------
           IF ((ksma(i,j).NE.0.0_rsh .AND. dzs(ksma(i,j),i,j).LE.0.0005_rsh) .OR. (ksma(i,j).EQ.0.0_rsh)) THEN
             ws3(1,iv_phyto_diat_N,i,j)=0.0_rsh
             ws3(1,iv_detr_N,i,j)=0.0_rsh
             ws3(1,iv_detr_Si,i,j)=0.0_rsh
             ws3(1,iv_detr_P,i,j)=0.0_rsh
             ws3(1,iv_spim,i,j)=0.0_rsh
           END IF
#endif

   ! ==================================
   ! Calcul du coefficient d extinction
   ! ==================================

   ! * Partie minerale:
   !   ---------------
           cmes=0.0_rsh
           DO iv=1,nvpc
             cmes=cmes+cv_wat_pos(iv,k,i,j)*1000.0_rsh  !cwat passe en mg/l
           END DO

#if defined key_messat
              IF (forcmes(i,j)*1000.0_rsh .GT. cmes) THEN  !forcage en g/l
                diag_3d_wat(irk_diag(id_spim_satused),k,i,j)=1.0_rsh
              ELSE
                diag_3d_wat(irk_diag(id_spim_satused),k,i,j)=0.0_rsh
              END IF
              cmes=MAX(cmes,forcmes(i,j)*1000.0_rsh)
#endif
           cmes=0.5_rsh*(cmes+ABS(cmes))     !Eq en mg/l

           diag_3d_wat(irk_diag(id_spm_total),k,i,j)=cmes

   ! calcul de l extinction
   ! ----------------------

   ! - Partie due au mineral (formule Gohin):
   ! --------------------------------------
           extinction(k,i,j)=p_extincspim*(cmes+epsilon)+p_extincwat

   ! - Partie due au phytoplancton (formule Gohin):
   ! --------------------------------------------
           cphyto=(cv_wat_pos(iv_phyto_diat_N,k,i,j)+ cv_wat_pos(iv_phyto_dino_N,k,i,j)+  &
                   cv_wat_pos(iv_phyto_nano_N,k,i,j)+epsilon)*p_phyto_ChlNratio
           cphyto=(ABS(cphyto)+cphyto)*0.5_rsh
           extinction(k,i,j)=extinction(k,i,j)+p_extincChl1*(cphyto)**p_extincChl2
           diag_3d_wat(irk_diag(id_extinctioncoeff),k,i,j)=extinction(k,i,j)
           dzu=f_dzu(h0(i,j),xe(i,j),k,i,j)
           attenuation(k,i,j)=EXP(-extinction(k,i,j)*dzu)

         END DO

   ! calcul de la lumiere au plafond de chaque maille
   ! ------------------------------------------------
         alumplafond(kmax,i,j)=rad(i,j)
         DO k=kmax-1,1,-1
           alumplafond(k,i,j)=alumplafond(k+1,i,j)*attenuation(k+1,i,j)
         END DO

       END IF
     END DO
   END DO
!$OMP END DO

   ! ---------------------------------------------
   ! Diagnostics
   ! ---------------------------------------------

!$OMP single
   obc_cv_ktherm_s(:)=0
   obc_cv_ktherm_n(:)=0
   obc_cv_ktherm_w(:)=0
   obc_cv_ktherm_e(:)=0
!$OMP END single

   ! calcul du max. de diat. dino. avec date du max et profondeur du max
   ! -------------------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)

   DO j=ljmin,ljmax
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       d=h0(i,j)+xe(i,j)
       IF (d.GT.h0fond) THEN

   ! diatomees
   ! ---------
         kmaxdiat=1
         xmaxdiat=0.0_rsh
         datemaxdiat=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_diat_N,k,i,j).GE.xmaxdiat) THEN
             xmaxdiat=cv_wat_pos(iv_phyto_diat_N,k,i,j)
             kmaxdiat=k
             datemaxdiat=jjulien
           END IF
         END DO

         IF (xmaxdiat.GT.diag_2d(irk_diag(id_diat_max),i,j)) THEN
           diag_2d(irk_diag(id_diat_max),i,j)=xmaxdiat
           diag_2d(irk_diag(id_diat_datemax),i,j)=datemaxdiat
           diag_2d(irk_diag(id_diat_depthmax),i,j)=-sig(kmaxdiat)*d
         END IF

   ! dinoflagelles
   ! -------------
         xmaxdino=0.0_rsh
         kmaxdino=1
         datemaxdino=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_dino_N,k,i,j).GE.xmaxdino) THEN
             xmaxdino=cv_wat_pos(iv_phyto_dino_N,k,i,j)
             kmaxdino=k
             datemaxdino=jjulien
           END IF
         END DO

         IF (xmaxdino.GT.diag_2d(irk_diag(id_dino_max),i,j)) THEN
           diag_2d(irk_diag(id_dino_max),i,j)=xmaxdino
           diag_2d(irk_diag(id_dino_datemax),i,j)=datemaxdino
           diag_2d(irk_diag(id_dino_depthmax),i,j)=-sig(kmaxdino)*d
         END IF

   ! nanoflagelles
   ! -------------
         xmaxnano=0.0_rsh
         kmaxnano=1
         datemaxnano=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_nano_N,k,i,j).GE.xmaxnano) THEN
             xmaxnano=cv_wat_pos(iv_phyto_nano_N,k,i,j)
             kmaxnano=k
             datemaxnano=jjulien
           END IF
         END DO

         IF (xmaxnano.GT.diag_2d(irk_diag(id_nano_max),i,j)) THEN
           diag_2d(irk_diag(id_nano_max),i,j)=xmaxnano
           diag_2d(irk_diag(id_nano_datemax),i,j)=datemaxnano
           diag_2d(irk_diag(id_nano_depthmax),i,j)=-sig(kmaxnano)*d
         END IF

   ! calcul de l intensite et profondeur de l halocline
   ! --------------------------------------------------
         dcdzmax=0.0_rsh
         kdcdz=kmax
         DO k=1,kmax-1
           dzw=f_dzw(h0(i,j),xe(i,j),k,i,j)
           dcdz=(sal(k,i,j)-sal(k+1,i,j))/dzw
           dcdz=ABS(dcdz)
           IF (dcdz.GT.dcdzmax) THEN
             kdcdz=k+1
             dcdzmax=dcdz
           END IF
         END DO
         diag_2d(irk_diag(id_gradsali_max),i,j)=dcdzmax
         diag_2d(irk_diag(id_gradsali_depthmax),i,j)=-sig(kdcdz)*d

   ! calcul de l intensite et profondeur de la thermocline
   ! -----------------------------------------------------
         dcdzmax=0.0_rsh
         kdcdz=0
         DO k=1,kmax-1
          dzw=f_dzw(h0(i,j),xe(i,j),k,i,j)
          dcdz=ABS(temp(k,i,j)-temp(k+1,i,j))/dzw
           IF (dcdz.GT.dcdzmax) THEN
             kdcdz=k+1
             dcdzmax=dcdz
           END IF
   !        diag_2d(irk_diag(40),i,j)=temp(kmax,i,j)-temp(1,i,j)
         END DO
         diag_2d(irk_diag(id_gradtemp_max),i,j)=dcdzmax
         diag_2d(irk_diag(id_gradtemp_depthmax),i,j)=-sig(kdcdz)*d

   ! niveau du gradient vertical de temperature max pour injection
   ! des concentrations surface-fond en limite ouest et sud
   ! -------------------------------------------------------------
         IF (ljmin==jmin .AND. j.EQ.jb(i)) obc_cv_ktherm_s(i)=kdcdz
         IF (limin==imin .AND. i.EQ.ig(j)) obc_cv_ktherm_w(j)=kdcdz

   ! calcul de la production cumulee integree sur la verticale
   ! ---------------------------------------------------------
         cumuldiat=0.0_rsh
         cumuldino=0.0_rsh
         cumulnano=0.0_rsh
         DO k=1,kmax
           cumuldiat=cumuldiat+cv_wat_pos(iv_phyto_diat_pp,k,i,j)
           cumuldino=cumuldino+cv_wat_pos(iv_phyto_dino_pp,k,i,j)
           cumulnano=cumulnano+cv_wat_pos(iv_phyto_nano_pp,k,i,j)
         END DO
         diag_2d(irk_diag(id_diat_columnprod),i,j)=cumuldiat
         diag_2d(irk_diag(id_dino_columnprod),i,j)=cumuldino
         diag_2d(irk_diag(id_nano_columnprod),i,j)=cumulnano

       END IF

     END DO
   END DO
!$OMP END DO
!$OMP END PARALLEL

   ! Introduction d une pression de Broutage, d apres Marie SAVINA
   ! -------------------------------------------------------------  
   ! txfiltbenth = taux de filtration du benthos l/j/dm2
!   txfiltbenth=5.0_rsh
   ! fonction sinusoidale
!   foncsinus=(1.0_rsh+SIN(2.0_rsh*3.14159_rsh/365.0_rsh*(jjulien-125)))/2.0_rsh
   ! txfiltbenth en m3/j/m2
!   txfiltbenth=txfiltbenth/1000.0_rsh*100.0_rsh*foncsinus

   ! calcul des processus selon z
!cval la vitesse verticale est traitee par les routine parsub* avec un sous pas de temps si necessaire
!   CALL verti_bio_nut4phy3zoo2(xe,cv_wat)

  END SUBROUTINE verti_nut4phy3zoo2

   !!======================================================================

  SUBROUTINE verti_bio_nut4phy3zoo2(xe,cv_wat)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE   ***
   !&E
   !&E ** Purpose : calcul du deplacement des var bio sur la verticale avec
   !&E              un simple schema d Euler (totalement conservatif)
   !&E ** Description :
   !&E
   !&E ** Called by : verti_dinodiatzoo
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E              25/07/2007  (M. Sourisseau) Original code
   !&E       !  2008-01 (M. Sourisseau) evolutions
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

   USE comvars2d,    ONLY : h0fond,id,ig,h0,hm
   USE comsubstance, ONLY : dtbio,nvp,ws3
   USE_MPI toolmpi,      ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT( inout )  :: cv_wat
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in )   :: xe


   !! * Local declarations
   INTEGER                  ::  i,j,k,l         ! loop indexes
   REAL(KIND=rsh)           ::  d
   REAL(KIND=rlg)           ::  sign_up,sign_do,test_in,test_out
   REAL(KIND=rlg) ,DIMENSION(kmax)    :: cv_wat_buff,cv_wat_buff_in
   REAL(KIND=rlg) ,DIMENSION(kmax)    :: epaiss

   !&E---------------------------------------------------------------------

!$OMP PARALLEL DEFAULT(none)                                            &
!$OMP& shared(h0fond,id,ig,h0,limin,limax,ljminp2,ljmaxm1,dtbio,hm,     &
!$OMP&        nvp,ws3,cv_wat,xe)                                        &
!$OMP& private(i,j,k,l,epaiss,cv_wat_buff,cv_wat_buff_in,sign_up,       &
!$OMP&         sign_do,d,test_in,test_out)

!$OMP DO SCHEDULE(RUNTIME)
   DO l=1,nvp
    DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       d=h0(i,j)+xe(i,j)
       IF (d.GT.hm) THEN   ! no sink in 2D : vertical homogeneity - see tssub_eq_hm.F90
       test_in=0.0_rlg
       test_out=0.0_rlg
       DO k=1,kmax
        epaiss(k)=f_dzu(h0(i,j),xe(i,j),k,i,j)
        cv_wat_buff(k)=0.0_rlg
        cv_wat_buff_in(k)=0.5_rlg*REAL(cv_wat(l,k,i,j)+ABS(cv_wat(l,k,i,j)),rlg)
!        cv_wat_buff_in(l,k,i,j)=cv_wat(l,k,i,j)
        test_in=test_in+cv_wat_buff_in(k)*epaiss(k)
 !       IF (cv_wat_buff_in(l,k,i,j).lt.0.) THEN
 !         write(*,*) 'pb avant verti_bio_nut4phy3zoo2',cv_wat_buff_in(l,k,i,j),l,k,i,j,h0(i,j)
 !         write(*,*) 'mod plus conservatif'
 !       END IF
       END DO

      IF (test_in .GT. 10.0_rlg**-20) THEN
       DO k=1,kmax
        IF (k.LT.kmax) THEN
         IF (ws3(k+1,l,i,j).GT.0.0_rsh) THEN    !vitesse positive vers le bas
          sign_up=1.0_rlg
         ELSE
          sign_up=0.0_rlg
         END IF
        ELSE
         sign_up=0.0_rlg
        END IF
        IF (k.GT.1) THEN
         IF (ws3(k-1,l,i,j).LT.0.0_rsh) THEN
          sign_do=1.0_rlg
         ELSE
          sign_do=0.0_rlg
         END IF
        ELSE
         sign_do=0.0_rlg
        END IF

        IF (k.EQ.1) THEN
         ws3(k,l,i,j)=MIN(0.0_rsh,ws3(k,l,i,j))
         cv_wat_buff(k)=cv_wat_buff_in(k)                                           &  !quitte la cell
              -cv_wat_buff_in(k)*MIN(0.95_rlg,REAL(ABS(ws3(k,l,i,j)),rlg)/epaiss(k)*dtbio)   &  !quitte la cell
              +sign_up*cv_wat_buff_in(k+1)*epaiss(k+1)/epaiss(k)               & !quitte la cell sup
                        *MIN(0.95_rlg,REAL(ws3(k+1,l,i,j),rlg)/epaiss(k+1)*dtbio)
        ELSE IF (k.EQ.kmax) THEN
         ws3(k,l,i,j)=MAX(0.0_rsh,ws3(k,l,i,j))
         cv_wat_buff(k)=cv_wat_buff_in(k)                                          &
                   -cv_wat_buff_in(k)*MIN(0.95_rlg,REAL(ABS(ws3(k,l,i,j)),rlg)/epaiss(k)*dtbio)  &  !quitte la cell
                   +sign_do*cv_wat_buff_in(k-1)*epaiss(k-1)/epaiss(k)              & !quitte la cell inf
                        *MIN(0.95_rlg,REAL(ws3(k-1,l,i,j),rlg)/epaiss(k-1)*dtbio)
        ELSE
         cv_wat_buff(k)=cv_wat_buff_in(k)                                          &
                   -cv_wat_buff_in(k)*MIN(0.95_rlg,REAL(ABS(ws3(k,l,i,j)),rlg)/epaiss(k)*dtbio)  &  !quitte la cell
                   +sign_up*cv_wat_buff_in(k+1)*epaiss(k+1)/epaiss(k)              &  !quitte la cell sup
                        *MIN(0.95_rlg,REAL(ws3(k+1,l,i,j),rlg)/epaiss(k+1)*dtbio)                &
                   +sign_do*cv_wat_buff_in(k-1)*epaiss(k-1)/epaiss(k)              &  !quitte la cell inf
                        *MIN(0.95_rlg,REAL(ws3(k-1,l,i,j),rlg)/epaiss(k-1)*dtbio)
        END IF
       ENDDO !enddo on k

       DO k=1,kmax
         test_out=test_out+cv_wat_buff(k)*epaiss(k)
       ENDDO

       IF (test_in .GT. 10.0_rlg**-20) THEN
!       IF (abs(test_in-test_out)/test_in*REAL(100/dtbio*24*60*60,rsh) .gt. 0.1_rsh) THEN
       IF (ABS(test_in-test_out)/test_in*100.0_rlg .GT. 0.0000000001_rlg) THEN
        WRITE(*,*) 'pb-verti_bio_nut4phy3zoo2',ABS(test_in-test_out)/test_in*100/dtbio*24*60*60,       &
                                 test_in,test_out,i,j,l,d
        ! write(*,*) 'cv_wat_buff_in',cv_wat_buff_in(1:kmax)
        ! write(*,*) 'cv_wat_buff',cv_wat_buff(1:kmax)
        ! write(*,*) 'ws3',ws3(1:kmax,l,i,j)
        STOP
       END IF
       END IF

       DO k=1,kmax
         cv_wat(l,k,i,j)=REAL(cv_wat_buff(k),rsh)
       ENDDO

       END IF !end if on test_in

       END IF !end if on h0fond
     END DO !end do on i
    END DO !end do on j
   END DO !end do on nvp
!$OMP END DO
!$OMP END PARALLEL

   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))

  END SUBROUTINE verti_bio_nut4phy3zoo2

   !!======================================================================

#endif

 END MODULE
