 MODULE bioloinit

#if defined key_substance && defined key_biolo

   !!======================================================================
   !!                   ***  MODULE  bioloinit  ***
   !! Ocean dynamics Bio :  Initializations, reading of files *.dat (rivers, discharges, bio variables...)
   !!
   !!   History :
   !!        !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !!
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters, ONLY : rsh,rlg,riosh,lchain,imin,imax,jmin,jmax,kmax, &
                          limin,limax,ljmin,ljmax
   USE comsubstance,  ONLY :phytomoy,phytocarre
   USE ionc4, ONLY : ionc4_openr,ionc4_read_dimt,ionc4_read_time,ionc4_read_xyt,ionc4_close

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC biolo_userinit                ! routine called by main.F90
   PUBLIC biolo_param                   ! routine called by init.F90
   PUBLIC biolo_init_iv,biolo_init_id   ! routine called by subreaddat.F90

   !! * Interface
   INTERFACE biolo_userinit
!     MODULE PROCEDURE biolo_init_dinodiatzoo
     MODULE PROCEDURE biolo_init_nut4phy3zoo2
   END INTERFACE

   !! * Shared or public module variables

   INTEGER, PUBLIC   :: &
            iv_nutr_NO3,    &   ! index for nitrate
            iv_nutr_SiOH,   &   ! index for silicium
            iv_nutr_PO4,    &   ! index for phosphate
            iv_nutr_Pads,   &   ! index for adsorbed phosphate
            iv_phyto_diat_N, &   ! index for diatoms as nitrogen
            iv_phyto_dino_N, &   ! index for dinoflagellates as nitrogen
            iv_detr_N,      &   ! index for organic detritus as nitrogen
            iv_detr_Si,     &   ! index for organic detritus as silicon
            iv_detr_P,      &   ! index for organic detritus as phosphate
            iv_nutr_NH4,    &   ! index for ammonium
            iv_zoo_meso_N,  &   ! index for mesozooplankton
            iv_phyto_nano_N,      &   ! index for nanopicoplankton
            iv_zoo_micr_N,  &   ! index for microzooplankton
            iv_spim,        &   ! index for suspended particulate inorganic matter
            iv_phyto_diat_pp,     &   ! index for diatoms production
            iv_phyto_dino_pp,     &   ! index for dinoflagellates production
            iv_phyto_nano_pp          ! index for nanoflagellates production

   REAL(KIND=rsh), PUBLIC       ::     &
                   p_N_remin,          &
                   p_P_remin,          &
                   p_Si_diss,          &
                   p_T_effect,         &
                   p_P_adsor,          &
                   p_P_desor,          &
                   p_P_adsormaxspim,   &
                   p_P_adsormaxsed,    &
                   p_phyto_ChlNratio,  &
                   p_phyto_SiNratio,   &
                   p_phyto_NPratio,    &
                   p_phyto_CNratio,    &
                   p_nano_mumax,       &
                   p_nano_kNO3,        &
                   p_nano_kNH4,        &
                   p_nano_kPO4,        &
                   p_nano_mort,        &
                   p_nano_iksmith,     &
                   p_diat_mumax,       &
                   p_diat_kNO3,        &
                   p_diat_kNH4,        &
                   p_diat_kSi,         &
                   p_diat_kPO4,        &
                   p_diat_iksmith,     &
                   p_diat_mort,        &
                   p_dino_mumax,       &
                   p_dino_kNO3,        &
                   p_dino_kNH4,        &
                   p_dino_kPO4,        &
                   p_dino_mort,        &
                   p_dino_iksmith,     &
                   p_mesz_thrChl,      &
                   p_mesz_kivlev,      &
                   p_mesz_mumax,       &
                   p_mesz_assim,       &
                   p_mesz_excret,      &
                   p_mesz_mort1,       &
                   p_mesz_mort2,       &
                   p_zoo_CDWratio,     &
                   p_zoo_CNratio,      &
                   p_micz_mumax,       &
                   p_micz_kgraz,       &
                   p_micz_assim,       &
                   p_micz_excret,      &
                   p_micz_mort,        &
                   p_micz_thrnano,     &
                   p_phyto_photoratio, &
                   p_Nbent_remin,      &
                   p_Pbent_remin,      &
                   p_Sibent_dissol,    &
                   p_O2_kremin,        &
                   p_nitrifwat,        &
                   p_nitrifsed,        &
                   p_zoo_resp,         &
                   p_extincwat,        &
                   p_extincspim,       &
                   p_extincChl1,       &
                   p_extincChl2,       &
                   p_parradratio,      &
                   p_erodflux,         &
                   p_erodcrittau,      &
                   p_detzoo_wsed,      &
                   p_detphy_wsed,      &
                   p_mesz_captdiat,    &
                   p_mesz_captdino,    &
                   p_mesz_captmicz,    &
                   p_micz_captdiat,    &
                   p_micz_captnano,    &
                   p_micz_captdet

   ! number of diagnostic variables
   INTEGER, PUBLIC :: ndiag_1d,     &  ! 1 dimension (time)
                      ndiag_2d,     &  ! 2 dimensions (i,j)
                      ndiag_3d,     &  ! 3 d
                      ndiag_3d_wat, &  ! 3 d (k,i,j)
                      ndiag_3d_sed, &  ! 3 d (i,j,k)
                      ndiag_tot

   ! indexes of diagnostic variables
   INTEGER, PUBLIC :: id_diat_max,          &
                      id_diat_datemax,      &
                      id_diat_depthmax,     &
                      id_dino_max,          &
                      id_dino_datemax,      &
                      id_dino_depthmax,     &
                      id_nano_max,          &
                      id_nano_datemax,      &
                      id_nano_depthmax,     &
                      id_gradsali_max,      &
                      id_gradsali_depthmax, &
                      id_gradtemp_max,      &
                      id_gradtemp_depthmax, &
                      id_spim_satused,      &
                      id_diat_limlight,     &
                      id_diat_limN,         &
                      id_diat_limSi,        &
                      id_diat_limP,         &
                      id_dino_limlight,     &
                      id_dino_limN,         &
                      id_dino_limP,         &
                      id_nano_limlight,     &
                      id_nano_limN,         &
                      id_nano_limP,         &
                      id_extinctioncoeff,   &
                      id_diat_columnprod,   &
                      id_dino_columnprod,   &
                      id_nano_columnprod,   &
                      id_totalchl,          &
                      id_columnprodtotal,   &
                      id_diatsettling,      &
                      id_detsettling,       &
                      id_spm_total

  ! arrays of diagnostic variables
   REAL(KIND=rsh),ALLOCATABLE,DIMENSION(:)      ,PUBLIC :: diag_1d     !(index of var, time)
   REAL(KIND=rsh),ALLOCATABLE,DIMENSION(:,:,:)  ,PUBLIC :: diag_2d     ! (index of var, i, j)
   REAL(KIND=rsh),ALLOCATABLE,DIMENSION(:,:,:,:),PUBLIC :: diag_3d_wat !(index of var,k,i,j)
#ifdef key_sedim_anc
   REAL(KIND=rsh),ALLOCATABLE,DIMENSION(:,:,:,:),PUBLIC :: diag_3d_sed !(index of var,i,j,k)
#endif
   INTEGER,ALLOCATABLE,DIMENSION(:),PUBLIC :: idimv_r
   LOGICAL,ALLOCATABLE,DIMENSION(:),PUBLIC :: l_diag_sortie

   ! names and attributes of diagnostic variables
   CHARACTER(LEN=lchain),ALLOCATABLE,DIMENSION(:),PUBLIC :: name_vardiag,         &
                                                            long_name_vardiag,    &
                                                            standard_name_vardiag

   ! units of diagnostic variables
   CHARACTER(LEN=lchain),ALLOCATABLE,DIMENSION(:),PUBLIC :: unit_vardiag

   ! rank of diagnostic variables
   INTEGER,ALLOCATABLE,DIMENSION(:),PUBLIC :: irk_diag ! (reading order)

   ! values of attributes of diagnostic variables
   REAL(KIND=rsh),ALLOCATABLE,DIMENSION(:),PUBLIC :: valid_min_vardiag, valid_max_vardiag

   !! * Private variables
   INTEGER                  :: idmin=99999,idmax=0, &  ! to get min and max indexes of diagnostic var.
                               ivmin=99999,ivmax=0     ! to get min and max indexes of substance var.

 CONTAINS

   !!======================================================================

  SUBROUTINE biolo_param

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE biolo_param  ***
   !&E
   !&E ** Purpose : Read namelist relative to the module of biology
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : init
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-10  (V. Garnier)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   !! * Arguments

   !! * Local declarations
   CHARACTER(LEN=lchain) :: filepc

   NAMELIST/namorgmat/ p_N_remin,p_P_remin,p_Si_diss,p_T_effect
   NAMELIST/nampsorb/ p_P_adsor,p_P_desor,p_P_adsormaxspim,p_P_adsormaxsed
   NAMELIST/namgenphyto/ p_phyto_ChlNratio,p_phyto_SiNratio,p_phyto_NPratio,p_phyto_CNratio
   NAMELIST/namnanophyto/ p_nano_mumax,p_nano_kNO3,p_nano_kNH4,p_nano_kPO4,p_nano_mort, &
                          p_nano_iksmith
   NAMELIST/namdiatom/ p_diat_mumax,p_diat_kNO3,p_diat_kNH4,p_diat_kSi,p_diat_kPO4,     &
                       p_diat_iksmith,p_diat_mort
   NAMELIST/namdino/ p_dino_mumax,p_dino_kNO3,p_dino_kNH4,p_dino_kPO4,p_dino_mort,      &
                     p_dino_iksmith
   NAMELIST/nammesozoo/ p_mesz_thrChl,p_mesz_kivlev,p_mesz_mumax,p_mesz_assim,      &
                        p_mesz_excret,p_mesz_mort1,p_mesz_mort2,p_zoo_CDWratio,    &
                        p_zoo_CNratio
   NAMELIST/nammicrozoo/ p_micz_mumax,p_micz_kgraz,p_micz_assim,p_micz_excret,p_micz_mort,   &
                         p_micz_thrnano
   NAMELIST/namoxygen/ p_phyto_photoratio,p_Nbent_remin,p_Pbent_remin,p_Sibent_dissol,p_O2_kremin,&
                       p_nitrifwat,p_nitrifsed,p_zoo_resp
   NAMELIST/namoptics/ p_extincwat,p_extincspim,p_extincChl1,p_extincChl2,p_parradratio
   NAMELIST/namsediment/ p_erodflux,p_erodcrittau,p_detzoo_wsed,p_detphy_wsed
   NAMELIST/namgrazing/ p_mesz_captdiat,p_mesz_captdino,p_mesz_captmicz,p_micz_captdiat,    &
                        p_micz_captnano,p_micz_captdet

   !!----------------------------------------------------------------------
   !! * Executable part

   filepc='./parabiolo.txt'
   IF_AGRIF (.NOT. Agrif_Root()) filepc='./parabiolo'//TRIM(Agrif_Cfixed())//'.txt'
   OPEN(50,file=filepc,status='old',form='formatted', access='sequential')
   READ(50,namorgmat)
   READ(50,nampsorb)
   READ(50,namgenphyto)
   READ(50,namnanophyto)
   READ(50,namdiatom)
   READ(50,namdino)
   READ(50,nammesozoo)
   READ(50,nammicrozoo)
   READ(50,namoxygen)
   READ(50,namoptics)
   READ(50,namsediment)
   READ(50,namgrazing)
   CLOSE(50)                                  ! end reading of namelists

  END SUBROUTINE biolo_param

   !!======================================================================

  SUBROUTINE biolo_init_iv(iv,standname)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE biolo_init_iv  ***
   !&E
   !&E ** Purpose : Initialize indexes of state variables (substances only)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : sub_read_var
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-10  (V. Garnier)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY   : nb_var
   USE comvars2d, ONLY    : iscreenlog
   USE comsubstance, ONLY : irk_mod
   USE toolmath, ONLY     : sumindex

   !! * Arguments
   INTEGER,          INTENT( in ) :: iv
   CHARACTER(LEN=*), INTENT( in ) :: standname

   !! * Local declarations
   INTEGER               :: sumverif

   !!----------------------------------------------------------------------
   !! * Executable part

   SELECT CASE( TRIM(ADJUSTL(ADJUSTR(standname))) )

     CASE('mole_concentration_of_nitrate_in_sea_water')
       iv_nutr_NO3 = irk_mod(iv)
     CASE('mole_concentration_of_silicate_in_sea_water')
       iv_nutr_SiOH = irk_mod(iv)
     CASE('mole_concentration_of_phosphate_in_sea_water')
       iv_nutr_PO4 = irk_mod(iv)
     CASE('mole_concentration_of_sorbed_phosphate_in_sea_water')
       iv_nutr_Pads = irk_mod(iv)
     CASE('mole_concentration_of_diatoms_expressed_as_nitrogen_in_sea_water')
       iv_phyto_diat_N = irk_mod(iv)
     CASE('mole_concentration_of_dinoflagellates_expressed_as_nitrogen_in_sea_water')
       iv_phyto_dino_N = irk_mod(iv)
     CASE('mole_concentration_of_organic_detritus_expressed_as_nitrogen_in_sea_water')
       iv_detr_N = irk_mod(iv)
     CASE('mole_concentration_of_organic_detritus_expressed_as_silicon_in_sea_water')
       iv_detr_Si = irk_mod(iv)
     CASE('mole_concentration_of_organic_detritus_expressed_as_phosphorus_in_sea_water')
       iv_detr_P = irk_mod(iv)
     CASE('mole_concentration_of_ammonium_in_sea_water')
       iv_nutr_NH4 = irk_mod(iv)
     CASE('mole_concentration_of_mesozooplankton_expressed_as_nitrogen_in_sea_water')
       iv_zoo_meso_N = irk_mod(iv)
     CASE('mole_concentration_of_nanoplankton_expressed_as_nitrogen_in_sea_water')
       iv_phyto_nano_N = irk_mod(iv)
     CASE('mole_concentration_of_microzooplankton_expressed_as_nitrogen_in_sea_water')
       iv_zoo_micr_N = irk_mod(iv)
     CASE('mass_concentration_of_suspended_matter_in_sea_water')
       iv_spim = irk_mod(iv)
     CASE('cumulative_diatom_production_expressed_as_carbon_in_sea_water')
       iv_phyto_diat_pp = irk_mod(iv)
     CASE('cumulative_dinoflagellate_production_expressed_as_carbon_in_sea_water')
       iv_phyto_dino_pp = irk_mod(iv)
     CASE('cumulative_nanoflagellate_production_expressed_as_carbon_in_sea_water')
       iv_phyto_nano_pp = irk_mod(iv)
     CASE DEFAULT
       print*, 'Invalid standard name of variable : ',TRIM(ADJUSTL(ADJUSTR(standname)))
   END SELECT

   ! smallest and largest indexes
   IF (iv >= ivmax) ivmax=iv
   IF (iv <= ivmin) ivmin=iv


   IF ( iv==nb_var) THEN
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) '********************************'
     WRITE(iscreenlog,*) '          BIOLO_INIT_IV         '
     WRITE(iscreenlog,*) ' Initialization of iv_variables '
     WRITE(iscreenlog,*) '********************************'
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) 'iv_nutr_NO3 = ',iv_nutr_NO3
     WRITE(iscreenlog,*) 'iv_nutr_SiOH = ',iv_nutr_SiOH
     WRITE(iscreenlog,*) 'iv_nutr_PO4 = ',iv_nutr_PO4
     WRITE(iscreenlog,*) 'iv_nutr_Pads = ',iv_nutr_Pads
     WRITE(iscreenlog,*) 'iv_phyto_diat_N = ',iv_phyto_diat_N
     WRITE(iscreenlog,*) 'iv_phyto_dino_N = ',iv_phyto_dino_N
     WRITE(iscreenlog,*) 'iv_detr_N = ',iv_detr_N
     WRITE(iscreenlog,*) 'iv_detr_Si = ',iv_detr_Si
     WRITE(iscreenlog,*) 'iv_detr_P = ',iv_detr_P
     WRITE(iscreenlog,*) 'iv_nutr_NH4 = ',iv_nutr_NH4
     WRITE(iscreenlog,*) 'iv_zoo_meso_N = ',iv_zoo_meso_N
     WRITE(iscreenlog,*) 'iv_phyto_nano_N = ',iv_phyto_nano_N
     WRITE(iscreenlog,*) 'iv_zoo_micr_N = ',iv_zoo_micr_N
     WRITE(iscreenlog,*) 'iv_spim = ',iv_spim
     WRITE(iscreenlog,*) 'iv_phyto_diat_pp = ',iv_phyto_diat_pp
     WRITE(iscreenlog,*) 'iv_phyto_dino_pp = ',iv_phyto_dino_pp
     WRITE(iscreenlog,*) 'iv_phyto_nano_pp = ',iv_phyto_nano_pp
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) '********************************'
     WRITE(iscreenlog,*) ' '

#if ! defined key_sedim && ! defined key_conta
     sumverif = iv_nutr_NO3 + iv_nutr_SiOH + iv_nutr_PO4 + iv_nutr_Pads + &
                iv_phyto_diat_N + iv_phyto_dino_N + iv_detr_N + iv_detr_Si + &
                iv_detr_P + iv_nutr_NH4 + iv_zoo_meso_N + iv_phyto_nano_N + &
                iv_zoo_micr_N + iv_spim + iv_phyto_diat_pp + iv_phyto_dino_pp + iv_phyto_nano_pp

     IF ( sumverif /= sumindex(ivmin,ivmax) ) THEN
       PRINT*,' '
       PRINT*,'BIOLO_INIT_IV'
       PRINT*,'the initialization of indexes of substances'
       PRINT*,'is not correct'
       PRINT*,'Have a look at simu.log and variable.dat files'
       PRINT*,'and inquire into the mistakes'
       PRINT*,' '
       PRINT*,'The simulation is stopped'
       STOP
     END IF
#endif

   END IF   ! ends test on nb_var

  END SUBROUTINE biolo_init_iv

   !!======================================================================

  SUBROUTINE biolo_init_id(id,standname)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE biolo_init_id  ***
   !&E
   !&E ** Purpose : Initialize indexes of diagnostic variables
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : sub_read_vardiag
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-10  (V. Garnier)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars2d, ONLY : iscreenlog
   USE toolmath,  ONLY : sumindex

   !! * Arguments
   INTEGER,          INTENT( in ) :: id
   CHARACTER(LEN=*), INTENT( in ) :: standname

   !! * Local declarations
   INTEGER               :: sumverif

   !!----------------------------------------------------------------------
   !! * Executable part

   SELECT CASE( TRIM(ADJUSTL(ADJUSTR(standname))) )

     CASE('maximum_diatom_mass_concentration_in_sea_water')
       id_diat_max = id
     CASE('date_of_maximum_diatom_mass_concentration_in_sea_water')
       id_diat_datemax = id
     CASE('depth_of_maximum_diatom_mass_concentration_in_sea_water')
       id_diat_depthmax = id
     CASE('maximum_dinoflagellate_mass_concentration_in_sea_water')
       id_dino_max = id
     CASE('date_of_maximum_dinoflagellate_mass_concentration_in_sea_water')
       id_dino_datemax = id
     CASE('depth_of_maximum_dinoflagellate_mass_concentration_in_sea_water')
       id_dino_depthmax = id
     CASE('maximum_nanoflagellate_mass_concentration_in_sea_water')
       id_nano_max = id
     CASE('date_of_maximum_nanoflagellate_mass_concentration_in_sea_water')
       id_nano_datemax = id
     CASE('depth_of_maximum_nanoflagellate_mass_concentration_in_sea_water')
       id_nano_depthmax = id
     CASE('maximum_vertical_gradient_of_sea_water_salinity')
       id_gradsali_max = id
     CASE('depth_of_maximum_vertical_gradient_of_sea_water_salinity')
       id_gradsali_depthmax = id
     CASE('maximum_vertical_gradient_of_sea_water_temperature')
       id_gradtemp_max = id
     CASE('depth_of_maximum_vertical_gradient_of_sea_water_temperature')
       id_gradtemp_depthmax = id
     CASE('corrected_concentration_of_suspended_matter_in_sea_water')
       id_spim_satused = id
     CASE('light_limitation_of_diatom_growth')
       id_diat_limlight = id
     CASE('nitrogen_limitation_of_diatom_growth')
       id_diat_limN = id
     CASE('silicon_limitation_of_diatom_growth')
       id_diat_limSi = id
     CASE('phosphate_limitation_of_diatom_growth')
       id_diat_limP = id
     CASE('light_limitation_of_dinoflagellate_growth')
       id_dino_limlight = id
     CASE('nitrogen_limitation_of_dinoflagellate_growth')
       id_dino_limN = id
     CASE('phosphate_limitation_of_dinoflagellate_growth')
       id_dino_limP = id
     CASE('light_limitation_of_nanoflagellate_growth')
       id_nano_limlight = id
     CASE('nitrogen_limitation_of_nanoflagellate_growth')
       id_nano_limN = id
     CASE('phosphate_limitation_of_nanoflagellate_growth')
       id_nano_limP = id
     CASE('light_extinction_in_sea_water')
       id_extinctioncoeff=id
     CASE('cumulated_production_of_diatoms_in_sea_water_column_expressed_in_nitrogen')
       id_diat_columnprod=id
     CASE('cumulated_production_of_dinoflagellates_in_sea_water_column_expressed_in_nitrogen')
       id_dino_columnprod=id
     CASE('cumulated_production_of_nanoflagellates_in_sea_water_column_expressed_in_nitrogen')
       id_nano_columnprod=id
     CASE('chlorophyll_mass_concentration_in_sea_water')
       id_totalchl=id
     CASE('cumulated_total_production_in_sea_water_column_expressed_in_nitrogen')
       id_columnprodtotal=id
     CASE('diatom_sinking_velocity_in_sea_water')
       id_diatsettling=id
     CASE('detritus_sinking_velocity_in_sea_water')
       id_detsettling=id
     CASE('total_suspended_matter_in_sea_water')
       id_spm_total=id
     CASE DEFAULT
       print*, 'Invalid standard name of diagnostic variable : ',TRIM(ADJUSTL(ADJUSTR(standname)))
   END SELECT

   ! smallest and largest indexes
   IF (id >= idmax) idmax=id
   IF (id <= idmin) idmin=id


   IF ( id==ndiag_tot) THEN
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) '********************************'
     WRITE(iscreenlog,*) '          BIOLO_INIT_ID         '
     WRITE(iscreenlog,*) ' Initialization of id_variables '
     WRITE(iscreenlog,*) '********************************'
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) 'id_diat_max = ',id_diat_max
     WRITE(iscreenlog,*) 'id_diat_datemax = ',id_diat_datemax
     WRITE(iscreenlog,*) 'id_diat_depthmax = ',id_diat_depthmax
     WRITE(iscreenlog,*) 'id_dino_max = ',id_dino_max
     WRITE(iscreenlog,*) 'id_dino_datemax = ',id_dino_datemax
     WRITE(iscreenlog,*) 'id_dino_depthmax = ',id_dino_depthmax
     WRITE(iscreenlog,*) 'id_nano_max = ',id_nano_max
     WRITE(iscreenlog,*) 'id_nano_datemax = ',id_nano_datemax
     WRITE(iscreenlog,*) 'id_nano_depthmax = ',id_nano_depthmax
     WRITE(iscreenlog,*) 'id_gradsali_max = ',id_gradsali_max
     WRITE(iscreenlog,*) 'id_gradsali_depthmax = ',id_gradsali_depthmax
     WRITE(iscreenlog,*) 'id_gradtemp_max = ',id_gradtemp_max
     WRITE(iscreenlog,*) 'id_gradtemp_depthmax = ',id_gradtemp_depthmax
     WRITE(iscreenlog,*) 'id_spim_satused = ',id_spim_satused
     WRITE(iscreenlog,*) 'id_diat_limlight = ',id_diat_limlight
     WRITE(iscreenlog,*) 'id_diat_limN = ',id_diat_limN
     WRITE(iscreenlog,*) 'id_diat_limSi = ',id_diat_limSi
     WRITE(iscreenlog,*) 'id_diat_limP = ',id_diat_limP
     WRITE(iscreenlog,*) 'id_dino_limlight = ',id_dino_limlight
     WRITE(iscreenlog,*) 'id_dino_limN = ',id_dino_limN
     WRITE(iscreenlog,*) 'id_dino_limP = ',id_dino_limP
     WRITE(iscreenlog,*) 'id_nano_limlight = ',id_nano_limlight
     WRITE(iscreenlog,*) 'id_nano_limN = ',id_nano_limN
     WRITE(iscreenlog,*) 'id_nano_limP = ',id_nano_limP
     WRITE(iscreenlog,*) 'id_extinctioncoeff = ',id_extinctioncoeff
     WRITE(iscreenlog,*) 'id_diat_columnprod = ',id_diat_columnprod
     WRITE(iscreenlog,*) 'id_dino_columnprod = ',id_dino_columnprod
     WRITE(iscreenlog,*) 'id_nano_columnprod = ',id_nano_columnprod
     WRITE(iscreenlog,*) 'id_totalchl = ',id_totalchl
     WRITE(iscreenlog,*) 'id_columnprodtotal = ',id_columnprodtotal
     WRITE(iscreenlog,*) 'id_diatsettling = ',id_diatsettling
     WRITE(iscreenlog,*) 'id_detsettling = ',id_detsettling
     WRITE(iscreenlog,*) 'id_spm_total = ',id_spm_total
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) '********************************'
     WRITE(iscreenlog,*) ' '

#if ! defined key_sedim && ! defined key_conta
     sumverif = id_diat_max + id_diat_datemax + id_diat_depthmax + &
              id_dino_max + id_dino_datemax + id_dino_depthmax + &
              id_nano_max + id_nano_datemax + id_nano_depthmax + &
              id_gradsali_max + id_gradsali_depthmax +           &
              id_gradtemp_max + id_gradtemp_depthmax + id_spim_satused + &
              id_diat_limlight + id_diat_limN + id_diat_limSi + id_diat_limP + &
              id_dino_limlight + id_dino_limN + id_dino_limP +   &
              id_nano_limlight + id_nano_limN + id_nano_limP +   &
              id_extinctioncoeff + id_diat_columnprod +          &
              id_dino_columnprod + id_nano_columnprod +          &
              id_totalchl + id_columnprodtotal + id_spm_total +  &
              id_diatsettling + id_detsettling

     IF ( sumverif /= sumindex(idmin,idmax) ) THEN
       PRINT*,' '
       PRINT*,'BIOLO_INIT_ID'
       PRINT*,'the initialization of indexes of diagnostic variables'
       PRINT*,'is not correct'
       PRINT*,'Have a look at simu.log and vardiag.dat files'
       PRINT*,'and inquire into the mistakes'
       PRINT*,' '
       PRINT*,'The simulation is stopped'
       STOP
     END IF
#endif

   END IF   ! ends test on ndiag_tot

  END SUBROUTINE biolo_init_id

   !!======================================================================

  SUBROUTINE biolo_init_nut4phy3zoo2(cv_wat)

#if defined key_substance && defined key_biolo

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE biolo_init_nut4phy3zoo2  ***
   !&E
   !&E ** Purpose : Local initialisations, reading of special files
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2006-10  (M. Sourisseau)  Original code
   !&E       !  2008-01  (M. Sourisseau)  Introduction MES satellitale (climato)
   !&E       !  2008-01  (P. Karleskind)  Initialisation idateinf=1
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,    ONLY : valmanq,nb_var,liminm1,limaxp2,ljminm1,ljmaxp2
   USE comvars2d,     ONLY : l_initfromfile,t,h0fond,h0,iscreenlog,cdate,id,ig
#ifdef key_agrif
   USE comvars2d,     ONLY : tdebagrif_messat
#endif
   USE comsubstance,  ONLY : irk_mod
#ifdef key_messat
   USE comsubstance,  ONLY : messat,l_messat_clim,filemessat_clim,idateinf,idatesup &
                             ,date_s_annee,idimt_messat,t_clim_messat
#endif
   USE_MPI toolmpi,      ONLY : ex_i_rsh,ex_j_rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER  :: k,i,j
#ifdef key_messat
   INTEGER                       :: itmes,it,ijour,imois,ian,iheure,iminu,isec
   INTEGER                       :: jjulien,tool_julien
   INTEGER, DIMENSION(51)        :: t_clim_messat_int
   CHARACTER(LEN=19)             :: date_start,tool_sectodat
   CHARACTER(LEN=lchain)         :: nomfic
   REAL(kind=rsh)                :: lon,lat,moy
   REAL(kind=riosh), DIMENSION(imin:imax,jmin:jmax) :: tab_mes
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) '**************************************************'
   WRITE(iscreenlog,*) '**************** BLO_USERINIT.F90 ****************'
   WRITE(iscreenlog,*) '**************************************************'

   ! Initializations
   !----------------
   phytomoy(:,:)  =0.0_rsh
   phytocarre(:,:)=0.0_rsh

   ! Modifications of global initializations
   !----------------------------------------
   IF(.NOT.l_initfromfile) THEN

   END IF

   ! Read special files
   !-------------------
#if defined key_messat

   ! initialization satellite MES
   messat(:,:,:)=0.0_rsh
   DO j=ljmin,ljmax
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
       IF (h0(i,j) .EQ. -valmanq) THEN
         messat(i,j,:)=valmanq
       END IF
     END DO
   END DO

   ! read  climato
   !----------------------------------------
   IF (l_messat_clim .EQV. .TRUE.) THEN

   WRITE(iscreenlog,*)'------------------------------------------------'
   WRITE(iscreenlog,*)'lecture fichier climato pour mes-sat  '
   WRITE(iscreenlog,*)'     donnees en g.l-1', TRIM(filemessat_clim)

   CALL ionc4_openr(filemessat_clim)
   idimt_messat=ionc4_read_dimt(filemessat_clim)
   !idimt_messat=26  !attention entree en dur de la taille du fichier- a modif
   DO it=1,idimt_messat
    CALL ionc4_read_time(filemessat_clim,it,t_clim_messat(it))
   END DO

   !!reading
   DO it=1,idimt_messat
     CALL ionc4_read_xyt(filemessat_clim,'MES',tab_mes,imin,imax,jmin,jmax,it,fillvar=valmanq)
     DO j=ljmin,ljmax
       DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
         IF(h0(i,j).GT.-valmanq) THEN
            IF (tab_mes(i,j).LT.valmanq) THEN
             messat(i,j,it)=tab_mes(i,j)
             IF(messat(i,j,it) .LT. 0.0_rsh) THEN
              WRITE(iscreenlog,*)'pb maille ',i,j,it,messat(i,j,it)
             END IF
            ELSE
             WRITE(iscreenlog,*)'pas de MES climato en ',i,j,it,tab_mes(i,j)
            END IF
          END IF
       END DO
     END DO
   WRITE(iscreenlog,*) t_clim_messat(it)
   END DO

   CALL ionc4_close(filemessat_clim)

#ifdef key_agrif
   IF (.NOT. l_initfromfile) THEN
     date_start=tool_sectodat(tdebagrif_messat)
   ELSE
     date_start=tool_sectodat(t)
   END IF
#else
   date_start=tool_sectodat(t)
#endif
   CALL tool_decompdate(date_start,ijour,imois,ian,iheure,iminu,isec)
   jjulien=tool_julien(ijour,imois,ian)-tool_julien(1,1,ian)+1
   date_s_annee=jjulien*24*60*60

   it=1
   idateinf=1
   t_clim_messat_int(1:idimt_messat)=INT(t_clim_messat(1:idimt_messat))
   IF (date_s_annee .LE. t_clim_messat_int(1)) THEN
    idatesup=1
    idateinf=idimt_messat
   ELSE
    DO WHILE ((it .LT. idimt_messat) .AND.      &
             (t_clim_messat_int(it+1) .LT. date_s_annee))
    it=it+1
    idateinf=it
    END DO
    IF (idateinf .EQ. idimt_messat) THEN
      idatesup=1
    ELSE
      idatesup=idateinf+1
    END IF
   END IF

   WRITE(iscreenlog,*) 'Fin de lecture fichier climato satellite MES'
   WRITE(iscreenlog,*) it,' champs MES lus'
   WRITE(iscreenlog,*) ' champs MES interp entre', idateinf,' et ',idatesup
   WRITE(iscreenlog,*) 'date du run  = ',date_s_annee
   WRITE(iscreenlog,*)'----------------------------------------------------'
   WRITE(*,*)''

   END IF !endif sur l_messat_clim

   IF (l_messat_clim .EQV. .TRUE.) THEN


   END IF

! if cv_wat has been assigned in this routine
!   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
!   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))

#endif

#else
  WRITE(*,*) 'biolo_init_nut4phy3zoo2 : You should not have seen this print! error?'
#endif

  END SUBROUTINE biolo_init_nut4phy3zoo2

   !!======================================================================

  SUBROUTINE biolo_init_dinodiatzoo(cv_wat)

#if defined key_substance && defined key_biolo

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE biolo_init_dinodiatzoo  ***
   !&E
   !&E ** Purpose : Local initialisations, reading of special files
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2006-10  (P. Cugier V. Garnier)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,    ONLY : valmanq,nb_var,liminm1,limaxp2,ljminm1,ljmaxp2
   USE comvars2d,     ONLY : l_initfromfile,t,h0fond,h0,iscreenlog
   USE comsubstance,  ONLY : irk_mod
#ifdef key_messat
   USE comsubstance,  ONLY : messat
#endif
   USE_MPI toolmpi,      ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
#ifdef key_messat
   INTEGER                       :: k,i,j,                                     &
                                    itmes,it,ijour,imois,ian,iheure,iminu,isec
   CHARACTER(LEN=19)             :: date_start,tool_sectodat
   CHARACTER(LEN=lchain)         :: nomfic
   REAL(kind=rsh)                :: lon,lat,moy 
   REAL(kind=rsh), DIMENSION(1:12)                             :: messatp
   REAL(kind=riosh), DIMENSION(imin:imax,jmin:jmax)            :: tab_mes

#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) '**************************************************'
   WRITE(iscreenlog,*) '**************** BLO_USERINIT.F90 ****************'
   WRITE(iscreenlog,*) '**************************************************'

   ! Initializations
   !----------------
   phytomoy(:,:)  =0.0_rsh
   phytocarre(:,:)=0.0_rsh

   ! Modifications of global initializations
   !----------------------------------------
   IF(.NOT.l_initfromfile) THEN
                                    ! initialize dino and diat
     WHERE(cv_wat(iv_phyto_diat_N,:,:,:) < 0.5_rsh) cv_wat(iv_phyto_diat_N,:,:,:)=0.5_rsh
     cv_wat(iv_phyto_dino_N,:,:,:)=0.025_rsh

                                    ! initialize MES1 and Phosp. part.
     cv_wat(iv_phyto_diat_pp,:,103:142,45)=0.05_rsh
     cv_wat(iv_nutr_Pads, :,103:142,45)=0.35_rsh

                                    ! no diat dino and zoo in Seine river
     cv_wat(iv_phyto_diat_N,:,103:142,45)=0.0_rsh
     cv_wat(iv_phyto_dino_N,:,103:142,45)=0.0_rsh
     cv_wat(iv_phyto_nano_N,:,103:142,45)=0.0_rsh

     ! Adaptation des parametres liee a la conversion des biomasses
     ! zooplanctoniques de microgrammes/L poids sec en micromoles/L azote
     p_mesz_mort2=p_mesz_mort2/p_zoo_CDWratio*p_zoo_CNratio*12.0_rsh

   END IF

   ! Read special files
   !-------------------
#if defined key_messat

   ! initialization satellite MES
   messat(:,:,:)=0.0_rsh
   DO i=imin,imax
     DO j=jmin,jmax
       IF (h0(i,j) > -valmanq) THEN
         messat(i,j,:)=0.002_rsh
       END IF
     END DO
   END DO

   ! read satellite MES
   !-------------------
   WRITE(iscreenlog,*)'------------------------------------------------'
   WRITE(iscreenlog,*)'debut de lecture des donnees satellite MES'
   WRITE(iscreenlog,*)'          donnees en mg.l-1 '

   OPEN(69,file='../../inputs/FORCAGES/climato_messat_roses.txt')
   READ(69,*)
   DO itmes=1,24440
     READ(69,12)i,j,lon,lat,(messatp(it),it=1,12)
     IF(h0(i,j) > -valmanq) THEN
       DO it=1,12
         IF (messatp(it) < 900.00_rsh) THEN
           messat(i,j,it)=messatp(it)/1000.0_rsh
         ELSE
           messat(i,j,it)=valmanq
           IF(h0(i,j) <= h0fond) messat(i,j,it)=valmanq
         END IF
       END DO
     END IF
   END DO
   WRITE(iscreenlog,*) 'lecmessat 1 ',messat(145,110,1:12)

   ! read monthly climatologies from SEAWiFS
   !----------------------------------------

   ! initialize name of file
   date_start=tool_sectodat(t)
   CALL tool_decompdate(date_start,ijour,imois,ian,iheure,iminu,isec)
   IF(imois.EQ.12) ian=ian+1
   WRITE(nomfic,102)ian

   WRITE(iscreenlog,*)'------------------------------------------------'
   WRITE(iscreenlog,*)'lecture fichier climatologies mensuelles ', TRIM(nomfic)
   WRITE(iscreenlog,*)'pour annee',ian

   !reading
   CALL ionc4_openr(nomfic)
   DO it=1,12
     CALL ionc4_read_xyt(nomfic,'MES',tab_mes,imin,imax,jmin,jmax,it,fillvar=valmanq)
     DO j=jmin,jmax
       DO i=imin,imax
         IF(h0(i,j).GT.-valmanq) THEN
           ! reinitialize cells for river mouths : escaut, tamise et seine
           IF ((j.EQ.45.AND.i.GE.103).OR.                                &
              (j.EQ.100.AND.i.GT.165).OR.                               &
              (j.EQ.102.AND.i.LT.110)) THEN
              messat(i,j,it)=0.03_rsh
           ! reinitialize cells for rade de Brest or for MES too low : min = 0.3 mg
           ELSE IF (i.GT.20.AND.j.LE.17.AND.j.GE.11.AND.it.LE.3)THEN
             IF (tab_mes(i,j) < 3.5_rsh)                                 &
               messat(i,j,it)=MAX(tab_mes(i,j)/1000.0_rsh,0.0035_rsh)
           ELSE
            IF (tab_mes(i,j).GT.0.0_rsh .AND. tab_mes(i,j).LT.30.0_rsh)         &
            messat(i,j,it)=tab_mes(i,j)/1000.0_rsh
            IF (tab_mes(i,j).EQ.0.0_rsh) WRITE(iscreenlog,*)'!!',i,j,it,tab_mes(i,j),h0(i,j)
           END IF
         END IF
       END DO
     END DO
   END DO

   !verification
   DO j=jmin,jmax
     DO i=imin,imax
       DO it=1,12
         IF (messat(i,j,it).LT.0.0_rsh) WRITE(iscreenlog,*)'pb maille ',i,j,it,messat(i,j,it) 
       END DO
     END DO
   END DO

   WRITE(iscreenlog,*) 'lecmessat 2 ',messat(145,110,1:12)
   WRITE(iscreenlog,*) 'Fin de lecture des donnees satellite MES'
   WRITE(iscreenlog,*)itmes,' donnees MES lues'
   WRITE(iscreenlog,*)'-----------------------------------------------'
   WRITE(*,*)''
   CLOSE(69)
 102  FORMAT(                                                           &
     &'../../inputs/CLIM/Climato_mensuelle_SeaWiFS_interpole_',i4,'.nc')
12    FORMAT(i3,1x,i3,2x,f6.3,2x,f6.3,12(1x,f7.3))

   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))

#endif

#else
  WRITE(*,*) 'biolo_init_dinodiatzoo : You should not have seen this print! error?'
#endif

  END SUBROUTINE biolo_init_dinodiatzoo

#endif

 END MODULE

