 MODULE ibm
#if defined key_ibm
   !!======================================================================
   !!                   ***  MODULE ibm   ***
   !! 
   !!         M. Huret 01/2011
   !!         IBM (Individual Based model)
   !!         To be adapted for each case for which a CPP key has to be defined
   !!         Several routines are generic
   !&E
   !&E ** History :
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E       !  2012-02 (M. Huret) Introduction of NAMELIST
   !&E
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters, ONLY :  imin,imax,jmin,jmax,kmax,ljmin,ljmax,ljmaxp1,  &
                           limaxp1,liminm1,limaxp2,ljminm1,ljmaxp2,       &
                           rsh,rlg,lchain,limin,liminm2,ljminm2,limax,    &
                           rg_valmanq_io,dg_valmanq_io
   USE comvars3d,  ONLY : sigw,sig
   USE comtraj

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC ibm_3d            ! routine called by step3dxy.F90 step3dyx.F90
   PUBLIC ibm_init          ! routine called by main.F90

   !! * Shared module variables
 
   !! * Private variables
   INTEGER,dimension(imin:imax,jmin:jmax),PUBLIC  ::    k_stratif
   REAL(kind=rsh),dimension(imin:imax,jmin:jmax),PUBLIC  :: depth_strat !-1 no stratif; >0 stratif
   REAL(kind=rsh),dimension(imin:imax,jmin:jmax),PUBLIC  ::  opt_depth_part
   
   REAL(kind=rsh)                  :: w_max,alpha_w

   REAL(KIND=rsh),  ALLOCATABLE, DIMENSION(:,:)  :: xep
   REAL(KIND=rsh),  ALLOCATABLE, DIMENSION(:,:)  :: dxe
   REAL(KIND=rsh),  ALLOCATABLE, DIMENSION(:,:)  :: dxep
   REAL(KIND=rlg),  ALLOCATABLE, DIMENSION(:,:)  :: deb_descend
   REAL(KIND=rlg),  ALLOCATABLE, DIMENSION(:,:)  :: deb_monte

 CONTAINS
   !!======================================================================

   SUBROUTINE ibm_init(xe,sal,temp,          &
#ifdef key_substance
                                   cv_wat,   &
#endif
                                   boz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ibm_init  ***
   !&E
   !&E ** Purpose : Initialize properties of particles with call to traj_init3d
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-11 (M. Huret) Original code
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE trajinitsave, ONLY: traj_init3d
#ifdef key_anchovy
   USE debmodel, ONLY : deb_init
#endif

! Arguments
     REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )       :: xe
     REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: sal,temp,boz
#ifdef key_substance
     REAL(KIND=rsh), DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in ) :: cv_wat
#endif

   !! * Local declarations
     CHARACTER(LEN=lchain)    :: fileibm,file_ibmrestart,file_saverestart
     REAL(KIND=rsh)           :: sal_surf,temp_surf,dens_surf,prof,sprof
     REAL(KIND=rsh)           :: tir,ec_type,deltas,v1
     REAL(KIND=rsh)           :: xe_init,h0_init
     INTEGER                  :: n,m,igg,idd,jhh,jbb,kp,km
     REAL(KIND=rsh)           :: px,py
     INTEGER                  :: hlb,hlt,hrb,hrt  
     INTEGER                  :: i_stage,i_number
     REAL(KIND=rsh)           :: i_w,i_age,i_size,i_density,i_Drate
     LOGICAL	              :: ibm_restart,ibm_pos,ibm_time,ibm_saverestart
     LOGICAL                  :: ibm_stage,ibm_number,ibm_age,ibm_size
   
!!----------------------------------------------------------------------
   !! * Executable part

   ! namelists in paraibm.txt
   !--------------------------

     NAMELIST/namibmin/ file_trajec,itypetraj
     NAMELIST/namibmphys/ ndtz
     NAMELIST/namibmrestart/ ibm_restart,file_ibmrestart,ibm_pos,ibm_time, &
          ibm_stage,ibm_number,ibm_age,ibm_size,ibm_saverestart, file_saverestart
     NAMELIST/namibminit/  i_stage, i_number,i_w,i_age,i_size,i_density,i_Drate
     NAMELIST/namibmbio/ w_max,alpha_w
!     NAMELIST/namibmdeb/ 
      
     fileibm='./paraibm.txt'
     OPEN(50,file=fileibm,status='old',form='formatted', access='sequential')
     READ(50,namibmin)
     READ(50,namibmphys)
     READ(50,namibmrestart)
     READ(50,namibminit)
     READ(50,namibmbio)
 !    READ(50,namibmdeb)
     CLOSE(50)

     CALL traj_init3d(xe)

 !    ALLOCATE(xep(nb_patch,nbmax_part) )
 !    xep(:,:)=0.0
 !    ALLOCATE( dxe(nb_patch,nbmax_part) )
 !    dxe(:,:)=0.0
 !    ALLOCATE( dxep(nb_patch,nbmax_part) )
 !    dxep(:,:)=0.0
 !    ALLOCATE( deb_monte(nb_patch,nbmax_part) )
 !    deb_monte(:,:)=0.0
 !    ALLOCATE( deb_descend(nb_patch,nbmax_part) )
 !    deb_descend(:,:)=0.0

     part(:,:)%stage=i_stage            
     part(:,:)%number=i_number           
     part(:,:)%opt_depth=0.0_rsh                                             
     part(:,:)%w=i_w
     part(:,:)%age=i_age
     part(:,:)%size=i_size !read in mm
     part(:,:)%density=i_density
     part(:,:)%temp=0.0_rsh   

#ifdef key_anchovy


     part(:,:)%size=part(:,:)%size*0.1_rsh ! egg diameter (mm ->cm)    

     call deb_init

  !Goarant et al. 2007 (egg density =f(surf. salinity))
     DO n = 1,nb_patch
        DO m = 1,nb_part(n)

           IF_MPI ( part(n,m)%inoutflag ) THEN

           ! Egg density ----------------

           prof=3.0_rsh ! surface
#ifdef key_siggen
           kint=-prof+part(n,m)%xe ! switch from immersion to real z
           CALL ztosiggen(kint,sprof,part(n,m)%xe,part(n,m)%h0,part(n,m)%hc)
#else
           sprof=-(prof/part(n,m)%d3)
#endif

           CALL ibm_loc_xyz(xe,part(n,m)%xpos,part(n,m)%ypos,sprof, &
                px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt,kp,km)

           !egg density =f(surf. salinity)
           sal_surf=ibm_traint(sal,xe,sprof,kp,km,px,py,igg,idd,jhh,jbb,hlb,hrb,hlt,hrt)
!           part(n,m)%density=-8.29_rsh + 0.990_rsh*sal_surf+1000.0_rsh 
           
           !egg density =f(surf. density)
           temp_surf=ibm_traint(temp,xe,sprof,kp,km,px,py,igg,idd,jhh,jbb,hlb,hrb,hlt,hrt)
           dens_surf=w_dens(temp_surf,sal_surf)
           part(n,m)%density=-0.36_rsh + 0.990_rsh*(dens_surf-1000.0_rsh)+1000.0_rsh 


           ! Egg density variability ----------------
           ec_type=0.75_rsh
           CALL gasdev_s(tir)
           part(n,m)%density=part(n,m)%density+tir*ec_type 

           ! Egg size variability----------------------
           call random_number(v1) ! 0 to +1 in each direction
           v1=2.0_rsh*v1-1.0_rsh  !-1 to +1 in each direction
           deltas=0.1_rsh*part(n,m)%size
           part(n,m)%size=part(n,m)%size+v1*deltas  

           ENDIF_MPI

        ENDDO
     ENDDO
#endif

  ! save initialization
  CALL ibm_save

  END SUBROUTINE ibm_init

   !!======================================================================

   SUBROUTINE ibm_3d(xe,uz,vz,sal,temp,    &
#ifdef key_substance
                                     cv_wat,   &
#endif
                                     boz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ibm_init  ***
   !&E
   !&E ** Purpose : Main routine with call to advection, behaviour, growth...
   !&E **  To be adapted by the user depending on case study
   !&E
   !&E ** Description :
   !&E ** Called by : traject3d
   !&E ** External calls :
   !&E ** Reference :
   !&E ** History : ! marc sourisseau 12/2008
   !&E              ! M. Huret 11/1010
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E---------------------------------------------------------------------

   !! * Modules used
   USE comvars2d, ONLY : t,dt
#ifdef key_substance
   USE parameters, ONLY : nb_var
#endif
   USE traject3d, ONLY : traj_3d
#if defined key_siggen
   USE trajectools, ONLY:ztosiggen
#endif
#ifdef key_anchovy
   USE debmodel, ONLY : deb_larve
#endif

   !! * Arguments
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )       :: xe
   REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: sal,temp,boz
   REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )  :: uz,vz
#ifdef key_substance
   REAL(KIND=rsh), DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in ) :: cv_wat
#endif

   !! * Local declarations
   INTEGER                    :: jj,mm,aaaa,hh,minu,sec,n,m
   REAL(KIND=rsh)             :: depth_day,depth_night,w1,w2
   REAL(KIND=rsh)             :: sal_part,zlag,slag
   REAL(KIND=rlg)             :: dtm
   LOGICAL                    :: update
   INTEGER                    :: kp,km
   INTEGER                    :: igg,idd,jhh,jbb  
   REAL(KIND=rsh)             :: px,py
   INTEGER                    :: hlb,hlt,hrb,hrt  
   INTEGER                    :: IERR_MPI

   !!----------------------------------------------------------------------
   !! * Executable part

   ! save particle properties (before any change for getting exact initial properties) 
   CALL ibm_save

!if advection, depend de itypevert passive transport
   CALL traj_3d(xe,uz,vz)

#ifdef key_dino
   !CALL ibm_detect_pycno(xe,boz,sal,temp)
   CALL ibm_opt_depth(xe)   !model of convergence to an optimal depth
#endif

!--------------------------------------------------------------
!Common part to all species
!---------------------------------------------------------------
   dtm=dt*0.5_rlg

   DO n = 1,nb_patch
      IF ((t >= part(n,1)%date_orig).and.(t <= tendtraj(n))) THEN
      DO m = 1,nb_part(n)

         IF_MPI ( part(n,m)%inoutflag ) THEN

         CALL ibm_loc_xyz(xe,part(n,m)%xpos,part(n,m)%ypos,part(n,m)%spos, &
              px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt,kp,km)

         ! get temperature at particle position
         part(n,m)%temp=ibm_traint(temp,xe,part(n,m)%spos,kp,km, &
              px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
         
         if (part(n,m)%temp < 0.0_rsh) THEN
            print*, part(n,m)%temp,n,m,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt
            CALL_MPI MPI_FINALIZE(IERR_MPI)
         endif

!Age determination
         part(n,m)%age=(t-part(n,m)%date_orig)/86400.0_rsh

!--------------------------------------------------------------
! Specific models with calls to subroutines
!---------------------------------------------------------------

!Anchovy eggs and larvae
#ifdef key_anchovy
! size in cm

   update=.TRUE.
       
   IF (part(n,m)%stage == 1) THEN ! stade 1, eggs, buoyancy  
      sal_part=ibm_traint(sal,xe,part(n,m)%spos,kp,km,px,py,igg,idd,jhh,jbb,hlb,hrb,hlt,hrt)
      part(n,m)%w = ibm_buoy(part(n,m)%density, part(n,m)%size,part(n,m)%temp,sal_part)
      
      !egg development rate (Regner, 1996)
      part(n,m)%Drate=part(n,m)%Drate+dtm/86400.0_rsh*0.000559_rsh*part(n,m)%temp**2.290236_rsh
     ! part(n,m)%number=(1.0_rsh-fracmort)*part(n,m)%number
      if (part(n,m)%Drate.ge.1.0_rsh) then 
         part(n,m)%stage = 2
         ! increase density at hatch
         part(n,m)%density=part(n,m)%density+0.8_rsh 
         update=.FALSE.
      endif
   ENDIF ! end of stage 1
      
   IF (part(n,m)%stage == 2.AND.update) THEN ! stage 2, larva without vertical migration
      part(n,m)%size=3.5_rsh/10.0_rsh ! on force eclosion  taille fixe (cm)
      !       part(n,m)%w = w from buoyancy  
      sal_part=ibm_traint(sal,xe,part(n,m)%spos,kp,km,px,py,igg,idd,jhh,jbb,hlb,hrb,hlt,hrt)
      part(n,m)%w = ibm_buoy(part(n,m)%density, part(n,m)%size,part(n,m)%temp,sal_part)

      !growth
      call deb_larve(n,m,dtm)
  !      if (part(n,m)%size.ge.0.6_rsh ) then 
      if (part(n,m)%age.ge.7.0_rsh) then 
         part(n,m)%stage = 3
         update=.FALSE.
      endif
   ENDIF ! end of stage 2
   
   IF (part(n,m)%stage == 3.AND.update) THEN ! stage 3, nycthemeral migration
      depth_day=25.0_rsh
      depth_night=0.0_rsh
      ! calculate an optimal depth ?
      w_max=part(n,m)%size/20.0_rsh/100.0_rsh
      part(n,m)%w=ibm_nycth_mig(depth_day,depth_night,w_max,w_max,n,m)       
      !growth
      call deb_larve(n,m,dtm)
   ENDIF  ! end of stage 3              

   
   ! mortality
   !part(n,m)%number=(1-fracmort)*part(n,m)%number
   

#endif   !anchovy
   
!------------------------------------------------------------------------------------

!Owenia fusiformis larvae (Sakina Ayata)
#ifdef key_owenia
         ! Ontogenic migrations for Owenia fusiformis larvae
         ! GUIZIEN ET AL 2006/THIEBAUT ET 94
         !      t<1d:     w=2.5mm/s
         !      1d<=t<20d:  w=0mm/s
         !      20d<=t<28d: w=-0.8mm/s
         !      28d<=t:    w=-2.5mm/s
      w_max= 0.0025   !      t < 1d:     w=2.5mm/s
      IF ((part(n,m)%age>= 1.0_rsh).AND.(part(n,m)%age < (20.0_rsh))) THEN    ! 1d <= t < 20d:  w=0mm/s 
         part(n,m)%stage=2
         w_max= 0.0_rsh
      ELSEIF ((part(n,m)%age >= (20.0_rsh)).AND.(part(n,m)%age < (28.0_rsh))) THEN  !    20d <= t < 28d: w=-0.8mm/s
         part(n,m)%stage=3
         w_max= -0.0008_rsh
      ELSEIF (part(n,m)%age >= (28.0_rsh)) THEN	 !     28d  <= t :    w=-2.5mm/s
         part(n,m)%stage=4
         w_max= -0.0025_rsh
      ENDIF   
      part(n,m)%w=w_max
#endif  ! Owenia


! Dynophysis
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#ifdef key_dino  !migration to an optimal depth
      !model de Neumann et Fennel : règles de migration
      ! a developper / Marc S.
      w_max=15.0_rsh/86400.0_rsh   !vitesse max de deplacement sur la verticale en m.j-1 conv en m.s-1
      part(n,m)%opt_depth=ibm_opt_depth(part(n,m)%xpos,part(n,m)%ypos)
      if (part(n,m)%opt_depth .lt. part(n,m)%d3) THEN
         part(n,m)%w=w_max*     &
              tanh(alpha_w*((part(n,m)%d3*(1.0_rsh-part(n,m)%zpos))-part(n,m)%opt_depth))
      else
         part(n,m)%w=0.0_rsh
      endif    
#endif  !dino

!--------------------------------------------------------------------------------

   ! Vertical movement


      IF (part(n,m)%w /= 0.0_rsh) THEN
         
         zlag=-part(n,m)%zpos+part(n,m)%xe ! switch from immersion to real z
         zlag=zlag+part(n,m)%w*dtm         ! update position
         zlag=MIN(MAX(zlag,-part(n,m)%h0),part(n,m)%xe) ! check boundaries

         part(n,m)%zpos=-zlag+part(n,m)%xe  ! immersion

#ifdef key_siggen 
         CALL ztosiggen(zlag,slag,part(n,m)%xe,part(n,m)%h0,part(n,m)%hc)
         part(n,m)%spos=slag
#else
         part(n,m)%spos=(zlag-part(n,m)%xe)/part(n,m)%d3                    
#endif 

      ENDIF
      
      ENDIF_MPI ! ends test on inoutflag
      ENDDO ! particle
   ENDIF ! date
   ENDDO ! patch

   END SUBROUTINE ibm_3d

!!======================================================================

   FUNCTION w_dens(tempw,salw)

   !&E---------------------------------------------------------------------
   !&E ** Purpose : Estimate the water density
   !&E ** 
   !&E ** Description : 
   !&E ** Called by : w_behav_part
   !&E ** External calls : 
   !&E ** Reference : 
   !&E ** History : ! M. Huret 11/1010
   !&E---------------------------------------------------------------------
   !! * Modules used
     USE comvars2d,  ONLY : ierrorlog,icon
     USE comvars3d,  ONLY : l_stateeq_lin,rhoref

   !! * Arguments
   REAL(KIND=rsh)   , INTENT( in )  :: salw,tempw
   REAL(KIND=rsh)                   :: w_dens

   !! * Local declarations
   INTEGER                 :: IERR_MPI
   REAL(KIND=rsh)          :: TP,S,P
   REAL(KIND=rlg)          :: row,rost,c,pcc
   REAL(kind=rsh)          :: alpha,beta

   !!----------------------------------------------------------------------
   !! * Executable part
 
   IF (icon >= 1 .AND. l_stateeq_lin) THEN

    alpha=0.00016_rsh
    beta=0.0008_rsh
    w_dens = (beta*salw-alpha*tempw)*rhoref

   ELSE IF (icon >= 1 .AND. .NOT.l_stateeq_lin) THEN
! water density (Mellor, 1991, see SUBROUTINE ts_stateeq_blumberg)
     P = 10.0_rsh ! pressure
     TP = MAX(tempw,0.001_rsh)
     S=MAX(salw,0.001_rsh)

   ! density of pure water
     row = 999.842594_rlg + (  6.793952d-02 + ( - 9.095290d-03 +                &
          (1.001685d-04 - 1.120083d-06*TP + 6.536332d-09*TP*TP)*TP )*TP)*TP

   ! density of sea water at the surface
     rost = row + S * (   0.824493_rlg + (  -4.0899d-03 + ( 7.6438d-05 +        &
        (-8.2467d-07+5.3875d-09*TP)*TP )*TP  )*TP   ) +           &
        S**1.5_rlg * ( -5.72466d-03 + (1.0227d-04-1.6546d-06*TP)*TP ) +     &
        S*S*4.8314d-04

   ! density of sea water at pressure p (decibars)
     c = 1449.2_rlg + 1.34_rlg*(S-35.0_rlg) + 4.55_rlg*TP - 0.045_rlg*TP*TP +   &
          0.00821_rlg*p + 15.0d-09*p*p
     pcc = p/(c*c)
     w_dens = rost + 1.0d4*(1.0_rlg-0.2_rlg*pcc)*pcc

!------ Fin MELLOR
   ELSE

     WRITE(ierrorlog,*) 'Function IBM_BUOY'
     WRITE(ierrorlog,*) 'No state equation have been chosen'
     WRITE(ierrorlog,*) 'because icon<1'
     WRITE(ierrorlog,*) 'The simulation is stopped'
     CALL_MPI MPI_FINALIZE(IERR_MPI)
     STOP

   END IF

   end FUNCTION w_dens ! w_buoy


!!======================================================================

   FUNCTION ibm_buoy(density,size,tempw,salw)

   !&E---------------------------------------------------------------------
   !&E ** Purpose : Estimate the terminal velocity from buoyancy scheme
   !&E ** 
   !&E ** Description : 
   !&E ** Called by : w_behav_part
   !&E ** External calls : 
   !&E ** Reference : 
   !&E ** History : ! M. Huret 11/1010
   !&E---------------------------------------------------------------------
   !! * Modules used
     USE parameters, ONLY : grav

   !! * Arguments
   REAL(KIND=rsh)   , INTENT( in )  :: salw,tempw,density,size
   REAL(KIND=rsh)                   :: ibm_buoy

   !! * Local declarations
   REAL(KIND=rsh)          :: w_buoy
   REAL(KIND=rsh)          :: wsto,wdal,cor,rayoeuf,diam,difdens
   REAL(KIND=rsh)          :: TP,S,wdens,dynvisc,reyn

   !!----------------------------------------------------------------------
   !! * Executable part

   cor=1.0_rsh
   rayoeuf=size*0.01_rsh/2.0_rsh    ! anchovy,  diameter to radius 

   TP = MAX(tempw,0.001_rsh)
   S=MAX(salw,0.001_rsh)
   
   wdens=w_dens(tempw,salw)

! Dynamic viscosity (kg.m-1.s-1) (approx. of table values, vertegg v1.0 toolbox, Adlandsvik, 2000)
   dynvisc=(1.7915_rsh - 0.0538_rsh*TP + 0.0007_rsh*TP*TP + 0.0023_rsh*S)*0.001_rsh

!  calcul de la vitesse de remontee des oeufs de poissons (Stokes)             
!  unites: m s-1
   difdens=density-REAL(wdens,rsh)
   wsto=2.0_rsh*grav*rayoeuf**2.0_rsh*difdens/(9_rsh*dynvisc)
   wsto=wsto*cor
   w_buoy=-wsto
              
! calcul du nombre de Reynolds (Re)
   reyn=(wdens*abs(wsto)*2*rayoeuf)/dynvisc

! choix de la vitesse de Dallavalle si Re > 0.5     
   if (reyn.gt.0.5_rsh) then
      diam=(9*(dynvisc**2)/(density*grav*abs(difdens)))**(1.0_rsh/3.0_rsh)      
      wdal=0.0875*abs(difdens)**(2.0/3.0)*((2*rayoeuf)-0.4*diam)/dynvisc**(1./3.)

      w_buoy=wdal*cor
      if(difdens.ge.0.0) then
         w_buoy=-wdal*cor
      endif
   
   endif

   ibm_buoy=w_buoy

  END FUNCTION ibm_buoy ! w_buoy

 !!======================================================================

  FUNCTION ibm_nycth_mig(depth1,depth2,w1,w2,n,m)

   !&E---------------------------------------------------------------------
   !&E
   !&E ** Purpose : Nycthemeral migration between depth1 (day) and depth2 (night)
   !&E ** Migration from depth1 to depth2 starts when rad=0 with w=w2
   !&E ** Migration from depth2 to depth1 starts when rad>0 with w=w1
   !&E **
   !&E ** Description :
   !&E **
   !&E ** Called by : ibm_3d
   !&E **
   !&E ** External calls :
   !&E ** 
   !&E ** Reference :
   !&E ** 
   !&E ** History : M. Huret 11/2010
   !&E---------------------------------------------------------------------

   !! * Modules used
   USE comvars2d, ONLY : t
   USE sflxatm,      ONLY : rad

   !! * Arguments
   REAL(KIND=rsh), INTENT( in )          ::depth1,depth2,w1,w2
   INTEGER, INTENT( in )                 :: n,m 
   REAL(KIND=rsh)                        :: ibm_nycth_mig

   !! * Local declarations
   REAL(kind=rsh)                        ::   depth,w
   character(len=19)                     ::  tool_sectodat,date
   integer                               :: jj,mm,aaaa,hh,minu,sec

   !!----------------------------------------------------------------------
   !! * Executable part

   depth=depth1
   w=w1
   if(rad(NINT(part(n,m)%xpos),NINT(part(n,m)%ypos)).eq.0.0_rsh) THEN
!   date=tool_sectodat(t)     ! Hour of the day
!   call tool_decompdate(date,jj,mm,aaaa,hh,minu,sec)
!   if(hh <= 6. .OR. hh >= 21.) THEN
      depth=depth2
      w=w2
   endif
   if (depth .lt. part(n,m)%d3) THEN  
      !      part(n,m)%w= MIN(w_max,w_max*(part(n,m)%zpos-depth)/depth)
      ! part(n,m)%zpos is immersion of particle
      ibm_nycth_mig= w*tanh(alpha_w*(part(n,m)%zpos-depth)) 
   else
      ibm_nycth_mig=0.
   endif
   
  END FUNCTION ibm_nycth_mig

!!======================================================================

  FUNCTION ibm_tidal_mig(xe,depth1,depth2,n,m)

   !&E---------------------------------------------------------------------
   !&E
   !&E ** Purpose : Tidal migration between depth1 (flood) and depth2 (ebb) 
   !&E ** 
   !&E ** Description :
   !&E ** Called by : w_behav_part
   !&E ** External calls :
   !&E ** 
   !&E ** Reference :
   !&E ** 
   !&E ** History : M. Huret 11/2010 (A ameliorer)
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E---------------------------------------------------------------------

   !! * Modules used
   USE comvars2d, ONLY : t

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in )   :: xe
   REAL(KIND=rsh), INTENT( in )       :: depth1,depth2,n,m
   REAL(KIND=rsh)                              :: ibm_tidal_mig
   
   !! * Local declarations
   INTEGER                 :: jj,mm,aaaa,hh,minu,sec
   REAL(kind=rsh) ::   depth

   !!----------------------------------------------------------------------
   !! * Executable part

   dxe(n,m) = xe(NINT(part(n,m)%xpos),NINT(part(n,m)%ypos))-xep(n,m) 
   
   ! Si chgmt d elevation decroissant apres au moins 6h
   if(dxep(n,m) > 0.0 .AND. dxe(n,m) <= 0.0 .AND. t >= deb_monte(n,m) + 6.0*3600.0) then
      deb_monte(n,m) = t + 4.0*3600.0 + 15.0*60.0 ! chgt de courant dans 4heures 15
   endif
   
   ! Si chgmt d elevation croissant apres au moins 6h
   if(dxep(n,m) <= 0.0 .AND. dxe(n,m) > 0.0 .AND. t >= deb_descend(n,m) + 6.0*3600.0) then
      deb_descend(n,m) = t + 4.0*3600.0 + 15.0*60.0 ! chgt de courant dans 4heures 15
   endif
   
   depth=depth1 ! en surface qd maree monte
   if(t < deb_monte(n,m) .AND. t >= deb_descend(n,m)) THEN 
      depth=depth2 ! au fond qd maree descend
   endif
   
   dxep(n,m) = dxe(n,m)
   xep(n,m)=xe(NINT(part(n,m)%xpos),NINT(part(n,m)%ypos))
   
   ibm_tidal_mig= w_max*tanh(alpha_w*(part(n,m)%zpos-depth)) 
   
  END FUNCTION ibm_tidal_mig


   !!======================================================================

  SUBROUTINE ibm_detect_pycno(xe,bz,sal,temp)

   !&E---------------------------------------------------------------------
   !&E ** Purpose : detect pycnocl for part behaviour
   !&E ** Description :
   !&E ** Called by : w_behav_part
   !&E ** External calls :
   !&E ** Reference :
   !&E ** History : Marc Sourisseau 2008 ! non terminée mais pas encore utilisée
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvars2d, ONLY : h0_g,h0fond,hm,ig,id
   USE toolgeom, ONLY : f_dzu

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in )   :: xe
   REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: sal,temp
   REAL(KIND=rsh), DIMENSION(kmax,imin:imax,jmin:jmax), INTENT( in )  :: bz

   !! * Local declarations
   INTEGER  :: k_strat_max,i,j,k
   REAL(kind=rsh) :: d,roref,dzu
   REAL(kind=rsh),dimension(kmax-1,imin:imax,jmin:jmax) ::  gradv_bz

   !!----------------------------------------------------------------------
   !! * Executable part
!
! calcul du gradient vertical de densite
!-----------------------
!       sig au centre des cellules et sigw position verticale entre des limites (calcul des w)

!cval Make sure you use the same value of roref than in tsstateeqblumberg.F90
     roref=1040.0_rsh

     do  j=ljmin,ljmax
!      do  i=ig(j),id(j)
      do i=MAX(limin,ig(j)+1),MIN(limax,id(j)-1)


       depth_strat(i,j)=0.0_rsh
       d=h0_g(i,j)+xe(i,j)

       if (d .gt. hm) THEN
        k_strat_max=1
        gradv_bz(1,i,j)=0.0_rsh            !grad initial ; grad > 0
        do k=2,kmax

        dzu=REAL(f_dzu(h0_g(i,j),xe(i,j),k,i,j),rsh)   !dsigu=sigw(k)-sigw(k-1) => dzu>0
        gradv_bz(k,i,j)=((roref-bz(k-1,i,j)/grav*roref) - (roref-bz(k,i,j)/grav*roref)) &
                         /dzu !grad between k-1 and k ; grad > 0 en m-1


          if (gradv_bz(k,i,j) .gt. gradv_bz(k_strat_max,i,j)) then
           k_strat_max=k           ! numero de la couche ou le gradv de flottab. (bz) est max.sur sa frontiere inf.
          END IF
        enddo !endo on k

        if ((abs(gradv_bz(k_strat_max,i,j)) .gt. 0.02_rsh) ) then 
                                 ! .and. (d*(1.0_rsh-(sigw(k_strat_max-1))) .gt. 60.0_rsh)
                                 ! 0.02 m-1 uniquement pour les panaches / stratif estivale non visible
                                 ! 0.000005 m-1 stratif obs partout de mars à sept

         k_stratif(i,j)=k_strat_max-1  !indic of sigma levels with the highest upper gradient
         depth_strat(i,j)=d*(1.0_rsh-(sigw(k_stratif(i,j))))   !depth in m of the highest buoanc. gradient
        else
         depth_strat(i,j)=-1.0_rsh
        END IF


       END IF
      enddo
     enddo

   END SUBROUTINE ibm_detect_pycno

   !!======================================================================

   SUBROUTINE ibm_opt_depth(xe)

   !&E---------------------------------------------------------------------
   !&E
   !&E ** Purpose : Estimate the optimal depth for part. behaviour.
   !&E ** Description :
   !&E ** Called by : w_behav_part
   !&E ** External calls :
   !&E ** Reference :
   !&E ** History : marc 11/2008 ! pour l instant opt_depth fixe
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars2d, ONLY : h0_g,h0fond,hm,ig,id

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in )   :: xe

   !! * Local declarations
   INTEGER  :: k_strat_max,i,j,k
   REAL(kind=rsh) :: d

   !!----------------------------------------------------------------------
   !! * Executable part
!
! estimation of the optimal vertical depth
!-----------------------
!
     do  j=ljmin,ljmax
      do  i=MAX(limin,ig(j)+1),MIN(limax,id(j)-1)

       d= h0_g(i,j)+xe(i,j)
       if (d .gt. hm) THEN
       !  depth_strat(i,j)=15.0_rsh;

         if (depth_strat(i,j) .gt. 0.0_rsh) then
          opt_depth_part(i,j)=depth_strat(i,j)
         else
          opt_depth_part(i,j)=0.0_rsh ! dpeth in m
         END IF

       END IF
      enddo
     enddo

   END SUBROUTINE ibm_opt_depth

!!======================================================================
   FUNCTION ibm_traint(varint,xe,spos,khi,klo,px,py,i1,i2,j1,j2,rh00,rh10,rh01,rh11)

   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION traint  ***
   !&E
   !&E ** Purpose : Linear interpolation of tracer values 
   !&E              at the location (xpos,ypos,zpos)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : traj_3d.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  08-2002 (F. Dumas, P. Lazure) 
   !&E       !  11-2010 M. Huret 
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sig,dsigw
   USE trajectools, ONLY : h_int_siggen

   !! * Arguments
   REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: varint
   REAL(KIND=rsh), INTENT( in )                :: spos,px,py
   REAL(KIND=rsh)                              :: ibm_traint
   INTEGER, INTENT( in )                       :: khi,klo
   INTEGER, INTENT( in )                       :: i1,i2,j1,j2
   INTEGER, INTENT( in )                       :: rh00,rh10,rh01,rh11
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )  :: xe


   !! * Local declarations
   REAL(KIND=rsh)                              :: traintm,traintp
   REAL(KIND=rsh)                              :: xe_lag,hc_sig_lag,h0_lag
   REAL(KIND=rsh)                              :: f_lag_hzp,f_lag_hzm,f_lag_hz
   
   !!----------------------------------------------------------------------
   !! * Executable part

   
   if ((khi>kmax+1) .or. (klo>kmax+1) .or. (khi<0) .or. (klo<0)) then
      write(*,*) 'mauvais sigma fonction wint'
      stop
   end if

   !-- interpolation proprement dite sur la nappe sigma au-dessus
   IF (khi /= kmax+1) THEN
      traintp=ibm_interp(varint,khi,i1,i2,j1,j2,px,py,rh00,rh10,rh01,rh11)
   ELSE
      ibm_traint=ibm_interp(varint,klo,i1,i2,j1,j2,px,py,rh00,rh10,rh01,rh11)
   END IF
   !-- interpolation proprement dite sur la nappe sigma au-dessous
   IF (klo /= 0) THEN
      traintm=ibm_interp(varint,klo,i1,i2,j1,j2,px,py,rh00,rh10,rh01,rh11)
   ELSE
      ibm_traint=ibm_interp(varint,khi,i1,i2,j1,j2,px,py,rh00,rh10,rh01,rh11)
   END IF

   IF (khi /= kmax+1 .and. klo /= 0) THEN
   !-- interpolation sur la verticale entre la valeur au-dessus (traintp)
   !-- et la valeur au-dessous (traintm).
        f_lag_hzp=1.0_rsh
        f_lag_hzm=1.0_rsh
        f_lag_hz=1.0_rsh
#ifdef key_siggen
       CALL h_int_siggen(xe_lag,h0_lag,hc_sig_lag,xe,px,py,i1,i2,j1,j2,rh00,rh10,rh01,rh11)
       CALL f_lag_sigz_tra(spos,khi,xe_lag,h0_lag,hc_sig_lag,f_lag_hzp,f_lag_hzm,f_lag_hz)
#endif
      ibm_traint=(f_lag_hzp*(sig(khi)-spos)*traintm+f_lag_hzm*(spos-sig(klo))*traintp) &
           /(f_lag_hz*dsigw(klo)) 
   ENDIF

   END FUNCTION ibm_traint

 !!======================================================================
   FUNCTION ibm_interp(varint,k,i1,i2,j1,j2,px,py,rh11,rh21,rh12,rh22)

   !&E---------------------------------------------------------------------
   !&E                 ***  SUBROUTINE interp  ***
   !&E
   !&E ** Purpose : Horizontal interpolation with verification of boundary limits
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : traj_3d
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  01-2011 (M. Huret)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars2d, ONLY  : h0_g

   !! * Arguments
   REAL(KIND=rsh)                 :: ibm_interp
   REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: varint

   REAL(KIND=rsh), INTENT( in )   :: px,py
   INTEGER, INTENT( in )          :: k
   INTEGER, INTENT( in )          :: i1,j1,i2,j2
   INTEGER, INTENT( in )         :: rh11,rh21,rh12,rh22


   !! * Local declarations
  ! REAL(KIND=rsh)    :: rh11,rh21,rh12,rh22

   !!----------------------------------------------------------------------
   !! * Executable part
 !  rh11=h0_g(i1,j1)
 !  rh21=h0_g(i2,j1)
 !  rh12=h0_g(i1,j2)
 !  rh22=h0_g(i2,j2)

   ! check if boundary condition
   IF (rh11 > 0 .and. rh21 > 0 .and. rh12 > 0 .and. rh22 > 0) THEN
      ibm_interp=px*(1-py)*varint(k,i2,j1) + px*py         *varint(k,i2,j2) &
           +(1-px)*py*varint(k,i1,j2) + (1-px)*(1-py) *varint(k,i1,j1)

   ELSE IF (rh11 <= 0 .and. rh21 <= 0 .and. rh12 <= 0 .and. rh22 <= 0) THEN
      ibm_interp=0

   ELSE IF (rh11 <= 0 .and. rh21 <= 0 .and. rh12 <= 0 .and. rh22 > 0) THEN
      ibm_interp=varint(k,i2,j2)

   ELSE IF (rh11 <= 0 .and. rh21 <= 0 .and. rh12 > 0 .and. rh22 <= 0) THEN
      ibm_interp=varint(k,i1,j2)

   ELSE IF (rh11 <= 0 .and. rh21 <= 0 .and. rh12 > 0 .and. rh22 > 0) THEN
      ibm_interp=px*varint(k,i2,j2)+(1-px)*varint(k,i1,j2)
      
   ELSE IF (rh11 <= 0 .and. rh21 > 0 .and. rh12 <= 0 .and. rh22 <= 0) THEN
      ibm_interp=varint(k,i2,j1)
      
   ELSE IF (rh11 <= 0 .and. rh21 > 0 .and. rh12 > 0 .and. rh22 <= 0) THEN
      if (px > 0.5_rsh) then
         ibm_interp=varint(k,i2,j1) 
      else
         ibm_interp=varint(k,i1,j2)
      end if
      
   ELSE IF (rh11 <= 0 .and. rh21 > 0 .and. rh12 <= 0 .and. rh22 > 0) THEN
      ibm_interp=(1-py)*varint(k,i2,j1) + py*varint(k,i2,j2)
      
   ELSE IF (rh11 <= 0 .and. rh21 > 0 .and. rh12 > 0 .and. rh22 > 0) THEN
      ibm_interp=px*(varint(k,i2,j2)-varint(k,i1,j2))+py*(varint(k,i2,j2)-varint(k,i2,j1))-varint(k,i2,j2) &
           +varint(k,i1,j2)+varint(k,i2,j1)
      
   ELSE IF (rh11 > 0 .and. rh21 <= 0 .and. rh12 <= 0 .and. rh22 <= 0) THEN
      ibm_interp=varint(k,i1,j1)
      
   ELSE IF (rh11 > 0 .and. rh21 <= 0 .and. rh12 <= 0 .and. rh22 > 0) THEN
      if (px > 0.5_rsh) then
         ibm_interp=varint(k,i2,j2)
      else
         ibm_interp=varint(k,i1,j1)
      end if
      
   ELSE IF (rh11 > 0 .and. rh21 > 0 .and. rh12 <= 0 .and. rh22 <= 0) THEN
      ibm_interp=px*varint(k,i2,j1)+(1-px)*varint(k,i1,j1)
      
   ELSE IF (rh11 > 0 .and. rh21 > 0 .and. rh12 <= 0 .and. rh22 > 0) THEN
      ibm_interp=px*(varint(k,i2,j1)-varint(k,i1,j1))+py*(varint(k,i2,j2)-varint(k,i2,j1))+varint(k,i1,j1)
      
   ELSE IF (rh11 > 0 .and. rh21 <= 0 .and. rh12 > 0 .and. rh22 <= 0) THEN
      ibm_interp=py*varint(k,i1,j2) + (1-py)*varint(k,i1,j1)
      
   ELSE IF (rh11 > 0 .and. rh21 > 0 .and. rh12 > 0 .and. rh22 <= 0) THEN
      ibm_interp=px*(varint(k,i2,j1)-varint(k,i1,j1))+py*(varint(k,i1,j2)-varint(k,i1,j1))+varint(k,i1,j1)
      
   ELSE IF (rh11 > 0 .and. rh21 <= 0 .and. rh12 > 0 .and. rh22 > 0) THEN
      ibm_interp=px*(varint(k,i2,j2)-varint(k,i1,j2))+py*(varint(k,i1,j2)-varint(k,i1,j1))+varint(k,i1,j1)
      
   END IF
   
 END FUNCTION ibm_interp
 
   !!======================================================================
   SUBROUTINE ibm_loc_xyz(xe,xpos,ypos,spos,px,py,i1,i2,j1,j2,rh11,rh21,rh12,rh22,khi,klo)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE location  ***
   !&E
   !&E ** Purpose : search index of the cells around particle
   !&E              and location of the particle inside the cell 
   !&E              for later interpolation 
   !&E              Valid relative to h0,xe,wkz... 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : traj_3d
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E               12/07 M. Huret
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
     USE comvars2d, ONLY  :h0_g
     USE parameters,    ONLY : valmanq
     USE trajectools, ONLY : ksupkinf
     
     
     !! * Arguments
     REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )  :: xe
     INTEGER , INTENT( out )            :: rh11,rh21,rh12,rh22
     REAL(KIND=rsh), INTENT( in )       :: xpos,ypos,spos
     INTEGER, INTENT( out )             ::i1,i2,j1,j2
     REAL(KIND=rsh), INTENT( out )      :: px,py 
     INTEGER, INTENT( out )             :: khi,klo
     
     !! * Local declarations
     REAL(KIND=rsh)                         :: xe_lag,h0_lag
     REAL(KIND=rsh),   DIMENSION(0:kmax+1)    :: spos_w  


   !!----------------------------------------------------------------------
   !! * Executable part

   ! indexes of the cell where the particle is
   ! reperage de la maille XE ou se situe la particule

   i1=INT(xpos)
   i2=INT(xpos)+1
   j1=INT(ypos)
   j2=INT(ypos)+1

   i1 = MIN(MAX(i1,imin),imax)
   i2 = MIN(MAX(i2,imin),imax)
   j1 = MIN(MAX(j1,jmin),jmax)
   j2 = MIN(MAX(j2,jmin),jmax)

   ! location of the particle inside the cell
   ! calcul des positions px et py dans la maille XE

   px=xpos-INT(xpos)
   py=ypos-INT(ypos)

   ! check for next cells in water
   rh11=1
   rh12=1
   rh21=1
   rh22=1

   IF (h0_g(i1,j1)==-valmanq) THEN
     rh11=0
   END IF
   IF (h0_g(i1,j2)==-valmanq) THEN
     rh12=0
   END IF
   IF (h0_g(i2,j1)==-valmanq) THEN
     rh21=0
   END IF
   IF (h0_g(i2,j2)==-valmanq) THEN
     rh22=0
   END IF

   ! get neighbooring indices on the vertical

   spos_w(0)=-1.0_rsh
   spos_w(kmax+1)=0.0_rsh
   spos_w(1:kmax)=sig(1:kmax)

   klo=0
   khi=kmax+1
   CALL ksupkinf(spos,spos_w,kmax+2,khi,klo,6)

 END SUBROUTINE ibm_loc_xyz


!====================================================================

 SUBROUTINE gasdev_s(harvest)

   REAL(rsh), INTENT(OUT) :: harvest
   !Returns in harvest a normally distributed deviate with zero mean and unit variance, using
   !ran1 as the source of uniform deviates.
   REAL(rsh) :: rsq,v1,v2
   REAL(rsh), SAVE :: g
   LOGICAL, SAVE :: gaus_stored=.false.
   
   if (gaus_stored) then ! We have an extra deviate handy,harvest=g so return it,
      gaus_stored=.false. !and unset the flag.
   else !We dont have an extra deviate handy, so
      do
         call random_number(v1) ! pick two uniform numbers in the square extending from 
         call random_number(v2) !-1 to +1 in each direction,
         v1=2.0_rsh*v1-1.0_rsh
         v2=2.0_rsh*v2-1.0_rsh
         rsq=v1**2+v2**2 !see if they are in the unit circle,
         if (rsq > 0.0 .and. rsq < 1.0) exit
      end do !otherwise try again.
      rsq=sqrt(-2.0_rsh*log(rsq)/rsq) !Now make the Box-Muller transformation to get two normal deviates. Return one and save the other for next time.
      harvest=v1*rsq
      g=v2*rsq
      gaus_stored=.true. ! Set flag.
   end if

 END SUBROUTINE gasdev_s


   !!======================================================================

 SUBROUTINE ibm_save

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ibm_save  ***
   !&E
   !&E ** Purpose : Save trajectories
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : step.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  01-2011 (M. Huret) 
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars2d,   ONLY : nbouc,date_ref,t,iscreenlog
   USE ionc4,       ONLY : ionc4_createfile_traj,ionc4_torigine,ionc4_tunite, &
                           ionc4_createvar_traj,ionc4_write_time,ionc4_sync,  &
                           ionc4_write_trajt
   USE_MPI toolmpi, ONLY : iproc,exchange_vectcpu_int

   !! * Arguments

   !! * Local declarations
   LOGICAL                                     :: l_out_nc4par
   INTEGER                                     :: n,m,k,kw,kwm,i,j,num1,num2,nbpart_out,p,nbproc,ind1
   REAL(kind=rsh)                              :: fillval
   REAL(KIND=rlg)                              :: tool_ind2lat,tool_ind2lon
   INTEGER,        ALLOCATABLE, DIMENSION(:)   :: num_out
   REAL(KIND=rlg), ALLOCATABLE, DIMENSION(:)   :: lat_out,lon_out
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: temp_out,flag_out,spos_out,zpos_out,xpos_out,ypos_out 
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: h0pos_out,age_out,size_out,stage_out,nb_out
   !!----------------------------------------------------------------------
   !! * Executable part

   IF(rsh==8) then 
      fillval = dg_valmanq_io
   ELSE
      fillval = rg_valmanq_io
   ENDIF

   ALLOCATE( xpos_out(0:nbmax_part),ypos_out(0:nbmax_part) )
   ALLOCATE( spos_out(0:nbmax_part),zpos_out(0:nbmax_part) )
   ALLOCATE( lat_out(0:nbmax_part),lon_out(0:nbmax_part) )
   ALLOCATE( temp_out(0:nbmax_part) )
   ALLOCATE( h0pos_out(0:nbmax_part),num_out(0:nbmax_part) )
   ALLOCATE( flag_out(0:nbmax_part) )
   ALLOCATE( age_out(0:nbmax_part),size_out(0:nbmax_part) )
   ALLOCATE( stage_out(0:nbmax_part),nb_out(0:nbmax_part) )

   xpos_out(:)=fillval ; ypos_out(:)=fillval ; spos_out(:)=fillval ; zpos_out(:)=-fillval ; h0pos_out(:)=-fillval
   flag_out(:)=fillval ; lat_out(:)=dg_valmanq_io ; lon_out(:)=dg_valmanq_io ; num_out(:)=0 ; temp_out(:)=fillval
   age_out(:)=fillval ; size_out(:)=fillval ; stage_out(:)=fillval ; nb_out(:)=fillval

   ! Create output file
   IF (nbouc == 0) THEN

#ifdef key_MPI_2D
     l_out_nc4par=.true.
#else
     l_out_nc4par=.false.
#endif

     DO n=1,nb_patch
#ifdef key_MPI_2D
        nbpart_out=nb_part(n)+1  ! part 0 fictive just for output of CPU without particles
#else
        nbpart_out=nb_part(n)
#endif
        CALL ionc4_createfile_traj(filetraj3dout(n),nbpart_out,0,0,l_out_nc4par=l_out_nc4par)
        CALL ionc4_torigine(filetraj3dout(n),date_ref)
        CALL ionc4_tunite(filetraj3dout(n),'seconds')
        CALL ionc4_createvar_traj(filetraj3dout(n),"DEPTH","m","depth as immersion",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"H0","m","h0",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"NUM","","number of the particle",fill_value=0)
        CALL ionc4_createvar_traj(filetraj3dout(n),"flag","nbr","flag",fill_value=-fillval) 
        CALL ionc4_createvar_traj(filetraj3dout(n),"TEMP","degrees_Celsius","temperature",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"AGE","Days","Age",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"SIZE","Centimeters","Size",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"STAGE","","Stage",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"NUMBER","Number","Number of individuals",fill_value=-fillval)
     END DO
  END IF

   DO n=1,nb_patch
     IF ((t >= tsavtraj(n)).and.(t <= tendtraj(n))) THEN

        ! get the number of particles per cpus
#ifdef key_MPI_2D
        nb_part_percpu(n,iproc)=count(part(n,:)%inoutflag)
        CALL_MPI exchange_vectcpu_int(nb_part_percpu(n,:))
#endif
        
        IF_MPI ( nb_part_percpu(n,iproc) /= 0 ) THEN
        p=0
       DO m = 1,nb_part(n)
          IF_MPI ( part(n,m)%inoutflag ) THEN
          p=p+1
          xpos_out(p)=part(n,m)%xpos
          ypos_out(p)=part(n,m)%ypos
          lat_out(p)=tool_ind2lat(part(n,m)%xpos,part(n,m)%ypos)
          lon_out(p)=tool_ind2lon(part(n,m)%xpos,part(n,m)%ypos)
          spos_out(p)=part(n,m)%spos
          zpos_out(p)=part(n,m)%zpos ! for output as immersion
          h0pos_out(p)=part(n,m)%d3 
          flag_out(p)=part(n,m)%flag   
          temp_out(p)=part(n,m)%temp
          age_out(p)=part(n,m)%age
          size_out(p)=part(n,m)%size
          stage_out(p)=part(n,m)%stage
          nb_out(p)=part(n,m)%number
          num_out(p)=m
          
          ENDIF_MPI

       END DO
       ENDIF_MPI

       CALL ionc4_write_time(filetraj3dout(n),0,t)

#ifdef key_MPI_2D
       nbproc=nb_part_percpu(n,iproc)
       IF_MPI ( nbproc /= 0 ) THEN
         ind1=1
         IF (iproc==0) THEN
            num1=2 ! 1 reserve a quand absence de part dans une CPU
         ELSE
            num1=SUM(nb_part_percpu(n,0:iproc-1))+2
         END IF
         num2=num1+nbproc-1
       ELSE_MPI  ! on met tout a 0 pour l indice 0.Indice d ecriture par defaut
          ind1=0
          num1=1
          num2=1
          lat_out(ind1)=REAL(ind1,rsh)
          lon_out(ind1)=lat_out(ind1)
          spos_out(ind1)=lat_out(ind1)
          zpos_out(ind1)=lat_out(ind1)
          xpos_out(ind1)=lat_out(ind1)
          ypos_out(ind1)=lat_out(ind1)
          h0pos_out(ind1)=lat_out(ind1)
          flag_out(ind1)=lat_out(ind1)
          temp_out(ind1)=lat_out(ind1)
          age_out(ind1)=lat_out(ind1)
          size_out(ind1)=lat_out(ind1)
          stage_out(ind1)=lat_out(ind1)
          nb_out(ind1)=lat_out(ind1)
          num_out(ind1)=ind1
       ENDIF_MPI
       CALL ionc4_write_trajt(filetraj3dout(n),'latitude',lat_out(ind1:nbproc),num1,num2,0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'longitude',lon_out(ind1:nbproc),num1,num2,0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'DEPTH',zpos_out(ind1:nbproc),num1,num2,0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'H0',h0pos_out(ind1:nbproc),num1,num2,0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUM',num_out(ind1:nbproc),num1,num2,0)
       CALL ionc4_write_trajt(filetraj3dout(n),'flag',flag_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'TEMP',temp_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'AGE',age_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'SIZE',size_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'STAGE',stage_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUMBER',nb_out(ind1:nbproc),num1,num2,0,fillval)
#else
       CALL ionc4_write_trajt(filetraj3dout(n),'latitude',lat_out(1:nb_part(n)),1,nb_part(n),0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'longitude',lon_out(1:nb_part(n)),1,nb_part(n),0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'DEPTH',zpos_out(1:nb_part(n)),1,nb_part(n),0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'H0',h0pos_out(1:nb_part(n)),1,nb_part(n),0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUM',num_out(1:nb_part(n)),1,nb_part(n),0)
       CALL ionc4_write_trajt(filetraj3dout(n),'flag',flag_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'TEMP',temp_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'AGE',age_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'SIZE',size_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'STAGE',stage_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUMBER',nb_out(1:nb_part(n)),1,nb_part(n),0,fillval)
#endif

       ! To write the data on the disk and not loose data in case of run crash
       CALL ionc4_sync(filetraj3dout(n))

       tsavtraj(n)=tsavtraj(n)+dtsave_traj(n)*3600.d0
     END IF
   END DO

   END SUBROUTINE ibm_save


#ifdef key_siggen

!!======================================================================
   SUBROUTINE f_lag_sigz_tra(spos,k,xe_lag,h0_lag,hc_sig_lag,f_lag_hzp,f_lag_hzm,f_lag_hz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE f_lag  ***
   !&E
   !&E ** Purpose : Scale factor from generalized sigma to z
   !&E              for vertical interpolation at particle location
   !&E
   !&E ** Description :
   !&Ex3
   !&E ** Called by : traj_3d
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History : M. Huret 12/1010
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comsiggen, ONLY : b_sig,theta_sig,Csu_sig,dcwsds
   USE comvars3d,  ONLY : sig


   !! * Arguments
   REAL(KIND=rsh), INTENT( in ) :: spos,xe_lag,h0_lag,hc_sig_lag
   REAL(KIND=rsh), INTENT(out ) :: f_lag_hzp,f_lag_hzm,f_lag_hz
   INTEGER,        INTENT( IN ) :: k

   !! * Local declarations
   REAL(KIND=rsh)      :: Csusig, dcwsdsp,dcwsdsm
   
   !!----------------------------------------------------------------------
   !! * Executable part

   

   Csusig=(1.0_rsh-b_sig)*SINH(theta_sig*spos)/SINH(theta_sig) &
         + 0.5_rsh*b_sig*(TANH(theta_sig*(spos+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)

   dcwsdsp=0.0_rsh
   dcwsdsm=0.0_rsh

   IF (spos /= sig(k)) THEN
      dcwsdsp=  (Csu_sig(k)-Csusig)/(sig(k)-spos)
   ENDIF
   IF (spos /= sig(k-1)) THEN
      dcwsdsm=  (Csusig-Csu_sig(k-1))/(spos-sig(k-1))
   ENDIF


   f_lag_hzp=xe_lag+hc_sig_lag+(h0_lag-hc_sig_lag)* dcwsdsp
   f_lag_hzm=xe_lag+hc_sig_lag+(h0_lag-hc_sig_lag)* dcwsdsm

   f_lag_hz=xe_lag+hc_sig_lag+(h0_lag-hc_sig_lag)* dcwsds(k-1)


  END SUBROUTINE f_lag_sigz_tra


! end key_siggen
#endif

!end key_ibm
#endif

 END MODULE
