SUBROUTINE ALLOC

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ALLOC  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       ! 2010-02 (G. Simon) add wave module
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY : kmax,valmanq,rsh,                    &
                         limin,limax,liminm1,limaxp1,limaxp2, &
                         ljmin,ljmax,ljminm1,ljmaxp1,ljmaxp2, &
                         liminm2,ljminm2,limaxp3,ljmaxp3
   USE comvars2d, ONLY : l_modele2d,alloc_comvars2d,hex,hey,  &
                         htx,hty,l_initfromfile
   USE comfluxdo, ONLY : alloc_comfluxdo,alloc_comfluxdo3d,alloc_fsalxy
! S.Petton 09/2013 : Utilisation d un profil moyen
   USE comvars3d, ONLY : alloc_comvars3d,wz,boz2,rho2,qp1,rho
#ifdef key_diag_physwz
   USE comvars3d, ONLY : wz_phys 
#endif    
   USE cometemp,  ONLY : alloc_cometemp,alloc_ftempxy
   USE sflxatm,   ONLY : alloc_sflxatm
   USE comdem,    ONLY : alloc_comdem
   USE output,    ONLY : output_alloc
   USE cometdyn,  ONLY : alloc_cometdyn
   USE comvarp2d, ONLY : sal,temp,salav,tempav,grad1T,grad1S,grad2T,grad2S
   USE comvarp2d, ONLY : u,v,ustar,vstar,ssh,ssh1,ssh2
   USE comvarp2d, ONLY : sal2,temp2
#ifdef key_tssub_macho3d
   USE comvarp2d, ONLY : sal3,temp3
#endif
#if defined key_tssub_adv_fifthorder || defined key_tssub_adv_fourthorder
   USE comvarp2d, ONLY : sal5,temp5  !VALERIE PROVISOIRE
#endif
   USE comvarp2d, ONLY : alloc_comvarp2d
   USE comvarp3d, ONLY : boz,uz,vz,ect,psi
   USE obccombine,ONLY : alloc_obc
   USE comturb,   ONLY : alloc_comturb
   USE comaree,   ONLY : alloc_comaree
#if defined key_tssub_wquickupwind || defined key_tssub_wcompact
   USE comfluvert,ONLY : alloc_comfluvert
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex,hzey
   USE comsiggen, ONLY : alloc_comsiggen
#endif
#ifdef key_wave
   USE comwave,   ONLY : alloc_comwave
#endif

#ifdef key_diagdyn
  USE  diagdyn,   ONLY : diagdyn_alloc
#endif
   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part
   CALL alloc_comvars2d
   CALL output_alloc
   CALL alloc_comdem
   CALL alloc_cometdyn
#ifdef key_wave
   CALL alloc_comwave
#endif
   CALL alloc_cometemp
   CALL alloc_comfluxdo
   CALL alloc_comvars3d
   CALL alloc_sflxatm
   CALL alloc_obc
   CALL alloc_ftempxy
   CALL alloc_fsalxy
   ! Allocation of variables : eps
   CALL alloc_comturb
   ! Allocation of variables : potgene
   CALL alloc_comaree
   ! Allocation of variables : u_vtmp,v_utmp
   CALL alloc_comvarp2d
#if defined key_tssub_wquickupwind || defined key_tssub_wcompact
   CALL alloc_comfluvert
#endif
   ! Allocation of variables of the different terms of the moment equations
   ! for diagnoses purpose
#ifdef key_diagdyn
    CALL diagdyn_alloc
#endif  

   IF(.NOT.l_modele2d) CALL alloc_comfluxdo3d

   ! Allocate dynamics variables
! S.Petton 09/2013 : Utilisation d un profil moyen
   ALLOCATE( boz2(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) ) !MANGA
   ALLOCATE( rho2(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) ) !MANGA
   ALLOCATE(  boz(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE(  qp1(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE(  rho(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   !FVDM
   boz(:,:,:)=0 ! valmanq
   qp1(:,:,:)=0 ! valmanq
   rho(:,:,:)=0 ! valmanq
      
   ALLOCATE( hex(liminm2:limaxp2,ljmin:ljmax) )
   ALLOCATE( hey(limin:limax,ljminm2:ljmaxp2) )
   hex(:,:)=0.0_rsh
   hey(:,:)=0.0_rsh
   !ALLOCATE( htx(liminm1:limax  ,ljminm1:ljmaxp1) )
   !ALLOCATE( hty(liminm1:limaxp1,ljminm1:ljmax  ) )
   ! larger because of interpvit (in avance) in trajectools.F90
   ALLOCATE( htx(liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( hty(liminm2:limaxp2,ljminm2:ljmaxp2) )
   htx(:,:)=0.0_rsh ! to manage with sleeping cpu
   hty(:,:)=0.0_rsh
   ALLOCATE( ect(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE( psi(1:kmax,limin:limax,ljmin:ljmax) )
   ect(:,:,:)=0.0_rsh
   psi(:,:,:)=0.0_rsh

   ALLOCATE( u(liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( v(liminm2:limaxp2,ljminm2:ljmaxp2) )
   u(:,:)=0.0_rsh ! to manage with sleeping cpu
   v(:,:)=0.0_rsh
   ALLOCATE( uz(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( vz(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2) )
   !ALLOCATE( wz(0:kmax,limin:limaxp1,ljmin:ljmaxp1) )
   ! larger extension because of traj_3d routine
   ! might be not sufficient because of the medium location
   ALLOCATE( wz(0:kmax,liminm1:limaxp1,ljminm1:ljmaxp1) )
#ifdef key_diag_physwz
   ALLOCATE( wz_phys(0:kmax,limin:limax,ljmin:ljmax) ) 
   wz_phys(:,:,:)=0.0_rsh
#endif
   IF(l_modele2d .AND. l_initfromfile) THEN
     ALLOCATE( vstar(liminm2:limaxp2,ljminm2:ljmaxp2) )
     vstar(:,:)=0.0_rsh
   END IF
#if ! defined key_dbg
   IF(.NOT.l_modele2d) THEN
#endif
     ALLOCATE( ustar(liminm2:limaxp2,ljminm2:ljmaxp2) )
     ALLOCATE( vstar(liminm2:limaxp2,ljminm2:ljmaxp2) )
     ustar(:,:)=0.0_rsh
     vstar(:,:)=0.0_rsh
#if ! defined key_dbg
   END IF
#endif
   ALLOCATE( ssh(liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE(ssh1(liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE(ssh2(liminm2:limaxp2,ljminm2:ljmaxp2) )
   ssh(:,:)=0.0_rsh
   ssh1(:,:)=0.0_rsh
   ssh2(:,:)=0.0_rsh

   ! Allocate thermodynamics variables
   ALLOCATE( sal(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE( temp(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE(salav(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE(tempav(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   ALLOCATE(grad1T(1:kmax,liminm1:limaxp1,ljminm1:ljmaxp1))   
   ALLOCATE(grad1S(1:kmax,liminm1:limaxp1,ljminm1:ljmaxp1))   
   ALLOCATE(grad2T(1:kmax,liminm1:limaxp1,ljminm1:ljmaxp1))   
   ALLOCATE(grad2S(1:kmax,liminm1:limaxp1,ljminm1:ljmaxp1))
   ALLOCATE( sal2(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE( temp2(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   salav(:,:,:)=0.0_rsh
   tempav(:,:,:)=0.0_rsh
   grad1T(:,:,:)=0.0_rsh ; grad1S(:,:,:)=0.0_rsh
   grad2T(:,:,:)=0.0_rsh ; grad2S(:,:,:)=0.0_rsh
   sal2(:,:,:)=0.0_rsh
   temp2(:,:,:)=0.0_rsh
#ifdef key_tssub_macho3d
   ALLOCATE( temp3(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE( sal3(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   sal3(:,:,:)=0.0_rsh
   temp3(:,:,:)=0.0_rsh
#endif
#if defined key_tssub_adv_fifthorder || defined key_tssub_adv_fourthorder
   ALLOCATE( temp5(1:kmax,liminm2:limaxp3,ljminm2:ljmaxp3) ) !VALERIE PROVISOIRE
   ALLOCATE( sal5(1:kmax,liminm2:limaxp3,ljminm2:ljmaxp3) )
   sal5(:,:,:)=0.0_rsh
   temp5(:,:,:)=0.0_rsh
#endif

   ! Variables relative to generalized vertical coordinates
#if defined key_siggen || defined key_gencoord
   ! Allocation of variables : hzutx,hzuty,hzwtx,hzwty
   CALL alloc_comsiggen
   ALLOCATE(hzex(1:kmax,liminm1:limax,ljmin  :ljmax) )
   ALLOCATE(hzey(1:kmax,limin  :limax,ljminm1:ljmax) )
   hzex(:,:,:)=0.0_rsh
   hzey(:,:,:)=0.0_rsh
#endif

#ifdef key_plus
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE( (liminm2:limaxp2,ljminm2:ljmaxp2) )
#endif

  END SUBROUTINE ALLOC
