
 MODULE comwave

#ifdef key_wave
   !!======================================================================
   !!                       ***  MODULE comwave   ***
   !!======================================================================

   !&E
   !&E ** Called by:
   !&E      module wave_read (in wave.F90)
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Creation  on the basis of cometdyn.F90
   !&E       !  2010-07 (R. Verney) Introduction of orbital velocity and wave period variables
   !&E
   !!======================================================================
   !! * Modules used
   USE parameters

   !! * Public variables
   INTEGER               :: idimt ! Number of sample in the wave file
   INTEGER               :: iwavemin,iwavemax,jwavemin,jwavemax ! Extrem spatial size of wave array
   INTEGER               :: ilec_wave ! For manage time sample
   INTEGER               :: ilecmem_wave ! For manage time sample
   INTEGER               :: iwmin0  ! shift to read the correct index in the netcdf file


   CHARACTER(LEN=lchain) :: file_wave ! Location of the NetCDF file
   REAL(KIND=rsh)        :: valfill_wave ! Fill value (ground, the same for each variable of waves file)
   INTEGER               :: cdf_id_wave
   REAL(KIND=rlg)        :: tnc1_wave,tnc2_wave

   !! For locate MARS grid from WAVE grid
   !!   Related to center of the MARS cell
   INTEGER,       DIMENSION(:,:),ALLOCATABLE :: i1xe_wave,j1xe_wave
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: xdpxe_wave,ydpxe_wave
   !!   Related to zonal velocity (East face)
   INTEGER,       DIMENSION(:,:),ALLOCATABLE :: i1u_wave,j1u_wave
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: xdpu_wave,ydpu_wave
   !!   Related to meridional velocity (North face)
   INTEGER,       DIMENSION(:,:),ALLOCATABLE :: i1v_wave,j1v_wave
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: xdpv_wave,ydpv_wave

   !! * Waves quantities -neww-
   !!   - Significant height
   LOGICAL               :: l_wave_rhs
   CHARACTER(LEN=lchain) :: name_wave_hs
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_hs1,wave_hs2
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: wave_hs
   !!   - Bottom orbital velocity
   LOGICAL               :: l_wave_rubot
   CHARACTER(LEN=lchain) :: name_wave_ubot
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_ubot1,wave_ubot2,wave_ubot
   LOGICAL               :: l_wave_ruubot
   CHARACTER(LEN=lchain) :: name_wave_uubot
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_uubot1,wave_uubot2,wave_uubot
   LOGICAL               :: l_wave_rvubot
   CHARACTER(LEN=lchain) :: name_wave_vubot
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_vubot1,wave_vubot2,wave_vubot
   !!  - Energie flux from waves to ocean
   LOGICAL               :: l_wave_rfoc
   CHARACTER(LEN=lchain) :: name_wave_foc
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_foc1,wave_foc2
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: wave_foc
   !!  - Directionality
   LOGICAL               :: l_wave_rdir
   CHARACTER(LEN=lchain) :: name_wave_dir
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_dir1,wave_dir2
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: wave_dir
   !!  - Wave peak period
   LOGICAL               :: l_wave_rtp
   CHARACTER(LEN=lchain) :: name_wave_tp
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_tp1,wave_tp2,wave_tp
   !!  - If data have been interpolated on the MARS grid in advance
   LOGICAL               :: l_wave_marsgrid
   ! 3D dynamics
!toseelater allouer ces variables
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE :: tosx3dm,tosy3dm

  CONTAINS

   !!======================================================================

  SUBROUTINE alloc_comwave

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE alloc_comwave  ***
   !&E
   !&E ** Purpose : Allocate array use wave
   !&E
   !&E ** Called by :
   !&E       ! module alloc (file alloc.F90)
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Creation  on the basis of alloc_cometdyn
   !&E
   !!----------------------------------------------------------------------
   !! * Executable part

   !! * Locate MARS cell from Wave cell 
   ALLOCATE(i1v_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(j1v_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(i1u_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(j1u_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(i1xe_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(j1xe_wave(limin:limax,ljmin:ljmax))

   ALLOCATE(xdpxe_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(xdpu_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(xdpv_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(ydpxe_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(ydpu_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(ydpv_wave(limin:limax,ljmin:ljmax))
   !! * Waves quantities -neww-
   !!   - directionality
   ALLOCATE(wave_dir1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_dir2(limin:limax,ljmin:ljmax))
   wave_dir1(:,:)=0.0_rsh ; wave_dir2(:,:)=0.0_rsh
   !!   - significant height
   ALLOCATE(wave_hs1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_hs2(limin:limax,ljmin:ljmax))
   wave_hs1(:,:) =0.0_rsh ; wave_hs2(:,:) =0.0_rsh
   !!   - energy flux from waves to ocean
   ALLOCATE(wave_foc1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_foc2(limin:limax,ljmin:ljmax))
   wave_foc1(:,:)=0.0_rsh ; wave_foc2(:,:)=0.0_rsh
   !!   - orbital velocity
   ALLOCATE(wave_ubot1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_ubot2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_ubot(limin:limax,ljmin:ljmax))
   wave_ubot1(:,:)=0.0_rsh ; wave_ubot2(:,:)=0.0_rsh ; wave_ubot(:,:)=0.0_rsh
   ALLOCATE(wave_uubot1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_uubot2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_uubot(limin:limax,ljmin:ljmax))
   wave_uubot1(:,:)=0.0_rsh ; wave_uubot2(:,:)=0.0_rsh ; wave_uubot(:,:)=0.0_rsh
   ALLOCATE(wave_vubot1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_vubot2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_vubot(limin:limax,ljmin:ljmax))
   wave_vubot1(:,:)=0.0_rsh ; wave_vubot2(:,:)=0.0_rsh ; wave_vubot(:,:)=0.0_rsh
   !!   - wave period
   ALLOCATE(wave_tp1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_tp2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_tp(limin:limax,ljmin:ljmax))
   wave_tp1(:,:)=0.0_rsh ; wave_tp2(:,:)=0.0_rsh ; wave_tp(:,:)=0.0_rsh
#ifdef key_palm_ww3
   ALLOCATE(tosx3dm(imin:imax,jmin:jmax,kmax))
   ALLOCATE(tosy3dm(imin:imax,jmin:jmax,kmax))
#endif
  END SUBROUTINE alloc_comwave

   !!======================================================================

  SUBROUTINE dealloc_comwave

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dealloc_comwave  ***
   !&E
   !&E ** Purpose : Deallocate array use wave
   !&E
   !&E ** Called by :
   !&E       ! module dealloc (file dealloc.F90)
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Creation  on the basis of dealloc_cometdyn
   !&E
   !&E---------------------------------------------------------------------
   !! * Executable part

   !! * Locate MARS cell from Wave cell 
   DEALLOCATE(xdpxe_wave,ydpxe_wave,xdpu_wave,ydpu_wave,xdpv_wave,ydpv_wave)
   DEALLOCATE(i1v_wave,j1v_wave,i1u_wave,j1u_wave,i1xe_wave,j1xe_wave)

   !! * Waves quantities -neww-
   !!   - directionality
   DEALLOCATE(wave_dir1,wave_dir2)
   !!   - significant height
   DEALLOCATE(wave_hs1,wave_hs2)
   !!   - energy flux from waves to ocean
   DEALLOCATE(wave_foc1,wave_foc2)
   !!   - orbital velocity
   DEALLOCATE(wave_ubot1,wave_ubot2,wave_ubot)
   DEALLOCATE(wave_uubot1,wave_uubot2,wave_uubot)
   DEALLOCATE(wave_vubot1,wave_vubot2,wave_vubot)
   !!   - wave period
   DEALLOCATE(wave_tp1,wave_tp2,wave_tp)

   !!   - radiative stress
#ifdef key_palm_ww3
   DEALLOCATE(tosx3dm,tosy3dm)
#endif
   
  END SUBROUTINE dealloc_comwave

   !!======================================================================

#endif

  END MODULE comwave
