 MODULE parameters

   !!======================================================================
   !!                   ***  MODULE parameters_rank1  ***
   !! 
   !! ** Description : parameters for a rank 1 configuration
   !! 
   !!
   !! ** Reference :
   !!                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !!                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !!
   !! ** History :
   !!       !  2008-07 (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !!       !  2008-09 (G. Galibert / ALTRAN) attributes of output files
   !!                   to respect NetCDF OCO v1.1 file format
   !!                   modification of rg_valmanq_io and addition of dg_valmanq_io
   !!
   !!======================================================================
   !! * Modules used

   !! * Shared or public module variables
#if defined key_agrif
   INTEGER,PARAMETER :: riosh=4,riolg=8,rlg=8,rsh=8,lchain=200
#else
   INTEGER,PARAMETER :: riosh=4,riolg=8,rlg=8,rsh=8,lchain=200
! S.T. 31 mars 
#if defined key_xios
   INTEGER :: MPI_COMM_MODEL
#endif
! S.T.
#endif
   CHARACTER(len=4),PARAMETER :: nrac='BACH'
   LOGICAL,PARAMETER :: l_testcase=.FALSE.
   INTEGER,PARAMETER :: num_testcase=0

   INTEGER,PARAMETER :: no=1 ! rank
   INTEGER           &
#if ! defined key_agrif
          ,PARAMETER &
#endif
                     :: imin=0,imax=1448,jmin=0,jmax=1281 ! configuration rang_1
!                     :: imin=0,imax=1100,jmin=0,jmax=462 ! configuration rang_1
! obc_width=number of lines (or columns) in your obc files - 1
   !INTEGER,PARAMETER :: kmax=20,obc_width=20
   INTEGER,PARAMETER :: kmax=100,obc_width=20

#ifdef key_tide_saverotatedobc
   INTEGER,PARAMETER :: imin_child=0,jmin_child=0,imax_child=131,jmax_child=100
#endif

#ifdef key_substance
   INTEGER,PARAMETER :: nb_var=17
#ifdef key_contaminant
   INTEGER,PARAMETER :: ntypreac=7         !! nb types de reactions déjà programmées
!!  4 parameters for preliminary and temporary arrays of contaminants
   INTEGER,PARAMETER  :: nfracvmax=10       !! nb max fractions possibles regroupées dans une variable
   INTEGER,PARAMETER  :: nflimcomax=10      !! nb max limiting functions for generic reaction of contaminant
   INTEGER,PARAMETER  :: nrecomax=20        !! nb max reaction affected contaminant
   INTEGER,PARAMETER  :: nvactcomax=10      !! nb max active variables for contaminant
#endif
#ifdef key_sedim
   INTEGER,PARAMETER :: ksdmin=1,ksdmax=10
#endif
#endif
   INTEGER,PARAMETER :: nbpartmax=1,nbtachmax=1
   INTEGER,PARAMETER :: ngridmax=10,nmarmax=143,nbchprox=1

   ! global variables for characteristics method (obc)
   ! coefficient used when correcting matrixes
   REAL(KIND=rsh),PARAMETER :: al1=1.0_rsh,al2=1.0_rsh,al3=-1.0_rsh,ac0=1.0_rsh-al1/4.0_rsh, &
                               ac1=al2+al3/2.0_rsh,ac2=al2/4.0_rsh+al3/2.0_rsh

   ! implicit and explicit coefficients
   REAL(kind=rsh)  ,PARAMETER :: cfroe=0.0_rsh,cfroi=1.0_rsh-cfroe
   ! alphap (p for previous) must be lower than 0.5
   ! alphan (n for next)
   REAL(kind=rsh)  ,PARAMETER :: alphap=0.45_rsh,alphan=1.0_rsh-alphap
   REAL(kind=rsh)  ,PARAMETER :: valmanq=999.0_rsh
   REAL(kind=riosh),PARAMETER :: rg_valmanq_io=999.0_riosh
   REAL(kind=riolg),PARAMETER :: dg_valmanq_io=-1.7e+38
   REAL(kind=rlg)  ,PARAMETER :: pi=3.141592654_rlg,tref=59958230400_rlg,epsilon=0.00000001_rlg
   REAL(KIND=rsh)  ,PARAMETER :: grav=9.81_rsh
   REAL(KIND=rlg)  ,PARAMETER :: exp1=2.718281828_rlg

#ifdef key_agrif
   INTEGER :: imaxm2=imax-2
   INTEGER :: jmaxm2=jmax-2
   INTEGER :: lax=imax+jmax
#endif

   INTEGER :: ljmin=jmin,ljminp1=jmin+1,ljminp2=jmin+2,ljmax=jmax,ljmaxm1=jmax-1,ljminm2=jmin, &
              ljmaxp2=jmax,ljminm1=jmin,ljmaxp1=jmax,ljmaxm1p1=jmax-1,ljminp1m1=jmin+1

   INTEGER :: limin=imin,limax=imax,limaxp1=imax,limaxm1=imax-1,liminp1=imin+1, &
              liminp2=imin+2,liminm1=imin,liminm2=imin,limaxp2=imax,limaxm1p1=imax-1, &
              liminp1m1=imin+1,liminp1m2=imin+1,ljminp1m2=jmin+1,ljmaxp3=jmax,limaxp3=imax

 END MODULE parameters
