!********************************************************************
!
      subroutine agrif_obc_ssh(xeuv)
!
!********************************************************************
!
!     objet    : routine de calculs des conditions aux limites
!                en niveau dans le cas d un modele emboite, pour lequel
!                on dispose d un fichier de conditions aux limites etabli
!                par un grand modele le contenant. Cas des frontieres
!                rectiligne.
!
!
!     appelant         : init_xe, stepu, stepv, step2dxy, step2dyx
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          :
!                      :
!     variables in     : nbouc, nbouc2d, filecl, ieac, iwac
!     variables in     : jnac, jsac, dgac, dfiac, dgp, dfip
!     variables in     : t, it
!                      :
!     variables out    : xe, ul, vl
!                      :
!     variables inout  : xlim, ylim
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas (test sur profondeur 2m au lieu de 2*h0fond)
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
USE parameters
use comvars2d
use comvarp2d
use agrifcommonid
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
implicit none
real(kind=rsh),dimension(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( inout ) :: xeuv
real(kind=rsh),dimension(liminm2:limaxp2,ljminm2:ljmaxp2)                 :: xetemp
!
integer              :: nb
real(kind=rlg)       :: TimeIndex
integer :: i,j
integer,pointer :: Nbouc_Parent
External :: interpssh
!
!************************************************************
!     Executable statements
!************************************************************
!
!     Initialisation
!************************************************************
!
  if (.not.Agrif_Root()) then
!   Nbouc_Parent => Agrif_Parent(nbouc)
!   Nb = Agrif_Nbstepint()
!   TimeIndex = (Agrif_Rhot()*dta+(Nb+1)*dt)/(Agrif_Rhot()*(dta+dt))
   Agrif_SpecialValue=valmanq
   Call Agrif_Bc_Variable(xetemp,ssh_id,procname=interpssh)
   Agrif_SpecialValue=valmanq
   IF_MPI(limin==imin) &
   xeuv(imin+1,ljminp1:ljmax)=xetemp(imin+1,ljminp1:ljmax)
   IF_MPI(limax==imax) &
   xeuv(imax  ,ljminp1:ljmax)=xetemp(imax  ,ljminp1:ljmax)
   IF_MPI(ljmin==jmin) &
   xeuv(liminp1:limax,jmin+1)=xetemp(liminp1:limax,jmin+1)
   IF_MPI(ljmax==jmax) &
   xeuv(liminp1:limax,jmax  )=xetemp(liminp1:limax,jmax  )
  end if

#ifdef key_quefairela
! d apres config BRES mieux sans mettre ces echanges (ce que vous aviez fait)
! cval echanges ajoutes a la revision 1268 (extension spatiale de ssh)
! cval la routine Agrif_User.F90 a ete adaptee au meme moment
! cval sauf que Franck me dit qu il y avait peut-etre une tres bonne raison pour
! ne pas faire les echanges. On les fait pour les vitesses.
! cval Aie que faut-il faire ici ?
! cval il est vrai que sur la config BRES, le pas de temps augmentait moins vite
! cval apres cette "correction", donc code moins stable ? Revenir en arriere ?
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(ssh)
#endif

#endif
end subroutine agrif_obc_ssh

#if defined key_agrif
subroutine interpssh(tabres,i1,i2,j1,j2)
USE parameters
use comvars2d
use comvarp2d
use agrif_types
implicit none

integer i1,i2,j1,j2
real(kind=rsh) :: tabres(i1:i2,j1:j2)
integer :: i,j
Do j=j1,j2
  Do i=i1,i2
!fdumas 12/12/2007
     IF (h0(i,j)+ssh(i,j) < 2.0_rsh*h0fond) THEN
!   IF (h0(i,j)+ssh(i,j) < 2.0_rsh) THEN
      tabres(i,j) = Agrif_SpecialValue
    ELSE
      tabres(i,j) = ssh(i,j)
    ENDIF
  End Do
EndDo

return
end

subroutine interph0(tabres,i1,i2,j1,j2)
USE parameters
use comvars2d
use comvarp2d
use agrif_types
implicit none

integer i1,i2,j1,j2
real(kind=rsh) :: tabres(i1:i2,j1:j2)
integer :: i,j
Do j=j1,j2
  Do i=i1,i2
    IF (h0(i,j)==-valmanq) THEN
      tabres(i,j) = Agrif_SpecialValue
    ELSE
      tabres(i,j) = h0(i,j)
    ENDIF
  End Do
EndDo

return
end
#endif
