!********************************************************************
!
      subroutine agrif_obc_u(uv)
!
!********************************************************************
!
!     objet    :  conditions aux limites ouvertes en vitesse.
!                 On remplit le cadre de u.
!
!
!
!     appelant         : stepu, step2dxy
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          : 
!                      :
!     variables in     : jh, jb, ig, id, hx, valmanq, h0fond
!                      :
!     variables out    :
!                      :
!     variables inout  : u
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
USE parameters
use comvars2d
use comvarp2d
use agrifcommonid
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send

implicit none
!
integer :: i,j
real(kind=rsh) :: htxtemp, bid0
real(kind=rsh),dimension(liminm2:limaxp2,ljminm2:ljmaxp2) :: utemp
real(kind=rsh),dimension(liminm2:limaxp2,ljminm2:ljmaxp2) ::uv
external interpu_stepu
!
!************************************************************
!     Executable statements
!************************************************************
!
 utemp(:,:) = 0.
 agrif_SpecialValue=0.
 call Agrif_Bc_variable(utemp,u_id,procname=interpu_stepu)
   IF_MPI(limin==imin) &
   uv(imin,ljminm2:ljmaxp2)  =utemp(imin,ljminm2:ljmaxp2)
   IF_MPI(limax==imax) &
   uv(imax,ljminm2:ljmaxp2)  =utemp(imax,ljminm2:ljmaxp2)
   IF_MPI(ljmax==jmax) &
   uv(liminm2:limaxp2,jmax)  =utemp(liminm2:limaxp2,jmax)
   IF_MPI(ljmin==jmin) &
   uv(liminm2:limaxp2,jmin+1)=utemp(liminm2:limaxp2,jmin+1)
 agrif_SpecialValue=valmanq

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,uv(liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull call ex_j_rsh(-2,2,1,liminm1,limax,ljminm2,ljmaxp2,uv(liminm1:limax,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,uv(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uv)

 return
 
   ! western boundary
 do i=imin,imin
 do j=jmin,jmax
 	   htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
	   if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif

 uv(i,j)=utemp(i,j)/bid0

 enddo
 enddo
 
   ! eastern boundary
 do i=imax,imax
 do j=jmin,jmax
 	   htxtemp = hx(i,j)+ssh(i,j)
	   if(htxtemp <= 0.0) then
            bid0=1.E20
           else
            bid0=htxtemp+dhx(i,j)
           endif

 uv(i,j)=utemp(i,j)/bid0

 enddo
 enddo
 

   ! southern boundary
 do j=jmin,jmin+1
  do i=imin+1,imax-1
 	   htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
	   if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif
 
 uv(i,j)=utemp(i,j)/bid0

 enddo
 enddo
 
   ! northern boundary
  do j=jmax,jmax
  do i=imin+1,imax-1
 	   htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
	   if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif
 
 uv(i,j)=utemp(i,j)/bid0

 enddo
 enddo

!
#endif
end subroutine agrif_obc_u

#ifdef key_agrif
      subroutine interpu_stepu(tabres,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
implicit none

       integer i1,i2,j1,j2
       real tabres(i1:i2,j1:j2)
       integer i,j
       real htxtemp
       real bid0
       
       do j=j1,j2
       do i=i1,i2
!   htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
!   if(htxtemp <= 0.0) then
!           bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
!           if(bid0 <= 0.0) bid0=0.0
!           if(bid0 >  0.0) bid0=bid0+dhx(i,j)
!          else
!           bid0=htxtemp+dhx(i,j)
!          endif
!            tabres(i,j) = bid0*u(i,j)
             tabres(i,j) = u(i,j)
       enddo
       enddo
      return
      end subroutine interpu_stepu

      subroutine interphx(tabres,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
use agrif_util
implicit none

       integer i1,i2,j1,j2
       real tabres(i1:i2,j1:j2)
       integer i,j
       real htxtemp
       real bid0

       do j=j1,j2
       do i=i1,i2
       if(hx(i,j)== -valmanq-h0fond) then
             tabres(i,j) = Agrif_specialvalue
       else
             tabres(i,j) = hx(i,j)
       end if
       enddo
       enddo
      return
      end subroutine interphx
#endif
