!********************************************************************
!
      subroutine agrif_obc_uz(uvz)
!
!********************************************************************
!
!     objet    :  conditions aux limites ouvertes en vitesse.
!                 On remplit le cadre de u.
!
!
!
!     appelant         : dyn3dx, stepu
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          :
!                      :
!     variables in     : jh, jb, ig, id, valmanq, h0fond, hx
!                      :
!     variables out    :
!                      :
!     variables inout  : uz
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
use agrifcommonid
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send

implicit none
!
integer        :: i,j,k
real(kind=rsh) :: htxtemp, bid0
real(kind=rsh),dimension(kmax,liminm2:limaxp2,ljminm2:ljmaxp2) :: uztemp,uvz
external interpuz_stepu
!
!************************************************************
!     Executable statements
!************************************************************
!
   ! Initialization
 uztemp(:,:,:)=0.0
 agrif_SpecialValue=0.
 call Agrif_Bc_variable(uztemp,uz_id,procname=interpuz_stepu)
   IF_MPI(limin==imin) &
   uvz(:,imin,ljminm2:ljmaxp2)  =uztemp(:,imin,ljminm2:ljmaxp2)
   IF_MPI(limax==imax) &
   uvz(:,imax,ljminm2:ljmaxp2)  =uztemp(:,imax,ljminm2:ljmaxp2)
   IF_MPI(ljmax==jmax) &
   uvz(:,liminm2:limaxp2,jmax)  =uztemp(:,liminm2:limaxp2,jmax)
   IF_MPI(ljmin==jmin) &
   uvz(:,liminm2:limaxp2,jmin+1)=uztemp(:,liminm2:limaxp2,jmin+1)
 agrif_SpecialValue=valmanq

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uvz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull CALL ex_j_rsh(-2,2,kmax,liminm1,limax,ljminm2,ljmaxp2,uvz(:,liminm1:limax,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uvz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uvz)

 return

   ! western boundary 
 do i=imin,imin
 do j=jmin,jmax
     htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
	   if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=uztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 
   ! eastern boundary 
 do i=imax,imax
 do j=jmin,jmax
 	   htxtemp = hx(i,j)+ssh(i,j)
	   if(htxtemp <= 0.0) then
            bid0=1.E20
           else
            bid0=htxtemp+dhx(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=uztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 

   ! southern boundary 
 do j=jmin,jmin+1
  do i=imin+1,imax-1
 	   htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
	   if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=uztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 
   ! northern boundary 
  do j=jmax,jmax
  do i=imin+1,imax-1
 	   htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
	   if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=uztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 
!
#endif
end subroutine agrif_obc_uz

#ifdef key_agrif
      subroutine interpuz_stepu(tabres,k1,k2,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
implicit none

       integer i1,i2,j1,j2,k1,k2
       real tabres(k1:k2,i1:i2,j1:j2)
       integer i,j,k
       real htxtemp
       real bid0
       
       do j=j1,j2
       do i=i1,i2
!   htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
!   if(htxtemp <= 0.0) then
!           bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
!           if(bid0 <= 0.0) bid0=0.0
!           if(bid0 >  0.0) bid0=bid0+dhx(i,j)
!          else
!           bid0=htxtemp+dhx(i,j)
!          endif
!     
           do k=k1,k2	   
!            tabres(k,i,j) = bid0*uz(k,i,j)
             tabres(k,i,j) = uz(k,i,j)
           enddo
       enddo
       enddo
      return
      end 
#endif
