!********************************************************************
!
      subroutine agrif_obc_v(uv)
!
!********************************************************************
!
!     objet    :  conditions aux limites ouvertes en vitesse.
!                 On remplit le cadre de v.
!
!
!
!     appelant         : stepv, step2dyx
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          :
!                      :
!     variables in     : jh, jb, ig, id, hy, h0fond, valmanq
!                      :
!     variables out    :
!                      :
!     variables inout  : v
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
   USE parameters
   USE comvars2d
   USE comvarp2d
   USE agrifcommonid
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send

!
implicit none
!
integer :: i,j
real(kind=rsh), dimension(liminm2:limaxp2,ljminm2:ljmaxp2) :: vtemp
real(kind=rsh), dimension(liminm2:limaxp2,ljminm2:ljmaxp2) :: uv
real(kind=rsh) :: htytemp, bid0
external interpv_stepu
!
!************************************************************
!     Executable statements
!************************************************************
!
 vtemp = 0.
 agrif_SpecialValue=0.
 call Agrif_Bc_variable(vtemp,v_id,procname=interpv_stepu)
   IF_MPI(limin==imin) &
   uv(imin+1,ljminm2:ljmaxp2)=vtemp(imin+1,ljminm2:ljmaxp2)
   IF_MPI(limax==imax) &
   uv(imax  ,ljminm2:ljmaxp2)=vtemp(imax,ljminm2:ljmaxp2)
   IF_MPI(ljmin==jmin) &
   uv(liminm2:limaxp2,jmin)=vtemp(liminm2:limaxp2,jmin)
   IF_MPI(ljmax==jmax) &
   uv(liminm2:limaxp2,jmax)=vtemp(liminm2:limaxp2,jmax)
   agrif_SpecialValue=valmanq

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,uv(liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull call ex_j_rsh(-2,2,1,limin,limaxp1,ljminm2,ljmaxp2,uv(limin:limaxp1,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,uv(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uv)


 return
 
   ! western boundary
  do j=jmin,jmax-1
  do i=imin,imin+1
     htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
	   if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif

 uv(i,j)=vtemp(i,j)/bid0
 enddo
 enddo
 
   ! eastern boundary
  do j=jmin,jmax-1
  do i=imax,imax
	   htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
	   if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif

 uv(i,j)=vtemp(i,j)/bid0

 enddo
 enddo
 

   ! southern boundary
  do j=jmin,jmin
  do i=imin,imax
	   htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
	   if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif

 uv(i,j)=vtemp(i,j)/bid0
 enddo
 enddo
 
   ! northern boundary
  do j=jmax,jmax
  do i=imin,imax
	   htytemp = hy(i,j)+ssh(i,j)
	   if(htytemp <= 0.0) then
            bid0=1.E20
           else
            bid0=htytemp+dhy(i,j)
           endif

 uv(i,j)=vtemp(i,j)/bid0
 enddo
 enddo 
 
!
#endif
end subroutine agrif_obc_v

#ifdef key_agrif
      subroutine interpv_stepu(tabres,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
implicit none

       integer i1,i2,j1,j2
       real tabres(i1:i2,j1:j2)
       integer i,j
       real htytemp
       real bid0
       
       do j=j1,j2
       do i=i1,i2
!   htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
!   if(htytemp <= 0.0) then
!           bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
!           if(bid0 <= 0.0) bid0=0.0
!           if(bid0 >  0.0) bid0=bid0+dhy(i,j)
!          else
!           bid0=htytemp+dhy(i,j)
!          endif      	   
!            tabres(i,j) = bid0*v(i,j)
             tabres(i,j) = v(i,j)
       enddo
       enddo
      return
      end subroutine interpv_stepu

      subroutine interphy(tabres,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
use agrif_util
implicit none

       integer i1,i2,j1,j2
       real tabres(i1:i2,j1:j2)
       integer i,j
       real htytemp
       real bid0
       
       do j=j1,j2
       do i=i1,i2
        if(hy(i,j)==-valmanq-h0fond) then
             tabres(i,j) = Agrif_SpecialValue
        else 
             tabres(i,j) = hy(i,j)
        endif 
       enddo
       enddo
      return
      end subroutine interphy 
#endif
