!********************************************************************
!
      subroutine agrif_obc_vz(uvz)
!
!********************************************************************
!
!     objet    :  conditions aux limites ouvertes en vitesse.
!                 On remplit le cadre de v.
!
!
!
!     appelant         : main, dyn3dy
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          : 
!                      :
!     variables in     : jh, jb, ig, id
!                      :
!     variables out    : u
!                      :
!     variables inout  : v, uvz
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
use agrifcommonid
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send
implicit none
!
integer :: i,j,k
real(kind=rsh)  ::  htytemp, bid0
real(kind=rsh),dimension(kmax,liminm2:limaxp2,ljminm2:ljmaxp2) :: vztemp,uvz
external interpvz_stepu
!
!************************************************************
!     Executable statements
!************************************************************
!
 vztemp(:,:,:)=0.0
 agrif_SpecialValue=0.
 call Agrif_Bc_variable(vztemp,vz_id,procname=interpvz_stepu)
   IF_MPI(limin==imin) &
   uvz(:,imin+1,ljminm2:ljmaxp2)=vztemp(:,imin+1,ljminm2:ljmaxp2)
   IF_MPI(limax==imax) &
   uvz(:,imax  ,ljminm2:ljmaxp2)=vztemp(:,imax,ljminm2:ljmaxp2)
   IF_MPI(ljmin==jmin) &
   uvz(:,liminm2:limaxp2,jmin)=vztemp(:,liminm2:limaxp2,jmin)
   IF_MPI(ljmax==jmax) &
   uvz(:,liminm2:limaxp2,jmax)=vztemp(:,liminm2:limaxp2,jmax)
 agrif_SpecialValue=valmanq

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uvz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull CALL ex_j_rsh(-2,2,kmax,limin,limaxp1,ljminm2,ljmaxp2,uvz(:,limin:limaxp1,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uvz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uvz)

 return

   ! western boundary
 do j=jmin,jmax-1
  do i=imin,imin+1
     htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
	   if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=vztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 
   ! eastern boundary
  do j=jmin,jmax-1
  do i=imax,imax
	   htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
	   if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=vztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 

   ! southern boundary
  do j=jmin,jmin
  do i=imin,imax
	   htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
	   if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0=1.E20
            if(bid0 >  0.0) bid0=bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=vztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 
   ! northern boundary
  do j=jmax,jmax
  do i=imin,imax
	   htytemp = hy(i,j)+ssh(i,j)
	   if(htytemp <= 0.0) then
            bid0=1.E20
           else
            bid0=htytemp+dhy(i,j)
           endif
 do k=1,kmax
 uvz(k,i,j)=vztemp(k,i,j)/bid0
 enddo
 enddo
 enddo
 

!
!************************************************************
!     Executable statements
!************************************************************
!
#endif
end subroutine agrif_obc_vz

#ifdef key_agrif
      subroutine interpvz_stepu(tabres,k1,k2,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
implicit none

       integer i1,i2,j1,j2,k1,k2
       real tabres(k1:k2,i1:i2,j1:j2)
       integer i,j,k
       real htytemp
       real bid0
       
       do j=j1,j2
       do i=i1,i2
!   htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
!   if(htytemp <= 0.0) then
!           bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
!           if(bid0 <= 0.0) bid0=0.0
!           if(bid0 >  0.0) bid0=bid0+dhy(i,j)
!          else
!           bid0=htytemp+dhy(i,j)
!          endif
      
           do k=k1,k2	   
!            tabres(k,i,j) = bid0*vz(k,i,j)
             tabres(k,i,j) = vz(k,i,j)
           enddo
       enddo
       enddo
      return
      end 
#endif
