#if defined key_agrif
!*************************************************
!
       subroutine update_agrif
!
!*************************************************
!
!     objet    : actualisation des valeurs des variables de la grille
!                parent par les valeur des variables sur la grille
!                fine sur tout ou partie de la zone de recouvrement.
!
!
!     appelant         : stepu (contexte agrif, voir stepuagrif.h)
!                      :
!     appeles          : Agrif_Root, Agrif_NbStepint
!     appeles          : Agrif_Rhox, Agrif_Rhoy, Agrif_ixl, Agrif_iyl
!     appeles          : COUNT, SUM
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          : key_agrif
!                      :
!     variables in     : sal, temp, ssh, uz, u
!                      :
!     variables out    : valmanq
!                      :
!     variables inout  :
!                      :
!     dernieres modifs : 19/09/02
!                      :
!***********************************************************
!

USE parameters
use comvars3d
use comvars2d
use comvarp2d
use comvarp3d
use comvarp3d
use agrifcommonid
#ifdef key_substance
use comsubstance
#endif
implicit none
!
	real(kind=rsh),dimension(kmax,liminm1:limaxp2,ljminm1:ljmaxp2) :: tabtemp
	real(kind=rsh),    dimension(liminm1:limaxp2,ljminm1:ljmaxp2)  :: tabtemp2d
        external :: updatetemp, updatesal, updateuz,updatevz,updatessh
#ifdef key_substance
	real(kind=rsh),dimension(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2) :: tabtemp4d
        real(kind=rsh),dimension(:,:,:,:),pointer  :: cv_wat_parent
        external updatecv_wat
#endif
!
!************************************************************
!     Executable statements
!************************************************************
!
      if (Agrif_Root()) return
      if (Agrif_NbStepint().NE.(Agrif_Rhot()-1)) return
!
      Agrif_UseSpecialValueInUpdate = .TRUE.
      Call Agrif_Update_Variable(tabtemp,sal_id ,procname = updatesal)        
      Call Agrif_Update_Variable(tabtemp,temp_id,procname = updatetemp)  
! pas terrible      Call Agrif_Update_Variable(tabtemp,ssh_id ,procname = updatessh)
! pas terrible      Call Agrif_Update_Variable(tabtemp,uz_id  ,procname = updateuz)        
! pas terrible      Call Agrif_Update_Variable(tabtemp,vz_id  ,procname = updatevz)        
! garder syntaxe pour update localise (la frontiere seule est updatee dans la grille mere)
!     Call Agrif_Update_Variable(tabtemp,sal_id ,locupdate=(/1,0/),procname = updatesal)        
!     Call Agrif_Update_Variable(tabtemp,temp_id,locupdate=(/1,0/),procname = updatetemp)  
!     Call Agrif_Update_Variable(tabtemp,ssh_id ,locupdate=(/1,0/),procname = updatessh)
!     Call Agrif_Update_Variable(tabtemp,uz_id  ,locupdate1=(/0,-1/),locupdate2=(/1,0/) ,procname = updateuz)        
!     Call Agrif_Update_Variable(tabtemp,vz_id  ,locupdate1=(/1,0/) ,locupdate2=(/0,-1/),procname = updatevz)        
  
#ifdef key_substance      
      cv_wat_parent => Agrif_Parent(cv_wat)
      Call Agrif_Update_Variable(tabtemp4d,cv_wat_id,procname = updatecv_wat)    
#endif    
      Agrif_UseSpecialValueInUpdate = .FALSE.
!
       end subroutine update_agrif

      Subroutine Updatesal(tabres,k1,k2,i1,i2,j1,j2,before)
USE parameters
use comvars3d
use comvars2d
use comvarp2d
use comvarp3d
      implicit none
      integer i1,i2,j1,j2,k1,k2
      real tabres(k1:k2,i1:i2,j1:j2)
      logical before
      
      integer i,j,k
      
      IF (before) THEN

          DO j=j1,j2
           DO i=i1,i2
           DO k=k1,k2
             tabres(k,i,j) = sal(k,i,j)
          ENDDO
          ENDDO
          ENDDO
       ELSE
        DO j=j1,j2
           DO i=i1,i2
             DO k=k1,k2
               IF ((sal(k,i,j).NE.valmanq).AND.(tabres(k,i,j).NE.valmanq)) THEN
                 sal(k,i,j) = tabres(k,i,j)
             END IF
            ENDDO
         ENDDO
        ENDDO        
       ENDIF
      
      return
      end
      
      Subroutine Updatetemp(tabres,k1,k2,i1,i2,j1,j2,before)
USE parameters
use comvars3d
use comvars2d
use comvarp2d
use comvarp3d
      implicit none
      integer i1,i2,j1,j2,k1,k2
      real tabres(k1:k2,i1:i2,j1:j2)
      logical before
      
      integer i,j,k
      
      IF (before) THEN

           DO j=j1,j2
           DO i=i1,i2
           DO k=k1,k2
             tabres(k,i,j) = temp(k,i,j)
          ENDDO
          ENDDO
          ENDDO
       ELSE
           DO j=j1,j2
           DO i=i1,i2
             DO k=k1,k2
               IF ((temp(k,i,j).NE.valmanq).AND.(tabres(k,i,j).NE.valmanq)) THEN
               temp(k,i,j) = tabres(k,i,j)
              ENDIF
            ENDDO
         ENDDO
        ENDDO        
       ENDIF
      
      return
      end  

#ifdef key_substance      
      Subroutine Updatecv_wat(tabres,l1,l2,k1,k2,i1,i2,j1,j2,before)
USE parameters
use comvars3d
use comvars2d
use comvarp2d
use comvarp3d
use comsubstance
    
      implicit none
      integer i1,i2,j1,j2,k1,k2,l1,l2
      real tabres(l1:l2,k1:k2,i1:i2,j1:j2)
      logical before
      
      integer i,j,k,l
      
      IF (before) THEN

           DO j=j1,j2
           DO i=i1,i2
           DO k=k1,k2
           DO l=l1,l2   
             tabres(l,k,i,j) = cv_wat(l,k,i,j)
          ENDDO
          ENDDO
          ENDDO
          ENDDO
       ELSE
          DO j=j1,j2
           DO i=i1,i2
             DO k=k1,k2
             DO l=l1,l2
             IF ( cv_wat(l,k,i,j)/=valmanq .AND. tabres(l,k,i,j)/=valmanq ) THEN
             cv_wat(l,k,i,j) = tabres(l,k,i,j)
            ENDIF
            ENDDO
            ENDDO
         ENDDO
        ENDDO        
       ENDIF
      
      return
      end 
#endif  

      Subroutine Updatessh(tabres,i1,i2,j1,j2,before)
USE parameters
use comvars3d
use comvars2d
use comvarp2d
use comvarp3d
      implicit none
      integer i1,i2,j1,j2
      real tabres(i1:i2,j1:j2)
      logical before

      integer i,j

      IF (before) THEN
        DO j=j1,j2
         DO i=i1,i2
           tabres(i,j) = ssh(i,j)
         ENDDO
        ENDDO
       ELSE
        DO j=j1,j2
         DO i=i1,i2
          IF (ssh(i,j)/=valmanq .AND. tabres(i,j)/=valmanq) THEN
             ssh(i,j) = tabres(i,j)
          ENDIF
         ENDDO
        ENDDO
       ENDIF

      return
      end
 
      
      Subroutine Updateuz(tabres,k1,k2,i1,i2,j1,j2,before)
USE parameters
use comvars3d
use comvars2d
use comvarp2d
use comvarp3d
      implicit none
      integer i1,i2,j1,j2,k1,k2
      real tabres(k1:k2,i1:i2,j1:j2)
      logical before
      
      integer i,j,k
      real umoy
      real htxtemp
      real bid0
      
      IF (before) THEN

   DO j=j1,j2
           DO i=i1,i2
   
          htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
          if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0=0.0
            if(bid0 >  0.0) bid0=bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif
      
           DO k=k1,k2
             tabres(k,i,j) = bid0*dsigu(k)*uz(k,i,j)
          ENDDO
          ENDDO
          ENDDO
       ELSE
          DO j=j1,j2
           DO i=i1,i2
           htxtemp = hx(i,j)+0.5*(ssh(i,j)+ssh(i+1,j))
           if(htxtemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i+1,j))+hx(i,j)
            if(bid0 <= 0.0) bid0 = 1.E20
            if(bid0 >  0.0) bid0 = bid0+dhx(i,j)
           else
            bid0=htxtemp+dhx(i,j)
           endif
           if (bid0 >= hm) then
             umoy = 0.
             DO k=k1,k2
             umoy = umoy + tabres(k,i,j)
             uz(k,i,j) = tabres(k,i,j)/(dsigu(k)*bid0)
            ENDDO
             u(i,j) = umoy/bid0
           endif
         ENDDO
        ENDDO        
       ENDIF
      
      return
      end        

      Subroutine Updatevz(tabres,k1,k2,i1,i2,j1,j2,before)
USE parameters
use comvars3d
use comvars2d
use comvarp2d
use comvarp3d
      implicit none
      integer i1,i2,j1,j2,k1,k2
      real tabres(k1:k2,i1:i2,j1:j2)
      logical before

      integer i,j,k
      real vmoy
      real htytemp
      real bid0

      IF (before) THEN

           DO j=j1,j2
           DO i=i1,i2
           htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
           if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0=0.0
            if(bid0 >  0.0) bid0=bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif

           DO k=k1,k2
             tabres(k,i,j) = bid0*dsigu(k)*vz(k,i,j)
          ENDDO
          ENDDO
          ENDDO
       ELSE
           DO j=j1,j2
           DO i=i1,i2
           htytemp = hy(i,j)+0.5*(ssh(i,j)+ssh(i,j+1))
           if(htytemp <= 0.0) then
            bid0 = max(ssh(i,j),ssh(i,j+1))+hy(i,j)
            if(bid0 <= 0.0) bid0 = 1.E20
            if(bid0 >  0.0) bid0 = bid0+dhy(i,j)
           else
            bid0=htytemp+dhy(i,j)
           endif
           if (bid0 >= hm) then
             vmoy = 0.
             DO k=k1,k2
             vmoy = vmoy + tabres(k,i,j)
             vz(k,i,j) = tabres(k,i,j)/(dsigu(k)*bid0)
            ENDDO
             v(i,j) = vmoy/bid0
             endif

         ENDDO
        ENDDO
       ENDIF

      return
      end


#else
        subroutine update_agrif_empty
        return
        end
#endif
