!**************** including dyn3dx_hdiff  *****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_hdiff.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to horizontal viscosity
   !&E              terms for the zonal momentum equation
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit part
   !&E                  (d (nu duz/dx)/dx+d (nu duz/dy)/dy)
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : yz_exp,mm,ms,dx,dx_u,dy_f,uz,vish_xe,vish_phi
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  1990-10  (F. Dumas ) new formulation
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E
   !&E---------------------------------------------------------------------

!     included from    : dyn3dx_tls_coef
!                      :
!     variables in     : uz
!     variables in     : vish_phi, vish_xe(comvers2d)
!     variables in     : dx,dx_u,dy_u,dy_f (comvers2d)
!     variables in     : cne(parameters)
!     variables in     : ms, mn (comcolig2d)
!     variables inout  : yz_exp(comcolig3d)
!                      :

     yz_exp(:,i,j)=yz_exp(:,i,j)+ &

!-----------------------------------
!   d nu(du/dx) /dx
!-----------------------------------

         (vish_xe(:,i+1,j)*(uz(:,i+1,j)-uz(:,i  ,j))/dx(i+1,j)    &
         -vish_xe(:,i  ,j)*(uz(:,i  ,j)-uz(:,i-1,j))/dx(i  ,j) )  &
                 /dx_u(i,j) &
!
!-----------------------------------
!   d nu(du/dy) /dy
!-----------------------------------
!
        +(vish_phi(:,i,j  )*mn(i,j)*(uz(:,i,j+1)-uz(:,i,j  ))/dy_f(i,j  ) &
         -vish_phi(:,i,j-1)*ms(i,j)*(uz(:,i,j  )-uz(:,i,j-1))/dy_f(i,j-1) )/dy_u(i,j)
!
!**************** end including dyn3dx_hdiff  *****************************
