!*********    including dyn3dx_ipg_pint.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_ipg_dj.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the density jacobian
   !&E                  (for sigma generalized framework)
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz2,dx_u,hc_sig,Csu_sig
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Marsaleix et al. (2009), low order pressure gradient
   !&E                schemes in sigma coordinate models : the seamount test revisited
   !&E
   !&E ** History :
   !&E       !  2009-04  (F. Dumas, V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------

     dxb=dx_u(i,j)

     htxee=h0(i+1,j)+xec(i+1,j)
     htxeo=h0(i,j)+xec(i,j)
! S.Petton 09/2013 : Utilisation d un profil moyen
   ! surface - estimate of the second pressure gradient term : boz2*dz/dx

     k = kmax

     p_prim_interface_i  (k)=0.0_rlg
     p_prim_interface_ip1(k)=0.0_rlg

#if defined key_siggen || defined key_gencoord
     hz_i   = f_hzu(h0(i,j),xec(i  ,j),k,i  ,j)
     hz_ip1 = f_hzu(h0(i+1,j),xec(i+1,j),k,i+1,j)
#else
     hz_i   = htxeo
     hz_ip1 = htxee
#endif
     zk_i   = f_zu(h0(i  ,j),xec(i  ,j),k,i  ,j)
     zk_ip1 = f_zu(h0(i+1,j),xec(i+1,j),k,i+1,j)
     ! g.r.dz/dx
! S.Petton 09/2013 : Utilisation d un profil moyen
     sec_gp(k)=(boz2(k,i+1,j)*hz_ip1+boz2(k,i,j)*hz_i)/(hz_ip1+hz_i)*(zk_ip1-zk_i)/dxb


   ! estimate of the second pressure gradient term : boz2*dz/dx (k level)
   ! estimate of the  first pressure gradient term : d/dx integral(boz2) (k+1 level)

     DO k=kmax-1,1,-1
       hz_i_kp1 = hz_i
       hz_ip1_kp1 = hz_ip1
#if defined key_siggen || defined key_gencoord
       hz_i   = f_hzu(h0(i,j),xec(i  ,j),k,i  ,j)
       hz_ip1 = f_hzu(h0(i+1,j),xec(i+1,j),k,i+1,j)
#else
       hz_i   = htxeo
       hz_ip1 = htxee
#endif
       zk_i   = f_zu(h0(i  ,j),xec(i  ,j),k,i  ,j)
       zk_ip1 = f_zu(h0(i+1,j),xec(i+1,j),k,i+1,j)

       ! horizontal gradient of pressure (sigma = cst)
! S.Petton 09/2013 : Utilisation d un profil moyen
       p_prim_interface_i(k)   = p_prim_interface_i(k+1)   + boz2(k+1,i  ,j)*hz_i_kp1  *dsigu(k+1)
       p_prim_interface_ip1(k) = p_prim_interface_ip1(k+1) + boz2(k+1,i+1,j)*hz_ip1_kp1*dsigu(k+1)
       p_primu_i(k+1)  =0.5_rlg*(p_prim_interface_i(k)  +p_prim_interface_i(k+1)  )
       p_primu_ip1(k+1)=0.5_rlg*(p_prim_interface_ip1(k)+p_prim_interface_ip1(k+1))

!cval 0.5_rlg*(p_prim_interface_jp1(k)+p_prim_interface_jp1(k+1)) alors que u n est pas
!cval necessairement au milieu de la couche ?

       ! g.r.dz/dx
! S.Petton 09/2013 : Utilisation d un profil moyen
       sec_gp(k)=(boz2(k,i+1,j)*hz_ip1+boz2(k,i,j)*hz_i)/(hz_ip1+hz_i)*(zk_ip1-zk_i)/dxb

     END DO
! S.Petton 09/2013 : Utilisation d un profil moyen
   ! bottom estimate of the first pressure gradient term : d/dx integral(boz2) (k=1 level)

     k = 0
     hz_i_kp1 = hz_i
     hz_ip1_kp1 = hz_ip1
     p_prim_interface_i(k)   = p_prim_interface_i(k+1)   + boz2(k+1,i  ,j)*hz_i_kp1  *dsigu(k+1)
     p_prim_interface_ip1(k) = p_prim_interface_ip1(k+1) + boz2(k+1,i+1,j)*hz_ip1_kp1*dsigu(k+1)
     p_primu_i(k+1)  =0.5_rlg*(p_prim_interface_i(k)  +p_prim_interface_i(k+1)  )
     p_primu_ip1(k+1)=0.5_rlg*(p_prim_interface_ip1(k)+p_prim_interface_ip1(k+1))


    do k=1,kmax
     yz_exp(k,i,j) = yz_exp(k,i,j)+(p_primu_ip1(k)-p_primu_i(k))/dxb+sec_gp(k)
    end do

!********* end  including dyn3dx_ipg_pint.h      ****************************
