!*********    including dyn3dx_ipg_pj_sig.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_ipg_pj_sig.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the pressure jacobian
   !&E                  (for sigma generalized framework)
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz,dx_u,hc_sig,csu_sig
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-04  (F. Dumas) generalized sigma
   !&E
   !&E---------------------------------------------------------------------


    dxb=dx_u(i,j)
    dhxx=(h0(i+1,j)-h0(i,j))/dxb                ! d h0 / dx
    dhcx=(hc_sig(i+1,j)-hc_sig(i,j))/dxb        ! d hc / dx
    dxex=(xec(i+1,j)-xec(i,j))/dxb              ! d xe / dx

    k=kmax

   ! pri = d int_z (b dz) / dx  (at u level)
    bhze=f_hzw(h0(i+1,j),xec(i+1,j),k,i+1,j)*boz(k,i+1,j)
    bhzo=f_hzw(h0(i  ,j),xec(i  ,j),k,i  ,j)*boz(k,i  ,j)
    bms=(bhze-bhzo)/dxb
    pri=bms*(0.0_rlg-sig(k))

   ! add b * dz/dx
!hccst    tbc=pri+(boz(k,i,j)+boz(k,i+1,j))*0.5_rlg*((sig(k)+1.0_rlg)*dxex+dhxx*csu_sig(k))
    tbc=pri+(boz(k,i,j)+boz(k,i+1,j))*0.5_rlg*((sig(k)+1.0_rlg)*dxex+sig(k)*dhcx+(dhxx-dhcx)*csu_sig(k))
    yz_exp(k,i,j) = yz_exp(k,i,j)+tbc

    DO k=kmax-1,1,-1

   ! d int_z (b dz) / dx (at u level)
      bhze=(f_hzw(h0(i+1,j),xec(i+1,j),k,i+1,j))*0.5_rlg*(boz(k,i+1,j)+boz(k+1,i+1,j))
      bhzo=(f_hzw(h0(i  ,j),xec(i  ,j),k,i  ,j))*0.5_rlg*(boz(k,i  ,j)+boz(k+1,i  ,j))
      bm=(bhze-bhzo)/dxb
      pri=pri+bm*dsigw(k)

   ! add b * dz/dx
!hccst     tbc=pri+(boz(k,i,j)+boz(k,i+1,j))*0.5_rlg*((sig(k)+1.0_rlg)*dxex+dhxx*csu_sig(k))
      tbc=pri+(boz(k,i,j)+boz(k,i+1,j))*0.5_rlg*((sig(k)+1.0_rlg)*dxex+sig(k)*dhcx+(dhxx-dhcx)*csu_sig(k))
      yz_exp(k,i,j) = yz_exp(k,i,j)+tbc
      bms=bm

    ENDDO

!********* end  including dyn3dx_ipg_pj_sig.h      ****************************
