   !&E   ***************   include dyn3dx_mtm   **********************************
   !&E
   !&E ** Purpose : remplissage des termes du systeme lineaire
   !&E              provenant de la discretisation de l equation de conservation
   !&E              de la quantite de mouvement du systeme d equations
   !&E              relatif au mode barotrope du modele 3D dans la direction Est-Ouest
   !&E              sur l ensemble de la grille.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : dyn3dx_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History : 2006-05
   !&E              2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2010-07  (L. Debreu, F. Dumas) rewritting of ADI
   !&E       !  2011-03  (F. Dumas) bottom stress parametrisation + remove case 2 and 3 for tidal flat processing
   !&E       !  2011-10  (L. Pineau, F. Dumas, V. garnier) correction bottom stress parametrisation
   !&E       !  2011-12  (S. Petton, F. Dumas) Implicitation of the bottom stress (consistence between 2D en 3D)
   !&E                                         removing of min,max
   !&E
   !&E
   !&E  RQ V Garnier vis2d est homogene spatialement donc OK
   !&E     neanmoins, il faudrait plutot utiliser vish_xe(1,:,:) et vis
   !&E     car ainsi le coef pourrait varier spatialement
   !&E     Aujourd hui, faux si cela varie spatialement, il faut definir
   !&E     coef de viscosite en u et v
   !&E
   !&E-------------------------------------------------------------------------

     gdt = grav*dt

   ! cas d un point u sans eau de part ni d autre
   ! drying u-point : no water on the right and left

     IF (igdu(i,j) == 4) THEN
       av(i,j)=0.0_rlg
       bv(i,j)=1.0_rlg
       cv(i,j)=0.0_rlg
       y2dv(i,j)=0.0_rlg


   ! cas d un point u avec de l eau de part et d autre
   ! u-point surrounded by water

     ELSE IF (igdu(i,j) == 1) THEN

       ! sea surface pressure gradient : implicit part
       av(i,j)=-alphan*0.5_rlg*gdt/dx_u(i,j)                ! i-1
       cv(i,j)= alphan*0.5_rlg*gdt/dx_u(i,j)                ! i

      ! time evolution (u at n+1/2)
#ifdef key_dyn_adv_linear

       ! time evolution (u at n+1/2)
       bv(i,j)=1.0_rlg

#else

       ! bottom stress (implicit part)
       v_utmp=0.25_rsh*(v(i,j)+v(i+1,j)+v(i,j-1)+v(i+1,j-1))
       vit=SQRT(u(i,j)*u(i,j)+v_utmp*v_utmp)
       ft=0.5_rsh*dt*vit/htx(i,j)*(0.4_rsh/LOG(htx(i,j)/(exp1*z0b(i,j))+1.0_rsh))**2
#ifdef key_no_bottom_stress
       ft=0.0_rsh
#elif key_linear_bottom_stress
       !ft=frot_lin/htx(i,j)
       ft=halfdt*frot_lin/MAX(htx(i,j),hminfrot)
#endif

       bv(i,j)=1.0_rlg/(                                &
               ! time evolution (u at n+1/2)
               1.0_rlg                                  &
               ! bottom stress : implicit part
               +ft*cfroi)

#endif

       ! rhs : explicit terms
       y2dv(i,j)=                                       &
                 ! time evolution (u at n-1/2)
                 u(i,j)                                 &
                 ! other explicit terms integrated from the bottom to the sea surface
                 +halfdt*intz_exp(i,j)                  &
                 ! explicit part of the sea surface pressure gradient
                 -0.5_rlg*gdt/dx_u(i,j)*alphap*(xec(i+1,j)-xec(i,j))


     ELSE IF (igdu(i,j) == 5) THEN
   ! low depth

       v_utmp=0.25_rsh*(v(i,j)+v(i+1,j)+v(i,j-1)+v(i+1,j-1))

       ! sea surface pressure gradient : implicit part
       av(i,j)=-alphan*0.5_rlg*gdt/dx_u(i,j)                ! i-1
       cv(i,j)= alphan*0.5_rlg*gdt/dx_u(i,j)                ! i

#ifdef key_dyn_adv_linear

       ! time evolution (u at n+1/2)
       bv(i,j)=1.0_rlg

#else

       ! bottom stress (implicit part)
       vit=SQRT(u(i,j)*u(i,j)+v_utmp*v_utmp)
!       factft=1.0_rsh+vit-botstressmax+ABS(vit-botstressmax)
!       ft=factft*0.5_rsh*dt*vit/htx(i,j)*MIN((0.4_rsh/LOG(MAX(MIN(htx(i,j),hmaxfrot),hminfrot)/(exp1*z0b(i,j))))**2,0.1_rsh)
       ft=0.5_rsh*dt*vit/htx(i,j)*(0.4_rsh/LOG(htx(i,j)/(exp1*z0b(i,j))+1.0_rsh))**2
#ifdef key_no_bottom_stress
       ft=0.0_rsh
#elif key_linear_bottom_stress
       !ft=frot_lin/htx(i,j)
       ft=halfdt*frot_lin/MAX(htx(i,j),hminfrot)
#endif

       bv(i,j)=1.0_rlg/(                                &
               ! time evolution (u at n+1/2)
               1.0_rlg                                  &
               ! bottom stress : implicit part
               +ft*cfroi)

#endif

       ! rhs : explicit terms
       y2dv(i,j)=                                       &
                 ! time evolution (u at n-1/2)
                 u(i,j)                                 &
                 ! bottom stress (u at n) explicit part
                 -ft*cfroe*u(i,j)                       &
                 ! coriolis
                 +fu(i,j)*v_utmp*halfdt                 &
                 ! explicit part of the sea surface pressure gradient
                 -0.5_rlg*gdt/dx_u(i,j)*alphap*(xec(i+1,j)-xec(i,j))      &
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
                 ! NL advection : u*v*tan(phi)/R (spherical coordinates)
                 +halfdt*u(i,j)*v_utmp*tfiusr(i,j)      &
                 ! non-linear (udu/dx) : explicit part
                 -halfdt*u(i,j)/(dx(i,j)+dx(i+1,j))*(u(i+1,j)-u(i-1,j))   &
#endif
#endif
                 ! viscosity (along x)
                 +halfdt*( (vis2d(i+1,j)*(u(i+1,j)-u(i  ,j)))/dx(i+1,j)   &
                          -(vis2d(i  ,j)*(u(i  ,j)-u(i-1,j)))/dx(i  ,j) )/dx_u(i,j)

       ! viscosity and non-linear term
       IF ((hty(i,j-1)+hty(i+1,j-1)) >= 0.0_rsh) THEN
         dusdy=(u(i,j  )-u(i,j-1))/dy_f(i,j-1)
         y2dv(i,j)=y2dv(i,j)                            &
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
                   ! non-linear (vdu/dy) : explicit part
                   -0.25_rlg*dt*v_utmp*dusdy             &
#endif
#endif
                   ! viscosity (along y) : explicit
                   -halfdt*vis2d(i,j-1)*dusdy/dy_u(i,j)
       END IF
       IF ((hty(i,j  )+hty(i+1,j  )) >= 0.0_rsh) THEN
         dusdy=(u(i,j+1)-u(i,j  ))/dy_f(i,j)
         y2dv(i,j)=y2dv(i,j)                            &
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
                   ! non-linear (vdu/dy) : explicit part
                   -0.25_rlg*dt*v_utmp*dusdy             &
#endif
#endif
                   ! viscosity (along y) : explicit
                   +halfdt*vis2d(i,j  )*dusdy/dy_u(i,j)
       END IF

       ! wind stress and atmospherical pressure gradient
       y2dv(i,j)=y2dv(i,j)                              &

                 ! wind stress in x-direction : explicit
                 +halfdt*tosx(i,j)/MAX(htx(i,j),hminfrot)/rhoref &

                 ! atmospherical pressure gradient : explicit
                 -halfdt*(pa(i+1,j)-pa(i,j))/(dx_u(i,j)*rhoref)

       ! generator potential : explicit
       IF (l_genpot) &
         y2dv(i,j)=y2dv(i,j)+halfdt/dx_u(i,j)*(potgene(i+1,j)-potgene(i,j))

     END IF 

   !&E   ***************   end include dyn3dx_mtm   ******************************
