!*******************    including dyn3dy_adv_centered.h      *******************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_adv_centered.h  ***
   !&E
   !&E ** Purpose : estimate the advection terms of the meridional momentum
   !&E              equation
   !&E
   !&E ** Description : explicit part
   !&E                  centered scheme
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz_v,vz,wz,mo,me,dy_f,dy,tfiusr
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E
   !&E---------------------------------------------------------------------


    k=1

    wdvi=0.5_rsh*(wz(k,i,j)+wz(k,i,j+1))*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)

   !   v * dv/dy

    yz_exp(k,i,j)=yz_exp(k,i,j)-(vz(k,i,j)*(vz(k,i,j+1)-vz(k,i,j-1))/(dy(i,j)+dy(i,j+1)) &

   !   w * dv/dz

                  +wdvi) &

   !   u * dv/dx

                  -((vz(k,i  ,j)-vz(k,i-1,j))*uz_v(k)*mo(i,j)/dx_f(i-1,j)       &
                   +(vz(k,i+1,j)-vz(k,i  ,j))*uz_v(k)*me(i,j)/dx_f(i  ,j) )*0.5_rsh &

   !   u**2 * tg(phi) / R

                  -uz_v(k)*uz_v(k)*tfivsr(i,j)

    DO k=2,kmax-1

      wdvs=0.5_rsh*(wz(k,i,j)+wz(k,i,j+1))*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)

   !   v * dv/dy

      yz_exp(k,i,j)=yz_exp(k,i,j)-(vz(k,i,j)*(vz(k,i,j+1)-vz(k,i,j-1))/(dy(i,j)+dy(i,j+1)) &

   !   w * dv/dz

                  +(wdvi*dsigw(k)+wdvs*dsigw(k-1))/(dsigw(k)+dsigw(k-1))) &

   !   u * dv/dx

                  -((vz(k,i  ,j)-vz(k,i-1,j))*uz_v(k)*mo(i,j)/dx_f(i-1,j)           &
                   +(vz(k,i+1,j)-vz(k,i  ,j))*uz_v(k)*me(i,j)/dx_f(i  ,j) )*0.5_rsh &

   !   u**2 * tg(phi) / R

                   -uz_v(k)*uz_v(k)*tfivsr(i,j)

       wdvi=wdvs

     END DO

     k = kmax

   !   v * dv/dy

     yz_exp(k,i,j)=yz_exp(k,i,j)-(vz(k,i,j)*(vz(k,i,j+1)-vz(k,i,j-1))/(dy(i,j)+dy(i,j+1)) &

   !   w * dv/dz

                   +wdvi*dsigw(k)/(dsigw(k)+dsigw(k-1))) &

   !   u * dv/dx

                  -((vz(k,i  ,j)-vz(k,i-1,j))*uz_v(k)*mo(i,j)/dx_f(i-1,j)           &
                   +(vz(k,i+1,j)-vz(k,i  ,j))*uz_v(k)*me(i,j)/dx_f(i  ,j) )*0.5_rsh &

   !   u**2 * tg(phi) / R

                  -uz_v(k)*uz_v(k)*tfivsr(i,j)

!********* end including dyn3dy_adv_centered.h      ******************************
