!*************include  dyn3dy_botstress   **********************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_botstress.h  ***
   !&E
   !&E ** Purpose : estimate the drag coefficient and the trend term
   !&E              relative to the bottom stress
   !&E              for the meridional momentum forcing (2D and 3D equation)
   !&E
   !&E ** Description : explicit
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz_v,vz,hzuty,hty,z0b,cdvit
   !&E
   !&E ** Modified variables : yz_exp,intz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original cod
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-07  (F. Dumas) Generalized sigmas
   !&E       !  2011-12  (S. Petton, F. Dumas) Implicitation of the bottom stress (consistence between 2D en 3D)
   !&E                                         removing of min,max
   !&E
   !&E---------------------------------------------------------------------

#ifdef key_linear_bottom_stress
    cdvit(i,j)=frot_lin
#else
    vitf=SQRT(uz_v(1)*uz_v(1)+vz(1,i,j)*vz(1,i,j))
#if defined key_siggen || defined key_gencoord
    cdf=(0.4_rsh/LOG(0.5_rsh*dsigu(1)*hzuty(1,i,j)/z0b(i,j)+1.0_rsh))**2
    !cdvit(i,j)=min(cdf*vitf,0.01_rsh)
    cdvit(i,j)=cdf*vitf
#else
    cdf=(0.4_rsh/LOG((sig(1)+1.0_rsh)*hty(i,j)/z0b(i,j)+1.0_rsh))**2
    cdvit(i,j)=cdf*vitf
#endif
#endif

    tofyexp=cdvit(i,j)*vz(1,i,j)/hty(i,j)*cfroe
    intz_exp(i,j)=intz_exp(i,j)-tofyexp

!*************   end include dyn3dy_botstress   *******************
