!*********    including dyn3dy_ipg_pint.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_ipg_dj.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the density jacobian
   !&E                  (for sigma generalized framework)
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz2,dy_v,hc_sig,Csu_sig
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Marsaleix et al. (2009), low order pressure gradient
   !&E                schemes in sigma coordinate models : the seamount test revisited
   !&E
   !&E ** History :
   !&E       !  2009-04  (F. Dumas, V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------

     dyb=dy_v(i,j)

     htxen=h0(i,j+1)+xec(i,j+1)
     htxes=h0(i,j  )+xec(i,j  )
! S.Petton 09/2013 : Utilisation d un profil moyen
   ! surface - estimate of the second pressure gradient term : boz2*dz/dx

     k = kmax

     p_prim_interface_j  (k)=0.0_rlg
     p_prim_interface_jp1(k)=0.0_rlg

#if defined key_siggen || defined key_gencoord
     hz_j   = f_hzu(h0(i,j),xec(i,j  ),k,i,j  )
     hz_jp1 = f_hzu(h0(i,j+1),xec(i,j+1),k,i,j+1)
#else
     hz_j   = htxes
     hz_jp1 = htxen
#endif
     zk_j   = f_zu(h0(i,j  ),xec(i,j  ),k,i,j  )
     zk_jp1 = f_zu(h0(i,j+1),xec(i,j+1),k,i,j+1)
     ! g.r.dz/dx
! S.Petton 09/2013 : Utilisation d un profil moyen
     sec_gp(k)=(boz2(k,i,j+1)*hz_jp1+boz2(k,i,j)*hz_j)/(hz_jp1+hz_j)*(zk_jp1-zk_j)/dyb


   ! estimate of the second pressure gradient term : boz2*dz/dx (k level)
   ! estimate of the  first pressure gradient term : d/dx integral(boz2) (k+1 level)
! S.Petton 09/2013 : Utilisation d un profil moyen
     DO k=kmax-1,1,-1
       hz_j_kp1 = hz_j
       hz_jp1_kp1 = hz_jp1
#if defined key_siggen || defined key_gencoord
       hz_j   = f_hzu(h0(i,j),xec(i,j  ),k,i,j  )
       hz_jp1 = f_hzu(h0(i,j+1),xec(i,j+1),k,i,j+1)
#else
       hz_j   = htxes
       hz_jp1 = htxen
#endif
       zk_j   = f_zu(h0(i,j  ),xec(i,j  ),k,i,j  )
       zk_jp1 = f_zu(h0(i,j+1),xec(i,j+1),k,i,j+1)

       ! horizontal gradient of pressure (sigma = cst)
! S.Petton 09/2013 : Utilisation d un profil moyen
       p_prim_interface_j(k)   = p_prim_interface_j(k+1)   + boz2(k+1,i,j  )*hz_j_kp1  *dsigu(k+1)
       p_prim_interface_jp1(k) = p_prim_interface_jp1(k+1) + boz2(k+1,i,j+1)*hz_jp1_kp1*dsigu(k+1)
       p_primu_j(k+1)  =0.5_rlg*(p_prim_interface_j(k)  +p_prim_interface_j(k+1)  )
       p_primu_jp1(k+1)=0.5_rlg*(p_prim_interface_jp1(k)+p_prim_interface_jp1(k+1))

!cval 0.5_rlg*(p_prim_interface_jp1(k)+p_prim_interface_jp1(k+1)) alors que u n est pas
!cval necessairement au milieu de la couche ?

       ! g.r.dz/dx
! S.Petton 09/2013 : Utilisation d un profil moyen
       sec_gp(k)=(boz2(k,i,j+1)*hz_jp1+boz2(k,i,j)*hz_j)/(hz_jp1+hz_j)*(zk_jp1-zk_j)/dyb

     END DO

   ! bottom estimate of the first pressure gradient term : d/dx integral(boz2) (k=1 level)
! S.Petton 09/2013 : Utilisation d un profil moyen
     k = 0
     hz_j_kp1 = hz_j
     hz_jp1_kp1 = hz_jp1
     p_prim_interface_j(k)   = p_prim_interface_j(k+1)   + boz2(k+1,i,j  )*hz_j_kp1  *dsigu(k+1)
     p_prim_interface_jp1(k) = p_prim_interface_jp1(k+1) + boz2(k+1,i,j+1)*hz_jp1_kp1*dsigu(k+1)
     p_primu_j(k+1)  =0.5_rlg*(p_prim_interface_j(k)  +p_prim_interface_j(k+1)  )
     p_primu_jp1(k+1)=0.5_rlg*(p_prim_interface_jp1(k)+p_prim_interface_jp1(k+1))


    do k=1,kmax
     yz_exp(k,i,j) = yz_exp(k,i,j)+(p_primu_jp1(k)-p_primu_j(k))/dyb+sec_gp(k)
    end do

!********* end  including dyn3dy_ipg_pint.h      ****************************
