!*********    including dyn3dy_ipg_pj.h      *******************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_ipg_pj.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the meridional momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the pressure jacobian
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz,dx_v
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E
   !&E---------------------------------------------------------------------

    dyb=dy_v(i,j)

!    dhyy=(h0(i,j+1)-h0(i,j))/dyb
    dhyy=(xec(i,j+1)-xec(i,j))/dyb
    htxen=h0(i,j+1)+xec(i,j+1)
    htxes=h0(i,j)+xec(i,j)
    dhty=(htxen-htxes)/dyb
    pri=0.0_rlg

    k=kmax
    bms=(htxen*boz(k,i,j+1)-htxes*boz(k,i,j))/dyb
   ! internal pressure gradient : dP/dy part
!    pri=pri+bms*(1.0-sig(kmax))
    pri=pri-bms*sig(kmax)
!    tbc=pri+(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg*(sig(k)*dhty-dhyy)
    tbc=pri+(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg*(sig(k)*dhty+dhyy)
    yz_exp(k,i,j)=yz_exp(k,i,j)+tbc
!
    DO k=kmax-1,1,-1
      bm=(htxen*boz(k,i,j+1)-htxes*boz(k,i,j))/dyb
      pri=pri+(bm+bms)*0.5_rlg*dsigw(k)
      tbc=pri+(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg*(sig(k)*dhty+dhyy)
      bms=bm
      yz_exp(k,i,j)=yz_exp(k,i,j)+tbc
    END DO

!********* end  including dyn3dy_ipg_pj.h      ***********************
