!*********    including dyn3dy_ipg_pj_sig.h      *******************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_ipg_pj_sig.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the meridional momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the pressure jacobian
   !&E                  (for sigma generalized framework)
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz,dx_v,hc_sig,csu_sig
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-04  (F. Dumas) generalized sigma
   !&E
   !&E---------------------------------------------------------------------


    dyb=dy_v(i,j)
    dhyy=( h0(i,j+1)- h0(i,j))/dyb               ! d h0 / dy
    dhcy=( hc_sig(i,j+1)- hc_sig(i,j))/dyb       ! d hc / dy
    dxey=(xec(i,j+1)-xec(i,j))/dyb               ! d xe / dy

    k=kmax

   ! pri = d int_z (b dz) / dy  (at u level)
    bhzn=f_hzw(h0(i,j+1),xec(i,j+1),k,i,j+1)*boz(k,i,j+1)
    bhzs=f_hzw(h0(i,j  ),xec(i,j  ),k,i,j  )*boz(k,i,j  )
    bms=(bhzn-bhzs)/dyb
    pri=bms*(0.0_rlg-sig(k))

   ! add b * dz/dy
!hccst    tbc=pri+(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg*((sig(k)+1.0_rlg)*dxey+dhyy*csu_sig(k))
    tbc=pri+(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg*((sig(k)+1.0_rlg)*dxey+sig(k)*dhcy+(dhyy-dhcy)*csu_sig(k))
    yz_exp(k,i,j)=yz_exp(k,i,j)+tbc

    DO k=kmax-1,1,-1

   ! d int_z (b dz) / dy (at u level)
      bhzn=(f_hzw(h0(i,j+1),xec(i,j+1),k,i,j+1))*0.5_rlg*(boz(k,i,j+1)+boz(k+1,i,j+1))
      bhzs=(f_hzw(h0(i,j  ),xec(i,j  ),k,i,j  ))*0.5_rlg*(boz(k,i,j  )+boz(k+1,i,j  ))
      bm=(bhzn-bhzs)/dyb
      pri=pri+bm*dsigw(k)

   ! add b * dz/dy
!hccst     tbc=pri+(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg*((sig(k)+1.0_rlg)*dxey+dhyy*csu_sig(k))
      tbc=pri+(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg*((sig(k)+1.0_rlg)*dxey+sig(k)*dhcy+(dhyy-dhcy)*csu_sig(k))
      bms=bm
      yz_exp(k,i,j)=yz_exp(k,i,j)+tbc

    END DO

!********* end  including dyn3dy_ipg_pj_sig.h      ***********************
