   !&E   ***************   include dyn3dy_mtm   **********************************
   !&E
   !&E ** Purpose : remplissage des termes du systeme lineaire
   !&E              provenant de la discretisation de l equation de conservation
   !&E              de la quantite de mouvement du systeme d equations
   !&E              de Saint-Venant (2D) dans la direction Nord-Sud
   !&E              sur l ensemble de la grille.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : dyn3dy_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History : 2006-05
   !&E              2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2010-07  (L. Debreu, F. Dumas) rewritting of ADI
   !&E       !  2011-03  (F. Dumas) bottom stress parametrisation + remove case 2 and 3 for tidal flat processing
   !&E       !  2011-10  (L. Pineau, F. Dumas, V. garnier) correction bottom stress parametrisation
   !&E       !  2011-12  (S. Petton, F. Dumas) Implicitation of the bottom stress (consistence between 2D en 3D)
   !&E                                         removing of min,max
   !&E
   !&E-------------------------------------------------------------------------

     gdt=grav*dt

     j1=jb(i)
     j2=jh(i)
     IF (j >= j1 .AND. j <= j2-1) THEN

   ! cas d un point v sans eau de part ni d autre
   ! drying v-point : no water on the right and left

      IF (igdv(i,j) == 4) THEN

        av(i,j)=0.0_rlg
        bv(i,j)=1.0_rlg
        cv(i,j)=0.0_rlg
        y2dv(i,j)=0.0_rlg

   ! cas d un point u avec de l eau de part et d autre
   ! u-point surrounded by water

      ELSE IF (igdv(i,j) == 1) THEN

        ! sea surface pressure gradient : implicit part
        av(i,j)=-alphan*0.5_rlg*gdt/dy_v(i,j)
        cv(i,j)= alphan*0.5_rlg*gdt/dy_v(i,j)

        ! time evolution (v at n+1/2)
#ifdef key_dyn_adv_linear

        ! time evolution (v at n+1)
        bv(i,j)=1.0_rlg

#else

        ! bottom stress (implicit)
        u_vtmp=0.25_rsh*(u(i,j)+u(i,j+1)+u(i-1,j)+u(i-1,j+1))
        vit=SQRT(u_vtmp*u_vtmp+v(i,j)*v(i,j))
        ft=0.5_rsh*dt*vit/hty(i,j)*(0.4_rsh/LOG(hty(i,j)/(exp1*z0b(i,j))+1.0_rsh))**2
#ifdef key_no_bottom_stress
        ft=0.0_rsh
#elif key_linear_bottom_stress
        !ft=frot_lin/hty(i,j)
        ft=halfdt*frot_lin/MAX(hty(i,j),hminfrot)
#endif

        bv(i,j)=1.0_rlg/(                                &
                ! time evolution (v at n+1)
                1.0_rlg                                  &
                ! bottom stress : implicit part
                +ft*cfroi)

#endif

        ! rhs : explicit terms
        y2dv(i,j)=                                            &
                  ! time evolution (u at n-1/2)
                  v(i,j)                                      &
                  ! other explicit terms integrated from the bottom to the
                  +halfdt*intz_exp(i,j)                       &
                  ! explicit part of the sea surface pressure gradient
                  -0.5_rlg*gdt/dy_v(i,j)*alphap*( xec(i,j+1)-xec(i,j) )


      ELSE IF (igdv(i,j) == 5) THEN
      ! low depth

        u_vtmp=0.25_rsh*(u(i,j)+u(i,j+1)+u(i-1,j)+u(i-1,j+1))

        ! sea surface pressure gradient : implicit part
        av(i,j)=-alphan*0.5_rlg*gdt/dy_v(i,j)         ! j-1
        cv(i,j)= alphan*0.5_rlg*gdt/dy_v(i,j)         ! j

#ifdef key_dyn_adv_linear

        ! time evolution (v at n+1)
        bv(i,j)=1.0_rlg

#else

        ! bottom stress (implicit part)
        vit=SQRT(u_vtmp*u_vtmp+v(i,j)*v(i,j))
        ft=0.5_rsh*dt*vit/hty(i,j)*(0.4_rsh/LOG(hty(i,j)/(exp1*z0b(i,j))+1.0_rsh))**2
#ifdef key_no_bottom_stress
        ft=0.0_rsh
#elif key_linear_bottom_stress
        !ft=frot_lin/hty(i,j)
        ft=halfdt*frot_lin/MAX(hty(i,j),hminfrot)
#endif

        bv(i,j)=1.0_rlg/(                                &
                ! time evolution (v at n+1)
                1.0_rlg                                  &
                ! bottom stress : implicit part
                +ft*cfroi)

#endif

        ! rhs : explicit terms
        y2dv(i,j)=                                       &
                  ! time evolution (v at n)
                  v(i,j)                                 &
                  ! bottom stress (v at n)
                  -ft*cfroe*v(i,j)                       &
                  ! coriolis
                  -fv(i,j)*u_vtmp*halfdt                 &
                  ! explicit part of the sea surface pressure gradient
                  -0.5_rlg*gdt/dy_v(i,j)*alphap*( xec(i,j+1)-xec(i,j) )  &
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
                  ! NL advection : u*u*tan(phi)/R (spherical coordinates)
                  -halfdt*tfivsr(i,j)*u_vtmp**2          &
                  ! non-linear (vdv/dy) : implicit part
                  -halfdt*v(i,j)/(dy(i,j)+dy(i,j+1))*(v(i,j+1)-v(i,j-1)) &
#endif
#endif
                  ! viscosity (along y)
                  +halfdt*( vis2d(i,j+1)*(v(i,j+1)-v(i,j  ))/dy(i,j+1)   &
                           -vis2d(i,j  )*(v(i,j  )-v(i,j-1))/dy(i,j  ) )/dy_v(i,j) 

        ! viscosity and non-linear term
        IF ((htx(i-1,j)+htx(i-1,j+1)) >= 0.0_rsh) THEN
          dvsdx=(v(i,j)-v(i-1,j))/dx_f(i-1,j)
          y2dv(i,j)=y2dv(i,j)                            &
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
                    ! non-linear (udv/dx) : explicit part
                    -0.25_rlg*dt*u_vtmp*dvsdx             &
#endif
#endif
                    ! viscosity (along x) : explicit
                    -halfdt*vis2d(i-1,j)*dvsdx/dx_v(i,j)
        END IF
        IF ((htx(i,j)+htx(i,j+1)) >= 0.0_rsh) THEN
          dvsdx=(v(i+1,j)-v(i,j))/dx_f(i,j)
          y2dv(i,j)=y2dv(i,j)                            &
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
                    ! non-linear (udv/dx) : explicit part
                    -0.25_rlg*dt*u_vtmp*dvsdx             &
#endif
#endif
                    ! viscosity (along x) : explicit
                    +halfdt*vis2d(i,j)*dvsdx/dx_v(i,j)
        END IF

        ! wind stress and atmospherical pressure gradient
        y2dv(i,j)=y2dv(i,j)                              &

                  ! wind stress in y-direction : explicit
                  +halfdt*tosy(i,j)/MAX(hty(i,j),hminfrot)/rhoref &

                  ! atmospherical pressure gradient : explicit
                  -halfdt*(pa(i,j+1)-pa(i,j))/(dy_v(i,j)*rhoref)

        ! generator potential : explicit
        IF (l_genpot) &
          y2dv(i,j)=y2dv(i,j)+halfdt/dy_v(i,j)*(potgene(i,j+1)-potgene(i,j))

      END IF
     END IF

   !&E   ***************   end include dyn3dy_mtm   ******************************
