   !!======================================================================

  SUBROUTINE init_flux

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE init_flux  ***
   !&E
   !&E ** Purpose : initialize all the variables that are cumulated over several half time-steps
   !&E              for the advection computation
   !&E
   !&E ** Description : step3dxy, step3dxy
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !          Original code
   !&E       !  2004-10-04  last modifications
   !&E       !  2011-11     (V. Garnier, B. Thouvenin) New organisation for the initializations of the flux
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used

   USE parameters
   USE comfluxdo,    ONLY : phieau
#ifdef key_castest
   USE comfluxdo,    ONLY : evap,rain
   USE comvars2d,    ONLY : surf
#endif
#ifdef key_substance
   USE comsubstance, ONLY : phicon
#if defined key_phicondrycell 
   USE comsubstance, ONLY : phicon_drycell
   USE comvars2d,    ONLY : h0fond,h0
   USE comvarp2d,    ONLY : ssh
#endif
#endif
   !USE cometemp,ONLY:phitemp,phisal
   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   INTEGER :: i,j
   REAL(kind=rsh)  :: d3
   
   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP PARALLEL DO PRIVATE(i,j,d3)
   DO j= ljmin,ljmax
     phieau(:,:,j)=0.0_rlg
#ifdef key_substance
     phicon(:,:,:,j)=0.0_rsh
#ifdef key_phicondrycell
     DO i=liminp1,limax
       d3=h0(i,j)+ssh(i,j)
       IF (d3 > h0fond) THEN
         phicon(:,:,i,j)=phicon_drycell(:,:,i,j)
         phicon_drycell(:,:,i,j)=0.0_rsh
       ENDIF
     END DO
#endif
#endif
   ENDDO
!$OMP END PARALLEL DO

#ifdef key_castest
   IF (num_testcase==5) THEN
!$OMP PARALLEL
!$OMP DO SCHEDULE(RUNTIME)
     DO j=ljminp1,ljmax
       DO i = liminp1,limax
         phieau(kmax,i,j) = 0.001_rsh*(rain(i,j)-evap(i,j))*surf(i,j)
       END DO
     END DO
!$OMP END DO
!$OMP END PARALLEL
   ELSE IF (num_testcase==6) THEN
     IF (limax==imax) phieau(:,imax-1,:)=200.0_rlg/REAL(kmax,rlg)
   END IF
#endif

   PRINT_DBG*, 'exit INIT_FLUX routine'

  END SUBROUTINE init_flux
