   !!======================================================================

  SUBROUTINE init_turbcoef(ect,psi)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE init_turbcoef  ***
   !&E
   !&E ** Purpose : initialize coefficients of turbulent parametrization
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : nzinit, kzinit
   !&E
   !&E ** Modified variables : nz, kz
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       ! 2004-08-17
   !&E       ! 2007-09-10  (Pascal Douillet) initialisation nbouc=1
   !&E       ! 2009-12-12  (Thomas Duhaut) initialisation for 2 equations (k-kl,keps,komega)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvars2d,ONLY:h0
   USE comvars3d,ONLY:nz,kz,nzinit,kzinit,turb_2eq_option,turb_nbeq
   USE comturb  ,ONLY:sigmaect,sigmapsi,c1turb,c2turb,c3pturb,ectmin,    &
                      psimin,epsmin,fwall,cmu,c3mturb,pturb,mturb,nturb, &
                      eps,emin

   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),DIMENSION(kmax,limin:limax,ljmin:ljmax),INTENT(out) :: ect,psi

   !! * Local declarations
   INTEGER :: i,j
   !!----------------------------------------------------------------------
   !! * Executable part

   IF(turb_2eq_option==1) THEN ! modele k-kl MY2.5
     pturb=0.0_rsh
     mturb=1.0_rsh
     nturb=1.0_rsh
     sigmaect=2.44_rsh
     sigmapsi=2.44_rsh
     c1turb=1.00_rsh
     c2turb=0.50_rsh
     c3pturb=1.00_rsh
     c3mturb=2.530_rsh
!     ectmin=5.0e-6 
     ectmin=1.0e-8 
     psimin=1.0e-8
     epsmin=1.0e-12
     fwall=1.0_rsh
     cmu=0.5544_rsh
   END IF

   IF(turb_2eq_option==2) THEN ! modele k-epsilon
     pturb= 3.0_rsh
     mturb= 1.5_rsh
     nturb=-1.0_rsh
     sigmaect=1.0_rsh
     sigmapsi=1.3_rsh
     c1turb=1.38_rsh ! c1 orig
     !c1turb=1.656_rsh ! c1 +20%
     !c1turb=1.104_rsh ! c1 -20%
     c2turb=1.98_rsh  ! c2 orig
     !c2turb=2.376_rsh !c2 +20%
     !c2turb=1.584_rsh  ! c2 -20%
     c3pturb=1.0_rsh
     c3mturb=-0.52_rsh
     ectmin=1.0e-8
     psimin=1.0e-12   ! keep this variable. Does not work wirh 1.0e-11 (AC. Bennis)
     epsmin=1.0e-12
     fwall=1.0_rsh
     cmu=0.5544_rsh
   END IF

   IF(turb_2eq_option==3) THEN ! modele k-omega
     pturb=-1.0_rsh
     mturb= 0.5_rsh
     nturb=-1.0_rsh
     sigmaect=2.0_rsh
     sigmapsi=2.0_rsh
!     c1turb=0.555_rsh
     c1turb=0.480_rsh
!     c2turb=0.833_rsh
     c2turb=0.840_rsh
     c3pturb=1.0_rsh
     c3mturb=-0.58_rsh
!     ectmin=7.6e-6
     ectmin=1.0e-8
     psimin=1.0e-12
     epsmin=1.0e-12 ! a modifier
     fwall=1.0_rsh
     cmu=0.5544_rsh
   END IF

  IF(turb_2eq_option==4) THEN ! modele Gen 
     pturb= 2.0_rsh
     mturb= 1.0_rsh
     nturb=-0.66666667_rsh
     sigmaect=2.0_rsh
     sigmapsi=2.0_rsh
     c1turb=1.00_rsh
     c2turb=1.22_rsh
     c3pturb=1.00_rsh
     c3mturb=-0.10_rsh
     ectmin=1.0e-8
     psimin=1.0e-12
     epsmin=1.0e-12 ! a modifier
     fwall=1.0_rsh
     cmu=0.5544_rsh
   END IF

!$OMP parallel
!$OMP do private(j) 
   DO j=ljmin,ljmaxp1
     DO i=limin,limaxp1
       IF (h0(i,j)/=-valmanq) THEN
         nz(:,i,j)=nzinit
       ELSE
         nz(:,i,j)=valmanq
       END IF
     ENDDO
   ENDDO
!$OMP end do

!$OMP do private(j) 
   DO j=ljminm1,ljmaxp1
     DO i=liminm1,limaxp1
       IF (h0(i,j)/=-valmanq) THEN
         kz(:,i,j)=kzinit
       ELSE
         kz(:,i,j)=valmanq
       END IF
     ENDDO
   ENDDO
!$OMP end do

!$OMP do private(j) 
   DO j=ljmin,ljmax
     DO i=limin,limax
       IF (h0(i,j)/=-valmanq) THEN
         IF (turb_nbeq==1) THEN
           emin=1.0e-8
           ect(:,i,j)=emin
         ELSE
           ect(:,i,j)=ectmin
         ENDIF
         psi(:,i,j)=psimin
         eps(:,i,j)=epsmin
       ELSE
         ect(:,i,j)=valmanq
         psi(:,i,j)=valmanq
         eps(:,i,j)=valmanq
       END IF
     ENDDO
   ENDDO
!$OMP end do
!$OMP end parallel

 END SUBROUTINE init_turbcoef
