 MODULE casinit

   !!======================================================================
   !!                   ***  MODULE  casinit  ***
   !! Casinit :  Initialization of the test cases
   !!            cas_init : call the correct test case
   !!            cas_init_* : global initialisation of each component
   !!
   !!   History :
   !!        !  2012-07 (V. Garnier) Reorganisation of the implementation of the test cases
   !!
   !!======================================================================

#include "toolcpp.h"

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC cas_init                    ! routine called by main.F90
   PUBLIC cas_init_param              ! routine called by each test case
   PUBLIC cas_init_grid               ! routine called by each test case
   PUBLIC cas_init_bathy              ! routine called by each test case
   PUBLIC cas_init_io                 ! routine called by each test case
   PUBLIC cas_init_obc                ! routine called by each test case
   PUBLIC cas_init_meteo              ! routine called by each test case
   PUBLIC cas_init_visc               ! routine called by each test case
   PUBLIC cas_init_turb               ! routine called by each test case
   PUBLIC cas_init_tspar              ! routine called by each test case
   PUBLIC cas_init_progvar            ! routine called by each test case
   PUBLIC cas_init_misc               ! routine called by each test case
   PUBLIC cas_init_subs               ! routine called by each test case
   PUBLIC cas_init_subs_part          ! routine called by each test case
   PUBLIC cas_init_traj               ! routine called by each test case

 CONTAINS
   !!======================================================================

  SUBROUTINE cas_init
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (F. Dumas) Original code
   !&E       !  2007-03  (F. Vandermeirch) Introduction of test cases 11 to 13
   !&E       !  2007-03  (V. Garnier, T. Duhaut) Introduction of test cases 11 to 13
   !&E       !  2010-03  (A.-C. Bennis) Introduction of test case (wave : Mellor approach)
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : num_testcase

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part
 
   IF (num_testcase==1)  CALL cas_cylinder

   IF (num_testcase==2)  CALL cas_gravadj

   IF (num_testcase==3)  CALL cas_kelvin

   IF (num_testcase==4)  CALL cas_upwelling

   IF (num_testcase==5)  CALL cas_evap

   IF (num_testcase==6)  CALL cas_channel

   IF (num_testcase==7)  CALL cas_deepening

   IF (num_testcase==8)  CALL cas_seamount

   IF (num_testcase==9)  CALL cas_profile

   IF (num_testcase==10) CALL cas_wetdrying

! cas test courant barotrope constant ...
!   IF (num_testcase==11) CALL cas_bt_current_anc

! cas test courant barotrope constant
   IF (num_testcase==11) CALL cas_bt_current

! cas test courant barocline constant ...
   IF (num_testcase==12) CALL cas_bc_current

! cas test jet barotrope  ...
!   IF (num_testcase==13) CALL cas_jet_barotrope

! cas test vortex barocline  ...
   IF (num_testcase==14) CALL cas_bcvortex

! cas test maree interne
   IF (num_testcase==15) CALL cas_intwave

! barotropic vortex
   IF (num_testcase==16) CALL cas_btvortex

! barotropic current
   IF (num_testcase==17) CALL cas_instbtjet

! advection of a spot
   IF (num_testcase==18) CALL cas_advspot

! does advection converve constants ? (smolarkiewicz test case)
   IF (num_testcase==19) CALL cas_advcst

! smolarkiewicz test case (time dependent velocity field)
   IF (num_testcase==20) CALL cas_advsmolt

! advection of a lense over a seamount
   IF (num_testcase==24) CALL cas_advspotmount

! sedimetology
   IF (num_testcase==21) CALL cas_sedim

! turbulence
   IF (num_testcase==22) CALL cas_turbkatophil

! Waves : Mellor approach
#ifdef key_palm_mellor
   IF (num_testcase==23) CALL cas_mellor
#endif

   IF (num_testcase==25)  CALL cas_thacker

   IF (num_testcase==26)  CALL cas_innershelf

   IF (num_testcase==30) CALL cas_chansed

   IF (num_testcase==33) CALL cas_tidalflat2dv

   PRINT_DBG*, 'exit CAS_INIT routine'

  END SUBROUTINE cas_init

   !!======================================================================

  SUBROUTINE cas_init_param
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_param  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : rsh
   USE comvars2d, ONLY : l_corio_cst,hm,h0fond,l_tssubadv_over_smalldepth,dhj,fwetmin,l_maxu,l_modele2d
   USE comvars3d, ONLY : l_stability
   USE comfluxdo,   ONLY : file_river
   USE cometemp,    ONLY : chp,coext
   USE cometdyn,    ONLY : rhoair

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   l_corio_cst=.TRUE.    ! true : f-plane, false : beta-plane

   l_maxu=.TRUE.    ! to list the maximum of the velocity in the log file

   l_modele2d=.FALSE.  ! if TRUE barotropic test case

   ! bathymetrie
   h0fond=0.0_rsh
   hm=0.0_rsh
   fwetmin=1.0_rsh
   dhj=0.0_rsh   ! > 0 if wet-drying
   l_tssubadv_over_smalldepth=.true.

   ! miscellaneous
   ! -------------

   l_stability=.FALSE.
   rhoair=1.25_rsh
   chp=3986.0_rsh
   coext=0.1_rsh
   file_river='e'

  END SUBROUTINE cas_init_param

   !!======================================================================

  SUBROUTINE cas_init_grid(dxcst,dycst)
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_grid  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : pi,rsh,rlg,imin,imax,jmin,jmax,kmax,limin,limax,ljmin,ljmax, &
                          liminm1,ljmaxp1,ljminm1,limaxp1
   USE comvars2d,  ONLY : fisac,gwac,dfiac,dgac,dx,dy,dx_u,dx_v,dy_u,dy_v,dx_f,dy_f,surf, &
                          fu,fv,tfiusr,tfivsr,h0,hx,hy, &
                          jb,jh,ig,id,goulet,govlet,radearth,l_corio_cst, &
                          lon2d_g,lat2d_g,lon2du_g,lon2dv_g,lon2df_g,lat2du_g,lat2dv_g,lat2df_g, &
                          lon2d,lat2d,lon2du,lon2dv,lon2df,lat2du,lat2dv,lat2df
   USE comvars3d,  ONLY : sig,sigw,dsigu,dsigw
#if defined key_siggen || defined key_gencoord
   USE comsiggen,  ONLY : b_sig,theta_sig,Csu_sig,Csw_sig,dcusds,dcwsds, &
#if ! defined key_siggen_g2
                          hzdcusds,hzdcwsds,hzdcwsds_x,hzdcusds_x,hzdcwsds_y,hzdcusds_y, &
#endif
                          hc_sig,hcx_sig,hcy_sig
#endif

   IMPLICIT NONE 

   !! * Arguments
   REAL(KIND=rsh), INTENT(IN)    :: dxcst,dycst

   !! * Local declarations
   INTEGER :: i,j,k
   REAL(KIND=rsh)    :: fi,f0
   REAL(KIND=rlg)    :: beta

   !!----------------------------------------------------------------------
   !! * Executable part

   ! Horizontal discretization
   ! -------------------------

   dx(:,:)=dxcst
   dx_u(:,:)=dxcst
   dx_v(:,:)=dxcst
   dx_f(:,:)=dxcst
   dy(:,:)=dycst
   dy_u(:,:)=dycst
   dy_v(:,:)=dycst
   dy_f(:,:)=dycst
   surf(:,:)=dxcst*dycst
   goulet(:,:)=1.0_rsh
   govlet(:,:)=1.0_rsh

   ! Vertical discretization
   ! -----------------------

   DO k=1,kmax
     sig(k)=(REAL(k-kmax,rsh)-0.5_rsh)/REAL(kmax,rsh)
   END DO
   sig(kmax+1)=-sig(kmax)
   sig(0)=ABS(sig(1))-2.0_rsh

   DO k=1,kmax
     dsigw(k)=sig(k+1)-sig(k)
     sigw(k)=0.5_rsh*(sig(k)+sig(k+1))
   END DO
   sigw(kmax+1)=-sigw(kmax)
   sigw(0)=-1.0_rsh

   DO k=1,kmax
     dsigu(k)=sigw(k)-sigw(k-1)
   END DO

#if defined key_siggen || defined key_gencoord

#ifdef key_siggen_g2
   DO k=0,kmax+1
     IF (theta_sig > 0.0_rsh) THEN
       Csu_sig(k)=(1.0_rsh-COSH(theta_sig*sig(k)))/(COSH(theta_sig)-1.0_rsh)
     ELSE
       Csu_sig(k)=-sig(k)*sig(k)
     END IF
     IF (b_sig > 0.0_rsh) &
       Csu_sig(k)=(EXP(b_sig*Csu_sig(k))-1.0_rsh)/(1.0_rsh-EXP(-b_sig))
   END DO
   DO k=0,kmax
     IF (theta_sig > 0.0_rsh) THEN
       Csw_sig(k)=(1.0_rsh-COSH(theta_sig*sigw(k)))/(COSH(theta_sig)-1.0_rsh)
     ELSE
       Csw_sig(k)=-sigw(k)*sigw(k)
     END IF
     IF (b_sig > 0.0_rsh) &
       Csw_sig(k)=(EXP(b_sig*Csw_sig(k))-1.0_rsh)/(1.0_rsh-EXP(-b_sig))
   END DO
#else
   DO k=0,kmax+1
     Csu_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sig(k))/SINH(theta_sig) &
                + 0.5_rsh*b_sig*(TANH(theta_sig*(sig(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO
   DO k=0,kmax
     Csw_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sigw(k))/SINH(theta_sig) &
                + 0.5_rsh*b_sig*(TANH(theta_sig*(sigw(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO
#endif

   DO k=1,kmax
     dcusds(k)=(Csw_sig(k)-Csw_sig(k-1))/(sigw(k)-sigw(k-1))
   END DO

   DO k=1,kmax-1
     dcwsds(k)=(Csu_sig(k+1)-Csu_sig(k))/(sig(k+1)-sig(k))
   END DO
   dcwsds(kmax)=(0.0_rsh-csu_sig(kmax))/(0.0_rsh-sig(kmax))

#if ! defined key_siggen_g2
   DO j=ljmin,ljmaxp1
     DO i=limin,limaxp1
       DO k=1,kmax
         hzdcwsds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcwsds(k)
         hzdcusds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcusds(k)
         hzdcwsds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcwsds(k)
         hzdcusds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcusds(k)
         hzdcwsds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcwsds(k)
         hzdcusds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcusds(k)
       END DO
     END DO
   END DO
#endif

#endif


   ! longitude latitude
   ! ------------------

   dfiac=dycst*360.0_rsh/(2*pi*radearth)
   dgac=dxcst*360.0_rsh/(2*pi*radearth)

   lon2d_g(:,0)=(/ (gwac + dgac *(i-1), i=0,imax) /)
   DO j=1,jmax
     lon2d_g(:,j)=lon2d_g(:,0)
   END DO
   lon2du_g(:,:)=lon2d_g(:,:)+dgac/2.0_rsh
   lon2dv_g(:,:)=lon2d_g(:,:)
   lon2df_g(:,:)=lon2du_g(:,:)
   lat2d_g(0,:)=(/ (fisac + dfiac *(j-1), j=0,jmax) /)
   DO i=1,imax
     lat2d_g(i,:)=lat2d_g(0,:)
   END DO
   lat2du_g(:,:)=lat2d_g(:,:)
   lat2dv_g(:,:)=lat2d_g(:,:)+dfiac/2.0_rsh
   lat2df_g(:,:)=lat2dv_g(:,:)

   lon2du(liminm1:limax,ljmin:ljmaxp1)=lon2du_g(liminm1:limax,ljmin:ljmaxp1)
   lon2d(limin:limax,ljmin:ljmax)=lon2d_g(limin:limax,ljmin:ljmax)
   lon2dv(limin:limax,ljmin:ljmax)=lon2dv_g(limin:limax,ljmin:ljmax)
   lon2df(limin:limax,ljmin:ljmax)=lon2du_g(limin:limax,ljmin:ljmax)

   lat2dv(limin:limax,ljminm1:ljmax)=lat2dv_g(limin:limax,ljminm1:ljmax)
   lat2d(limin:limax,ljmin:ljmax)=lat2d_g(limin:limax,ljmin:ljmax)
   lat2du(limin:limax,ljmin:ljmax)=lat2du_g(limin:limax,ljmin:ljmax)
   lat2df(limin:limax,ljmin:ljmax)=lat2dv_g(limin:limax,ljmin:ljmax)

   ! coriolis parameter
   ! ------------------

   f0 = 2.0_rsh * 2.0_rsh*pi/(3600.0_rsh*24.0_rsh)
   fi = REAL(fisac,rsh)
   IF (l_corio_cst) THEN
     ! f-plane
     DO j=ljmin,ljmax
       fu(:,j)=f0*SIN(fi*pi/180.0_rsh)
       fv(:,j)=f0*SIN(fi*pi/180.0_rsh)
     END DO
   ELSE
     ! beta-plane
     beta=f0*COS(fi*pi/180.0_rlg)/radearth
     DO j=ljmin,ljmax
       fu(:,j)=f0*SIN(fi*pi/180.0_rsh)+beta*dycst*(REAL(j,rsh)-REAL(jmin+jmax,rsh)/2.0_rsh)
       fv(:,j)=f0*SIN(fi*pi/180.0_rsh)+beta*dycst*(REAL(j,rsh)-REAL(jmin+jmax,rsh)/2.0_rsh+0.5_rsh)
     END DO
   END IF

   ! Oh la attention à revoir
   tfiusr(:,:)=0.0_rsh
   tfivsr(:,:)=0.0_rsh

  END SUBROUTINE cas_init_grid

   !!======================================================================

  SUBROUTINE cas_init_bathy(depthcst,ni_wall_e,ni_wall_w,nj_wall_n,nj_wall_s)
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_bathy  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : valmanq,epsilon,rsh,imin,imax,jmin,jmax, &
                          limin,limax,ljmin,ljmax,liminm2,limaxp2,ljminm2,ljmaxp2, &
                          limin,limaxp1,ljmin,ljmaxp1,liminm1,ljminm1
   USE comvars2d,  ONLY : h0_g,hx_g,hy_g,h0,hx,hy,dhxy,dhx,dhy,dhx_g,dhy_g,dhj,h0fond,mslshift, &
                          dhxy_g,dhxy,hrdef
   USE comcolig2d, ONLY : me,mn,ms,mo
#if defined key_siggen || defined key_gencoord
   USE comsiggen,  ONLY : hc_sig_g,hcx_sig_g,hcy_sig_g,hc,hc_sig,hcx_sig,hcy_sig
#endif

   IMPLICIT NONE 

   !! * Arguments
   INTEGER, OPTIONAL :: ni_wall_e,ni_wall_w,nj_wall_n,nj_wall_s
   REAL(KIND=rsh), INTENT(IN) :: depthcst

   !! * Local declarations
   INTEGER :: i,j
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: hx_t,hy_t


   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CAS_INIT_BATHY routine'

   mslshift=0.0_rsh
   hrdef=0.0_rsh

   ! depth of the domain
   ! -------------------
   IF (depthcst/=0.0_rsh) THEN
     hx_g(:,:)=depthcst
     hy_g(:,:)=depthcst
     h0_g(:,:)=depthcst+h0fond
   END IF

   ! wall mask
   ! ---------
   me(:,:)=1
   mo(:,:)=1
   ms(:,:)=1
   mn(:,:)=1

   ! walls of the domain
   ! -------------------
   IF (PRESENT(ni_wall_e)) THEN
     hx_g(imax-ni_wall_e  :imax,:)=-valmanq-h0fond
     hy_g(imax-ni_wall_e+1:imax,:)=-valmanq-h0fond
     h0_g(imax-ni_wall_e+1:imax,:)=-valmanq
     IF_MPI (limax==imax) THEN
     me(imax-ni_wall_e  :imax,:)=0
     mo(imax-ni_wall_e+1:imax,:)=0
     ms(imax-ni_wall_e  :imax,:)=0
     mn(imax-ni_wall_e  :imax,:)=0
     ENDIF_MPI
   END IF

   IF (PRESENT(ni_wall_w)) THEN
     hx_g(imin:imin+ni_wall_w-1,:)=-valmanq-h0fond
     hy_g(imin:imin+ni_wall_w-1,:)=-valmanq-h0fond
     h0_g(imin:imin+ni_wall_w-1,:)=-valmanq
     IF_MPI (limin==imin) THEN
     me(imin:imin+ni_wall_w-1,:)=0
     mo(imin:imin+ni_wall_w  ,:)=0
     ms(imin:imin+ni_wall_w-1,:)=0
     mn(imin:imin+ni_wall_w-1,:)=0
     ENDIF_MPI
   END IF

   IF (PRESENT(nj_wall_n)) THEN
     hx_g(:,jmax-nj_wall_n+1:jmax)=-valmanq-h0fond
     hy_g(:,jmax-nj_wall_n  :jmax)=-valmanq-h0fond
     h0_g(:,jmax-nj_wall_n+1:jmax)=-valmanq
     IF_MPI (ljmax==jmax) THEN
     me(:,jmax-nj_wall_n  :jmax)=0
     mo(:,jmax-nj_wall_n  :jmax)=0
     ms(:,jmax-nj_wall_n+1:jmax)=0
     mn(:,jmax-nj_wall_n  :jmax)=0
     ENDIF_MPI
   END IF

   IF (PRESENT(nj_wall_s)) THEN
     hx_g(:,jmin:jmin+nj_wall_s-1)=-valmanq-h0fond
     hy_g(:,jmin:jmin+nj_wall_s-1)=-valmanq-h0fond
     h0_g(:,jmin:jmin+nj_wall_s-1)=-valmanq
     IF_MPI (ljmin==jmin) THEN
     me(:,jmin:jmin+nj_wall_s-1)=0
     mo(:,jmin:jmin+nj_wall_s-1)=0
     ms(:,jmin:jmin+nj_wall_s  )=0
     mn(:,jmin:jmin+nj_wall_s-1)=0
     ENDIF_MPI
   END IF

   !
   ! ----------------
   hx_t(:,:)=100000.0_rsh
   hy_t(:,:)=100000.0_rsh
   WHERE (hx_g(:,:) /= -valmanq -h0fond) hx_t(:,:)=hx_g(:,:)
   WHERE (hy_g(:,:) /= -valmanq -h0fond) hy_t(:,:)=hy_g(:,:)
   dhxy_g(:,:)=1.0_rsh
   DO j=jmin+1,jmax
      DO i=imin+1,imax
         dhxy_g(i,j)=MAX(hx_g(i,j),hx_g(i-1,j),hy_g(i,j),hy_g(i,j-1))- &
                     MIN(hx_t(i,j),hx_t(i-1,j),hy_t(i,j),hy_t(i,j-1))+epsilon
      END DO
   END DO

   dhx_g(:,:)=0.0_rsh
   dhy_g(:,:)=0.0_rsh
   WHERE (hx_g(:,:) < hrdef) dhx_g(:,:)=dhj
   WHERE (hy_g(:,:) < hrdef) dhy_g(:,:)=dhj

#if defined key_siggen || defined key_gencoord
   hc_sig_g(:,:)=0.0_rsh ; hcx_sig_g(:,:)=0.0_rsh ; hcy_sig_g(:,:)=0.0_rsh
   WHERE (h0_g(:,:)/=-valmanq) hc_sig_g(:,:)=MIN(h0_g(:,:)-0.00001_rsh,hc)
   WHERE (hx_g(:,:)/=-valmanq-h0fond) hcx_sig_g(:,:)=MIN(hx_g(:,:)-0.00001_rsh,hc)
   WHERE (hy_g(:,:)/=-valmanq-h0fond) hcy_sig_g(:,:)=MIN(hy_g(:,:)-0.00001_rsh,hc)
   hc_sig(:,:)  = hc_sig_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hcx_sig(liminm1:limaxp1,ljmin  :ljmaxp1)= hc_sig_g(liminm1:limaxp1,ljmin  :ljmaxp1)
   hcy_sig(limin  :limaxp1,ljminm1:ljmaxp1)= hc_sig_g(limin  :limaxp1,ljminm1:ljmaxp1)
#endif


   ! local variables
   ! ---------------
   h0(:,:) = h0_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hx(:,:) = hx_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hy(:,:) = hy_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   dhx(:,:) = dhx_g(limin:limax,ljmin:ljmax)
   dhy(:,:) = dhy_g(limin:limax,ljmin:ljmax)
   dhxy(:,:) = dhxy_g(limin:limax,ljmin:ljmax)

  END SUBROUTINE cas_init_bathy

   !!======================================================================


  SUBROUTINE cas_init_io(date_start,date_end,date_start3d,date_startagrif, &
                         suffix,l_nc4par,nb_file)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_io  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : rsh,rlg,riosh,lchain,imin,imax,jmin,jmax,kmax,limin,limax,ljmin,ljmax
   USE comvars2d, ONLY : date_ref,tdeb,t,tfin,tdeb3d,tdebagrif,date_startshift, &
                         cdate,ijour,imois,ian,iheure,iminu,isec, &
                         cflcrt,tobs,tdebobs,cflumaxmax,cflvmaxmax,l_obc2drank_save, &
                         l_saverestart_1file,l_initfromfile,l_saverestart_bydate,l_init_rtime, &
                         l_init_rdt,l_init_rssh,l_init_rbtvel,l_init_r3dvel,l_init_rwz, &
                         l_init_rturb,l_init_rsal,l_init_rtemp, &
                         l_init_restart,file_saverestart,file_init,saverestart_step, &
                         name_bathy_hx,name_bathy_hy,name_bathy_h0,name_in_dt,name_in_xev,   &
                         name_in_u,name_in_v,name_in_uz,name_in_vz,      &
                         name_in_wz,name_in_temp,name_in_sal,name_in_psi,&
                         name_in_ect,name_in_kz,name_in_nz,name_in_vis2d,&
                         name_in_vis3d_xe,name_in_vis3d_phi, &
                         iscreen,iscreenlog,ierrorlog,iwarnlog
   USE comvars2d, ONLY : dx,dy
   USE comvars2d, ONLY : cg_conv_title,cg_conv_comment,cg_conv_source,      &
                         cg_conv_area,cg_conv_institution,                  &
                         cg_conv_institution_references,cg_conv_data_centre,&
                         cg_conv_data_centre_references,                    &
                         cg_conv_forecast_range,cg_conv_forecast_type,      &
                         cg_conv_field_type,cg_conv_operational_status
   USE comvars2d, ONLY : riog_valid_min_h0,riog_valid_max_h0,         &
                         riog_valid_min_u,riog_valid_max_u,             &
                         riog_valid_min_temp,riog_valid_max_temp,       &
                         riog_valid_min_sal,riog_valid_max_sal,         &
                         riog_valid_min_xe,riog_valid_max_xe,           &
                         riog_valid_min_wstress,riog_valid_max_wstress, &
                         riog_valid_min_flux,riog_valid_max_flux,       &
                         riog_valid_min_visc,riog_valid_max_visc,       &
                         riog_valid_min_nz,riog_valid_max_nz,           &
                         riog_valid_min_kz,riog_valid_max_kz,           &
                         riog_valid_min_bz,riog_valid_max_bz, &
                         riog_valid_min_dx,riog_valid_min_dy,         &
                         riog_valid_max_dx,riog_valid_max_dy
   USE comvars2d, ONLY : name_out_h0,name_out_hx,name_out_hy,name_out_u, &
                         name_out_v,name_out_uz,name_out_vz,name_out_temp, &
                         name_out_sal,name_out_xe,name_out_tosx,name_out_tosy, &
                         name_out_vis2d,name_out_vishxe,name_out_vishphi, &
                         name_out_nz,name_out_kz,name_out_bz

   USE output, ONLY : fileresu,tdebsor,tfinsor,nb_out,tinter,tsort,pasor,   &
                      l_out_precdate,l_out_demer,l_out_ssh,l_out_ubt,l_out_u3d, &
                      l_out_sal,l_out_temp,l_out_wind,l_out_buoy,           &
                      l_out_visc,l_out_turb,l_out_off,l_out_atmsflx,        &
                      iextmin,iextmax,l_out_nc4par,l_out_pack,l_out_zaxis_w,&
                      jextmin,jextmax,kextmin,kextmax,stepi,stepj,stepk,    &
                      igextmin,igextmax,jgextmin,jgextmax,l_filebydate,     &
                      l_out_timeave,nbave
   USE comdiag,     ONLY : l_diag,file_diag,tdebdiag,tfindiag
   USE_MPI toolmpi, ONLY : ADD_TAG,MASTER,min_all_mpi_rsh,max_all_mpi_rsh



   IMPLICIT NONE

   !! * Arguments
   CHARACTER(LEN=19), INTENT(IN)     :: date_start,date_end,date_start3d,date_startagrif
   CHARACTER(len=lchain), INTENT(IN) :: suffix
   LOGICAL, OPTIONAL,     INTENT(IN) :: l_nc4par
   INTEGER, OPTIONAL,     INTENT(IN) :: nb_file

   !! * Local declarations
   CHARACTER(LEN=lchain) :: logfilename,chain
   INTEGER               :: i
   REAL(KIND=rsh)        :: value
   REAL(KIND=rlg)        :: tool_datosec
   CHARACTER(LEN=19)     :: tool_sectodat


   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CAS_INIT_IO routine'

   ! dates
   ! -----

   date_ref = '01-Jan-1900 00:00:00'
   tdeb     = tool_datosec(date_start)
   t=tdeb
   tfin     = tool_datosec(date_end)
   tdeb3d   = tool_datosec(date_start3d)
   tdebagrif = tool_datosec(date_startagrif)
   date_startshift= 0.0_rlg
   cflcrt=0.6_rsh
   tobs=12.0_rlg*3600.0_rlg
   tdebobs=t
   cdate = tool_sectodat(t)
   CALL tool_decompdate(cdate,ijour,imois,ian,iheure,iminu,isec)

   ! restart file
   ! ------------

   l_initfromfile=.FALSE.
   file_init = 'save.nc'
   l_saverestart_1file=.FALSE.
   l_saverestart_bydate=.FALSE.
   file_saverestart='save.nc'
   saverestart_step=5.0_rlg*3600.0_rlg   ! in hours

   l_init_restart=.FALSE.
   l_init_rtime=.FALSE.
   l_init_rdt=.FALSE.
   l_init_rssh=.FALSE.
   l_init_rbtvel=.FALSE.
   l_init_r3dvel=.FALSE.
   l_init_rwz=.FALSE.
   l_init_rturb=.FALSE.
   l_init_rsal=.FALSE.
   l_init_rtemp=.FALSE.

   ! input parametrization
   ! ---------------------

   name_in_dt='dt'
   name_bathy_h0='h0'
   name_bathy_hx='hx'
   name_bathy_hy='hy'
   name_in_xev='xev'
   name_in_u='u'
   name_in_v='v'
   name_in_uz='uz'
   name_in_vz='vz'
   name_in_wz='wz'
   name_in_temp='temp'
   name_in_sal='sal'
   name_in_ect='ect'
   name_in_psi='psi'
   name_in_kz='kz'
   name_in_nz='nz'
   name_in_vis2d='name_in_vis2d'
   name_in_vis3d_xe='name_in_vis3d_xe'
   name_in_vis3d_phi='name_in_vis3d_phi'

   ! output parametrization
   ! ----------------------

   IF ( PRESENT(nb_file) ) THEN
     nb_out = nb_file
   ELSE
     nb_out = 1
   END IF
   ALLOCATE( fileresu(nb_out),tdebsor(nb_out),tfinsor(nb_out),tinter(nb_out) )
   ALLOCATE( tsort(nb_out),pasor(nb_out),l_filebydate(nb_out),l_out_precdate(nb_out) )
   ALLOCATE( l_out_off(nb_out),l_out_atmsflx(nb_out),l_out_visc(nb_out),l_out_turb(nb_out) )
   ALLOCATE( l_out_timeave(nb_out),l_out_buoy(nb_out), l_out_zaxis_w(nb_out) )
   ALLOCATE( l_out_demer(nb_out),l_out_ubt(nb_out),l_out_u3d(nb_out) )
   ALLOCATE( l_out_sal(nb_out),l_out_temp(nb_out),l_out_ssh(nb_out),l_out_wind(nb_out) )
   ALLOCATE( iextmin(nb_out),iextmax(nb_out),jextmin(nb_out),jextmax(nb_out) )
   ALLOCATE( igextmin(nb_out),igextmax(nb_out),jgextmin(nb_out),jgextmax(nb_out))
   ALLOCATE( kextmin(nb_out),kextmax(nb_out),stepi(nb_out),stepj(nb_out),stepk(nb_out) )
   fileresu(1)='results.nc'
   IF (nb_out >= 2) THEN
     DO i=2,nb_out
       WRITE(chain,123) nb_out
       fileresu(nb_out)='results_'//TRIM(chain)//'.nc'
     END DO
   END IF
123 FORMAT(I1)
   tdebsor(1:nb_out)=tool_datosec(date_start)
   tfinsor(1:nb_out)=tool_datosec(date_end)
   pasor(1:nb_out)=3600.0_rlg
   l_filebydate(1:nb_out)=.FALSE.
   l_out_off(1:nb_out)=.FALSE.
   l_out_precdate(1:nb_out)=.FALSE.
   l_out_demer(1:nb_out)=.FALSE.
   igextmin(1:nb_out)=imin
   igextmax(1:nb_out)=imax
   jgextmin(1:nb_out)=jmin
   jgextmax(1:nb_out)=jmax
   iextmin(1:nb_out)=limin
   iextmax(1:nb_out)=limax
   jextmin(1:nb_out)=ljmin
   jextmax(1:nb_out)=ljmax
   kextmin(1:nb_out)=1
   kextmax(1:nb_out)=kmax
   stepi(1:nb_out)=1
   stepj(1:nb_out)=1
   stepk(1:nb_out)=1
   l_out_ssh(1:nb_out)=.TRUE.
   l_out_ubt(1:nb_out)=.TRUE.
   l_out_u3d(1:nb_out)=.TRUE.
   l_out_sal(1:nb_out)=.TRUE.
   l_out_temp(1:nb_out)=.TRUE.
   l_out_wind(1:nb_out)=.FALSE.
   l_out_atmsflx(1:nb_out)=.FALSE.
   l_out_timeave(1:nb_out)=.FALSE.
   l_out_visc(1:nb_out)=.FALSE.
   l_out_turb(1:nb_out)=.FALSE.
   l_out_buoy(1:nb_out)=.FALSE.
   l_out_zaxis_w(1:nb_out)=.FALSE.
   l_obc2drank_save=.FALSE.
   l_obc2drank_save=.FALSE.
   IF ( PRESENT(l_nc4par) ) THEN
     l_out_nc4par = l_nc4par
   ELSE
     l_out_nc4par = .FALSE.
   END IF
   IF ( .NOT. l_out_nc4par) THEN
     DO i=1,nb_out
      IF_MPI ( .NOT. l_filebydate(nb_out) ) THEN
        CALL_MPI ADD_TAG(fileresu(nb_out),LEN(fileresu(nb_out)))
      ENDIF_MPI
     END DO
   END IF
   l_out_pack = .FALSE.
   nbave=0    ! no file for averaged variables

   name_out_h0='h0'
   name_out_hx='hx'
   name_out_hy='hy'
   name_out_u='u'
   name_out_v='v'
   name_out_uz='uz'
   name_out_vz='vz'
   name_out_temp='temp'
   name_out_sal='sal'
   name_out_xe='xe'
   name_out_vis2d='VIS2D'
   name_out_vishxe='VISH_XE'
   name_out_vishphi='VISH_PHI'
   name_out_nz='NZ'
   name_out_kz='KZ'
   name_out_bz='BZ'
   name_out_tosx='taux'
   name_out_tosy='tauy'

   ! initialization of attributes (max and min valid values for each variable)
   ! -------------------------------------------------------------------------

#if ! defined key_agrif
! it is not possible to initialize riog_valid_ variables
! because they are in riosh (=4) while agrif allow rlg (=8) only
! consequently, l_out_pack is not allowed for the test cases with agrif
   riog_valid_min_h0=-50.0_riosh
   riog_valid_max_h0=10000.0_riosh
   riog_valid_min_u=-20.0_riosh
   riog_valid_max_u=20.0_riosh
   riog_valid_min_temp=-5.0_riosh
   riog_valid_max_temp=50.0_riosh
   riog_valid_min_sal=-5.0_riosh
   riog_valid_max_sal=50.0_riosh
   riog_valid_min_xe=-50.0_riosh
   riog_valid_max_xe=50.0_riosh
   riog_valid_min_wstress=-2.0_riosh
   riog_valid_max_wstress=2.0_riosh
   riog_valid_min_flux=-1000.0_riosh
   riog_valid_max_flux=2000.0_riosh
   riog_valid_min_visc=0.0_riosh
   riog_valid_max_visc=1000.0_riosh
   riog_valid_min_nz=0.0_riosh
   riog_valid_max_nz=1000.0_riosh
   riog_valid_min_kz=0.0_riosh
   riog_valid_max_kz=1000.0_riosh
   riog_valid_min_bz=-1.0_riosh
   riog_valid_max_bz=1.0_riosh
#endif
   value=MINVAL(dx(:,:))
   CALL_MPI MIN_ALL_MPI_RSH(value)
   riog_valid_min_dx=value-1.0_riosh
   value=MAXVAL(dx(:,:))
   CALL_MPI MAX_ALL_MPI_RSH(value)
   riog_valid_max_dx=value+1.0_riosh
   value=MINVAL(dy(:,:))
   CALL_MPI MIN_ALL_MPI_RSH(value)
   riog_valid_min_dy=value-1.0_riosh
   value=MAXVAL(dy(:,:))
   CALL_MPI MAX_ALL_MPI_RSH(value)
   riog_valid_max_dy=value+1.0_riosh

   ! initialization of other attributes
   ! ----------------------------------

   cg_conv_title="name of area"
   cg_conv_comment=" "
   cg_conv_source="MARS CONF"
   cg_conv_area="area"
   cg_conv_institution="IFREMER"
   cg_conv_institution_references="http://www.ifremer.fr"
   cg_conv_data_centre="DYNECO/PHYSED"
   cg_conv_data_centre_references="http://wwz.ifremer.fr/mars3d"
   cg_conv_forecast_range="n/a"
   cg_conv_forecast_type="n/a"
   cg_conv_field_type="n/a"
   cg_conv_operational_status="experimental"

   ! log files
   ! ---------

   iscreen=12
   iscreenlog=18
   ierrorlog=17
   iwarnlog=16
   IF_MPI (MASTER) THEN
   logfilename= 'simu_'//TRIM(suffix)//'.log'
   IF (iscreenlog/=6) OPEN(unit=iscreenlog,file=logfilename,status="replace")
   logfilename='error_'//TRIM(suffix)//'.log'
   IF (ierrorlog/=6) OPEN(unit=ierrorlog,file=logfilename,status="replace")
   logfilename='warning_'//TRIM(suffix)//'.log'
   IF (iwarnlog/=6) OPEN(unit=iwarnlog,file=logfilename,status="replace")
   ENDIF_MPI

   ! diagnoses
   ! ---------

   l_diag=.FALSE.
   file_diag='diagnostic.txt'
   tdebdiag=tdebsor(1)
   tfindiag=tfinsor(1)

   ! max of velocity
   ! ---------------

   cflumaxmax=0.0_rsh
   cflvmaxmax=0.0_rsh

   PRINT_DBG*, 'exit CAS_INIT_IO routine'

  END SUBROUTINE cas_init_io

   !!======================================================================

  SUBROUTINE cas_init_obc
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_obc  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : rsh
   USE comvars2d,  ONLY : l_obc_cycl_x,l_obc_cycl_y
   USE comvars3d,  ONLY : saliref,tetaref
   USE obccombine, ONLY : l_obc_south, l_obc_north,l_obc_east,l_obc_west, &
                          l_obc_mars,l_obc_diri,l_obc_char,l_obc_cycl,    &
                          l_obc_tide,l_obc_ogcm, &
                          obc_tide_sshn,obc_tide_sshs,obc_tide_sshe,obc_tide_sshw, &
                          obc_tide_us,obc_tide_vs,obc_tide_uw,obc_tide_vw, &
                          obc_tide_un,obc_tide_vn, obc_tide_ue,obc_tide_ve
   USE obcogcm,    ONLY : obc_te,obc_tw,obc_ts,obc_tn,obc_se,obc_sw,obc_ss,obc_sn 
   USE obcogcm,    ONLY : obc_coefrel
   USE comaree,    ONLY : l_tide_harmcompo


   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   ! geometry (where are the open boundaries ?)
   l_obc_south=.FALSE.
   l_obc_north=.FALSE.
   l_obc_west=.FALSE.
   l_obc_east=.FALSE.

   ! used scheme to manage obc

   l_obc_mars = .FALSE.
   l_obc_diri = .FALSE.
   l_obc_char = .FALSE.
   l_obc_cycl = .FALSE.
   l_obc_cycl_x=.FALSE.
   l_obc_cycl_y=.FALSE.

   ! physics introduced into the simulated area (tide or ogcm solution)

   l_obc_tide=.FALSE.
   l_tide_harmcompo=.FALSE.
   l_obc_ogcm=.FALSE.

   ! tide at the open boundaries

   obc_tide_sshs(:)=0.0_rsh
   obc_tide_sshn(:)=0.0_rsh
   obc_tide_sshe(:)=0.0_rsh
   obc_tide_sshw(:)=0.0_rsh

   obc_coefrel=0.0_rsh


  END SUBROUTINE cas_init_obc

   !!======================================================================

  SUBROUTINE cas_init_meteo
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_meteo  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : rsh,rlg
   USE cometdyn,   ONLY : l_meteo_stat,l_meteo_hom,wind_veloc,wind_dir,cds, &
                          windamp_duration
   USE cometemp,   ONLY : l_sflx_solarcst,sflx_solarcst,l_sflx_radlossbot
   USE output,     ONLY : nbatm
   USE sflxatm,    ONLY : imeteo_dragtype,imeteo_exchtype

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   l_meteo_stat=.TRUE.
   l_meteo_hom=.TRUE.
   wind_veloc=0.0_rsh
   wind_dir=0.0_rsh
   windamp_duration = 0.0_rlg
   l_sflx_solarcst=.TRUE.
   sflx_solarcst=0.0_rsh
   l_sflx_radlossbot = .FALSE.
   nbatm=0
   imeteo_dragtype=0
   imeteo_exchtype=0
   cds=0.0016_rsh


  END SUBROUTINE cas_init_meteo

   !!======================================================================

  SUBROUTINE cas_init_visc
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_visc  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : rsh
   USE comvars2d,  ONLY : vis2d,vish_phi,vish_xe,vismin,vismax,cosmag,l_smagor, &
                          l_spongw,l_sponge,l_spongs,l_spongn,sponge_nbcell, &
                          fvisc,vismul

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   fvisc=0.0_rsh
   vis2d(:,:) = 0.0_rsh
   vish_xe(:,:,:) = 0.0_rsh
   vish_phi(:,:,:)= 0.0_rsh

   l_smagor=.FALSE.
   cosmag=0.1_rsh
   vismin=0.001_rsh
   vismax=4.0_rsh

   vismul=1.0_rsh
   sponge_nbcell=5
   l_spongs=.FALSE.
   l_spongn=.FALSE.
   l_sponge=.FALSE.
   l_spongw=.FALSE.

  END SUBROUTINE cas_init_visc

   !!======================================================================

  SUBROUTINE cas_init_turb(kzcst,nzcst,z0scst,z0bcst)
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_turb  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters,ONLY : rsh
   USE comvars2d, ONLY : z0b,z0s,l_incbotstress,botstressmax,frot_lin, &
                         hminfrot,hmaxfrot
   USE comvars3d, ONLY : turb_nbeq,turb_0eq_option,turb_2eq_option,nz,kz, &
                         bgdiff
   USE comvarp3d, ONLY : ect,psi
   USE comturb,   ONLY : eps

   IMPLICIT NONE 

   !! * Arguments
   REAL(KIND=rsh), INTENT(IN) :: kzcst,nzcst,z0scst,z0bcst

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   turb_nbeq=0
   turb_0eq_option=1
   turb_2eq_option=0
   z0b(:,:)  =z0bcst
   z0s(:,:)  =z0scst
   ect(:,:,:)=0.0_rsh
   eps(:,:,:)=0.0_rsh
   psi(:,:,:)=0.0_rsh
   nz(:,:,:) = nzcst
   kz(:,:,:) = kzcst
   l_incbotstress=.FALSE.
   frot_lin=0.0_rsh  ! used if key_linear_bottom_stress
   botstressmax=100.0_rsh
   hminfrot=0.5_rsh
   hmaxfrot=5.0_rsh
   bgdiff=0.0_rsh

  END SUBROUTINE cas_init_turb

   !!======================================================================

  SUBROUTINE cas_init_tspar
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_tspar  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : rsh,rlg
   USE comvars2d,  ONLY : icon,qmax,kx,ky
   USE comvars3d,  ONLY : saliref,tetaref,rhoref,saliref_lin,tetaref_lin,qmaxz, &
                          l_stateeq_lin
   USE cometemp,   ONLY : phitemp,phisal

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   icon=2
   kx=0.0_rsh
   ky=0.0_rsh
   qmax=120000.0_rsh
   qmaxz=1000.0_rsh
   rhoref=1027.34_rsh
   saliref=35.5_rsh
   tetaref=10.0_rsh
   l_stateeq_lin=.TRUE.
   saliref_lin=saliref
   tetaref_lin=tetaref
   phisal(:,:,:) = 0.0_rlg
   phitemp(:,:,:) = 0.0_rlg

  END SUBROUTINE cas_init_tspar

   !!======================================================================

  SUBROUTINE cas_init_progvar
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_progvar  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : valmanq,rsh,ljmin,ljmax,limin,limax,liminm2,limaxp2,ljminm2,ljmaxp2, &
                          liminm1,ljminm1,kmax
   USE comvars2d,  ONLY : h0
   USE comvarp2d,  ONLY : sal,temp,ssh,u,v
   USE comvarp3d,  ONLY : uz,vz
   USE comvars3d,  ONLY : wz,saliref,tetaref

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations
   INTEGER :: k

   !!----------------------------------------------------------------------
   !! * Executable part

   ssh(:,:)  = MAX(0.0_rsh,-h0(liminm2:limaxp2,ljminm2:ljmaxp2))
   u(:,:)    = 0.0_rsh
   v(:,:)    = 0.0_rsh
   uz(:,:,:) = 0.0_rsh
   vz(:,:,:) = 0.0_rsh
   wz(:,:,:) = 0.0_rsh
   temp(:,:,:) = tetaref
   sal(:,:,:) = saliref

   ! over land
   WHERE( h0(liminm2:limaxp2,ljminm2:ljmaxp2)==-valmanq ) ssh(liminm2:limaxp2,ljminm2:ljmaxp2)=valmanq
   DO k=1,kmax
     WHERE( h0(liminm1:limaxp2,ljminm1:ljmaxp2)==-valmanq ) sal(k,liminm1:limaxp2,ljminm1:ljmaxp2)=valmanq
     WHERE( h0(liminm1:limaxp2,ljminm1:ljmaxp2)==-valmanq ) temp(k,liminm1:limaxp2,ljminm1:ljmaxp2)=valmanq
   END DO

  END SUBROUTINE cas_init_progvar

   !!======================================================================

  SUBROUTINE cas_init_misc
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_misc  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : rsh,kmax,ljmin,ljmax,limin,limax,imin,imax,jmin,jmax,valmanq, &
                          liminm1,limaxp1,ljminm1,ljmaxp1,liminm2,limaxp2,ljminm2,ljmaxp2
   USE comvars2d,  ONLY : fwet,fwetp,fwetmin,dhxy,h0,jb,jh,ig,id,h0,h0_g, &
                          hex,hey,hx,hy,dhx,dhy,goulet,govlet,halfdt
   USE comvars2d,  ONLY : l_obc_cycl_x,l_obc_cycl_y
   USE comvarp2d,  ONLY : ssh
   USE comfluxdo, ONLY : fxeau,fyeau
   USE comvarp3d, ONLY : uz,vz
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex,hzey,hcx_sig,hcy_sig,dcusds
#endif
   USE_MPI toolmpi, ONLY: ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   INTEGER :: i,j,k

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CAS_INIT_MISC routine'

   ! Domain of calculation
   ! ---------------------
   DO i=imin,imax
     jb(i)=jmin+1
     IF ( .NOT. l_obc_cycl_y) THEN
       DO j = jmin+1,jmax-2
         IF (h0_g(i,j) > -valmanq .OR. h0_g(i,j+1) > -valmanq .OR. h0_g(i,j+2) > -valmanq) THEN
           jb(i) = j
           GOTO 1050
         ELSE
           jb(i) = j
         END IF
       END DO
 1050 CONTINUE
     END IF
   END DO

   DO i=imin,imax
     jh(i)=jmax
     IF ( .NOT. l_obc_cycl_y) THEN
       DO j = jmax-2,jmin+1,-1
         IF (h0_g(i,j) > -valmanq .OR. h0_g(i,j+1) > -valmanq .OR. h0_g(i,j+2) > -valmanq) THEN
           jh(i) = j + 2
           GOTO 1100
         ELSE
           jh(i) = j
         END IF
       END DO
 1100 CONTINUE
     END IF
   END DO

   DO j=jmin,jmax
     ig(j)=imin+1
     IF ( .NOT. l_obc_cycl_x) THEN
       DO i = imin+1,imax-2
         IF (h0_g(i,j) > -valmanq .OR. h0_g(i+1,j) > -valmanq .OR. h0_g(i+2,j) > -valmanq) THEN
           ig(j) = i
           GOTO 1200
         ELSE
           ig(j) = i
         END IF
       END DO
 1200 CONTINUE
     END IF
   END DO

   DO j=jmin,jmax
     id(j)=imax
     IF ( .NOT. l_obc_cycl_x) THEN
       DO i = imax-2,imin+1,-1
         IF (h0_g(i,j) > -valmanq .OR. h0_g(i+1,j) > -valmanq .OR. h0_g(i+2,j) > -valmanq) THEN
           id(j) = i + 2
           GOTO 1300
         ELSE
           id(j) = i
         END IF
       END DO
 1300  CONTINUE
     END IF
   END DO


   ! Fraction of wet surface on each cell
   ! ------------------------------------

   DO j=ljmin,ljmax
     DO i=limin,limax
       IF (dhxy(i,j) == 0.0_rsh) THEN
          fwet(i,j)=1.0_rsh
         fwetp(i,j)=1.0_rsh
       ELSE
          fwet(i,j)=MIN(MAX(fwetmin,(ssh(i,j)+h0(i,j))/dhxy(i,j)),1.0_rsh)
         fwetp(i,j)=MIN(MAX(fwetmin,(ssh(i,j)+h0(i,j))/dhxy(i,j)),1.0_rsh)
       ENDIF
     END DO
   END DO

   ! Water depth at T and u locations
   ! --------------------------------

   hex(limin:limax,ljmin:ljmax)=(hx(limin:limax,ljmin:ljmax)+ssh(limin:limax,ljmin:ljmax)+ &
                                dhx(limin:limax,ljmin:ljmax))*goulet(limin:limax,ljmin:ljmax)
   hey(limin:limax,ljmin:ljmax)=(hy(limin:limax,ljmin:ljmax)+ssh(limin:limax,ljmin:ljmax)+ &
                                dhy(limin:limax,ljmin:ljmax))*govlet(limin:limax,ljmin:ljmax)
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljmin,ljmax,hex(liminm2:limaxp2,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-2,2,1,limin,limax,ljminm2,ljmaxp2,hey(limin:limax,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(hex,hey)
#if defined key_siggen || defined key_gencoord
   DO j=ljmin,ljmax
   DO i=limin,limax
   DO k=1,kmax
     hzex(k,i,j)=ssh(i,j)+hcx_sig(i,j)+(hx(i,j)+dhx(i,j)-hcx_sig(i,j))*dcusds(k)
     hzey(k,i,j)=ssh(i,j)+hcy_sig(i,j)+(hy(i,j)+dhy(i,j)-hcy_sig(i,j))*dcusds(k)
   END DO
   END DO
   END DO
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax,ljmin,ljmax,hzex(:,liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,0,kmax,limin,limax,ljminm1,ljmax,hzey(:,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(hzex,hzey)
#endif

#if defined key_siggen || defined key_gencoord
   fxeau(:,liminm1:limax,ljmin:ljmax)=hzex(:,liminm1:limax,ljmin:ljmax)*uz(:,liminm1:limax,ljmin:ljmax)*halfdt
   fyeau(:,limin:limax,ljminm1:ljmax)=hzey(:,limin:limax,ljminm1:ljmax)*vz(:,limin:limax,ljminm1:ljmax)*halfdt
#else
   DO k=1,kmax
     fxeau(k,liminm1:limax,ljmin:ljmax)=hex(liminm1:limax,ljmin:ljmax)*uz(k,liminm1:limax,ljmin:ljmax)*halfdt
     fyeau(k,limin:limax,ljminm1:ljmax)=hey(limin:limax,ljminm1:ljmax)*vz(k,limin:limax,ljminm1:ljmax)*halfdt
   END DO
#endif

   PRINT_DBG*, 'exit CAS_INIT_MISC routine'

  END SUBROUTINE cas_init_misc

   !!======================================================================

  SUBROUTINE cas_init_traj
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_traj  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
#if defined key_trajec3d || defined key_ibm
   USE comtraj,    ONLY : file_trajec,itypetraj,ndtz
#endif

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_trajec3d || defined key_ibm
!   file_trajec='../inputs/traject3d_type1circ.dat'
   file_trajec='../inputs/traject3d_type2rect.dat'
   itypetraj=2
   ndtz  = 50
#endif


  END SUBROUTINE cas_init_traj

   !!======================================================================

  SUBROUTINE cas_init_subs(nvp,nvpc,nv_cp,nv_dis,nv_fix,nv_dri,nv_int,date_start)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_subs  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : imin,imax,jmin,jmax,kmax,limin,limax,ljmin,ljmax,obc_width, &
                          rsh,rlg,liminm1,limaxp2,ljminm1,ljmaxp2
   USE comvars2d, ONLY : hm,iscreenlog,ierrorlog
   USE comvars3d, ONLY : dsigu
#if defined key_substance
   USE parameters, ONLY : nb_var
   USE parameters, ONLY : liminm2,ljminm2,limaxp3,ljmaxp3   ! VALERIE PROVISOIRE
   USE comsubstance, ONLY : nv_adv,nv_state,nv_tot,f90,t90,irk_fil,l_out_subs, &
                            name_var,unit_var,long_name_var,standard_name_var, &
                            valid_min_var,valid_max_var,cv_wat,c2,c3,tdebsubs, &
                            xws,sub_flx_atm,cv_rain,cini_wat,tocd,obc_cv_name, &
                            obc_cv_wat_e,obc_cv_wat_w,obc_cv_wat_n,obc_cv_wat_s, &
                            phicon,alloc_fxfyfz
#if defined key_tssub_adv_fifthorder || defined key_tssub_adv_fourthorder
   USE comsubstance,ONLY : c5
#endif
#ifdef key_phicondrycell
   USE comsubstance, ONLY : phicon_drycell
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc,dcusds
#endif
#endif


   IMPLICIT NONE

   !! * Arguments
   CHARACTER(LEN=19), INTENT(IN) :: date_start
   INTEGER, INTENT(IN) :: nvp,nvpc,nv_cp,nv_dis,nv_fix,nv_dri,nv_int

   !! * Local declarations
   INTEGER :: n,k,isubs,IERR_MPI,iv,iiv,indgrp
   REAL(KIND=rsh)    :: dhmin
   REAL(KIND=rlg)    :: tool_datosec

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_substance

   nv_adv=nb_var
   nv_state=nb_var
   nv_tot=nv_state+nv_dri+nv_int
   ALLOCATE(cv_wat(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   cv_wat(:,:,:,:)=0.0_rsh
   ALLOCATE(f90(nb_var))
   f90(1:nb_var) = 1.0_rsh
   ALLOCATE(t90(nv_state))
   t90(1:nv_state) = 0.0_rsh
   ALLOCATE(irk_fil(nv_tot))
   DO n=1,nv_tot
     irk_fil(n)=n
   END DO
   ALLOCATE(l_out_subs(nv_state))
   ALLOCATE(obc_cv_name(nv_state))
   obc_cv_name(:)='none'
   ALLOCATE(xws(13,nv_adv))
   ALLOCATE(tocd(nvp))

   unit_var(:)='unknown'
   long_name_var(:)=name_var(:)
   standard_name_var(:)=long_name_var(:)
   valid_min_var(:)=-3.4e12
   valid_max_var(:)=3.4e12
   tdebsubs=tool_datosec(date_start)
   l_out_subs(:)=.TRUE.

   ALLOCATE(sub_flx_atm(nv_adv))
   sub_flx_atm(:)=0.0_rsh
   ALLOCATE(cv_rain(nv_adv))
   cv_rain(:)=0.0_rsh
   ALLOCATE(cini_wat(nv_state))
   cini_wat(:)=0.0_rsh
   ALLOCATE(obc_cv_wat_e(nv_adv,kmax,imax-obc_width+1:imax,ljmin:ljmax))
   ALLOCATE(obc_cv_wat_w(nv_adv,kmax,imin:imin+obc_width,ljmin:ljmax))
   ALLOCATE(obc_cv_wat_n(nv_adv,kmax,limin:limax,jmax-obc_width+1:jmax))
   ALLOCATE(obc_cv_wat_s(nv_adv,kmax,limin:limax,jmin:jmin+obc_width))
   obc_cv_wat_e(:,:,:,:)=0.0_rsh
   obc_cv_wat_w(:,:,:,:)=0.0_rsh
   obc_cv_wat_n(:,:,:,:)=0.0_rsh
   obc_cv_wat_s(:,:,:,:)=0.0_rsh
   ALLOCATE(phicon(nv_tot,kmax,limin:limax,ljmin:ljmax))
   phicon(:,:,:,:) = 0.0_rsh
#ifdef key_phicondrycell
   ALLOCATE(phicon_drycell(nv_tot,kmax,limin:limax,ljmin:ljmax))
   phicon_drycell(:,:,:,:) = 0.0_rsh
#endif

   ! Allocate intermediate variable used in ultimate quickest macho scheme and macho3d organisation
   ALLOCATE(c2(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   c2(:,:,:,:)=0.0_rsh
#ifdef key_tssub_macho3d
   ALLOCATE(c3(nv_adv,kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   c3(:,:,:,:)=0.0_rsh
#endif
#if defined key_tssub_adv_fifthorder || defined key_tssub_adv_fourthorder
   ALLOCATE(c5(nv_adv,kmax,liminm2:limaxp3,ljminm2:ljmaxp3))
   c5(:,:,:,:)=0.0_rsh
#endif

   CALL alloc_fxfyfz

#endif

  END SUBROUTINE cas_init_subs

   !!======================================================================

  SUBROUTINE cas_init_subs_part

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init_subs_part  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : imin,imax,jmin,jmax,kmax,limin,limax,ljmin,ljmax,obc_width, &
                          rsh,rlg,liminm1,limaxp2,ljminm1,ljmaxp2
   USE comvars2d, ONLY : hm,iscreenlog,ierrorlog
   USE comvars3d, ONLY : dsigu
#if defined key_substance
   USE parameters, ONLY : nb_var
   USE comsubstance, ONLY : nvpc,nvp,irk_fil,name_var,xws,iv_ws_max,nv_ws_max, &
                            ndt_part,sv_part,ap_ws_gr,nb_ws_max,nvpi
#ifdef key_phicondrycell
   USE comsubstance, ONLY : phicon_drycell
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc,dcusds
#endif
#endif
   USE_MPI toolmpi, ONLY : MASTER


   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations
   INTEGER :: n,k,isubs,IERR_MPI,iv,iiv,indgrp
   REAL(KIND=rsh)    :: dhmin
   REAL(KIND=rlg)    :: tool_datosec

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_substance

   ! variables with settling velocities
   isubs=1
   xws(1,isubs)=.0001_rsh
   xws(2,isubs)=.001_rsh
   xws(3,isubs)=20.0_rsh
   xws(4,isubs)=0.085_rsh
   xws(5,isubs)=0.5_rsh
   xws(6,isubs)=60.0_rsh
   xws(7,isubs)=0.0_rsh
   xws(8,isubs)=0.0_rsh
   xws(9,isubs)=1.0_rsh

   ! Mise en groupe des variables particulaires
   ! ------------------------------------------

   IF (nvp>0) THEN
     ! Allocation des tableaux se rapportant aux variables non dissoutes
     ALLOCATE(iv_ws_max(nvp,nvp))
     ALLOCATE(nv_ws_max(nvp))
     ALLOCATE(ndt_part(nvp))
     ALLOCATE(sv_part(nvp))
     ALLOCATE(ap_ws_gr(nvp))

     ! Initialisation des différentes variables et tableaux
     nb_ws_max=0
     iv_ws_max(:,:)=0
     nv_ws_max(:)=0
     ap_ws_gr(:)=0
     ndt_part(:)=0
     sv_part(:)=0.0_rsh


     ! Equation of substances with settling velocity
     ! is solved with a sub time step

     ! estimate of the minimum layer thickness
     ! ---------------------------------------

     dhmin=hm
#if defined key_siggen || defined key_gencoord
     DO k=1,kmax
       dhmin=MIN(dhmin,(hc+(hm-hc)*dcusds(k))*dsigu(k))
     ENDDO
#else
     DO k=1,kmax
       dhmin=MIN(dhmin,hm*dsigu(k))
     ENDDO
#endif

#ifdef key_subs_part_eqsubdt
     DO iv=1,nvp                      ! loop on particulate variables
       IF (ap_ws_gr(iv)==0) THEN

     ! estimate the number of different settling velocities
         nb_ws_max=nb_ws_max+1
     ! number of variables with the same settling velocity
         nv_ws_max(nb_ws_max)=1
         indgrp=1
     ! index of variable associated with a settling velocity
         iv_ws_max(nb_ws_max,indgrp)=iv
         ap_ws_gr(iv)=nb_ws_max
     ! number of sub time step
!         ndt_part(nb_ws_max)=NINT(dt/MIN((dhmin/xws(2,iv)),dt))
         ndt_part(nb_ws_max)=1
         ndt_part(nb_ws_max)=5
         sv_part(nb_ws_max)=xws(2,iv)
         IF (iv<nvp) THEN
           DO iiv=iv+1,nvp
             IF ((ap_ws_gr(iiv)==0) .AND. (xws(2,iiv)==sv_part(nb_ws_max))) THEN
     ! number of variables with the same settling velocity
               nv_ws_max(nb_ws_max)=nv_ws_max(nb_ws_max)+1
               indgrp=indgrp+1
     ! index of variable associated with a settling velocity
               iv_ws_max(nb_ws_max,indgrp)=iiv
               ap_ws_gr(iiv)=nb_ws_max
             ENDIF
           ENDDO
         ENDIF
       ENDIF
     ENDDO

#else

     nb_ws_max=0
     DO iv=1,nvp
       nb_ws_max=iv     ! number of variables with different settling velocities
                        ! all of them for key_sedim because ws3 is a 4D variable
       nv_ws_max(iv)=1  ! each particulate variable is relative to a unique group
                        ! because ws3 is a 4D variable
       sv_part(iv)=xws(2,iv)   ! important to determinate the number of iterations
     ! index of variable associated with a settling velocity
       iv_ws_max(iv,1)=iv
       IF (xws(2,iv) == 0.0_rsh) THEN
         IF_MPI (MASTER) THEN
         WRITE(ierrorlog,*) ''
         WRITE(ierrorlog,*) ' ******* Error detected from SUB_READ_VAR from SUBREADDAT.F90 ******* '
         WRITE(ierrorlog,*) 'the variable :',name_var(irk_fil(isubs))
         WRITE(ierrorlog,*) 'is a particulate one'
         WRITE(ierrorlog,*) 'but its settling velocity is set to 0.0 in cascylinder.F90'
         WRITE(ierrorlog,*) 'Consequently, the sub time step used to solve substance equation cannot be estimated'
         WRITE(ierrorlog,*) 'The simulation is stopped'
         CALL_MPI MPI_FINALIZE(IERR_MPI)
         STOP
         ENDIF_MPI
       END IF
!       ndt_part(nb_ws_max)=NINT(dt/MIN((dhmin/xws(2,iv)),dt))
       ndt_part(nb_ws_max)=1
       ndt_part(nb_ws_max)=5
     ENDDO

#endif

   ENDIF

   nvpi=nvp
   IF_MPI (MASTER) THEN
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) 'SUBSTANCES WITH A TIME CONSTANT SETTLING VELOCITY '
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*)'NUMBER GROUP OF SUBSTANCES            : ',nb_ws_max
   ENDIF_MPI
   DO iiv=1,nb_ws_max
     IF_MPI (MASTER) THEN
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*)'group number                   : ',iiv
     WRITE(iscreenlog,*)'number of substances           : ',nv_ws_max(iiv)
     WRITE(iscreenlog,*)'group settling velocity        : ',sv_part(iiv)
     WRITE(iscreenlog,*)'group number of iterations     : ',ndt_part(iiv)
     ENDIF_MPI
     DO iv=1,nv_ws_max(iiv)
       IF_MPI (MASTER) THEN
       WRITE(iscreenlog,*) 'substance number        name'
       WRITE(iscreenlog,'(4x,i4,6x,a18,2x)')  &
       iv,TRIM(name_var(irk_fil(iv_ws_max(iiv,iv))))
       ENDIF_MPI
     ENDDO
   ENDDO

#endif

  END SUBROUTINE cas_init_subs_part

   !!======================================================================

  SUBROUTINE cas_init_sedim
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

  END SUBROUTINE cas_init_sedim

   !!======================================================================

  SUBROUTINE cas_init_bio
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

  END SUBROUTINE cas_init_bio

   !!======================================================================

  SUBROUTINE cas_init_agrif
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_init  ***
   !&E
   !&E ** Purpose : initialize a test case
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code : reorganization of the test cases
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules

   IMPLICIT NONE 

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

  END SUBROUTINE cas_init_agrif

   !!======================================================================

  END MODULE casinit
