SUBROUTINE bathy
    !&E---------------------------------------------------------------------
    !&E                 ***  ROUTINE bathy  ***
    !&E
    !&E ** Purpose : read the bathymetry and mean sea level file
    !&E              in the historical DEL/AO format
    !&E
    !&E ** Description :
    !&E
    !&E ** Called by : main
    !&E
    !&E ** External calls : 
    !&E
    !&E ** Used ij-arrays : hnivmoy,zlit
    !&E
    !&E ** Modified variables : hx_g, hy_g, h0_g, dhx, dhy
    !&E
    !&E ** Reference :
    !&E
    !&E ** History :
    !&E       !  2005-10-14
    !&E       !  2009-04  (F. Dumas) generalized sigma
    !&E       !  01-2011 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
    !&E       !  09-2012 (S. Petton) key_siggen hc = MIN(h0_g(:,:),hc)
    !&E
    !&E---------------------------------------------------------------------
    !
#include "toolcpp.h"
    !
    !! * Modules used
    USE parameters
    USE ionc4, ONLY : ionc4_openr,ionc4_read_xy,ionc4_close,&
    ionc4_createfile,ionc4_createvar,ionc4_write_xy
    USE comvars2d
    !    USE comvars3d, ONLY : sig
    !
#if defined key_siggen || defined key_gencoord
    USE comsiggen, ONLY : hc,hc_sig_g,hcx_sig_g,hcy_sig_g
#endif
    USE_MPI toolmpi,   ONLY : MASTER
    USE obccombine,ONLY : l_obc_tide
    !
    IMPLICIT NONE
    !
    !! * Local declarations
    INTEGER,DIMENSION(imin:imax,jmin:jmax)        :: icx,icy
    REAL(kind=rsh)                                :: critr,cff,rtarget,nu,hmin
    !    REAL(kind=rlg)                                :: hmax_coast
    REAL(kind=rsh),DIMENSION(imin:imax,jmin:jmax) :: rx2,ry2,h0m,fxh,fyh,diff
    CHARACTER(lchain)                             :: chain
    INTEGER                                       :: i,j,k,test1,test2,test3, &
    n_filter_deep_topo,n_filter_final
    REAL(kind=rsh),DIMENSION(imin:imax,jmin:jmax) :: hnivmoy
    REAL(kind=riosh),DIMENSION(imin:imax,jmin:jmax) :: zlit
    REAL(kind=riolg),dimension(imin:imax,jmin:jmax) :: tmp2drlg
    REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: hx_t,hy_t
    REAL(kind=rsh),DIMENSION(imin:imax,jmin:jmax) :: coef
    LOGICAL,DIMENSION(imin:imax,jmin:jmax)          :: maskbathy_T, maskbathy_T_reel, maskbathy_U, maskbathy_V
    !
    !!----------------------------------------------------------------------
    !! * Executable part
    !

    !
    ! ouverture du fichier de bathy issu de l interpolation (e.g. bmgtools)
    CALL ionc4_openr(file_bathy)
    !    lecture des coordonnees longitude, latitude aux points T, U, V et F 
    CALL ionc4_read_xy(file_bathy,name_in_longitude,tmp2drlg,imin,imax,jmin,jmax)
    lon2d_g(:,:)=tmp2drlg(:,:)
    CALL ionc4_read_xy(file_bathy,name_in_latitude,tmp2drlg,imin,imax,jmin,jmax)
    lat2d_g(:,:)=tmp2drlg(:,:)
    CALL ionc4_read_xy(file_bathy,name_in_longitude_u,tmp2drlg,imin,imax,jmin,jmax)
    lon2du_g(:,:)=tmp2drlg(:,:)
    CALL ionc4_read_xy(file_bathy,name_in_latitude_u,tmp2drlg,imin,imax,jmin,jmax)
    lat2du_g(:,:)=tmp2drlg(:,:)
    CALL ionc4_read_xy(file_bathy,name_in_longitude_v,tmp2drlg,imin,imax,jmin,jmax)
    lon2dv_g(:,:)=tmp2drlg(:,:)
    CALL ionc4_read_xy(file_bathy,name_in_latitude_v,tmp2drlg,imin,imax,jmin,jmax)
    lat2dv_g(:,:)=tmp2drlg(:,:)
    CALL ionc4_read_xy(file_bathy,name_in_longitude_f,tmp2drlg,imin,imax,jmin,jmax)
    lon2df_g(:,:)=tmp2drlg(:,:)
    CALL ionc4_read_xy(file_bathy,name_in_latitude_f,tmp2drlg,imin,imax,jmin,jmax)
    lat2df_g(:,:)=tmp2drlg(:,:)
    !
#if ! defined key_bathy_traitement
    ! On considere que le niveau moyen a deja ete pris en compte dans l interpolation
    l_bathy_meanlev=.false.
    !    Lecture de H0
    CALL ionc4_read_xy(file_bathy,name_bathy_h0,zlit,imin,imax,jmin,jmax,fillvar=-rg_valmanq_io)
    h0_g(:,:)=zlit(:,:)
    !
    !    Lecture de HX
    CALL ionc4_read_xy(file_bathy,name_bathy_hx,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
    hx_g(:,:)=zlit(:,:)
    where( hx_g(:,:)<=(-rg_valmanq_io-REAL(h0fond,riosh)))
        hx_g(:,:)=-rg_valmanq_io-h0fond
    end where
    !
    !    Lecture de HY
    CALL ionc4_read_xy(file_bathy,name_bathy_hy,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
    hy_g(:,:)=zlit(:,:)
    where( hy_g(:,:)<=(-rg_valmanq_io-REAL(h0fond,riosh)))
        hy_g(:,:)=-rg_valmanq_io-h0fond
    end where
    !
#endif
    !
#ifdef key_bathy_traitement
    !    Lecture de H0
    CALL ionc4_read_xy(file_bathy,name_bathy_h0,zlit,imin,imax,jmin,jmax,fillvar=-rg_valmanq_io)
    h0_g(:,:)=zlit(:,:)
    !
    ! - masque Terre vrai :
    ! Les points qui ont des valeurs manquantes issues de l interpolation
    ! sont masquees
    maskbathy_t_reel(:,:)=.true.
    !
    WHERE (h0_g(:,:) == -rg_valmanq_io) maskbathy_t_reel(:,:)=.false.
    !
    IF (l_bathy_meanlev) THEN
        hnivmoy(:,:)=0.0_rsh
        CALL ionc4_read_xy(file_bathy,name_bathy_meanlev,zlit,imin,imax,jmin,jmax)
        hnivmoy(:,:)=zlit(:,:)
        h0_g(:,:)=h0_g(:,:)+hnivmoy(:,:)-mslshift
    END IF
    !
    !    Masquage de H0
     maskbathy_T(:,:)=.true.
     !
     ! Les points dont la valeur de la bathymetrie est inferieure à la profondeur hminim
     ! sont consideres comme des points a Terre ==> le "trait de cote" peut donc etre
     ! modifie par rapport au vrai trait de cote 
     ! 
     WHERE (h0_g(:,:) < hminim) maskbathy_T(:,:)=.false.
     !
     ! Les valeurs de H0 qui sont < hminim sont mis à la valeur de hminim
     ! afin de pouvoir effectuer les operations de traitement de facon continue a la limite Terre-Mer
     where(h0_g(:,:) < hminim) h0_g(:,:)=hminim
     !---------------------------------------------------------------------------   
     !
     ! Slope parameter (r=grad(h)/h) maximum value for topography smoothing
     !
     rtarget = 0.1
     !
     ! Number of pass of a selective filter to reduce the isolated
     ! seamounts on the deep ocean.
     !
     n_filter_deep_topo=4;
     !
     ! Number of pass of a single hanning filter at the end of the
     ! smooting procedure to ensure that there is no 2DX noise in the 
     ! topography.
     !
     n_filter_final=2;
     !
     test1 =1
     test2 =1
     test3 =1
     !
     !---------------------------------------------------------------------------   
     if (test1==1) then
         !
         if (n_filter_deep_topo>=1) then
             !
             !  Build a smoothing coefficient that is a linear function 
             !  of a smooth topography.
             !
             coef(:,:)=h0_g(:,:)
             do i=1,8
                 call hanning_smoother(coef)            ! coef is a smoothed bathy
             end do
             !
             coef(:,:)=0.125*(coef(:,:)/maxval(coef(:,:)))        ! rescale the smoothed bathy
             !
             DO i=1,n_filter_deep_topo;
                 CALL hanning_smoother_coef2d(coef,h0_g)      ! smooth with avariable coef
                 !    h(maskbathyr==0 & h>hmax_coast)=hmax_coast;
             end do
         end if
     end if
     !
     !---------------------------------------------------------------------------   
     if (test2==1) then
         ! FD -  FVDM (revue en septembre 2014): filtrage de la bathymetrie de Sacha S.
         !
         IF_MPI (MASTER) print*,'rang no = ',no
         IF(no==1) THEN
             !
             icx(:,:)=0
             icy(:,:)=0
             rx2(:,:)=0.0
             ry2(:,:)=0.0
             cff=0.25
             nu=3./16.
             !
             h0m(:,:)=h0_g(:,:)
             DO j=jmin+2,jmax-1
                 DO i=imin+1,imax-1
                     ! faire un test (lequel ?)...
                     !    
                     !    if( (h0m(i,j) /= 0.) .and. (h0m(i+1,j) /= 0.) ) then
                     !            print*,h0m(i,j),h0m(i+1,j),h0m(i,j),h0m(i+1,j)
                     !            print*,abs(h0m(i,j)-h0m(i+1,j)),(h0m(i,j)+h0m(i+1,j))
                     !            print*,abs(h0m(i,j)-h0m(i+1,j))/(h0m(i,j)+h0m(i+1,j))
                     !         
                     rx2(i,j)=abs(h0m(i,j)-h0m(i+1,j))/(h0m(i,j)+h0m(i+1,j))
                     !     endif
                     !    
                     !     IF(rx2(i,j) > rtarget) icx(i,j)=1
                 END DO
             END DO
             !
             DO j=jmin+1,jmax-1
                 DO i=imin+2,imax-1
                     !     if ( (h0m(i,j) /= 0.)  .and. (h0m(i,j+1) /= 0.) ) then
                     ry2(i,j)=abs(h0m(i,j)-h0m(i,j+1))/(h0m(i,j)+h0m(i,j+1))
                     !     endif
                     !     IF(ry2(i,j) > rtarget) icy(i,j)=1
                 END DO
             END DO
             !
             critr=max(maxval(rx2(:,:)),maxval(ry2(:,:)))
             IF_MPI (MASTER) print*,'critr',critr
             h0m=log(h0m(:,:))
             k = 0
             !
             DO while(critr>rtarget)
                 k=k+1
                 DO j=jmin+1,jmax-1
                     DO i=imin,imax-1
                         fxh(i,j)=((h0m(i+1,j)-h0m(i,j))*5./6.+(h0m(i+1,j-1)-h0m(i,j-1)+h0m(i+1,j+1)-h0m(i,j+1))/12.)
                     END DO
                 END DO
                 fxh(:,jmin) = fxh(:,jmin+1)
                 fxh(:,jmax) = fxh(:,jmax-1)
                 !
                 DO j=jmin,jmax-1
                     DO i=imin+1,imax-1
                         fyh(i,j)=((h0m(i,j+1)-h0m(i,j))*5./6.+(h0m(i-1,j+1)-h0m(i-1,j)+h0m(i+1,j+1)-h0m(i+1,j))/12.)
                     END DO
                 END DO
                 fyh(imin,:) = fyh(imin+1,:)
                 fyh(imax,:) = fyh(imax-1,:)
                 !
                 DO j=jmin+1,jmax-1
                     DO i=imin+1,imax-1
                         h0m(i,j)=h0m(i,j)+nu*(fxh(i,j)-fxh(i-1,j)+fyh(i,j)-fyh(i,j-1))
                     END DO
                 END DO
                 !
                 h0m(imin,:)=h0m(imin+1,:)
                 h0m(imax  ,:)=h0m(imax-1,:)
                 h0m(:,jmin)=h0m(:,jmin+1)
                 h0m(:,jmax  )=h0m(:,jmax-1)
                 !
                 h0m(:,:)=exp(h0m(:,:))
                 !
                 rx2(:,:)=0.0
                 ry2(:,:)=0.0
                 icx(:,:)=0
                 icy(:,:)=0
                 DO j=jmin+2,jmax-1
                     DO i=imin+1,imax-1
                         rx2(i,j)=abs(h0m(i,j)-h0m(i+1,j))/(h0m(i,j)+h0m(i+1,j))
                     END DO
                 END DO
                 !
                 DO j=jmin+1,jmax-1
                     DO i=imin+2,imax-1
                         ry2(i,j)=abs(h0m(i,j)-h0m(i,j+1))/(h0m(i,j)+h0m(i,j+1))
                     END DO
                 END DO
                 !
                 critr=max(maxval(rx2(:,:)),maxval(ry2(:,:)))
                 !  print*,maxloc(rx2(:,:)),maxloc(ry2(:,:))
                 !    print*,critr
                 IF_MPI (MASTER) print*,k,' iterations - rtarget = ',critr
                 h0m(:,:)=log(h0m(:,:))        
             END DO  
             !
             diff(:,:)=0
             !         WHERE (h0_g(:,:)>=hmin) diff(:,:)=abs(h0_g(:,:)-exp(h0m(:,:)))
             !  print*,maxval(diff)
             !  print*,maxloc(diff)
             !
             ! Utile si l on veut garder les valeurs sous hmin
             !         WHERE (h0_g(:,:)/=-valmanq)    h0_g(:,:)=exp(h0m(:,:))
             h0_g(:,:)=exp(h0m(:,:))
         END IF
     end if
     !---------------------------------------------------------------------------   
     if (test3==1) then
         !     %
         !     %  Smooth the topography again to prevent 2D noise
         !     %
         if ( n_filter_final > 1 ) then
             !       disp(' Smooth the topography a last time to prevent 2DX noise:')
             !       disp(['   ',num2str(n_filter_final),' pass of a hanning smoother.'])
             do i=1,n_filter_final
                 call hanning_smoother(h0_g)
                 !         h(maskbathyr==0 & h>hmax_coast)=hmax_coast;
             end do
         end if
     end if
     !---------------------------------------------------------------------------
     !  Redefine hx_g and hy_g
     !  Recalcul des Hx,Hy comme moyenne spatiale (demi-somme) de H0
     DO i=imin,imax-1
         DO j=jmin,jmax
             hx_g(i,j)=(h0_g(i,j)+h0_g(i+1,j))/2.0_rsh
         END DO
     END DO
     DO i=imin,imax
         DO j=jmin,jmax-1
             hy_g(i,j)=(h0_g(i,j)+h0_g(i,j+1))/2.0_rsh
         END DO
     END DO
     !      Est ce aberrant de completer en imax et jmax comme ci-dessous ?
     hx_g(imax,:)=hx_g(imax-1,:)
     hy_g(:,jmax)=hy_g(:,jmax-1)
     !
     ! masquage des points Terre
!      WHERE (maskbathy_T(:,:).eqv..false.) h0_g(:,:) = -valmanq
    WHERE (maskbathy_T_REEL(:,:).eqv..false.) h0_g(:,:) = -valmanq
    !
    !
    maskbathy_u(:,:)=maskbathy_t_reel(:,:)
    maskbathy_v(:,:)=maskbathy_t_reel(:,:)
    
    do j=jmin,jmax-1
        do i=imin,imax-1
            if(maskbathy_t_reel(i,j) == .true. .and. maskbathy_t_reel(i+1,j)==.false.) maskbathy_u(i,j)=.false.
            if(maskbathy_t_reel(i,j) == .true. .and. maskbathy_t_reel(i,j+1)==.false.) maskbathy_v(i,j)=.false.
        end do
    end do
    
    WHERE (maskbathy_u(:,:).eqv..false.) hx_g(:,:) = -valmanq-h0fond
    WHERE (maskbathy_v(:,:).eqv..false.) hy_g(:,:) = -valmanq-h0fond
         
    !
    !---------------------------------------------------------------------------
    ! Lignes ci-dessous concernent les bancs decouvrant.
    ! les dhx_g et dhy_ ne sont pas a priori utilises
    !
    hx_t(:,:)=100000.0_rsh
    hy_t(:,:)=100000.0_rsh
    WHERE (hx_g(:,:) /= -valmanq -h0fond) hx_t(:,:)=hx_g(:,:)
    WHERE (hy_g(:,:) /= -valmanq -h0fond) hy_t(:,:)=hy_g(:,:)
    dhxy_g(:,:)=1.0_rsh
    DO j=jmin+1,jmax
        DO i=imin+1,imax
            dhxy_g(i,j)=MAX(hx_g(i,j),hx_g(i-1,j),hy_g(i,j),hy_g(i,j-1))- &
            MIN(hx_t(i,j),hx_t(i-1,j),hy_t(i,j),hy_t(i,j-1))+epsilon
        END DO
    END DO
    !
    dhx_g(:,:)=0.0_rsh
    dhy_g(:,:)=0.0_rsh
    WHERE (hx_g(:,:) < hrdef) dhx_g(:,:)=dhj
    WHERE (hy_g(:,:) < hrdef) dhy_g(:,:)=dhj
!---------------------------------------------------------------------------
#endif
!
#if defined key_siggen || defined key_gencoord
!    A priori lignes ci-dessous pour tester dans obcogcm la compatibilite de la bathy/OBC/Coord. Sigma ???
! Ne sert qu a afficher des warning ....
! en plus ce n est implemente que pour hc_sig_g  ....
   hc_sig_g(:,:)=-valmanq
   hcx_sig_g(:,:)=-valmanq
   hcy_sig_g(:,:)=-valmanq
   IF (l_bathy_meanlev) THEN
      WHERE (h0_g(:,:)/=-valmanq) hc_sig_g(:,:)=MIN(h0_g(:,:),MAX(hnivmoy(:,:),hc))
      WHERE (hx_g(:,:)/=-valmanq-h0fond) hcx_sig_g(:,:)=MIN(hx_g(:,:),MAX(hnivmoy(:,:),hc))
      WHERE (hy_g(:,:)/=-valmanq-h0fond) hcy_sig_g(:,:)=MIN(hy_g(:,:),MAX(hnivmoy(:,:),hc))
   ELSE
      WHERE (h0_g(:,:)/=-valmanq) hc_sig_g(:,:)=MIN(h0_g(:,:),hc)
      WHERE (hx_g(:,:)/=-valmanq-h0fond) hcx_sig_g(:,:)=MIN(hx_g(:,:),hc)
      WHERE (hy_g(:,:)/=-valmanq-h0fond) hcy_sig_g(:,:)=MIN(hy_g(:,:),hc)
   END IF
#endif
!
!---------------------------------------------------------------------------
   ! Save the bathymetry
   ! -------------------

   IF (l_bathy_save) THEN

     IF_MPI (MASTER) THEN
       CALL ionc4_createfile('bathy_verif.nc',lon2d_g,lat2d_g, &
                             imin,imax,1, &
                             jmin,jmax,1, &
                             1,kmax,1,kmax,0,l_out_nc4par=.false.)
       CALL ionc4_createvar('bathy_verif.nc','longitude_u',                            &
                            units='degrees_east',long_name='longitude at u location',   &
                            standard_name='longitude_at_u_location',location_in_cell='u', &
                            valid_min=-180.0_riolg,valid_max=180.0_riolg,              &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lon2du_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','longitude_u',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','latitude_u',                             &
                            units='degrees_north',long_name='latitude at u location',   &
                            standard_name='latitude_at_u_location',location_in_cell='u',  &
                            valid_min=-90.0_riolg,valid_max=90.0_riolg,                &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lat2du_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','latitude_u',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','longitude_v',                            &
                            units='degrees_east',long_name='longitude at v location',   &
                            standard_name='longitude_at_v_location',location_in_cell='v', &
                            valid_min=-180.0_riolg,valid_max=180.0_riolg,              &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lon2dv_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','longitude_v',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','latitude_v',                             &
                            units='degrees_north',long_name='latitude at v location',   &
                            standard_name='latitude_at_v_location',location_in_cell='v',  &
                            valid_min=-90.0_riolg,valid_max=90.0_riolg,                &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lat2dv_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','latitude_v',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','longitude_f',                            &
                            units='degrees_east',long_name='longitude at f location',   &
                            standard_name='longitude_at_f_location',location_in_cell='f', &
                            valid_min=-180.0_riolg,valid_max=180.0_riolg,              &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lon2du_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','longitude_f',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','latitude_f',                             &
                            units='degrees_north',long_name='latitude at f location',   &
                            standard_name='latitude_at_f_location',location_in_cell='f',  &
                            valid_min=-90.0_riolg,valid_max=90.0_riolg,                &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lat2dv_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','latitude_f',tmp2drlg,imin,imax,jmin,jmax)

       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level'
       ELSE
         chain='model_sea_floor_depth_below_geoid'
       END IF
       CALL ionc4_createvar('bathy_verif.nc','H0',units='m',                     &
                            long_name='bathymetry relative to the mean level',   &
                            standard_name = TRIM(chain),                         &
                            fill_value=-rg_valmanq_io,dims='xy')
       zlit(:,:)=h0_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','H0',zlit,imin,imax,jmin,jmax)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_u_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_u_location'
       END IF
       CALL ionc4_createvar('bathy_verif.nc','HX',units='m',location_in_cell='u',              &
                            long_name = 'bathymetry at u-location relative to the mean level', &
                            standard_name = TRIM(chain),                         &
                           fill_value=-rg_valmanq_io-REAL(h0fond,riosh),dims='xy')
       zlit(:,:)=hx_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','HX',zlit,imin,imax,jmin,jmax)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_v_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_v_location'
       END IF
       CALL ionc4_createvar('bathy_verif.nc','HY',units='m',location_in_cell='v',              &
                            long_name = 'bathymetry at v-location relative to the mean level', &
                            standard_name = TRIM(chain),                         &
                           fill_value=-rg_valmanq_io-REAL(h0fond,riosh),dims='xy')
       zlit(:,:)=hy_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','HY',zlit,imin,imax,jmin,jmax)

       CALL ionc4_close('bathy_verif.nc')
     ENDIF_MPI
   END IF



   CALL ionc4_close(file_bathy)
#if defined key_bathy_traitement
IF_MPI (MASTER) print*, "END of bathy pre-processing " !(because key_bathy_final is unused)
!print*, "CODE is stopped"
!STOP
#endif
   PRINT_DBG*, 'exit BATHY routine'

  END SUBROUTINE bathy
  !------------------------------------------------------------------------------------  
  
    SUBROUTINE hanning_smoother(coef)
        USE parameters
        implicit none
        REAL(kind=rsh),DIMENSION(imin:imax,jmin:jmax) :: coef,h
        INTEGER                                       :: i,j

        h(:,:) =coef(:,:)    

!    do i=imin,imax-3
!     do j=jmin,jmax-3  
   do i=imin,imax-2
    do j=jmin,jmax-2
       coef(i+1,j+1)=0.125*(h(i+1,j)+h(i+1,j+2)+h(i,j+1)+h(i+2,j+1)+4*h(i+1,j+1))
    enddo
   enddo

  coef(imin,:)  =coef(imin+1,:)
!       coef(:,jmax-1)=coef(:,jmax-2)
coef(:,jmax)=coef(:,jmax-1)
  coef(:,jmin)  =coef(:,jmin+1)
!       coef(imax-1,:)=coef(imax-2,:)
coef(imax,:)=coef(imax-1,:)
    END SUBROUTINE 
  !------------------------------------------------------------------------------------  
  SUBROUTINE hanning_smoother_coef2D(coef,hh)
       USE parameters
        implicit none
        REAL(kind=rsh),DIMENSION(imin:imax,jmin:jmax) :: coef,h,hh
        INTEGER                                       :: i,j

        h(:,:) =hh(:,:)    

!    do i=imin,imax-3
!     do j=jmin,jmax-3  
        do i=imin,imax-2
            do j=jmin,jmax-2
        hh(i+1,j+1)=coef(i+1,j+1)*(h(i+1,j)+h(i+1,j+2)+h(i,j+1)+h(i+2,j+1))+(1-4*coef(i+1,j+1))*h(i+1,j+1)
    enddo
   enddo 
  hh(imin,:)  =hh(imin+1,:)
!       hh(:,jmax-1)=hh(:,jmax-2)
    hh(:,jmax)=hh(:,jmax-1)
  hh(:,jmin)  =hh(:,jmin+1)
!       hh(imax-1,:)=hh(imax-2,:)
    hh(imax,:)=hh(imax-1,:)
    END SUBROUTINE 
