  MODULE diagdyn

   !!======================================================================
   !!                   ***  MODULE diagdyn  ***
   !!
   !! ** Description :
   !!
   !! ** Reference :
   !!
   !! ** History :  !  2011-04  (P. Lazure)  Original code
   !!
   !!======================================================================

   !! * Modules used
   USE parameters, ONLY : lchain,rsh,rlg,riolg,riosh,kmax,imin,imax,jmin,jmax, &
                          limin,limax,ljmin,ljmax

   IMPLICIT NONE

#ifdef key_MPI_2D
   INCLUDE 'mpif.h'
#endif

   PRIVATE

   !! * Accessibility

   PUBLIC diagdyn_alloc,   &       ! called by alloc.F90
          diagdyn_dealloc, &       ! called by dealloc.F90
          diagdyn_init,   &       ! called by main.F90
          diagdyn_output          ! called by dyn3dxztlssolve.F90, dyn3dyztlssolve.F90

   !! * Shared module variables

   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:),PUBLIC :: diagx_cor,diagx_nlt,diagx_ipg,diagx_hdif,diagx_epg, &
                                                         diagx_corcor,diagx_vdif,diagx_dudt,diagx_vel,diag_boz
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:),PUBLIC :: diagy_cor,diagy_nlt,diagy_ipg,diagy_hdif,diagy_epg, &
                                                         diagy_corcor,diagy_vdif,diagy_dudt,diagy_vel
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:),PUBLIC   :: diag_xe
   LOGICAL,ALLOCATABLE,DIMENSION(:,:),PUBLIC          :: mask_diagx,mask_diagy

   !! * Private variables

   REAL(KIND=rlg)  :: tdebdiagdyn,tfindiagdyn,dtsordiagdyn,tnextout,tcumul,tprecmoy,dtmoy
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: diagx_cor_save,diagx_nlt_save,diagx_ipg_save,diagx_hdif_save,diagx_epg_save,&
                                                  diagx_corcor_save,diagx_vdif_save,diagx_dudt_save,diagx_vel_save,diag_boz_save
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: diagy_cor_save,diagy_nlt_save,diagy_ipg_save,diagy_hdif_save,diagy_epg_save,&
                                                  diagy_corcor_save,diagy_vdif_save,diagy_dudt_save,diagy_vel_save
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:) :: diag_xe_save

   LOGICAL,ALLOCATABLE,DIMENSION(:,:)  :: mask_h0_diag,mask_hx_diag,mask_hy_diag

   LOGICAL     :: l_moy_save
   INTEGER     :: nbmoy
   CHARACTER(LEN=lchain) :: filediag
   INTEGER      :: iextmindiag,iextmaxdiag,stepidiag,jextmindiag,jextmaxdiag,stepjdiag,kextmindiag,kextmaxdiag,stepkdiag
   INTEGER      :: igextmindiag,igextmaxdiag,jgextmindiag,jgextmaxdiag
   LOGICAL      :: l_trend,l_nlt,l_cor,l_epgrad,l_ipg,l_hdif,l_epg,l_corcor,l_vdif,l_dudt,l_vel,l_boz,l_xe

   CHARACTER(len=lchain) :: name_outdiag_h0,name_outdiag_hx,name_outdiag_hy, &
                            name_outdiagx_cor,name_outdiagx_nlt,name_outdiagx_ipg,name_outdiagx_hdif, &
                            name_outdiagx_epg,name_outdiagx_corcor,name_outdiagx_vdif,name_outdiagx_dudt, &
                            name_outdiagx_vel,name_outdiag_xe,name_outdiag_boz

   CHARACTER(len=lchain) :: name_outdiagy_cor,name_outdiagy_nlt,name_outdiagy_ipg,name_outdiagy_hdif, &
                            name_outdiagy_epg,name_outdiagy_corcor,name_outdiagy_vdif,name_outdiagy_dudt,name_outdiagy_vel


  CONTAINS

   !!======================================================================

  SUBROUTINE diagdyn_alloc

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE diagdyn_alloc  ***
   !&E
   !&E ** Purpose :
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-09 (S. Michel, HOCER) correct i dimension of the arrays
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   ALLOCATE(diagx_cor(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_nlt(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_ipg(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_epg(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_corcor(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_vdif(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_hdif(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_dudt(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_vel(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diagy_cor(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_nlt(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_ipg(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_epg(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_corcor(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_vdif(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_hdif(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_dudt(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_vel(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diagx_cor_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_nlt_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_ipg_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_epg_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_corcor_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_vdif_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_hdif_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_dudt_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_vel_save(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diagy_cor_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_nlt_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_ipg_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_epg_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_corcor_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_vdif_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_hdif_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_dudt_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_vel_save(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diag_boz(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_boz_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_xe(limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_xe_save(limin:limax,ljmin:ljmax) )

   ALLOCATE(mask_h0_diag(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_hx_diag(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_hy_diag(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_diagx(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_diagy(limin:limax,ljmin:ljmax) )

   diagx_cor(:,:,:)=0.0_rlg
   diagx_nlt(:,:,:)=0.0_rlg
   diagx_ipg(:,:,:)=0.0_rlg
   diagx_epg(:,:,:)=0.0_rlg
   diagx_corcor(:,:,:)=0.0_rlg
   diagx_vdif(:,:,:)=0.0_rlg
   diagx_hdif(:,:,:)=0.0_rlg
   diagx_dudt(:,:,:)=0.0_rlg
   diagx_vel(:,:,:)=0.0_rlg

   diagy_cor(:,:,:)=0.0_rlg
   diagy_nlt(:,:,:)=0.0_rlg
   diagy_ipg(:,:,:)=0.0_rlg
   diagy_epg(:,:,:)=0.0_rlg
   diagy_corcor(:,:,:)=0.0_rlg
   diagy_vdif(:,:,:)=0.0_rlg
   diagy_hdif(:,:,:)=0.0_rlg
   diagy_dudt(:,:,:)=0.0_rlg
   diagy_vel(:,:,:)=0.0_rlg

  END SUBROUTINE diagdyn_alloc

   !!======================================================================

  SUBROUTINE diagdyn_dealloc

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE diagdyn_dealloc  ***
   !&E
   !&E ** Purpose :
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   DEALLOCATE(diagx_cor,diagx_nlt,diagx_ipg,diagx_hdif,diagx_epg, &
              diagx_corcor,diagx_vdif,diagx_dudt,diagx_vel)

   DEALLOCATE(diagy_cor,diagy_nlt,diagy_ipg,diagy_hdif,diagy_epg, &
              diagy_corcor,diagy_vdif,diagy_dudt,diagy_vel)

   DEALLOCATE(diagx_cor_save,diagx_nlt_save,diagx_ipg_save,diagx_hdif_save,diagx_epg_save, &
              diagx_corcor_save,diagx_vdif_save,diagx_dudt_save,diagx_vel_save)

   DEALLOCATE(diagy_cor_save,diagy_nlt_save,diagy_ipg_save,diagy_hdif_save,diagy_epg_save, &
              diagy_corcor_save,diagy_vdif_save,diagy_dudt_save,diagy_vel_save)

   DEALLOCATE(diag_xe,diag_boz,diag_xe_save,diag_boz_save)

   DEALLOCATE(mask_h0_diag,mask_hx_diag,mask_hy_diag,mask_diagx,mask_diagy)

  END SUBROUTINE diagdyn_dealloc

   !!======================================================================

  SUBROUTINE diagdyn_init

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE diagdyn_init  ***
   !&E
   !&E ** Purpose : Initialize diagnostics of dynamical terms
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :  !  2011-04  (P. Lazure)  Original code
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE comvars2d,   ONLY : lon2d_g,lat2d_g,lon2d,lat2d,ierrorlog
   USE parameters,  ONLY : rsh,rlg,riosh,kmax,imin,imax,limin,limax,ljmin,ljmax,  &
                           rg_valmanq_io,ljminp1,riolg,valmanq,lchain
#if defined key_xios
   USE parameters,  ONLY : MPI_COMM_VAR
#endif
   USE ionc4,       ONLY : ionc4_createfile,ionc4_gatt,ionc4_createvar,           &
                           ionc4_write,ionc4_write_sig,ionc4_write_xy,            &
                           ionc4_write_time,ionc4_vatt_chain
   USE output,      ONLY : l_out_nc4par,l_out_pack,l_out_zaxis_w
   USE comvars2d,   ONLY : hx,hy,h0,h0fond,riog_valid_min_h0,riog_valid_max_h0,   &
                           name_out_xe,name_out_h0
   USE comvars2d,   ONLY : ig,id,dx_u,dy_v,jb,jh,htx,hty,tdeb3d
   USE comvars3d,   ONLY : sig
   USE_MPI toolmpi, ONLY : ADD_TAG,IPROC,NPROCS,ADD_ALL_MPI_INT

#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : b_sig,theta_sig,hc_sig,Csu_sig,Csw_sig
#endif
   USE obccombine,  ONLY : l_obc_tide

   !! * Local declarations
   CHARACTER(LEN=lchain) :: chain
   REAL(kind=riosh),DIMENSION(limin:limax,ljmin:ljmax) :: tecrit
   REAL(KIND=riosh),DIMENSION(0:kmax+1)            :: sigecrit
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)      :: h0_local,hx_local,hy_local
   REAL(KIND=riosh),DIMENSION(kmax,limin:limax,ljmin:ljmax) :: tzecrit
   REAL(KIND=riolg),DIMENSION(imin:imax)                    :: lon
   REAL(KIND=riosh)                            :: riog_valid_min_diag,riog_valid_max_diag
   REAL(kind=rlg)        :: tool_datosec
   INTEGER :: i,j,IERR_MPI
   INTEGER :: grp_world,grp_zoom,comm_zoom,ncpu,ip
   INTEGER, DIMENSION(:), ALLOCATABLE  :: numrank,listrank
   REAL(KIND=riosh)                                              :: vecrit

   !!----------------------------------------------------------------------
   !! * Executable part

   filediag=TRIM('diag.nc')

   tdebdiagdyn=tool_datosec('01/01/1900 00:00:00')
   tdebdiagdyn=MAX(tdebdiagdyn, tdeb3d)
   tfindiagdyn=tool_datosec('01/01/2870 00:00:00')
   dtsordiagdyn=3.*3600.0_rlg
   l_moy_save=.TRUE.


   l_cor=.TRUE.
   l_ipg=.TRUE.
   l_nlt=.TRUE.
   l_hdif=.TRUE.
   l_epg=.TRUE.
   l_corcor=.TRUE.
   l_vdif=.TRUE.
   l_dudt=.TRUE.
   l_vel=.TRUE.
   l_xe=.TRUE.
   l_boz=.TRUE.

   igextmindiag=imin
   igextmaxdiag=imax
   iextmindiag=imin
   iextmaxdiag=imax
   stepidiag=3
   jextmindiag=jmin
   jextmaxdiag=jmax
   jgextmindiag=jmin
   jgextmaxdiag=jmax
   stepjdiag=3
#if defined key_MPI_2D
   ! If we do not use the parallel writing, the spatial extension
   ! must be bounded by limin and limax
   IF (.NOT.l_out_nc4par) THEN
     DO WHILE ( iextmindiag < limin )
       iextmindiag=iextmindiag+stepidiag
     END DO
     iextmaxdiag=MIN(limax,iextmaxdiag)
     DO WHILE ( jextmindiag < ljmin )
       jextmindiag=jextmindiag+stepjdiag
     END DO
     jextmaxdiag=MIN(ljmax,jextmaxdiag)
   END IF
#endif
   kextmindiag=1  ! should not be changed
   kextmaxdiag=kmax ! id
   stepkdiag=1 ! id

   riog_valid_min_diag=-10000.0_riosh
   riog_valid_max_diag=10000.0_riosh

   name_outdiag_h0='h0'
   name_outdiag_hx='hx'
   name_outdiag_hy='hy'


   name_outdiagx_cor='cor_x'
   name_outdiagx_ipg='ipg_x'
   name_outdiagx_nlt='nlt_x'
   name_outdiagx_hdif='hdif_x'
   name_outdiagx_epg='epg_x'
   name_outdiagx_corcor='cor2_x'
   name_outdiagx_vdif='vdif_x'
   name_outdiagx_dudt='dudt_x'
   name_outdiagx_vel='u'

   name_outdiagy_cor='cor_y'
   name_outdiagy_ipg='ipg_y'
   name_outdiagy_nlt='nlt_y'
   name_outdiagy_hdif='hdif_y'
   name_outdiagy_epg='epg_y'
   name_outdiagy_corcor='cor2_y'
   name_outdiagy_vdif='vdif_y'
   name_outdiagy_dudt='dudt_y'
   name_outdiagy_vel='v'

   name_outdiag_xe='xe'
   name_outdiag_boz='boz'


   tnextout=tdebdiagdyn+dtsordiagdyn


! Verifications
! -------------

   IF ( jextmindiag > jextmaxdiag .OR. iextmindiag > iextmaxdiag ) THEN
     WRITE(ierrorlog,*) ' '
     WRITE(ierrorlog,*) ' Routine DIAGDYN_INIT in DIAGDYN.F90'
     WRITE(ierrorlog,*) ' '
     WRITE(ierrorlog,*) 'Check your domain : because'
     WRITE(ierrorlog,*) ' jextmindiag > jextmaxdiag '
     WRITE(ierrorlog,*) '           or'
     WRITE(ierrorlog,*) ' iextmindiag > iextmaxdiag '
     WRITE(ierrorlog,*) ' '
     WRITE(ierrorlog,*) ' The simulation is stopped '
     WRITE(ierrorlog,*) ' '
     CALL_MPI MPI_FINALIZE(IERR_MPI)
     STOP
   END IF


! Masks
! -----

   mask_diagx(:,:)=.FALSE.
   DO j=jextmindiag,jextmaxdiag,stepjdiag
     DO i=iextmindiag,iextmaxdiag,stepidiag
       IF ( i >= limin .AND. i <= limax .AND. j >= ljmin .AND. j <= ljmax) THEN
         mask_diagx(i,j)=.TRUE.
       END IF
     END DO
   END DO

   mask_diagy(:,:)=.FALSE.
   DO j=jextmindiag,jextmaxdiag,stepjdiag
     DO i=iextmindiag,iextmaxdiag,stepidiag
       IF ( i >= limin .AND. i <= limax .AND. j >= ljmin .AND. j <= ljmax) THEN
         mask_diagy(i,j)=.TRUE.
       END IF
     END DO
   END DO



   h0_local(limin:limax,ljmin:ljmax)=h0(limin:limax,ljmin:ljmax)
   hx_local(limin:limax,ljmin:ljmax)=hx(limin:limax,ljmin:ljmax)
   hy_local(limin:limax,ljmin:ljmax)=hy(limin:limax,ljmin:ljmax)

   mask_h0_diag(:,:)=.TRUE.
   DO j=ljmin,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       IF ( h0_local(i,j)> h0fond ) mask_h0_diag(i,j)=.FALSE.
     END DO
   END DO

! pascal
! mouais pas clair c t affaire
   mask_hx_diag(:,:)=.TRUE.
   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)-1),MIN(limax,id(j))
       IF ( hx_local(i,j) /= -valmanq -h0fond ) mask_hx_diag(i,j)=.FALSE.
     END DO
   END DO
   mask_hx_diag(:,:)=.FALSE.

   mask_hy_diag(:,:)=.TRUE.
   DO j=ljmin,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       IF ( (j >= jb(i)-1 .AND. j <= jh(i)) .AND. (hy_local(i,j) /= -valmanq -h0fond) ) mask_hy_diag(i,j)=.FALSE.
     END DO
   END DO


   comm_zoom=0
#ifdef key_MPI_2D
     comm_zoom=MPI_COMM_VAR
     ! what a code ! Not a clever code probably but not clear in my head at the moment
     ! Count the number of processors active over the area to be saved
     ncpu=0
     IF ( jextmindiag <= jextmaxdiag .AND. iextmindiag <= iextmaxdiag ) ncpu=ncpu+1
     CALL ADD_ALL_MPI_INT(ncpu)
     ! Save the number of active processors
     ALLOCATE (numrank(ncpu))
     ALLOCATE (listrank(nprocs))
     numrank(:)=0   ! must be initialized if the proc 0 is active on the saved zoom
     listrank(:)=0
     ! which cpu is active ?
     IF ( jextmindiag <= jextmaxdiag .AND. iextmindiag <= iextmaxdiag ) listrank(1)=iproc
     ! all cpus get the number of active cpus plus sleeping cpus (set to 0)
     CALL MPI_ALLGATHER(listrank(1),1,MPI_INTEGER,listrank,1,MPI_INTEGER,MPI_COMM_VAR,ierr_mpi)
     ! all cpus keep the active cpus only
     IF (ncpu/=nprocs) THEN
       ip=0
       DO i=1,nprocs
         IF_MPI ( listrank(i) /= 0 ) THEN
         ip=ip+1
         numrank(ip)=listrank(i)
         ENDIF
       END DO
       ! Group relative to the MPI_COMM_VAR
       CALL MPI_COMM_GROUP(MPI_COMM_VAR,grp_world,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_GROUP_INCL(grp_world,ncpu,numrank,grp_zoom,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_COMM_CREATE(MPI_COMM_VAR,grp_zoom,comm_zoom,ierr_mpi)
     ENDIF
#endif


   ! to work with acive cpus over a small area
   IF ( jextmindiag <= jextmaxdiag .AND. iextmindiag <= iextmaxdiag ) THEN


   sigecrit(:)=sig(:)
   IF ( l_out_nc4par ) THEN
     ! Create a global file because of the parallel writing
     CALL ionc4_createfile(filediag,lon2d_g,lat2d_g,igextmindiag,igextmaxdiag,stepidiag, &
                           jgextmindiag,jgextmaxdiag,stepjdiag, &
                           kextmindiag,kextmaxdiag,stepkdiag,   &
                           kmax,0,l_out_nc4par = .TRUE.,comm_active=comm_zoom, &
                           zaxis_var=sigecrit(1:kmax))
   ELSE
     ! Create a local file bounded by limin,limax,ljmin,ljmax
     CALL_MPI ADD_TAG(filediag,LEN(filediag))
     CALL ionc4_createfile(filediag,lon2d,lat2d,iextmindiag,iextmaxdiag,stepidiag, &
                           jextmindiag,jextmaxdiag,stepjdiag, &
                           kextmindiag,kextmaxdiag,stepkdiag, &
                           kmax,0,zaxis_var=sigecrit(1:kmax))
   END IF

   ! save attribute of the spatial extension of the domain
   ! -----------------------------------------------------
   CALL ionc4_gatt(filediag,'global_imin',iextmindiag)
   CALL ionc4_gatt(filediag,'global_imax',iextmaxdiag)
   CALL ionc4_gatt(filediag,'global_jmin',jextmindiag)
   CALL ionc4_gatt(filediag,'global_jmax',jextmaxdiag)

   ! Add the vertical axis
   CALL out_zaxis_w(filediag,kextmindiag,kextmaxdiag,stepkdiag,comm_zoom)

   CALL ionc4_createvar(filediag,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                        standard_name= 'ocean_s_variable',&
#else
                        standard_name= 'ocean_sigma_variable',&
#endif
                        dims='z',valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io)
   CALL ionc4_write_sig(filediag,'SIG',sigecrit,kmax)

#if defined key_siggen || defined key_gencoord

   sigecrit(:)=Csu_sig(:)
   CALL ionc4_createvar(filediag,'Csu_sig',' ','C(s) law at mid layer',dims='z', &
                        standard_name='ocean_s_coordinate_function_at_midlayer', &
                        valid_min=-1.0_riosh,valid_max=10000.0_riosh,fill_value= rg_valmanq_io)
   CALL ionc4_write_sig(filediag,'Csu_sig',sigecrit,kmax)
   CALL ionc4_vatt_chain(filediag,'Csu_sig','formula_definition', &
          'C(k) = (1-b)*sinh(a*s(k))/sinh(a) + b*[tanh(a*(s(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')

   sigecrit(0:kmax)=Csw_sig(0:kmax)
   sigecrit(kmax+1)=0.0_riosh
   CALL ionc4_createvar(filediag,'Csw_sig',' ','C(s) law at interface',dims='z', &
                        standard_name='ocean_s_coordinate_function_at_interface', &
                        valid_min= -1.0_riosh,valid_max= 10000.0_riosh,fill_value= rg_valmanq_io)
   CALL ionc4_write_sig(filediag,'Csw_sig',sigecrit,kmax)
   CALL ionc4_vatt_chain(filediag,'Csw_sig','formula_definition', &
          'C_w(k) = (1-b)*sinh(a*s_w(k))/sinh(a) + b*[tanh(a*(s_w(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')

   CALL ionc4_createvar(filediag,'hc','m', 'depth over which the resolution is increased', &
                        standard_name='ocean_s_coordinate_thick_of_surface_resolution',dims='xy', &
                        valid_min= 0.0_riosh,valid_max= 1000.0_riosh,fill_value= rg_valmanq_io,&
                        l_out_nc4par=l_out_nc4par,l_pack=l_out_pack )
   tecrit(limin:limax,ljmin:ljmax)=hc_sig(limin:limax,ljmin:ljmax)
   CALL ionc4_write_xy(filediag,'hc',tecrit,limin,limax,ljmin,ljmax,fill_value=rg_valmanq_io,l_out_nc4par=l_out_nc4par)

   CALL ionc4_createvar(filediag,'b',' ','bottom control parameter',  &
                        standard_name='ocean_s_coordinate_bottom_control', &
                        valid_min=0.0_riosh,valid_max=1.0_riosh,fill_value=rg_valmanq_io)
   vecrit=b_sig
   CALL ionc4_write(filediag,'b',vecrit)

   CALL ionc4_createvar(filediag,'theta',' ','surface control parameter', &
                        standard_name ='ocean_s_coordinate_surface_control', &
                        valid_min=0.0_riosh,valid_max=20.0_riosh,fill_value=rg_valmanq_io)
   vecrit=theta_sig
   CALL ionc4_write(filediag,'theta',vecrit)

#endif

   ! Add attributes to follow OCO netCDF convention
   CALL out_netcdf_conv(filediag,1,comm_zoom,igextmindiag,igextmaxdiag,jgextmindiag,jgextmaxdiag)

   IF_MPI (l_out_nc4par .AND. comm_zoom/=MPI_COMM_VAR) CALL MPI_COMM_FREE(comm_zoom,ierr_mpi)


! Save the bathymetry
! -------------------

! h0
! --

#if ! defined key_agrif
   ! avec agrif on ne peut compiler car The type of the actual argument differs from the type of the dummy argument.   [ARRAY0]
   !  &,  Agrif_tabvars(606) % var% array0 ,  Agrif_tabvars(605) &
   ! Pas de souci si on utilise riog_valid_min_diag a la place de riog_valid_min_h0 !!!!!!!!!!
   ! Cela m exaspere, et c est un pb recurant, voir mail du 3/3/2010 a Romain Ramel (PREVI_OMPMPI)
   ! Laurent peux-tu faire qq chose STP ou donner une explication rationnelle !!!!

   IF (l_obc_tide) THEN
     chain='model_sea_floor_depth_below_sea_level'
   ELSE
     chain='model_sea_floor_depth_below_geoid'
   END IF
   CALL ionc4_createvar(filediag,name_outdiag_h0,units='m',&
                        long_name='bathymetry relative to the mean level',&
                        standard_name = TRIM(chain),&
                        fill_value = -rg_valmanq_io,&
                        valid_min = riog_valid_min_h0,&
                        valid_max = riog_valid_max_h0,&
                        dims = 'xy',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)
   tecrit(limin:limax,ljmin:ljmax)=h0_local(limin:limax,ljmin:ljmax)
   CALL ionc4_write_xy(filediag,TRIM(name_outdiag_h0),tecrit,limin,limax,ljmin,ljmax,&
                       fill_value=-rg_valmanq_io,l_out_nc4par = l_out_nc4par )


! hx
! --
   IF (l_obc_tide) THEN
     chain='model_sea_floor_depth_below_sea_level_at_u_location'
   ELSE
     chain='model_sea_floor_depth_below_geoid_at_u_location'
   END IF
   CALL ionc4_createvar(filediag,name_outdiag_hx,units='m',&
                        long_name = 'bathymetry at u-location relative to the mean level',&
                        standard_name = TRIM(chain),&
                        fill_value = -REAL(rg_valmanq_io+h0fond,riosh),&
                        valid_min = riog_valid_min_h0,&
                        valid_max = riog_valid_max_h0,&
                        dims = 'xy',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)
   tecrit(:,:)=hx_local(:,:)
   CALL ionc4_write_xy(filediag,TRIM(name_outdiag_hx),tecrit,limin,limax,ljmin,ljmax ,&
                       fill_value = -REAL(rg_valmanq_io+h0fond,riosh),l_out_nc4par = l_out_nc4par)

! hy
! --

   IF (l_obc_tide) THEN
     chain='model_sea_floor_depth_below_sea_level_at_v_location'
   ELSE
     chain='model_sea_floor_depth_below_geoid_at_v_location'
   END IF
   CALL ionc4_createvar(filediag,name_outdiag_hy,units='m',&
                        long_name = 'bathymetry at v-location relative to the mean level',&
                        standard_name = TRIM(chain),&
                        fill_value = -REAL(rg_valmanq_io+h0fond,riosh),&
                        valid_min = riog_valid_min_h0,&
                        valid_max = riog_valid_max_h0,&
                        dims = 'xy',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)
   tecrit(:,:)=hy_local(:,:)
   CALL ionc4_write_xy(filediag,TRIM(name_outdiag_hy),tecrit,limin,limax,ljmin,ljmax,&
                       fill_value = -REAL(rg_valmanq_io+h0fond,riosh),l_out_nc4par = l_out_nc4par)

#endif

   IF (l_cor) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_cor,units= 'm s-2',&
                          long_name = 'Coriolis term at u location',&
                          standard_name = 'coriolis_term_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_cor,units= 'm s-2',&
                          long_name = 'Coriolis term at v location',&
                          standard_name = 'coriolis_term_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)
   END IF

   IF (l_ipg) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_ipg,units= 'm s-2',&
                          long_name = 'internal pressure gradient at u location',&
                          standard_name = 'internal_pressure_gradient_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_ipg,units= 'm s-2',&
                          long_name = 'internal pressure gradient at v location',&
                          standard_name = 'internal_pressure_gradient_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_nlt) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_nlt,units= 'm s-2',&
                          long_name = 'non linear terms at u location',&
                          standard_name = 'non_linear_terms_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_nlt,units= 'm s-2',&
                          long_name = 'non linear terms at v location',&
                          standard_name = 'non_linear_terms_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_hdif) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_hdif,units= 'm s-2',&
                          long_name = 'horizontal diffusion at u location',&
                          standard_name = 'horizontal_diffusion_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_hdif,units= 'm s-2',&
                          long_name = 'horizontal diffusion at v location',&
                          standard_name = 'horizontal_diffusion_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_epg) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_epg,units= 'm s-2',&
                          long_name = 'external pressure gradient at u location',&
                          standard_name = 'external_pressure_gradient_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_epg,units= 'm s-2',&
                          long_name = 'external pressure gradient at v location',&
                          standard_name = 'external_pressure_gradient_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_corcor) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_corcor,units= 'm s-2',&
                          long_name = 'corrected Coriolis term at u location',&
                          standard_name = 'corrected coriolis_term_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_corcor,units= 'm s-2',&
                          long_name = 'corrected Coriolis term at v location',&
                          standard_name = 'corrected coriolis_term_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_vdif) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_vdif,units= 'm s-2',&
                          long_name = 'vertical diffusion at u location',&
                          standard_name = 'vertical_diffusion_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_vdif,units= 'm s-2',&
                          long_name = 'vertical diffusion at v location',&
                          standard_name = 'vertical_diffusion_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_dudt) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_dudt,units= 'm s-2',&
                          long_name = 'time trend at u location',&
                          standard_name = 'time_trend_u_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_dudt,units= 'm s-2',&
                          long_name = 'time trend at v location',&
                          standard_name = 'time_trend_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_vel) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_vel,units= 'm s-1',&
                          long_name = '3d zonal velocity',&
                          standard_name = 'sea_water_x_velocity_at_u_location',&
                          fill_value = 0.0_riosh,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_vel,units= 'm s-1',&
                          long_name = '3d meridional velocity',&
                          standard_name = 'sea_water_y_velocity_at_y_location',&
                          fill_value = 0.0_riosh,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_xe) &
   CALL ionc4_createvar(filediag,name_outdiag_xe,units= 'm',&
                        long_name = 'sea surface height',&
                        standard_name = 'sea surface height',&
                        fill_value = rg_valmanq_io,&
                        valid_min = riog_valid_min_diag,&
                        valid_max = riog_valid_max_diag,&
                        dims= 'xyt',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)

   IF (l_boz) &
   CALL ionc4_createvar(filediag,name_outdiag_boz,units= 'm s-2',&
                        long_name = 'Buoyancy of sea water at atmospheric pressure',&
                        standard_name = 'sea_water_potential_buoyancy',&
                        fill_value = rg_valmanq_io,&
                        valid_min = riog_valid_min_diag,&
                        valid_max = riog_valid_max_diag,&
                        dims= 'zxyt',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)

   END IF  ! ends test on iextmin...

  END SUBROUTINE diagdyn_init

   !!======================================================================

  SUBROUTINE diagdyn_output

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE outputdiagdyn  ***
   !&E
   !&E ** Purpose : Save diagnostics of dynamical terms
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :  !  2011-04  (P. Lazure)  Original code
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters,  ONLY : rsh,rlg,riosh,kmax,imin,imax,limin,limax,ljmin,ljmax,  &
                           rg_valmanq_io,ljminp1,riolg,valmanq,lchain
   USE outtools,    ONLY : out_write_xytvar,out_write_zxytvar
   USE ionc4,       ONLY : ionc4_write_time,ionc4_sync
   USE output,      ONLY : l_out_nc4par
   USE comvars2d,   ONLY : hx,hy,h0,h0fond,riog_valid_min_h0,riog_valid_max_h0
   USE comvars2d,   ONLY : ig,id,dx_u,dy_v,jb,jh,htx,hty
   USE comvars2d,   ONLY : t,h0fond
   USE comvars3d,   ONLY : sig

   !! * Local declarations
   REAL(kind=riosh),DIMENSION(imin:imax,jmin:jmax) :: tecrit
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)      :: h0_local,hx_local,hy_local
   REAL(KIND=riosh),DIMENSION(kmax,limin:limax,ljmin:ljmax) :: tzecrit
   REAL(KIND=riolg),DIMENSION(imin:imax)                    :: lon
   REAL(KIND=riolg)                            :: riog_valid_min_diag,riog_valid_max_diag
   INTEGER :: i,j
   CHARACTER (len=19) :: tool_sectodat

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter OUTPUT_DIAG_DYN routine'

   IF ( t < tdebdiagdyn .OR. t > tfindiagdyn )  RETURN

!print*, ' prochaine ecriture:',tool_sectodat(tnextout)

   IF (l_moy_save) THEN
     nbmoy=nbmoy+1

     IF (nbmoy ==1) THEN
       tprecmoy=t
       tcumul=0
     ELSE
       dtmoy=t-tprecmoy
       tcumul=tcumul+dtmoy

       diagx_cor_save=diagx_cor_save+diagx_cor*dtmoy
       diagx_nlt_save=diagx_nlt_save+diagx_nlt*dtmoy
       diagx_ipg_save=diagx_ipg_save+diagx_ipg*dtmoy
       diagx_hdif_save=diagx_hdif_save+diagx_hdif*dtmoy
       diagx_epg_save=diagx_epg_save+diagx_epg*dtmoy
       diagx_corcor_save=diagx_corcor_save+diagx_corcor*dtmoy
       diagx_vdif_save=diagx_vdif_save+diagx_vdif*dtmoy
       diagx_dudt_save=diagx_dudt_save+diagx_dudt*dtmoy
       diagx_vel_save=diagx_vel_save+diagx_vel*dtmoy


       diagy_cor_save=diagy_cor_save+diagy_cor*dtmoy
       diagy_nlt_save=diagy_nlt_save+diagy_nlt*dtmoy
       diagy_ipg_save=diagy_ipg_save+diagy_ipg*dtmoy
       diagy_hdif_save=diagy_hdif_save+diagy_hdif*dtmoy
       diagy_epg_save=diagy_epg_save+diagy_epg*dtmoy
       diagy_corcor_save=diagy_corcor_save+diagy_corcor*dtmoy
       diagy_vdif_save=diagy_vdif_save+diagy_vdif*dtmoy
       diagy_dudt_save=diagy_dudt_save+diagy_dudt*dtmoy
       diagy_vel_save=diagy_vel_save+diagy_vel*dtmoy

       diag_boz_save=diag_boz_save+diag_boz*dtmoy
       diag_xe_save=diag_xe_save+diag_xe*dtmoy
     END IF

   ELSE

     nbmoy=1
     tcumul=1._rlg
     diagx_cor_save=diagx_cor
     diagx_nlt_save=diagx_nlt
     diagx_ipg_save=diagx_ipg
     diagx_hdif_save=diagx_hdif
     diagx_epg_save=diagx_epg
     diagx_corcor_save=diagx_corcor
     diagx_vdif_save=diagx_vdif
     diagx_dudt_save=diagx_dudt
     diagx_vel_save=diagx_vel

     diagy_cor_save=diagy_cor
     diagy_nlt_save=diagy_nlt
     diagy_ipg_save=diagy_ipg
     diagy_hdif_save=diagy_hdif
     diagy_epg_save=diagy_epg
     diagy_corcor_save=diagy_corcor
     diagy_vdif_save=diagy_vdif
     diagy_dudt_save=diagy_dudt
     diagy_vel_save=diagy_vel

     diag_boz_save=diag_boz
     diag_xe_save=diag_xe
   END IF


   IF ( t >= tnextout) THEN

     print*,' on va ecrire'
     print*,' date:',tool_sectodat(t)
     print*, 'date sortie prevue',tool_sectodat(tnextout)
     print*, 'longueur moyenne (en h)',tcumul/3600.

     tnextout=tnextout+dtsordiagdyn

     diagx_cor_save=diagx_cor_save/tcumul
     diagx_nlt_save=diagx_nlt_save/tcumul
     diagx_ipg_save=diagx_ipg_save/tcumul
     diagx_hdif_save=diagx_hdif_save/tcumul
     diagx_epg_save=diagx_epg_save/tcumul
     diagx_corcor_save=diagx_corcor_save/tcumul
     diagx_vdif_save=diagx_vdif_save/tcumul
     diagx_dudt_save=diagx_dudt_save/tcumul
     diagx_vel_save=diagx_vel_save/tcumul

     diagy_cor_save=diagy_cor_save/tcumul
     diagy_nlt_save=diagy_nlt_save/tcumul
     diagy_ipg_save=diagy_ipg_save/tcumul
     diagy_hdif_save=diagy_hdif_save/tcumul
     diagy_epg_save=diagy_epg_save/tcumul
     diagy_corcor_save=diagy_corcor_save/tcumul
     diagy_vdif_save=diagy_vdif_save/tcumul
     diagy_dudt_save=diagy_dudt_save/tcumul
     diagy_vel_save=diagy_vel_save/tcumul

     diag_boz_save=diag_boz_save/tcumul
     diag_xe_save=diag_xe_save/tcumul

     CALL ionc4_write_time(filediag,0,t)

     IF (l_cor) CALL out_write_zxytvar(filediag,name_outdiagx_cor,rg_valmanq_io,mask_hx_diag, &
                                       diagx_cor_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_cor) CALL out_write_zxytvar(filediag,name_outdiagy_cor,rg_valmanq_io,mask_hx_diag, &
                                       diagy_cor_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_ipg) CALL out_write_zxytvar(filediag,name_outdiagx_ipg,rg_valmanq_io,mask_hx_diag, &
                                       diagx_ipg_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_ipg) CALL out_write_zxytvar(filediag,name_outdiagy_ipg,rg_valmanq_io,mask_hx_diag, &
                                       diagy_ipg_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_nlt) CALL out_write_zxytvar(filediag,name_outdiagx_nlt,rg_valmanq_io,mask_hx_diag, &
                                       diagx_nlt_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_nlt) CALL out_write_zxytvar(filediag,name_outdiagy_nlt,rg_valmanq_io,mask_hx_diag, &
                                       diagy_nlt_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_hdif) CALL out_write_zxytvar(filediag,name_outdiagx_hdif,rg_valmanq_io,mask_hx_diag, &
                                        diagx_hdif_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_hdif) CALL out_write_zxytvar(filediag,name_outdiagy_hdif,rg_valmanq_io,mask_hx_diag, &
                                        diagy_hdif_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_epg) CALL out_write_zxytvar(filediag,name_outdiagx_epg,rg_valmanq_io,mask_hx_diag, &
                                       diagx_epg_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_epg) CALL out_write_zxytvar(filediag,name_outdiagy_epg,rg_valmanq_io,mask_hx_diag, &
                                       diagy_epg_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_corcor) CALL out_write_zxytvar(filediag,name_outdiagx_corcor,rg_valmanq_io,mask_hx_diag, &
                                          diagx_corcor_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_corcor) CALL out_write_zxytvar(filediag,name_outdiagy_corcor,rg_valmanq_io,mask_hx_diag, &
                                          diagy_corcor_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_vdif) CALL out_write_zxytvar(filediag,name_outdiagx_vdif,rg_valmanq_io,mask_hx_diag, &
                                        diagx_vdif_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_vdif) CALL out_write_zxytvar(filediag,name_outdiagy_vdif,rg_valmanq_io,mask_hx_diag, &
                                        diagy_vdif_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_dudt) CALL out_write_zxytvar(filediag,name_outdiagx_dudt,rg_valmanq_io,mask_hx_diag, &
                                        diagx_dudt_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_dudt) CALL out_write_zxytvar(filediag,name_outdiagy_dudt,rg_valmanq_io,mask_hx_diag, &
                                        diagy_dudt_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_vel) CALL out_write_zxytvar(filediag,name_outdiagx_vel,0.0_riosh,mask_hx_diag, &
                                       diagx_vel_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     IF (l_vel) CALL out_write_zxytvar(filediag,name_outdiagy_vel,0.0_riosh,mask_hx_diag, &
                                       diagy_vel_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_xe) CALL out_write_xytvar(filediag,name_outdiag_xe,rg_valmanq_io,mask_hx_diag, &
                                     diag_xe_save(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_boz) CALL out_write_zxytvar(filediag,name_outdiag_boz,rg_valmanq_io,mask_hx_diag, &
                                       diag_boz_save(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     ! To write the data on the disk and not loose data in case of run crash
     CALL ionc4_sync(filediag)

     !print*, ' avant reinitialisation'

     nbmoy=0
     diagx_cor_save=0.0_rsh
     diagx_ipg_save=0.0_rsh
     diagx_nlt_save=0.0_rsh
     diagx_hdif_save=0.0_rsh
     diagx_epg_save=0.0_rsh
     diagx_corcor_save=0.0_rsh
     diagx_vdif_save=0.0_rsh
     diagx_dudt_save=0.0_rsh
     diagx_vel_save=0.0_rsh

     diagy_cor_save=0.0_rsh
     diagy_ipg_save=0.0_rsh
     diagy_nlt_save=0.0_rsh
     diagy_hdif_save=0.0_rsh
     diagy_epg_save=0.0_rsh
     diagy_corcor_save=0.0_rsh
     diagy_vdif_save=0.0_rsh
     diagy_dudt_save=0.0_rsh
     diagy_vel_save=0.0_rsh

     diag_boz_save=0.0_rsh
     diag_xe_save=0.0_rsh

   END IF

   PRINT_DBG*, 'exit OUTPUT_DIAG_DYN routine'

  END SUBROUTINE diagdyn_output

   !!======================================================================

  END MODULE diagdyn
