   !!======================================================================

  SUBROUTINE grid_metric

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE grid_metric  ***
   !&E
   !&E ** Purpose : estimate of grid parametrization
   !&E
   !&E ** Description : estimate of surface cell, begining and ending cells,
   !&E                  sigma repartition,...
   !&E
   !&E ** Called by : main.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !         (J.-C. Salomon, P. Lazure)  Original code
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  01-2011 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE ionc4,     ONLY : ionc4_open,ionc4_createvar,ionc4_write_xy,ionc4_close,ionc4_openr,ionc4_read_xy
   USE comvars2d, ONLY : dx,dy,dfiac,goulet,govlet,cosg,sing,cos2g,sin2g
   USE comvars2d, ONLY : dx_u,dx_v,dx_f,dy_v,dy_f,dy_u,l_corio_cst,surf,ig,id,h0,radearth
   USE comvars2d, ONLY : sinfiu,sin2fi,coscafi,fv,tfivsr,l_modele2d,iwarnlog
   USE comvars2d, ONLY : cosfiu,tfiusr,fu,fisac,dgac,gwac,lonmin,latmin,lonmax,latmax,h0fond
   USE comvars2d, ONLY : ierrorlog,l_bathy_save,lon2du,lon2dv,lat2du,lat2dv,lon2d,lat2d,file_bathy
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   USE comvars2d, ONLY : jbound_omp
#endif
   USE comvars2d, ONLY : lon2d_g,lat2d_g,lon2du_g,lat2du_g,lon2dv_g,lat2dv_g,lon2df_g,lat2df_g
   USE comvars3d, ONLY : l_equisig,sig,dsigu,dsigw,sigw,rho2,boz2
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc_sig,hcx_sig,hcy_sig,Csu_sig,Csw_sig,dcusds,dcwsds,b_sig,theta_sig
#if ! defined key_siggen_g2
   USE comsiggen, ONLY : hzdcusds,hzdcwsds,hzdcusds_x,hzdcwsds_x,hzdcusds_y,hzdcwsds_y
   USE comvars2d, ONLY : hx,hy
#endif
#else
   USE comvars2d, ONLY : hx,hy
#endif
   USE_MPI toolmpi,   ONLY : MASTER

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   INTEGER        :: i,j,k,ixl,iyl,nb,kk
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   INTEGER        :: nb_wetpoint,OMP_GET_NUM_THREADS,nb_point,nb_thread
#endif
   REAL(KIND=rlg) :: rhox,rhoy
   REAL(KIND=rsh) :: fi,g,radeg
#ifdef key_grid_verif
   INTEGER        :: IERR_MPI
   REAL(kind=rsh),dimension(limin:limax,ljmin:ljmax) :: tmp2drsh
#endif

!MANGA
   INTEGER,parameter :: nb_niv=43
   REAL(KIND=rsh),dimension(nb_niv) :: sal_profil,temp_profil,depth,densite
   REAL(KIND=rsh) :: zr,c,cc,p,s,tp,row,rost,pcc
   REAL(kind=rsh) :: crit,zjp1kp12,zjp1km12,zjkp12,zjkm12
   REAL(kind=rsh) ::      zip1kp12,zip1km12,zikp12,zikm12

   !! * spetton
   REAL(kind=riosh),DIMENSION(imin:imax,jmin:jmax) :: zlit
! END MANGA

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter GRIDMETRIC routine'

!$OMP PARALLEL DEFAULT(none) PRIVATE(i,j,k,radeg,g,fi,        &
!$OMP& lonmin,latmin,lonmax,latmax)                           &
!$OMP& SHARED(dx,dy,govlet,goulet,cosg,sing,cos2g,sin2g       &
!$OMP& ,l_corio_cst,fu,tfiusr,cosfiu,sinfiu,sin2fi,coscafi    &
!$OMP& ,dx_u,dx_v,dx_f,dy_v,dy_f,dy_u                         &
!$OMP& ,lon2du,lon2dv,lat2du,lat2dv,lon2d,lat2d               &
!$OMP& ,lon2d_g,lat2d_g,lon2du_g,lat2du_g,lon2dv_g            &
!$OMP& ,lat2dv_g,lon2df_g,lat2df_g,zlit                            &
#if defined key_MPI_2D
!$OMP& ,MASTER                                                &
#endif
!$OMP& ,fv,tfivsr,surf,ig,id,sig,dsigu,l_modele2d,iwarnlog    &
!$OMP& ,l_equisig, h0,dsigw,sigw,dfiac,fisac,dgac,gwac,file_bathy        &
#if defined key_siggen || defined key_gencoord
!$OMP& ,hc_sig,Csu_sig,Csw_sig,dcusds,dcwsds,b_sig,theta_sig  &
!$OMP& ,hcx_sig,hcy_sig,hx,hy,hzdcusds,hzdcwsds               &
!$OMP& ,hzdcusds_x,hzdcwsds_x,hzdcusds_y,hzdcwsds_y           &
#endif
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
!$OMP& ,nb_thread                                             &
#endif
!$OMP& ,limin,limax,ljmin,ljmax,limaxp1,ljmaxp1,liminp1       &
!$OMP& ,liminm2,limaxp2,ljminm2,ljmaxp2                       &
!$OMP& ,ljminp1,liminm1,ljminm1,limaxm1,ljmaxm1,ierrorlog)

   radeg = pi/180.0_rsh

  ! ATTENTION quelle declaration des variables pour Agrif et OMP ????
#ifdef key_agrif
   IF (.NOT.agrif_root()) THEN
     rhox=Agrif_Rhox()
     rhoy=Agrif_Rhoy()

     ixl=Agrif_ix()
     iyl=Agrif_iy()

     dfiac = (Agrif_Parent(dfiac))/rhoy
     fisac = Agrif_Parent(fisac)+(iyl-0.5_rlg-0.5_rlg/rhoy) &
             *Agrif_Parent(dfiac)
     dgac = ( Agrif_Parent(dgac) )/rhox
     gwac = Agrif_Parent(gwac)+(ixl-0.5_rlg-0.5_rlg/rhox) &
             *Agrif_Parent(dgac)
   END IF
#endif

   ! Initially, we know the location of points hx and hy at each cell i,j :
   ! The longitudes and latitudes are read from file_bathy
   !
   !       ---- V(i,j) --- phi(i,j) 
   !       ^                  |                        <-- dx_v(i,j)--->
   !       |                  |                       |
   !       |                  |                       |
   !      dy(    XE(i,j)     U(i,j) and hx(i,j)       |<----dx(i,j)---->
   !       |                  |                       |
   !       |                  |                       |
   !       <---dx_v(i,j-1)--->|                       |-----------------
   !
   !
   !       |<-- dx_v(i,j)--->|            |--------|--------|
   !       |                 |            |        |        |
   !       |                 |            |        |        |
   !       |<----dx(i,j)---->|            |     dy(i,j)   dy_u(i,j)
   !       |                 |            |        |        |
   !       |                 |            |        |        |
   !       |-----------------|            |--------|--------| 
   !
   ! so we know dx and dy at the center of the cell and we estimate :
   ! dx_u and dy_u at location u or hx
   ! dx_v and dy_v at location v or hy

!-----------------------------------------------------------
! coriolis parameter, estimates of dx,dy at the different locations, etc...
!-----------------------------------------------------------

! FVDM 26/03/14: pas utile pour WE10
! MODIF MANGA
!S.P. lecture facteur d echelle pour geometrie des fleuves (correction de la section pour conserver le debit)
!$OMP single
! IF (no==1) THEN
!  CALL ionc4_openr(file_bathy)
!  CALL ionc4_read_xy(file_bathy,"goulet",zlit,imin,imax,jmin,jmax)
!  goulet(limin:limax,ljmin:ljmax)=zlit(limin:limax,ljmin:ljmax)
!  CALL ionc4_read_xy(file_bathy,"govlet",zlit,imin,imax,jmin,jmax)
!  govlet(limin:limax,ljmin:ljmax)=zlit(limin:limax,ljmin:ljmax)
!  CALL ionc4_close(file_bathy)
!END IF 
!$OMP end single
!END MANGA


!$OMP DO SCHEDULE(STATIC)
   DO j=ljmin,ljmax
     DO i = limin,limax

!  IF (no==0) THEN
       goulet(i,j)=1.0_rsh
       govlet(i,j)=1.0_rsh
!  END IF
       ! T location
       g = REAL(lon2d(i,j),rsh)
       fi = REAL(lat2d(i,j),rsh)

       cosg(i,j) = COS(g*radeg)
       sing(i,j) = SIN(g*radeg)
       cos2g(i,j) = COS(2.0_rsh*g*radeg)
       sin2g(i,j) = SIN(2.0_rsh*g*radeg)

       IF (l_corio_cst) fi=fisac
       cosfiu(i,j) = COS(fi*radeg)
       sinfiu(i,j) = SIN(fi*radeg)
       sin2fi(i,j) = SIN(2.0_rsh*fi*radeg)
       coscafi(i,j) = cosfiu(i,j)*cosfiu(i,j)

       IF (j==jmin) THEN
         dy(i,j)=radearth*REAL(lat2dv_g(i,j+1)-lat2dv_g(i,j),rsh)*radeg
       ELSE
         dy(i,j)=radearth*REAL(lat2dv_g(i,j)-lat2dv_g(i,j-1),rsh)*radeg
       END IF
       IF (i==imin) THEN
         dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i+1,j)-lon2du_g(i,j),rsh)*radeg
       ELSE
         dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i,j)-lon2du_g(i-1,j),rsh)*radeg
       END IF

       ! U location
       fi = REAL(lat2du(i,j),rsh)
       IF (l_corio_cst) fi=fisac
       fu(i,j)=0.0001458_rsh*SIN(fi*radeg)
       tfiusr(i,j)=TAN(fi*radeg)/radearth
       IF (l_corio_cst) tfiusr(i,j)=0.0_rsh

       IF (j==jmin) THEN
         dy_u(i,j)=radearth*REAL(lat2df_g(i,j+1)-lat2df_g(i,j),rsh)*radeg
       ELSE
         dy_u(i,j)=radearth*REAL(lat2df_g(i,j)-lat2df_g(i,j-1),rsh)*radeg
       END IF
       IF (i==imax) THEN
         dx_u(i,j)=radearth*COS(fi*radeg)*REAL(lon2d_g(i,j)-lon2d_g(i-1,j),rsh)*radeg
       ELSE
         dx_u(i,j)=radearth*COS(fi*radeg)*REAL(lon2d_g(i+1,j)-lon2d_g(i,j),rsh)*radeg
       END IF

       ! V location
       fi = REAL(lat2dv(i,j),rsh)
       IF (l_corio_cst) fi=fisac
       fv(i,j)=0.0001458_rsh*SIN(fi*radeg)
       tfivsr(i,j)=TAN(fi*radeg)/radearth
       IF (l_corio_cst) tfivsr(i,j)=0.0_rsh

       IF (j==jmax) THEN
         dy_v(i,j)=radearth*REAL(lat2d_g(i,j)-lat2d_g(i,j-1),rsh)*radeg
       ELSE
         dy_v(i,j)=radearth*REAL(lat2d_g(i,j+1)-lat2d_g(i,j),rsh)*radeg
       END IF
       IF (i==imin) THEN
         dx_v(i,j)=radearth*COS(fi*radeg)*REAL(lon2df_g(i+1,j)-lon2df_g(i,j),rsh)*radeg
       ELSE
         dx_v(i,j)=radearth*COS(fi*radeg)*REAL(lon2df_g(i,j)-lon2df_g(i-1,j),rsh)*radeg
       END IF

       ! F location
       fi = REAL(lat2df_g(i,j),rsh)
       IF (l_corio_cst) fi=fisac

       IF (j==jmax) THEN
         dy_f(i,j)=radearth*REAL(lat2du_g(i,j)-lat2du_g(i,j-1),rsh)*radeg
       ELSE
         dy_f(i,j)=radearth*REAL(lat2du_g(i,j+1)-lat2du_g(i,j),rsh)*radeg
       END IF
       IF (i==imax) THEN
         dx_f(i,j)=radearth*COS(fi*radeg)*REAL(lon2dv_g(i,j)-lon2dv_g(i-1,j),rsh)*radeg
       ELSE
         dx_f(i,j)=radearth*COS(fi*radeg)*REAL(lon2dv_g(i+1,j)-lon2dv_g(i,j),rsh)*radeg
       END IF

     END DO
   END DO
!$OMP END DO

!$OMP SINGLE
   i=liminm1
   DO j=ljmin,ljmax
     IF (j==jmin) THEN
       dy_u(i,j)=radearth*REAL(lat2df_g(i,j+1)-lat2df_g(i,j),rsh)*radeg
       dy(i,j)=radearth*REAL(lat2dv_g(i,j+1)-lat2dv_g(i,j),rsh)*radeg
     ELSE
       dy_u(i,j)=radearth*REAL(lat2df_g(i,j)-lat2df_g(i,j-1),rsh)*radeg
       dy(i,j)=radearth*REAL(lat2dv_g(i,j)-lat2dv_g(i,j-1),rsh)*radeg
     END IF
     fi = REAL(lat2d_g(i,j),rsh)
     IF (i==imin) THEN
       dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i+1,j)-lon2du_g(i,j),rsh)*radeg
     ELSE
       dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i,j)-lon2du_g(i-1,j),rsh)*radeg
     END IF
     fi = REAL(lat2du_g(i,j),rsh)
     dx_u(i,j)=radearth*COS(fi*radeg)*REAL(lon2d_g(i+1,j)-lon2d_g(i,j),rsh)*radeg
     fi = REAL(lat2df_g(i,j),rsh)
     dx_f(i,j)=radearth*COS(fi*radeg)*REAL(lon2dv_g(i+1,j)-lon2dv_g(i,j),rsh)*radeg
   END DO

   i=limaxp1
   DO j=ljmin,ljmax
     fi = REAL(lat2d_g(i,j),rsh)
     dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i,j)-lon2du_g(i-1,j),rsh)*radeg
     fi = REAL(lat2dv_g(i,j),rsh)
     dx_v(i,j)=radearth*COS(fi*radeg)*REAL(lon2df_g(i,j)-lon2df_g(i-1,j),rsh)*radeg
     IF (j==jmin) THEN
       dy(i,j)=radearth*REAL(lat2dv_g(i,j+1)-lat2dv_g(i,j),rsh)*radeg
     ELSE
       dy(i,j)=radearth*REAL(lat2dv_g(i,j)-lat2dv_g(i,j-1),rsh)*radeg
     END IF
   END DO

   j=ljminm1
   govlet(limin:limax,j)=1.0_rsh
   DO i=limin,limaxp1
     fi = REAL(lat2dv_g(i,j),rsh)
     IF (i==imin) THEN
       dx_v(i,j)=radearth*COS(fi*radeg)*REAL(lon2df_g(i+1,j)-lon2df_g(i,j),rsh)*radeg
     ELSE
       dx_v(i,j)=radearth*COS(fi*radeg)*REAL(lon2df_g(i,j)-lon2df_g(i-1,j),rsh)*radeg
     END IF
   END DO
   DO i=liminm1,limaxp1
     fi = REAL(lat2d_g(i,j),rsh)
     IF (i==imin) THEN
       dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i+1,j)-lon2du_g(i,j),rsh)*radeg
     ELSE
       dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i,j)-lon2du_g(i-1,j),rsh)*radeg
     END IF
   END DO
   IF (j==jmin) THEN
     dy(liminm1:limaxp1,j)=radearth*REAL(lat2dv_g(liminm1:limaxp1,j+1)-lat2dv_g(liminm1:limaxp1,j),rsh)*radeg
   ELSE
     dy(liminm1:limaxp1,j)=radearth*REAL(lat2dv_g(liminm1:limaxp1,j)-lat2dv_g(liminm1:limaxp1,j-1),rsh)*radeg
   END IF
   dy_v(limin:limax,j)=radearth*REAL(lat2d_g(limin:limax,j+1)-lat2d_g(limin:limax,j),rsh)*radeg
   dy_f(limin:limax,j)=radearth*REAL(lat2du_g(limin:limax,j+1)-lat2du_g(limin:limax,j),rsh)*radeg

   j=ljmaxp1
   DO i=liminm1,limaxp1
     fi = REAL(lat2d_g(i,j),rsh)
     IF (i==imin) THEN
       dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i+1,j)-lon2du_g(i,j),rsh)*radeg
     ELSE
       dx(i,j)=radearth*COS(fi*radeg)*REAL(lon2du_g(i,j)-lon2du_g(i-1,j),rsh)*radeg
     END IF
   END DO
   dy(liminm1:limaxp1,j)=radearth*REAL(lat2dv_g(liminm1:limaxp1,j)-lat2dv_g(liminm1:limaxp1,j-1),rsh)*radeg
   dy_u(liminm1:limax,j)=radearth*REAL(lat2df_g(liminm1:limax,j)-lat2df_g(liminm1:limax,j-1),rsh)*radeg
!$OMP END SINGLE

#ifdef key_nocoriolis
!$OMP SINGLE
   fu(:,:)=0.0_rsh
   fv(:,:)=0.0_rsh
!$OMP END SINGLE
#endif

   !-------------------------------------------------------
   ! boundaries for meteorological forcings :
   !--------------------------------------------------------
   latmax=maxval(maxval(lat2dv,dim=2))
   latmin=minval(minval(lat2d,dim=2))
   lonmin=minval(minval(lon2d,dim=1))
   lonmax=maxval(maxval(lon2du,dim=1))

   IF_MPI (MASTER) THEN
   IF ( ABS(latmax) < epsilon .OR. &
        ABS(latmin) < epsilon .OR. &
        ABS(lonmax) < epsilon .OR. &
        ABS(lonmin) < epsilon ) THEN
     WRITE(iwarnlog,*) " "
     WRITE(iwarnlog,*) "ERROR: routine BATHY.F90 ?"
     WRITE(iwarnlog,*) " "
     WRITE(iwarnlog,*) 'lonmin=',lonmin
     WRITE(iwarnlog,*) 'lonmax=',lonmax
     WRITE(iwarnlog,*) 'latmax=',latmax
     WRITE(iwarnlog,*) 'latmin=',latmin
     WRITE(iwarnlog,*) "Make sure the max/min latitude and longitude are correct"
     WRITE(iwarnlog,*) " "
   END IF
   ENDIF_MPI

!$OMP DO SCHEDULE(STATIC) 
   DO j = ljminp1,ljmax
     DO i = limin,limax
       IF (l_corio_cst) THEN
         surf(i,j)=dy(i,j)*govlet(i,j)*dx(i,j)*goulet(i,j)
       ELSE
         surf(i,j)=goulet(i,j)*dy(i,j)*(govlet(i,j)*dx_v(i,j)+govlet(i,j-1)*dx_v(i,j-1))*0.5_rsh &
             *SQRT(1.0_rsh-(0.5_rsh*(govlet(i,j)*dx_v(i,j)-govlet(i,j-1)*dx_v(i,j-1))/dy(i,j))**2)
       END IF
     END DO
   END DO
!$OMP END DO
!$OMP SINGLE
   IF_MPI (ljmin==jmin) THEN
     surf(limin:limax,jmin)=surf(limin:limax,jmin+1)
     surf(limin:limax,jmin)=surf(limin:limax,jmin+1)
   ENDIF_MPI
!$OMP END SINGLE

   ! sigma repartition
   !------------------

!$OMP single
#if defined key_siggen || defined key_gencoord
  ! pour l instant on calcule les coefs de contraction avec sig regulier mais a terme on pourra choisir regulier ou non
   l_equisig=.TRUE.
#endif
   IF (l_equisig) THEN
     DO k=1,kmax
       sig(k)=(REAL(k-kmax,rsh)-0.5_rsh)/REAL(kmax,rsh)
     END DO
   END IF

   sig(kmax+1)=-sig(kmax)
   sig(0)=ABS(sig(1))-2.0_rsh

   DO k=1,kmax
     IF (.NOT.(sig(k) > sig(k-1) .AND. sig(k) < sig(k+1))) THEN
       WRITE(ierrorlog,*) ' '
       WRITE(ierrorlog,*) 'ERROR: paraspec.txt input file'
       WRITE(ierrorlog,*) 'ERROR: test in grid_metric.F90 routine'
       WRITE(ierrorlog,*) 'Sigma level must be defined with increasing number.'
       WRITE(ierrorlog,*) 'There are not in paraspec.txt namelist.'
       STOP
     END IF
   END DO

   DO k=1,kmax
     !cval  dsigu(k)=0.5*(sig(k+1)-sig(k-1))
     dsigw(k)=sig(k+1)-sig(k)
     sigw(k)=0.5_rsh*(sig(k)+sig(k+1))
   END DO
   sigw(kmax+1)=-sigw(kmax)
   sigw(0)=-1.0_rsh
   IF (l_modele2d) THEN
     dsigu(1)=1.0_rsh
     dsigu(2:kmax)=0.0_rsh
   END IF

   IF (.NOT.l_modele2d) THEN
     DO k=1,kmax
       dsigu(k)=sigw(k)-sigw(k-1)
     END DO
   END IF

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
   DO k=0,kmax+1
     IF (theta_sig > 0.0_rsh) THEN
       Csu_sig(k)=(1.0_rsh-COSH(theta_sig*sig(k)))/(COSH(theta_sig)-1.0_rsh)
     ELSE
       Csu_sig(k)=-sig(k)*sig(k)
     END IF
     IF (b_sig > 0.0_rsh) &
       Csu_sig(k)=(EXP(b_sig*Csu_sig(k))-1.0_rsh)/(1.0_rsh-EXP(-b_sig))
   END DO
   DO k=0,kmax
     IF (theta_sig > 0.0_rsh) THEN
       Csw_sig(k)=(1.0_rsh-COSH(theta_sig*sigw(k)))/(COSH(theta_sig)-1.0_rsh)
     ELSE
       Csw_sig(k)=-sigw(k)*sigw(k)
     END IF
     IF (b_sig > 0.0_rsh) &
       Csw_sig(k)=(EXP(b_sig*Csw_sig(k))-1.0_rsh)/(1.0_rsh-EXP(-b_sig))
   END DO
#else
   DO k=0,kmax+1
!   Csu_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*(sig(k)-1.0_rsh))/SINH(theta_sig) &
!         + 0.5_rsh*b_sig*(TANH(theta_sig*(sig(k)-0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
     Csu_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sig(k))/SINH(theta_sig) &
         + 0.5_rsh*b_sig*(TANH(theta_sig*(sig(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO
   DO k=0,kmax
!   Csw_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*(sigw(k)-1.0_rsh))/SINH(theta_sig) &
!         + 0.5_rsh*b_sig*(TANH(theta_sig*(sigw(k)-0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
     Csw_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sigw(k))/SINH(theta_sig) &
         + 0.5_rsh*b_sig*(TANH(theta_sig*(sigw(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO
#endif

   DO k=1,kmax
     dcusds(k)=(Csw_sig(k)-Csw_sig(k-1))/(sigw(k)-sigw(k-1))
   END DO

   DO k=1,kmax-1
     dcwsds(k)=(Csu_sig(k+1)-Csu_sig(k))/(sig(k+1)-sig(k))
   END DO
   dcwsds(kmax)=(0.0_rsh-csu_sig(kmax))/(0.0_rsh-sig(kmax))
   !IF_MPI (MASTER) THEN
   !DO k=1,kmax
   !  PRINT*,k,sig(k),sigw(k),csw_sig(k)+1,csu_sig(k)+1,dcusds(k)
   !END DO
   !PRINT*,'sum(dcusds(1:kmax)*dsigu(1:kmax)) must be equal to 1'
   !PRINT*,SUM(dcusds(1:kmax)*dsigu(1:kmax))
   !ENDIF_MPI

#if ! defined key_siggen_g2
   DO j=ljmin,ljmaxp1
     DO i=limin,limaxp1
! S.P. 2013 correction boggue
       DO k=1,kmax-1
         hzdcwsds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcwsds(k)
         hzdcusds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcusds(k)
         hzdcwsds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcwsds(k)
         hzdcusds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcusds(k)
         hzdcwsds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcwsds(k)
         hzdcusds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcusds(k)
       END DO
       k=kmax
       hzdcusds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcusds(k)
       hzdcusds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcusds(k)
       hzdcusds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcusds(k)
     END DO
   END DO
#endif

#endif

!$OMP END single

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
!$ nb_thread=OMP_GET_NUM_THREADS()
#endif

!$OMP END PARALLEL

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
  ! estimate limits for switching thread with OMP : jbound_omp
  ! with balancing the number of wet cells of each thread

! initialization of locales variables + jbound_omp (public)
!$ ALLOCATE( jbound_omp(0:nb_thread) )
!$ jbound_omp(0)=0
!$ j=jmin
!$ nb=0
!$ nb_point=0
! number of wet cells by thread
!$ nb_wetpoint=INT( SUM(id(:)-ig(:)+1)/nb_thread ) + 1

! set jbound_omp(num thread) to limit of switching
!$ DO WHILE(j<=jmax)
!$   IF ( j==jmax .OR. nb_point>=nb_wetpoint ) THEN
!$     nb=nb+1
!$     jbound_omp(nb)=j
!$     nb_point=0
!$   ELSE IF ( nb_point<nb_wetpoint ) THEN
!$     nb_point=nb_point+id(j)-ig(j)+1
!$   END IF
!$   j=j+1
!$ END DO
!$ PRINT*,jbound_omp,'jbound_omp'
#endif

! MANGA
  OPen(622,file="../inputs/profil_sud.txt",form="formatted")
   do k=1,nb_niv
    read(622,*),depth(k),sal_profil(k),temp_profil(k)
    p=10.d0
    tp=temp_profil(k)
    S = sal_profil(k)
    row = 999.842594d0 + (  6.793952d-02 + ( - 9.095290d-03 + &
          (1.001685d-04 - 1.120083d-06*TP + 6.536332d-09*TP*TP)*TP )*TP  )*TP
    rost = row + S * (   0.824493d0 + (  -4.0899d-03 + ( 7.6438d-05 +          &
                     (-8.2467d-07+5.3875d-09*TP)*TP )*TP  )*TP   ) +           &
           S**1.5d0 * ( -5.72466d-03 + (1.0227d-04-1.6546d-06*TP)*TP ) +       &
           S*S*4.8314d-04
    c = 1449.2d0 + 1.34d0*(S-35.0d0) + 4.55d0*TP - 0.045d0*TP*TP +             &
          0.00821d0*p + 15.0d-09*p*p
    pcc = p/(c*c)
    densite(k) = rost + 1.0d4*(1.0d0-0.2d0*pcc)*pcc
   end do
  close(622)


#if defined key_siggen || defined key_gencoord
  DO j=ljminp2,ljmaxm1
    DO i=liminp2,limaxm1
     if(hx(i,j) /= -valmanq -h0fond) then
       do k=1,kmax
         zikp12   = hc_sig(i  ,j)*sigw(k  )+(h0(i  ,j)-hc_sig(i  ,j))*Csw_sig(k  )
         zikm12   = hc_sig(i  ,j)*sigw(k-1)+(h0(i  ,j)-hc_sig(i  ,j))*Csw_sig(k-1)
         zip1kp12 = hc_sig(i+1,j)*sigw(k  )+(h0(i+1,j)-hc_sig(i+1,j))*Csw_sig(k  )
         zip1km12 = hc_sig(i+1,j)*sigw(k-1)+(h0(i+1,j)-hc_sig(i+1,j))*Csw_sig(k-1)
         if(zip1kp12-zip1km12+zikp12-zikm12/=0.0 ) crit=abs((zip1kp12+zip1km12-(zikp12+zikm12))/(zip1kp12-zip1km12+zikp12-zikm12)) 
       end do
     end if
     if(hy(i,j) /= -valmanq -h0fond) then
       do k=1,kmax
         zjkp12   = hc_sig(i,j  )*sigw(k  )+(h0(i,j  )-hc_sig(i,j  ))*Csw_sig(k  )
         zjkm12   = hc_sig(i,j  )*sigw(k-1)+(h0(i,j  )-hc_sig(i,j  ))*Csw_sig(k-1)
         zjp1kp12 = hc_sig(i,j+1)*sigw(k  )+(h0(i,j+1)-hc_sig(i,j+1))*Csw_sig(k  )
         zjp1km12 = hc_sig(i,j+1)*sigw(k-1)+(h0(i,j+1)-hc_sig(i,j+1))*Csw_sig(k-1)
         if(zjp1kp12-zjp1km12+zjkp12-zjkm12/=0.0 ) crit=abs((zjp1kp12+zjp1km12-(zjkp12+zjkm12))/(zjp1kp12-zjp1km12+zjkp12-zjkm12))
       end do
     end if
    END DO
  END DO
!
!
   DO j=ljmin,ljmaxp1
     DO i=limin,limaxp1
       IF (h0(i,j) > -valmanq) THEN
         DO k=1,kmax
!gandre            zr=hc_sig(i,j)*(sig(k)-1.0_rsh)+(h0(i,j)-hc_sig(i,j))*csu_sig(k)
           zr=hc_sig(i,j)*(sig(k))+(h0(i,j)-hc_sig(i,j))*csu_sig(k)
           zr=zr*(-1)
!          temp(k,i,j)=temp_profil(1)
!          sal(k,i,j)=sal_profil(1)
            rho2(k,i,j)=densite(1)
           kk=1
           do  while(depth(kk) < zr .and. kk < nb_niv)
            kk=kk+1
           end do
           if(kk /= 1) then
!           temp(k,i,j)= ((zr-depth(kk-1))*temp_profil(kk)+(depth(kk)-zr)*temp_profil(kk-1))/(depth(kk)-depth(kk-1))
!           sal(k,i,j) = ((zr-depth(kk-1))*sal_profil(kk) +(depth(kk)-zr)* sal_profil(kk-1))/(depth(kk)-depth(kk-1))
            rho2(k,i,j)= ((zr-depth(kk-1))*densite(kk) +(depth(kk)-zr)* densite(kk-1))/(depth(kk)-depth(kk-1))
           end if
         END DO
       ELSE
!        temp(:,i,j)=temp_profil(1)
!        sal(:,i,j)=sal_profil(1)
         rho2(:,i,j)=densite(1)
       END IF
     END DO
   END DO
#else
  DO j=ljminp2,-ljmaxm1
    DO i=liminp2,limaxm1
     if(hx(i,j) /= -valmanq -h0fond) then
       do k=1,kmax
         zikp12   = h0(i  ,j)*sigw(k  )
         zikm12   = h0(i  ,j)*sigw(k-1)
         zip1kp12 = h0(i+1,j)*sigw(k  )
         zip1km12 = h0(i+1,j)*sigw(k-1)
         if(zip1kp12-zip1km12+zikp12-zikm12/=0.0 ) crit=abs((zip1kp12+zip1km12-(zikp12+zikm12))/(zip1kp12-zip1km12+zikp12-zikm12))
         if(crit > 3.0) write(566,'(3i4,1x,f6.3,1x,3(e10.4,1x))') k,i,j,crit,h0(i,j),h0(i+1,j),hx(i,j)
       end do
     end if
     if(hy(i,j) /= -valmanq -h0fond) then
       do k=1,kmax
         zjkp12   = h0(i,j  )*sigw(k  )
         zjkm12   = h0(i,j  )*sigw(k-1)
         zjp1kp12 = h0(i,j+1)*sigw(k  )
         zjp1km12 = h0(i,j+1)*sigw(k-1)
         if(zjp1kp12-zjp1km12+zjkp12-zjkm12/=0.0 ) crit=abs((zjp1kp12+zjp1km12-(zjkp12+zjkm12))/(zjp1kp12-zjp1km12+zjkp12-zjkm12))
          if(crit > 3.0) write(567,'(3i4,1x,f6.3,1x,3(e10.4,1x))') k,i,j,crit,h0(i,j),h0(i,j+1),hy(i,j)
       end do
     end if
    END DO
  END DO
!
!
   DO j=ljmin,ljmaxp1
     DO i=limin,limaxp1
       IF (h0(i,j) > -valmanq) THEN
         DO k=1,kmax
!gandre            zr=hc_sig(i,j)*(sig(k)-1.0_rsh)+(h0(i,j)-hc_sig(i,j))*csu_sig(k)
           zr=h0(i,j)*sig(k)
           zr=zr*(-1)
!          temp(k,i,j)=temp_profil(1)
!          sal(k,i,j)=sal_profil(1)
           rho2(k,i,j)=densite(1)
           kk=1
           do  while(depth(kk) < zr .and. kk < nb_niv)
            kk=kk+1
           end do
           if(kk /= 1) then
!           temp(k,i,j)= ((zr-depth(kk-1))*temp_profil(kk)+(depth(kk)-zr)*temp_profil(kk-1))/(depth(kk)-depth(kk-1))
!           sal(k,i,j) = ((zr-depth(kk-1))*sal_profil(kk) +(depth(kk)-zr)* sal_profil(kk-1))/(depth(kk)-depth(kk-1))
            rho2(k,i,j)= ((zr-depth(kk-1))*densite(kk) +(depth(kk)-zr)* densite(kk-1))/(depth(kk)-depth(kk-1))
           end if
         END DO
       ELSE
!        temp(:,i,j)=temp_profil(1)
!        sal(:,i,j)=sal_profil(1)
         rho2(:,i,j)=densite(1)
       END IF
     END DO
   END DO

#endif
! END MANGA


#ifdef key_grid_verif
   ! saving of dx, dy variables for verification. Optional (in sequential only)
   IF (l_bathy_save) THEN

#ifdef key_MPI_2D
     WRITE(ierrorlog,*) 'ERROR: routine GRDIMETRICS.F90'
     WRITE(ierrorlog,*) 'Check the grid (dx,dy,dx_u...) in sequential only'
     CALL_MPI MPI_FINALIZE(IERR_MPI) ; STOP
#endif

     write(1231,*) 'j,MINVAL(lon2d(i,:)),MAXVAL(lon2d(i,:))'
     DO i = imin,imax
       write(1231,*) j,MINVAL(lon2d(i,:)),MAXVAL(lon2d(i,:))
     END DO
     write(1231,*) 'j,MINVAL(lon2du(i,:)),MAXVAL(lon2du(i,:))'
     DO i = imin,imax
       write(1231,*) j,MINVAL(lon2du(i,:)),MAXVAL(lon2du(i,:))
     END DO
     write(1231,*) 'j,MINVAL(lon2dv(i,:)),MAXVAL(lon2dv(i,:))'
     DO i = imin,imax
       write(1231,*) j,MINVAL(lon2dv(i,:)),MAXVAL(lon2dv(i,:))
     END DO

     write(1232,*) 'j,MINVAL(lat2d(:,j)),MAXVAL(lat2d(:,j))'
     DO j = jmin,jmax
       write(1232,*) j,MINVAL(lat2d(:,j)),MAXVAL(lat2d(:,j))
     END DO
     write(1232,*) 'j,MINVAL(lat2du(:,j)),MAXVAL(lat2du(:,j))'
     DO j = jmin,jmax
       write(1232,*) j,MINVAL(lat2du(:,j)),MAXVAL(lat2du(:,j))
     END DO
     write(1232,*) 'j,MINVAL(lat2dv(:,j)),MAXVAL(lat2dv(:,j))'
     DO j = jmin,jmax
       write(1232,*) j,MINVAL(lat2dv(:,j)),MAXVAL(lat2dv(:,j))
     END DO

     write(1233,*) 'j,MINVAL(dx(:,j)),MAXVAL(dx(:,j))'
     DO j = jmin,jmax
       write(1233,*) j,MINVAL(dx(:,j)),MAXVAL(dx(:,j))
     END DO
     write(1233,*) 'j,MINVAL(dx_u(:,j)),MAXVAL(dx_u(:,j))'
     DO j = jmin,jmax
       write(1233,*) j,MINVAL(dx_u(:,j)),MAXVAL(dx_u(:,j))
     END DO
     write(1233,*) 'j,MINVAL(dx_v(:,j)),MAXVAL(dx_v(:,j))'
     DO j = jmin,jmax
       write(1233,*) j,MINVAL(dx_v(:,j)),MAXVAL(dx_v(:,j))
     END DO
     write(1233,*) 'j,MINVAL(dx_f(:,j)),MAXVAL(dx_f(:,j))'
     DO j = jmin,jmax
       write(1233,*) j,MINVAL(dx_f(:,j)),MAXVAL(dx_f(:,j))
     END DO

     write(1234,*) 'dy etc = cst or not ?'
     write(1234,*) 'dy',MINVAL(dy(:,:)),MAXVAL(dy(:,:))
     write(1234,*) 'dy_u',MINVAL(dy_u(:,:)),MAXVAL(dy_u(:,:))
     write(1234,*) 'dy_v',MINVAL(dy_v(:,:)),MAXVAL(dy_v(:,:))
     write(1234,*) 'dy_f',MINVAL(dy_f(:,:)),MAXVAL(dy_f(:,:))
     write(1234,*) 'j,MINVAL(dy(:,j)),MAXVAL(dy(:,j))'
     DO j = jmin,jmax
       write(1234,*) j,MINVAL(dy(:,j)),MAXVAL(dy(:,j))
     END DO
     write(1234,*) 'j,MINVAL(dy_u(:,j)),MAXVAL(dy_u(:,j))'
     DO j = jmin,jmax
       write(1234,*) j,MINVAL(dy_u(:,j)),MAXVAL(dy_u(:,j))
     END DO
     write(1234,*) 'j,MINVAL(dy_v(:,j)),MAXVAL(dy_v(:,j))'
     DO j = jmin,jmax
       write(1234,*) j,MINVAL(dy_v(:,j)),MAXVAL(dy_v(:,j))
     END DO
     write(1234,*) 'j,MINVAL(dy_f(:,j)),MAXVAL(dy_f(:,j))'
     DO j = jmin,jmax
       write(1234,*) j,MINVAL(dy_f(:,j)),MAXVAL(dy_f(:,j))
     END DO


     IF_MPI (MASTER) THEN
     CALL ionc4_open('bathy_verif.nc')
     CALL ionc4_createvar('bathy_verif.nc','dx','degres','dx', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dx(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dx',tmp2drsh,imin,imax,jmin,jmax)
     CALL ionc4_createvar('bathy_verif.nc','dx_u','degres','dx_u', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dx_u(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dx_u',tmp2drsh,imin,imax,jmin,jmax)
     CALL ionc4_createvar('bathy_verif.nc','dx_v','degres','dx_v', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dx_v(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dx_v',tmp2drsh,imin,imax,jmin,jmax)
     CALL ionc4_createvar('bathy_verif.nc','dx_f','degres','dx_f', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dx_u(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dx_f',tmp2drsh,imin,imax,jmin,jmax)

     CALL ionc4_createvar('bathy_verif.nc','dy','degres','dy', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dy(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dy',tmp2drsh,imin,imax,jmin,jmax)
     CALL ionc4_createvar('bathy_verif.nc','dy_u','degres','dy_u', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dy_u(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dy_u',tmp2drsh,imin,imax,jmin,jmax)
     CALL ionc4_createvar('bathy_verif.nc','dy_v','degres','dy_v', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dy_v(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dy_v',tmp2drsh,imin,imax,jmin,jmax)
     CALL ionc4_createvar('bathy_verif.nc','dy_f','degres','dy_f', &
                          dims='xy',fill_value=dg_valmanq_io)
     tmp2drsh(:,:)=dy_v(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy('bathy_verif.nc','dy_f',tmp2drsh,imin,imax,jmin,jmax)
     CALL ionc4_close('bathy_verif.nc')
     ENDIF_MPI
   ENDIF
#endif

   PRINT_DBG*, 'exit GRID_METRIC routine'
   
  END SUBROUTINE grid_metric
