   !&E---------------------------------------------------------------------

#ifdef key_palm
  SUBROUTINE mars2d3d()
#else
  PROGRAM mars2d3d
#endif

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE main  ***
   !&E
   !&E ** Purpose : main program
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : ionc4_init, init, bathy, grid_mask,grid_metric
   !&E                   : tool_decompdate, meteo, init_xe, init_uvz, init_uv, init_viscoef
   !&E                   : init_turbcoef, traceur, river, outflow, init_tssub, ts_stateeq_linear
   !&E                   : ts_stateeq_blumberg, sortie, diagnostic, track_points, step
   !&E                   : Agrif_Step
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History : 
   !&E        !  1987-02  (J.-C. Salomon, P. Lazure) original code
   !&E        !  2007-07  (M. Sourisseau) elimination var_read_obc
   !&E        !  2009-02  (V. Garnier) MPI2D
   !&E        !  2009-02  (P. Le Hir, F. Cayocca) introduction "new" sand/mud module
   !&E        !  2009-07  (N. Bruneau, F. Dumas) coupling with swan
   !&E        !  2009-09  (B. Thouvenin) coupling with MET&OR (contaminant)
   !&E        !  2009-10  (V. Garnier) reproductibility when using restart file
   !&E        !  2010-03  (A.-C. Bennis, T. Odaka) use of palm coupling
   !&E        !  2010-06  (V.M. Faure, N. Bousserez, M. Baklouti) coupling with eco3m
   !&E        !  2010-06  (F. Cayocca) coupling with the module of sedimentology
   !&E        !  2010-10  (B. Thouvenin) Distribution of the river discharge throughout a thickness
   !&E        !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E        !  2010-10  (B. Thouvenin) Distribution of the river discharge throughout a thickness
   !&E        !  2011-03  (M. Huret) Introduction of Individual Based Model
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE ionc4, ONLY : ionc4_init,ionc4_closeall
   USE comvars2d
   USE comvars3d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d,    ONLY : igdv_old
   USE comdiag,       ONLY : l_diag,file_diag
   USE output,        ONLY : output_init,output_mng,l_out_demer,fileresu,nb_out
   USE comdem
   USE comfluxdo
   USE comturb,       ONLY : eps
   USE obcogcm,       ONLY : obc_tsref
#ifdef key_waveforcing
   USE waveforcing,   ONLY : lec_para_swan,init_wave_forcing_fric, &
                             lect_wave,compute_wave_forcing
#endif
#ifdef key_substance
   USE subreaddat,    ONLY : sub_read_var,sub_read_vardiag,sub_sflx_read
   USE comsubstance,  ONLY : nv_dri, nv_state, cv_wat,l_cvrain_readfile,l_subflxatm_readfile,l_bilan
   USE usersubstance, ONLY : user_init_vdriv
   USE obcogcm,       ONLY : obc_subsref
   USE submassbalance,ONLY : sub_budget_readdomain
#ifdef key_biolo
   USE bioloinit,     ONLY : biolo_userinit
#endif
#ifdef key_eco3m
   USE DEF_TYPE
   USE VAR_USER
   USE VAR_GLOBAL
   USE COUPLEUR_PHY_BIO
   USE F_PROCESS
#endif
#ifdef key_contaminant                             
   USE contareaddat,  ONLY : conta_read_react  
#endif 
#endif
#ifdef key_sedim
   USE sedimento,     ONLY : sed_alloc,l_repsed,sed_init_fromfile,sed_init,   &
                             sed_settlveloc_ini,sed_morphoinit,l_morphocoupl, &
                             cv_sed
#endif
   USE sflxatm,     ONLY : drag,exch,imeteo_dragtype,imeteo_exchtype
   USE sflxturb,    ONLY : sflx_coef_init
   USE trackpoints, ONLY : l_points,track_points
#ifdef key_trajec3d
   USE trajinitsave,   ONLY : traj_init3d
#endif
#ifdef key_trajec2d
   USE traject2d,   ONLY : traj_init2d
#endif
#ifdef key_ibm
   USE ibm,         ONLY : ibm_init
#endif
#ifdef key_diagdyn
   USE diagdyn,     ONLY : diagdyn_init
#endif
   USE casinit,     ONLY : cas_init
   USE_MPI toolmpi,     ONLY : MPI_START1,MPI_START2,MPI_START3,MPI_END1
   USE_MPI toolmpi,     ONLY : ADD_TAG,ex_j_rsh,ex_i_rsh,MASTER

   !! * Arguments

   !! * Local declarations

   IMPLICIT NONE

#ifdef key_VT
   INCLUDE '/usr/local/vampir/vampirtraceold/include/VT.inc'
   INTEGER ierr
#endif

   CHARACTER(LEN=lchain)              :: modele
   INTEGER                            :: i,j,k,taille_germe,narg,iargc,nb,IERR_MPI
   INTEGER, ALLOCATABLE, DIMENSION(:) :: germe
#ifdef key_agrif
   EXTERNAL :: step
#endif
#ifdef key_MPI_2D
   CHARACTER(LEN=3)     :: tag
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_VT
   CALL VTTRACEON(ierr)
#endif
   CALL_MPI MPI_START1

#ifdef key_agrif
   CALL Agrif_Init_Grids()
#endif

   !  Init des boucles
   !------------------------
   nbouc=0

   ! initialisation de la procedure de tirage des nombres aleatoires
   !-----------------------------------------------------------------
   root='./'
   CALL RANDOM_SEED(size=taille_germe)
   ALLOCATE(germe(taille_germe))
   germe(:)=3
   CALL RANDOM_SEED(put=germe(:))

   CALL ionc4_init()
   narg=iargc()

   IF (l_testcase) THEN
     CALL_MPI MPI_START2(0,0)
     CALL_AGRIF Agrif_Declare_Variables

   ! Allocation of variables 
   ! -----------------------
     ! Dynamical variables
     CALL ALLOC
     ! Intermediate variables
     CALL alloc_inter
     ! Stationnary variable (grid metrics)
     CALL alloc_stat
   ! Initialize test case configuration
     CALL cas_init

   ! Initialize coefficients of turbulence
     IF (turb_nbeq==1 .OR. turb_nbeq==2) CALL init_turbcoef(ect,psi)

   ! Initialize xeadv
     xeadv(limin:limax,ljmin:ljmax)=ssh(limin:limax,ljmin:ljmax)

     CALL_MPI MPI_START3

   ELSE

   ! Read namelists (paramain.txt, paracom.txt, paraspec.txt)
   ! Read head.conf
     CALL init(narg)

   ! Read parameters of XXXXX
#ifdef key_waveforcing
     CALL lec_para_swan
#endif

   ! Initialize flux exchange coefficients (to prepare the bilinear interpolation)
     IF (imeteo_exchtype==1) CALL sflx_coef_init(drag,exch,imeteo_dragtype)

   ! For MARS_WEB
     IF (narg>=2) CALL init_marsweb_clim
     IF (narg==3) CALL init_marsweb_simu

   ! Read the bathymetry
     CALL bathy

   ! Initialize ig,id,jb,jh
     CALL grid_mask

   ! Divide the domain for MPI run
     CALL_MPI MPI_START2(1,1)
     CALL_MPI MPI_START3
     CALL_AGRIF Agrif_Declare_Variables


   ! Allocation of variables 
   ! -----------------------
     ! Dynamical variables
     CALL ALLOC
     ! Intermediate variables
     CALL alloc_inter
     ! Stationnary variables (grid metrics)
     CALL alloc_stat

   ! Initialize metrics (dx,dy,sig,lon,lat...)
     CALL grid_metric


#ifdef key_substance
   ! Read file variables.dat describing substances
   ! Allocate substance variables 
   ! Make variables in order
     CALL sub_read_var

   ! Initialize open boundary arrays for substance
     CALL obc_subsref

#ifdef key_sedim
       CALL sed_alloc
       IF (l_repsed) THEN
         CALL sed_init_fromfile
       ELSE
         CALL sed_init
       END IF
       CALL sed_settlveloc_ini
       IF(l_morphocoupl)CALL sed_morphoinit
#endif

   ! Read file vardiag.dat describing diagnoses
   ! Allocate diagnoses variables
#ifdef key_biolo
     CALL sub_read_vardiag
#endif
#ifdef key_eco3m
! Initialization of ECO3M
     IF (nbouc == 0) THEN
       nbcallbio = -1
       CALL main_bio
       CALL alloc_var_eco3m
       !In order to compute the modulo of the call to ECO3M
       dt_phy_ini= dt
     END IF
#endif
#endif

   ! Initialize sea surface height
     CALL init_xe(ssh)
   ! Initialize height variables for vertical integration
     CALL init_height(ssh)
   ! Initialize 3D velocities
     CALL init_uvz(uz,vz)
   ! Initialize 2D velocities
     CALL init_uv(u,v,uz,vz)
   ! Initialize viscosity coefficients
     CALL init_viscoef
   ! Initialize coefficients of turbulence
     CALL init_turbcoef(ect,psi)
   ! Initialize sal, temp, cv_wat
     CALL init_tssub(sal,temp   &
#ifdef key_substance
                        ,cv_wat &
#endif
                             )
     CALL obc_tsref

   ! Restart from a file ( restart file from MARS or climatological file )
     IF (l_initfromfile) THEN
       CALL init_fromfile(u,uz,v,vz,ssh,sal,temp,        &
                                        vstar,igdv_old,  &
#ifdef key_substance
                                            cv_wat,      &
#endif
                                            ect,psi,eps)
     END IF


   ! Initialize wave fields
#ifdef key_waveforcing
     CALL init_wave_forcing_fric
     CALL lect_wave
!cval ?     CALL bathy
     CALL compute_wave_forcing
#endif

   ! Initialize ecosystem variables
#ifdef key_substance
#ifdef key_biolo
   ! User special initialization of cv_wat.
     CALL biolo_userinit(cv_wat)
#endif
!!!! CAR INITIALISE DANS STEP POUR SEDIM REVOIR ?#if defined key_sedim_anc || defined key_contaminant
#if defined key_contaminant
!!!!! deja fait avec sedim??     IF (.NOT.l_initfromfile) CALL sed_settlveloc_ini
     CALL conta_read_react
!!!!!     CALL conta_userinit
#endif
#endif

#ifdef key_substance
   ! initialization of driving variables
     IF (nv_dri > 0) &
       CALL user_init_vdriv(cv_wat                       &
#ifdef key_sedim                                
                           ,cv_sed                       &
#endif   
                                )

#endif
   ! Read output.dat and allocate variables with the correct number of output files
     CALL output_init

   END IF   ! ends test on l_testcase

#ifdef key_substance
   !!! introduction of submassbalance
     IF (l_bilan) CALL sub_budget_readdomain(ssh)
     PRINT_DBG*, 'call read_limits'
   !!! end submassbalance
#endif

   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,v(liminm2:limaxp2,ljminm2:ljmaxp2))   ! for dyn2d_dt val
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,u(liminm2:limaxp2,ljminm2:ljmaxp2))   ! for dyn2d_dt val

   ! Save summary of simulation
   IF_MPI (MASTER) THEN
   CALL summary
   ENDIF_MPI

   ! Initialize listing file
   IF (narg == 0) THEN
     filename= 'listing'
     CALL_MPI ADD_TAG(filename,LEN(filename))
     CALL_MPI ADD_TAG(file_diag,LEN(file_diag))
!     CALL edge_initialise_MPI
     IF_MPI (MASTER) THEN
     IF (iscreen/=6) OPEN(unit=iscreen,file=filename,status="replace")
     ENDIF_MPI
   END IF

   ! Initialize meteorological variables if l_meteo_stat or l_meteo_hom
   IF (.NOT. l_testcase) CALL meteo

   ! Read file river.dat, initialize river discharges inputs with time interpolation
   CALL river(ssh)

#ifdef key_substance
   ! Read file outflow.dat, file describing discharges
   ! initialize discharges inputs with time interpolation
   CALL outflow

   ! Read files sflx_cvrain and/or sflx_subflxatm 
   ! if temporal variations of cvrain or of atm. fluxes for at least one substance
   !  (l_cvrain_readfile=.true. or l_subflxatm_readfile=.true.)
   IF (l_cvrain_readfile .or. l_subflxatm_readfile) CALL sub_sflx_read
#endif


   ! Estimate buoyancy from sal and temp
   IF (l_modele2d) THEN
     IF (icon == 2) CALL ts_stateeq2d(sal)
   ELSE
     IF (icon >= 1 .AND. l_stateeq_lin) CALL ts_stateeq_linear (sal,temp,boz)
     IF (icon >= 1 .AND. .NOT.l_stateeq_lin) CALL ts_stateeq_blumberg(sal,temp,boz)
   END IF

   ! Intialize variables for traking
   IF (l_points) CALL track_points(u,v,uz,vz,ssh,sal,temp, &
#ifdef key_substance
                                   cv_wat,                 &
#endif
                                   ect,boz)

   ! Initialize diagnoses variables
   IF (l_diag) CALL diagnostic(u,v,uz,vz,ssh,sal,temp, &
#ifdef key_substance
                               cv_wat,                 &
#endif
                               ect,boz)

   ! Initialize diagnoses variables relative to terms of the momentum equation
#ifdef key_diagdyn
   CALL diagdyn_init
#endif

   ! manage outputs files
   CALL output_mng

   ! Initialize demerliac filter
   DO nb=1,nb_out

     IF (l_out_demer(nb)) &
        CALL out_demerliac(ssh(limin:limax,ljmin:ljmax),                                     &
                           u(limin:limax,ljmin:ljmax),uz(:,limin:limax,ljmin:ljmax),         &
                           v(limin:limax,ljmin:ljmax),vz(:,limin:limax,ljmin:ljmax),         &
                           sal(:,limin:limax,ljmin:ljmax),temp(:,limin:limax,ljmin:ljmax),   &
                           xed(limin:limax,ljmin:ljmax),                                     &
                           ud(limin:limax,ljmin:ljmax),uzd(:,limin:limax,ljmin:ljmax),       &
                           vd(limin:limax,ljmin:ljmax),vzd(:,limin:limax,ljmin:ljmax),       &
                           sald(:,limin:limax,ljmin:ljmax),tempd(:,limin:limax,ljmin:ljmax), &
#ifdef key_substance
                           cv_wat(:,:,limin:limax,ljmin:ljmax),cv_watd(:,:,limin:limax,ljmin:ljmax), &
#endif
                           nsdem)
   END DO


#ifdef key_trajec3d
   IF (.NOT.l_modele2d) CALL traj_init3d(ssh)
#endif
#ifdef key_trajec2d
   IF (l_modele2d) CALL traj_init2d
#endif

#ifdef key_ibm
   IF (.NOT.l_modele2d) CALL ibm_init(ssh,sal,temp, &
#ifdef key_substance
                                      cv_wat,       &
#endif
                                      boz)
#endif

   IF_MPI (MASTER) THEN
   WRITE(iscreen,*) ' '
   WRITE(iscreen,*) '**************************************************'
   WRITE(iscreen,*) '**************** Start time integration **********'
   WRITE(iscreen,*) '**************************************************'
   WRITE(iscreen,*) ' '
   ENDIF_MPI

   halfdt = dt*0.5_rlg

   ! Time integration
#ifdef key_agrif
   DO WHILE (t <= tdebagrif .OR. MOD(nbouc,2) ==1 )  
     CALL step()
   END DO
   DO WHILE (t <= tfin .OR. MOD(nbouc,2)==0)
     CALL Agrif_Step(step)
   END DO
#else
   DO WHILE (t <= tfin .OR. MOD(nbouc,2)==0)
#ifdef key_waveforcing
     CALL compute_wave_forcing
#endif
#ifdef key_eco3m
! Call of ECO3M (in order to compute the dcdt term
! with a variable time step, modulo of dt ini of MARS
   IF (mod(nint(dt_bio/(2*dt_phy_ini)),2)==0) THEN
     IF (mod(nbouc,nint(dt_bio/(2*dt_phy_ini)))==0) THEN
       CALL transfert_marseco(ssh)
       CALL main_bio
       CALL transfert_ecomars
     END IF
   ELSE
     IF (mod(nbouc,1+nint(dt_bio/(2*dt_phy_ini)))==0) THEN
       CALL transfert_marseco(ssh)
       CALL main_bio
       CALL transfert_ecomars
     END IF
   END IF
#endif

     CALL step()
   END DO
#endif

   CALL dealloc_inter
   CALL dealloc_stat
   CALL ionc4_closeall()
   PRINT*, 'fin normale'

   CALL DEALLOC

   CALL_MPI MPI_FINALIZE(IERR_MPI)

#ifdef key_eco3m
   CALL DEALLOC_VAR_Eco3M
#endif
   CALL_MPI MPI_END1
#ifdef key_VT
   CALL VTTRACEOFF(ierr)
#endif

#ifdef key_palm
  END subroutine mars2d3d
#else
  END PROGRAM mars2d3d 
#endif
