 MODULE obcapplyssh

   !!======================================================================
   !!                   ***  MODULE  obcapplyssh  ***
   !! Ocean dynamics:  apply sea surface height open boundary conditions
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE obccombine, ONLY : obc_sshs,obc_sshn,obc_sshe,obc_sshw,              &
                          l_obc_west,l_obc_east,l_obc_north,l_obc_south

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC obc_apply_ssh                ! routine called by stepu*.F90 stepv*.F90

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

   SUBROUTINE obc_apply_ssh(xe)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE obc_apply_ssh  ***
   !&E
   !&E ** Purpose : save sea surface height open boundary into xe
   !&E
   !&E ** Description : open boundary sea surface heights (combination of ogcm !and tide)
   !&E                  are saved in variables obc_sshs
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-04 (V. Garnier) Original code
   !&E       !  2009-05  (F. Dumas)  AGRIF
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E ** RQ : this routine can not be parallelized at all until xe is not in dyn3dy.F90
   !&E
   !&E---------------------------------------------------------------------

   !! * Modules used
   USE parameters, ONLY : imin,imax,jmin,jmax,liminp1,limax,ljminp1,ljmax, &
                          limin,liminm2,limaxp2,ljmin,ljminm2,ljmaxp2,rsh,valmanq
   USE comvars2d,  ONLY : h0

   !! * Declaration Subroutine

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( out )   :: xe

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   ! western boundary
   IF_MPI (limin==imin) THEN
   IF ( l_obc_west ) WHERE(h0(imin+1,ljminm2:ljmaxp2)/=-valmanq) xe(imin+1,ljminm2:ljmaxp2) = obc_sshw(ljminm2:ljmaxp2)
   ENDIF_MPI

   ! eastern boundary
   IF_MPI (limax==imax) THEN 
   IF ( l_obc_east ) WHERE(h0(imax,ljminm2:ljmaxp2)/=-valmanq) xe(imax,ljminm2:ljmaxp2) = obc_sshe(ljminm2:ljmaxp2)
   ENDIF_MPI

   ! southern boundary
   IF_MPI (ljmin==jmin) THEN
   IF ( l_obc_south ) WHERE(h0(liminm2:limaxp2,jmin+1)/=-valmanq) xe(liminm2:limaxp2,jmin+1) = obc_sshs(liminm2:limaxp2)
   ENDIF_MPI

   ! northern boundary
   IF_MPI (ljmax==jmax) THEN
   IF ( l_obc_north ) WHERE(h0(liminm2:limaxp2,jmax)/=-valmanq) xe(liminm2:limaxp2,jmax) = obc_sshn(liminm2:limaxp2)
   ENDIF_MPI


   END SUBROUTINE obc_apply_ssh

   !!======================================================================

 END MODULE obcapplyssh
