!**************************************************
!
       subroutine tide_harmcompo
!                           
!***************************************************
!
!     objet    : calcul des niveaux de la surface libre a la
!                limite par composition harmonique (type Simon).
!                Composition harmonique basee sur 8 ondes :
!                M2, K1, K2, 2N2, M4, MF, MM, MSQM, MTM, N2, O1, P1, Q1, S2 
!                issues du modele numerique de maree mondiale FES2004.
!                On peut construire un modele de maree partout dans le monde
!                a l aide de ces donnees. Possibilite d enrichir le spectre
!                par la methode d admittance.
!                Cette option est activee a l aide de la cle key_tide_fes2004
!                Pour l interpolation spatiale sur la grille du domaine d etude,
!                seuls les points ou la composante M2 est renseignee sont 
!                pris en compte.
!                Rappel : la composante M2 doit imperativement etre enregistree
!                en premier indice d onde pour pouvoir n utiliser que l onde M2
!                (option l_tide_M2harm)
!
!     appelant         : main
!     appeles          : ionc4_openr, ionc4_corres, ionc4_flimax
!     appeles          : ionc4_fljmax, ionc4_read_lat, ionc4_read_lon
!     appeles          : ionc4_read_subxy, admittances, interpolfes, tide_predic
!     appeles          : tide_data
!     canal d ecriture :
!     canal de lecture : file_tide_harmcp
!                      :
!     cle cpp          :
!                      :
!     variables in     : t, h0, valmanq, l_tide_M2harm, fisac, gwac, dgac, diac
!     variables in     : nbouc
!                      :
!     variables out    : l_presence (-> admittance)
!                      :
!     variables inout  :
!                      :
!     dernieres modifs : 14/06/2004
   !&E
   !&E ** History :
   !&E       !          Original code
   !&E       !  2009-03 (V. Garnier)  removing of xeuv argument
   !&E       !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E
!                      :
!*********************************************************************
!
!
USE parameters
use comvars2d
use commar
use comaree
use ionc4, only : ionc4_openr,ionc4_corres,ionc4_flimax,ionc4_fljmax, &
                  ionc4_read_lat,ionc4_read_lon,ionc4_read_subxy
implicit none
!
integer                                      :: nbfes
character(len=lchain),dimension(:),pointer   :: mar1,mar2
integer                                      :: delta
integer                                      :: i,j,n,kount
integer                                      :: ixmar,jxmar,cdf_id
integer                                      :: imarmin,imarmax,jmarmin,jmarmax
real(kind=rsh)                               :: xt,g0fes,phi0fes,dgfes,dphifes
real(kind=riolg),dimension(:),pointer          :: lonmar,latmar
real(kind=rsh)                               :: long,latit
real(kind=riosh),dimension(:,:),pointer      :: wra,wrg
real(kind=rsh),dimension(:,:,:),pointer      :: wrp,wip
real(kind=riosh),parameter                   :: valmanq_tide = 999.
!
common /fes99/ g0fes,phi0fes,dgfes,dphifes,wrp,wip
common /fes99/ imarmin,imarmax,jmarmin,jmarmax
!
!************************************************************
!     Executable statements
!************************************************************
!
!   initialisation
!---------------------
!
      if(nbouc==0) then
       nbfes=14
       allocate(mar1(nbfes)) 
       allocate(mar2(nbfes)) 
       mar1(1)='amp_M2'
       mar1(2)='amp_K1'
       mar1(3)='amp_K2'
       mar1(4)='amp_2N2'
       mar1(5)='amp_M4'
       mar1(6)='amp_Mf'
       mar1(7)='amp_Mm'
       mar1(8)='amp_Msqm'
       mar1(9)='amp_Mtm'
      mar1(10)='amp_N2'
      mar1(11)='amp_O1'
      mar1(12)='amp_P1'
      mar1(13)='amp_Q1'
      mar1(14)='amp_S2'
!
       mar2(1)='pha_M2'
       mar2(2)='pha_K1'
       mar2(3)='pha_K2'
       mar2(4)='pha_2N2'
       mar2(5)='pha_M4'
       mar2(6)='pha_Mf'
       mar2(7)='pha_Mm'
       mar2(8)='pha_Msqm'
       mar2(9)='pha_Mtm'
      mar2(10)='pha_N2'
      mar2(11)='pha_O1'
      mar2(12)='pha_P1'
      mar2(13)='pha_Q1'
      mar2(14)='pha_S2'
!
       tide_nbharmcp=nbfes
       l_presence(:)=.false.
       nu(:)=int(valmanq)
       a0sud(:,:)=valmanq
       ph0sud(:,:)=valmanq
       a0nord(:,:)=valmanq
       ph0nord(:,:)=valmanq
       a0ouest(:,:)=valmanq
       ph0ouest(:,:)=valmanq
       a0est(:,:)=valmanq
       ph0est(:,:)=valmanq
!
! initialisation du tableau des constituants harmoniques
!--------------------------------------------------------
!
       call tide_data
!
       nu(1)=im2
       nu(2)=ik1
       nu(3)=ik2
       nu(4)=i2n22nm2
       nu(5)=im4
       nu(6)=imf
       nu(7)=imm
       nu(8)=imsf
       nu(9)=imsm
       nu(10)=in2
       nu(11)=io1
       nu(12)=ip1
       nu(13)=iq1
       nu(14)=is2
       l_presence(nu(1:nbfes))=.true.    ! pour l admittance
!
! ouverture du fichier des constantes harmoniques
! reperage des indices correspondant a la zone geographique
!--------------------------------------------------------------
!
       call ionc4_openr(file_tide_harmcp)
       call ionc4_corres(file_tide_harmcp,cdf_id)
       ixmar=ionc4_flimax(cdf_id)
       jxmar=ionc4_fljmax(cdf_id)
       allocate(latmar(1:jxmar),lonmar(1:ixmar))
       call ionc4_read_lat(file_tide_harmcp,latmar,1,jxmar)
       call ionc4_read_lon(file_tide_harmcp,lonmar,1,ixmar)
!
       do i=1,ixmar-1
        if(gwac >= lonmar(i) .and. gwac < lonmar(i+1)) imarmin=i
        if(gwac+real(imax-1)*dgac >= lonmar(i)-epsilon .and. &
           gwac+real(imax-1)*dgac <= lonmar(i+1)+epsilon) imarmax=i+1
       enddo
!
       do j=1,jxmar-1
        if(fisac >= latmar(j) .and. fisac < latmar(j+1)) jmarmin=j
        if(fisac+real(jmax-1)*dfiac >= latmar(j)-epsilon .and. &
           fisac+real(jmax-1)*dfiac <= latmar(j+1)+epsilon) jmarmax=j+1
       enddo
!
       delta=3
       imarmin=max(1,imarmin-delta)
       imarmax=min(ixmar,imarmax+delta)
       jmarmin=max(1,jmarmin-delta)
       jmarmax=min(jxmar,jmarmax+delta)
       allocate(wrp(imarmin:imarmax,jmarmin:jmarmax,nbfes))
       allocate(wip(imarmin:imarmax,jmarmin:jmarmax,nbfes))
       allocate(wra(imarmin:imarmax,jmarmin:jmarmax))
       allocate(wrg(imarmin:imarmax,jmarmin:jmarmax))
!
! lecture des constantes harmoniques (phase et amplitude)
!--------------------------------------------------------------
!
       do n=1,nbfes
        call ionc4_read_subxy(file_tide_harmcp,mar1(n),wra,&
	                      imarmin,imarmax,jmarmin,jmarmax,1,1)
        call ionc4_read_subxy(file_tide_harmcp,mar2(n),wrg,&
	                      imarmin,imarmax,jmarmin,jmarmax,1,1)
        do j=jmarmin,jmarmax
         do i=imarmin,imarmax
          if (wra(i,j) /= valmanq_tide .and. wrg(i,j) /= valmanq_tide) then
           wrp(i,j,n)= wra(i,j)*cos(-wrg(i,j)*pi/180)
           wip(i,j,n)=-wra(i,j)*sin(-wrg(i,j)*pi/180)
          else
           wrp(i,j,n)=valmanq
           wip(i,j,n)=valmanq
          endif
         enddo
        enddo
       end do
!
       g0fes=lonmar(imarmin)
       phi0fes=latmar(jmarmin)
       dgfes=lonmar(2)-lonmar(1)
       dphifes=latmar(2)-latmar(1)
!
! interpolation des constantes harmoniques le
! long des frontieres ouest et est.
!----------------------------------------------
!
       do j=jmin,jmax
        long=gwac
        latit=fisac+(j-1)*dfiac
        call interpolfes(long,latit,nbfes,a0ouest(1,j),ph0ouest(1,j),h0_g(1,j))
        if(h0_g(1,j)/=-valmanq .and. a0ouest(im2,j)==valmanq) print*,j,'ouest'
        long=gwac+(imax-1)*dgac
        latit=fisac+(j-1)*dfiac
        call interpolfes(long,latit,nbfes,a0est(1,j),ph0est(1,j),h0_g(imax,j)) 
        if(h0_g(imax,j)/=-valmanq .and. a0est(im2,j)==valmanq) print*,j,'est'
       end do
!
! interpolation des constantes harmoniques le long des frontieres nord
! et sud.
!----------------------------------------------
!
       do i=imin,imax
        long=gwac+(i-1)*dgac
        latit=fisac
        call interpolfes(long,latit,nbfes,a0sud(1,i),ph0sud(1,i),h0_g(i,1))
        if(h0_g(i,1)/=-valmanq .and. a0sud(im2,i)==valmanq) print*,i,'sud'
        long=gwac+(i-1)*dgac
        latit=fisac+(jmax-1)*dfiac
        call interpolfes(long,latit,nbfes,a0nord(1,i),ph0nord(1,i),h0_g(i,jmax))
        if(h0_g(i,jmax)/=-valmanq .and. a0nord(im2,i)==valmanq) print*,i,'nord'
       end do 
       deallocate(wip,wrp,latmar,lonmar,mar1,mar2)
!
! On procede a un calcul d admittance pour ajouter les ondes secondaires
!-------------------------------------------------
!
!      if(l_tide_admittance.and..not.l_tide_M2harm) then
!       call tide_admittances(tide_nbharmcp2)
!       tide_nbharmcp=tide_nbharmcp2
!      end if
      if(l_tide_M2harm) tide_nbharmcp=1    ! on ne conserve que l onde M2
!
! on reordonne les ondes pour decrire le spectre de facon compatible
! avec la liste SIMON, c est a dire par nu croissant
!-------------------------------------------------
!
     kount=1
     do while(kount/=0)
      kount=0
      do n=1,tide_nbharmcp-1
       if(nu(n) > nu(n+1)) then
        kount=kount+1
        xt=nu(n)
        nu(n)=nu(n+1)
        nu(n+1)=xt
       end if
      end do
     end do
!
   end if
!
! deroulement normal : tide_prediction harmonique de SIMON
!----------------------------------------------------
!
   call tide_predic
!
  end

!****************************************************************************
!
       subroutine interpolfes(long,latit,nbfes,a0,ph0,prof)
!                           
!***************************************************************************
!
!     objet    : interpolation spatiale des constantes harmoniques en
!                complexe.
!
!     appelant         : tide_harmcompo
!     appeles          : 
!     canal d ecriture :
!     canal de lecture : 
!                      :
!     cle cpp          :
!                      :
!     variables in     : 
!                      :
!     variables out    : a0, ph0
!                      :
!     variables inout  :
!
!*********************************************************************
!
!
USE parameters
use commar
implicit none
!
logical                                :: iflag
integer                                :: nbfes,i0,j0,i,j,n,nb,is
integer                                :: imarmin,imarmax,jmarmin,jmarmax
real(kind=rsh)                         :: prof,g0fes,phi0fes,dgfes,dphifes
real(kind=rsh)                         :: long,latit,x,y,dx,dy,x1,x2
real(kind=rsh),dimension(nbfes)        :: sre,sim
real(kind=rsh),dimension(nmarmax)      :: a0,ph0
real(kind=rsh),dimension(:,:,:),pointer:: wrp,wip
     
common /fes99/ g0fes,phi0fes,dgfes,dphifes,wrp,wip
common /fes99/ imarmin,imarmax,jmarmin,jmarmax
!
!************************************************************
!     Executable statements
!************************************************************
!
       sre(:)=0.0_rsh
       sim(:)=0.0_rsh
!
!      long=long+360.0_rsh
       i0=int((long-g0fes)/dgfes)+imarmin
       j0=int((latit-phi0fes)/dphifes)+jmarmin
       x=g0fes+(i0-imarmin)*dgfes
       y=phi0fes+(j0-jmarmin)*dphifes
       dx=(long-x)/dgfes
       dy=(latit-y)/dphifes
!
       iflag=.true.
       do i=i0,i0+1
        do j=j0,j0+1
         do n=1,nbfes
          if(wrp(i,j,n)== valmanq .or.wip(i,j,n)== valmanq) iflag=.false.
         end do
        enddo
       enddo
!
      if(iflag) then
       do n=1,nbfes
        x1=(1.0_rsh-dx)*wrp(i0,j0,n)+dx*wrp(i0+1,j0,n)
        x2=(1.0_rsh-dx)*wrp(i0,j0+1,n)+dx*wrp(i0+1,j0+1,n)
        sre(n)=(1.0_rsh-dy)*x1+dy*x2
        x1=(1.0_rsh-dx)*wip(i0,j0,n)+dx*wip(i0+1,j0,n)
        x2=(1.0_rsh-dx)*wip(i0,j0+1,n)+dx*wip(i0+1,j0+1,n)
        sim(n)=(1.0_rsh-dy)*x1+dy*x2
       enddo
      end if
!
      if(prof/=-valmanq .and. .not.iflag) then
       is=0
       do while(.not.iflag)
        is=is+1
        sre(:)=0.0_rsh
        sim(:)=0.0_rsh
        nb=0
        do j=max(j0-is+1,jmarmin),min(j0+is,jmarmax)
         do i=max(i0-is+1,imarmin),min(i0+is,imarmax)
          do n=1,nbfes
           if(wrp(i,j,n)/= valmanq .or.wip(i,j,n)/= valmanq) then
            sre(n)=sre(n)+wrp(i,j,n)
            sim(n)=sim(n)+wip(i,j,n)
            if(n==1) then
               nb=nb+1
               iflag=.true.
            end if
           end if
          end do
         enddo
        enddo
       enddo 
       sre(:)=sre(:)/nb
       sim(:)=sim(:)/nb
      end if
!
      do n=1,nbfes
       if(iflag) then
        a0(nu(n))=sqrt(sre(n)*sre(n)+sim(n)*sim(n))  !/100.0
        if(sim(n)/=0.0_rsh .or.sre(n)/=0.0_rsh) then
	   a0(nu(n))=sqrt(sre(n)*sre(n)+sim(n)*sim(n))
	   ph0(nu(n))=atan2(sim(n),sre(n))*180.0_rsh/pi
	else
	   a0(nu(n))=0.0_rsh
	   ph0(nu(n))=0.0_rsh
	end if
        if(ph0(nu(n)) < 0.0_rsh) ph0(nu(n))=ph0(nu(n))+360.0_rsh
       else
        a0(nu(n))=valmanq
        ph0(nu(n))=valmanq
       endif
      enddo   
!
      end subroutine interpolfes
