!**********************************************************************
!                                                                     
  SUBROUTINE tide_harmcompo
!                                                                      
!**********************************************************************
!
!     objet    : programme de calcul de la hauteur de maree en un port 
!                principal d apres l annuaire du SHOM. utilise pour les
!                graus de Sete.
!
!
!     appelant         : step
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          :
!                      :
!     variables in     : tjour
!                      :
!     variables out    : haut
!                      :
!     variables inout  :
!                      :
!     dernieres modifs : 19/08/2004
!                      :
!***********************************************************

#include "toolcpp.h"

!
   USE parameters
   USE comvars2d,  ONLY : nbouc,t,igrau,jgrau,nbgrau,type_grau,hnivmoy
   USE cometdyn,   ONLY : pa,paref
   USE obccombine, ONLY : obc_tide_sshn,obc_tide_sshs,obc_tide_sshw,obc_tide_sshe, &
                          obc_tide_un,obc_tide_us,obc_tide_uw,obc_tide_ue,         &
                          obc_tide_vn,obc_tide_vs,obc_tide_vw,obc_tide_ve
   
   IMPLICIT NONE

   INTEGER :: i,ng,i0,j0
   INTEGER :: ndmx,ndmz,ndmq,ndmh,ndmv,ndmvp
   INTEGER :: ndmo,ndmap,ndmf,ndmp,ndmco,ndmlam
#ifdef key_tide_lagoon_banyuls
   ! A Banyuls
   REAL(KIND=rsh),PARAMETER :: z0=38.0_rsh
   REAL(KIND=rsh),PARAMETER :: Asa=62.0_rsh, Aq1=3.3_rsh
   REAL(KIND=rsh),PARAMETER :: Ao1=20.0_rsh, Ak1=30.0_rsh, An2=10.0_rsh
   REAL(KIND=rsh),PARAMETER :: Am2=49.0_rsh, As2=19.0_rsh, Amn4=1.7_rsh
   REAL(KIND=rsh),PARAMETER :: Am4=5.0_rsh,  Ams4=3.8_rsh
   REAL(KIND=rsh),PARAMETER :: Gsa=262.0_rsh,Gq1=46.8_rsh, Go1=125.0_rsh
   REAL(KIND=rsh),PARAMETER :: Gk1=192.0_rsh,Gn2=245.0_rsh
   REAL(KIND=rsh),PARAMETER :: Gm2=255.0_rsh,Gs2=275.0_rsh,Gmn4=26.0_rsh
   REAL(KIND=rsh),PARAMETER :: Gm4=64.0_rsh, Gms4=125.0_rsh
#endif
#ifdef key_tide_lagoon_marseille   
   ! a marseille
   REAL(KIND=rsh),PARAMETER :: z0=45.0_rsh
   REAL(KIND=rsh),PARAMETER :: Asa=9.0_rsh, Aq1=3.0_rsh,  Ao1=19.0_rsh
   REAL(KIND=rsh),PARAMETER :: Ak1=32.3_rsh, An2=13.0_rsh
   REAL(KIND=rsh),PARAMETER :: Am2=62.0_rsh, As2=23.0_rsh, Amn4=2.0_rsh
   REAL(KIND=rsh),PARAMETER :: Am4=6.0_rsh,  Ams4=1.0_rsh
   REAL(KIND=rsh),PARAMETER :: Gsa=318.0_rsh,Gq1=38.3_rsh, Go1=109.0_rsh
   REAL(KIND=rsh),PARAMETER :: Gk1=181.0_rsh,Gn2=219.0_rsh
   REAL(KIND=rsh),PARAMETER :: Gm2=230.0_rsh,Gs2=230.0_rsh,Gmn4=323.0_rsh
   REAL(KIND=rsh),PARAMETER :: Gm4=4.0_rsh, Gms4=72.0_rsh
#endif

   REAL(KIND=rsh)           :: heure,s,h,p,np,p1

   REAL(KIND=rsh),dimension(21) :: aa,vv,gg
   REAL(KIND=rsh)    :: haut
   REAL(KIND=rlg)    :: tjour,tool_datosec

   CHARACTER(len=19) ::datte
!
!************************************************************
!     Executable statements
!************************************************************
!
! ---------------------------------------------------------------------
!       reactualisation de tjour
!       ------------------------
!
   datte = '01/01/1980 00:00:00'
   IF(nbouc == 0) THEN
     obc_tide_sshn(:)=valmanq
     obc_tide_sshe(:)=valmanq
     obc_tide_sshs(:)=valmanq
     obc_tide_sshw(:)=valmanq
     
     obc_tide_un(:,:)=0.0_rsh
     obc_tide_ue(:,:)=0.0_rsh
     obc_tide_us(:,:)=0.0_rsh
     obc_tide_uw(:,:)=0.0_rsh
     
     obc_tide_vn(:,:)=0.0_rsh
     obc_tide_ve(:,:)=0.0_rsh
     obc_tide_vs(:,:)=0.0_rsh
     obc_tide_vw(:,:)=0.0_rsh
   END IF
   
   tjour=(t-tool_datosec(datte))/86400.0_rlg 
   heure=(tjour-aint(tjour))*24.0_rlg
!
!       calcul de s,h,p,Np,p1
!----------------------------
!
   s =  78.16_rlg + 13.17639673_rlg*tjour
   h = 279.82_rlg +  0.98564734_rlg*tjour
   p = 349.50_rlg +  0.11140408_rlg*tjour
   np= 208.10_rlg +  0.05295392_rlg*tjour
   p1= 282.60_rlg +  0.000047069_rlg*tjour
!
!     calcul par groupes des arguments astronomiques,
!     des situations et des amplitudes
!     -----------------------------------------------
!
!     onde annuelle :
!     ---------------
!
   aa(1)=Asa
   gg(1)=Gsa
   vv(1)=h
!
!     ondes diurnes :
!     ---------------
!
   aa(2)=Ak1
   gg(2)=Gk1
   vv(2)=15.0_rsh*heure+h+90.0_rsh
   aa(3)=Ao1
   gg(3)=Go1
   vv(3)=15.0_rsh*heure-2.0_rsh*s+h-90.0_rsh
   aa(4)=Aq1
   gg(4)=Gq1
   vv(4)=15.0_rsh*heure-3.0_rsh*s+h+p-90.0_rsh
   aa(5)=-Ak1/3.0_rsh
   
   gg(5)=Gk1
   vv(5)=15.0_rsh*heure-h+90.0_rsh
   aa(6)=Ao1/5.3_rsh
   gg(6)=Go1
   vv(6)=15.0_rsh*heure-2.0_rsh*s+h-Np-90.0_rsh
   aa(7)=Ak1/7.4_rsh
   gg(7)=Gk1
   vv(7)=15.0_rsh*heure+h+Np+90.0_rsh
!
!     ondes semi-diurnes :
!     --------------------
!
   aa(8)=Am2
   gg(8)=Gm2
   vv(8)=30.0_rsh*heure-2.0_rsh*s+2.0_rsh*h
   aa(9)=An2
   gg(9)=Gn2
   vv(9)=30.0_rsh*heure-3.0_rsh*s+2.0_rsh*h+p
   aa(10)=As2
   gg(10)=Gs2
   vv(10)=30.0_rsh*heure
   aa(11)=An2/7.6_rsh
   gg(11)=Gn2
   vv(11)=30.0_rsh*heure-4.0_rsh*s+2.0_rsh*h+2.0_rsh*p
   aa(12)=An2/6.3_rsh
   gg(12)=Gn2
   vv(12)=30.0_rsh*heure-4.0_rsh*s+4.0_rsh*h
   aa(13)=An2/5.3_rsh
   gg(13)=Gn2
   vv(13)=30.0_rsh*heure-3.0_rsh*s+4.0_rsh*h-p
   aa(14)=-Am2/35.0_rsh
   gg(14)=Gm2
   vv(14)=30.0_rsh*heure-s+2.0_rsh*h-p
   aa(15)=As2/3.7_rsh
   gg(15)=Gs2
   vv(15)=30.0_rsh*heure+2.0_rsh*h
   aa(16)=As2/17.0_rsh
   gg(16)=Gs2-283.0_rsh
   vv(16)=30.0_rsh*heure-h
   aa(17)=-Am2/27.0_rsh
   gg(17)=Gm2
   vv(17)=30.0_rsh*heure-2.0_rsh*s+2.0_rsh*h-Np
   aa(18)=As2/12.0_rsh
   gg(18)=Gs2
   vv(18)=30.0_rsh*heure+2.0_rsh*h+Np
!
!     ondes quart-diurnes :
!     ---------------------
!
   aa(19)=Amn4
   gg(19)=Gmn4
   vv(19)=60.0_rsh*heure-5.0_rsh*s+4.0_rsh*h+p
   aa(20)=Am4
   gg(20)=Gm4
   vv(20)=60.0_rsh*heure-4.0_rsh*s+4.0_rsh*h
   aa(21)=Ams4
   gg(21)=Gms4
   vv(21)=60.0_rsh*heure-2.0_rsh*s+2.0_rsh*h
!
!     l_tide_harmcompo des ondes
!     ---------------------
!     (a priori z0 est en cm, Aij en mm et Gij en degre)
!
   haut=z0*10.0_rsh
   DO i=1,21
     haut=haut+aa(i)*cos(0.0174533_rsh*(vv(i)-gg(i)))
   END DO
   haut=haut/1000.0_rsh
!
   DO ng=1,nbgrau
     i0=igrau(ng)
     j0=jgrau(ng)

     IF_MPI (ljmin <= j0 .AND. j0<=ljmaxp1 .AND. limin <= i0 .AND. i0<=limaxp1 ) THEN
     
     IF (type_grau(ng)==1) THEN
     ! grau sur la frontiere Nord
       obc_tide_sshn(i0)=haut+(paref-pa(i0,j0))/10000.0_rsh-hnivmoy(i0,j0)
     END IF

     IF (type_grau(ng)==2) THEN
     ! grau sur la frontiere Est
       obc_tide_sshe(i0)=haut+(paref-pa(i0,j0))/10000.0_rsh-hnivmoy(i0,j0)
     END IF
     
     IF (type_grau(ng)==3) THEN
     ! grau sur la frontiere Sud
       obc_tide_sshs(i0)=haut+(paref-pa(i0,j0))/10000.0_rsh-hnivmoy(i0,j0)
     END IF

     IF (type_grau(ng)==4) THEN
     ! grau sur la frontiere Ouest
       obc_tide_sshw(i0)=haut+(paref-pa(i0,j0))/10000.0_rsh-hnivmoy(i0,j0)
     END IF

     ENDIF_MPI

   END DO

  END SUBROUTINE tide_harmcompo
