!***************************************************
!
       subroutine tide_harmcompo
!                           
!***************************************************
!
!     objet    : calcul des niveaux de la surface libre a la
!                limite pour un modele de rang 0 par composition
!                harmonique (type Simon). Composition harmonique
!                basee sur 8 ondes (M2, N2, S2, K2, O1, K1, P1, Q1).
!
!     appelant         : main
!     appeles          : tide_astroarg, tide_calcoef
!     canal d ecriture :
!     canal de lecture : 7
!                      :
!     cle cpp          :
!                      :
!     variables in     : t, h0, methom, parcoef, omegat
!                      :
!     variables out    :
!                      :
!     variables inout  :
   !&E
   !&E ** History :
   !&E       !          Original code
   !&E       !  2009-03 (V. Garnier)  removing of xeuv argument
   !&E       !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E
!
!*********************************************************************
!
USE parameters
USE ionc4, only : ionc4_openr,ionc4_corres,ionc4_flimax,ionc4_fljmax,&
                  ionc4_read_lat,ionc4_read_lon,ionc4_read_subxy,ionc4_read_subxy
use comvars2d
use commar
use comaree
implicit none
!
      integer,parameter                   :: nbfes=114
      character(len=100)                  :: nomlu
      integer                             :: i,j,n,nuu,l,tide_nbharmcp2,kount,nifes,njfes,k
      integer                             :: imarmin,imarmax,jmarmin,jmarmax,ixmar,jxmar,cdf_id
      real(kind=rlg)                      :: xt,g0fes,phi0fes,dgfes,dphifes
      real(kind=rlg),dimension(:),pointer :: lonmar,latmar
      real(kind=rlg)                      :: long,latit
      real(kind=riosh),dimension(:,:),pointer       :: wra,wrg
      real(kind=rsh),dimension(:,:,:),pointer       :: wrp,wip
      real(kind=riosh),parameter                    :: valmanq_tide = 999.
!
      common /fes99/ g0fes,phi0fes,dgfes,dphifes,wrp,wip
      common /fes99/ imarmin,imarmax,jmarmin,jmarmax
!
!************************************************************
!     Executable statements
!************************************************************
!
!   initialisation
!---------------------
!
      if(nbouc==0) then
       tide_nbharmcp=nbfes
       l_presence(:)=.false.
       nu(:)=int(valmanq)
       a0sud(:,:)=valmanq
       ph0sud(:,:)=valmanq
       a0nord(:,:)=valmanq
       ph0nord(:,:)=valmanq
       a0ouest(:,:)=valmanq
       ph0ouest(:,:)=valmanq
       a0est(:,:)=valmanq
       ph0est(:,:)=valmanq
!
! initialisation du tableau des constituants harmoniques
!-------------------------------------------------
!
       call tide_data
       do i=2,nbfes+1
         nu(i-1)=i
       enddo
        l_presence(4:115)=.true.
!
!
! lecture des conditions limites en phase et en amplitude
!--------------------------------------------------------
!
        IF (l_tide_surge) file_tide_harmcp = file_tide_harmcp_surge
        call ionc4_openr(file_tide_harmcp)
        call ionc4_corres(file_tide_harmcp,cdf_id)
        ixmar=ionc4_flimax(cdf_id)
        jxmar=ionc4_fljmax(cdf_id)
        allocate(latmar(1:jxmar),lonmar(1:ixmar))
        call ionc4_read_lat(file_tide_harmcp,latmar,1,jxmar)
        call ionc4_read_lon(file_tide_harmcp,lonmar,1,ixmar)
!
        do i=1,ixmar-1
        if(gwac >= lonmar(i) .and. gwac < lonmar(i+1)) imarmin=i
        if(gwac+real(imax-1)*dgac >= lonmar(i)-epsilon .and. &
           gwac+real(imax-1)*dgac <= lonmar(i+1)+epsilon) imarmax=i+1
       enddo
!
       do j=1,jxmar-1
        if(fisac >= latmar(j) .and. fisac < latmar(j+1)) jmarmin=j
        if(fisac+real(jmax-1)*dfiac >= latmar(j)-epsilon .and. &
           fisac+real(jmax-1)*dfiac <= latmar(j+1)+epsilon) jmarmax=j+1
       enddo
!
       allocate(wrp(imarmin:imarmax,jmarmin:jmarmax,nbfes))
       allocate(wip(imarmin:imarmax,jmarmin:jmarmax,nbfes))
       allocate(wra(imarmin:imarmax,jmarmin:jmarmax))
       allocate(wrg(imarmin:imarmax,jmarmin:jmarmax))
!
       do n=1,nbfes
       nomlu='amp_'//trim(nommar(nu(n)))
       call ionc4_read_subxy(file_tide_harmcp,nomlu,wra,&
	                      imarmin,imarmax,jmarmin,jmarmax,1,1)
       nomlu='phas_'//trim(nommar(nu(n)))
       call ionc4_read_subxy(file_tide_harmcp,nomlu,wrg,&
                      imarmin,imarmax,jmarmin,jmarmax,1,1)
         do j=jmarmin,jmarmax
          do i=imarmin,imarmax
           if (wra(i,j) /= valmanq_tide .and. wrg(i,j) /= valmanq_tide) then
            wrp(i,j,n)= wra(i,j)*cos(-wrg(i,j)*pi/180)
            wip(i,j,n)=-wra(i,j)*sin(-wrg(i,j)*pi/180)
           else
            wrp(i,j,n)=valmanq
            wip(i,j,n)=valmanq
           endif
          enddo
         enddo
       end do
!
        g0fes=lonmar(imarmin)
	phi0fes=latmar(jmarmin)
	dgfes=lonmar(2)-lonmar(1)
	dphifes=latmar(2)-latmar(1)
!
        do j=jmin,jmax
         long=gwac
         latit=fisac+(j-1)*dfiac
         call interpolfes(long,latit,nbfes,a0ouest(1,j),ph0ouest(1,j))
         long=gwac+(imax-1)*dgac
         latit=fisac+(j-1)*dfiac
         call interpolfes(long,latit,nbfes,a0est(1,j),ph0est(1,j)) 
        end do
!
        do i=imin,imax
         long=gwac+(i-1)*dgac
         latit=fisac
         call interpolfes(long,latit,nbfes,a0sud(1,i),ph0sud(1,i))
         long=gwac+(i-1)*dgac
         latit=fisac+(jmax-1)*dfiac
         call interpolfes(long,latit,nbfes,a0nord(1,i),ph0nord(1,i))
        end do 
      deallocate(wip,wrp,latmar,lonmar)
!
!
! on reordonne les ondes pour decrire le spectre de facon compatible
! avec la liste SIMON
!-------------------------------------------------
!
      kount=1
      do while(kount/=0)
      kount=0
      do n=1,tide_nbharmcp-1
       if(nu(n) > nu(n+1)) then
        kount=kount+1
        xt=nu(n)
        nu(n)=nu(n+1)
        nu(n+1)=xt
       end if
      end do
      end do
!
      end if
      call tide_predic
!
      end

!****************************************************************************
!
       subroutine interpolfes(long,latit,nbfes,a0,ph0)
!                           
!***************************************************************************
!
!     objet    : calcul des niveaux de la surface libre a la
!                limite pour un modele de rang 0 par composition
!                harmonique (type Simon). Composition harmonique
!                basee sur 8 ondes (M2, N2, S2, K2, O1, K1, P1, Q1).
!
!     appelant         : main
!     appeles          : tide_astroarg, tide_calcoef
!     canal d ecriture :
!     canal de lecture : 7
!                      :
!     cle cpp          :
!                      :
!     variables in     : t, h0, methom, parcoef, omegat
!                      :
!     variables out    :
!                      :
!     variables inout  :
   !&E ** History :
   !&E       !  19XX (F. Dumas) Original code
   !&E       !  2011-10 (F. Dumas) Integration of a bilinear interpolation for high resolution grids
   !&E                  key_tide_shom_bilinear
   !&E                  The two types of interpolation are not compatible yet
!
!*********************************************************************
!
USE parameters
use commar
implicit none
!
      integer                                :: nbfes,i0,j0,i,j,n
      integer                                :: imarmin,imarmax,jmarmin,jmarmax
      real(kind=rlg)                         :: ptot,g0fes,phi0fes,dgfes,dphifes,long,latit,pds
#ifdef key_tide_shom_bilinear
      real(kind=rlg)                         :: var1,var2,xm,ym,xdp,ydp
#endif
      real(kind=rsh),dimension(nbfes)                  :: sre,sim
      real(kind=rsh),dimension(nmarmax)                :: a0,ph0
      real(kind=rsh),dimension(:,:,:),pointer          :: wrp,wip
     
      common /fes99/ g0fes,phi0fes,dgfes,dphifes,wrp,wip
      common /fes99/ imarmin,imarmax,jmarmin,jmarmax
!
!     Executable statements
!************************************************************
!
   ! Assignement of sre and sim

       sre(:)=0.0_rsh
       sim(:)=0.0_rsh
!
       i0=int((long-g0fes)/dgfes)+imarmin
       j0=int((latit-phi0fes)/dphifes)+jmarmin

#ifdef key_tide_shom_bilinear
       xm=(long-g0fes)/dgfes+imarmin
       ym=(latit-phi0fes)/dphifes+jmarmin
       xdp=xm-i0
       ydp=ym-j0

       do n=1,nbfes
        var1=wrp(i0+1,j0  ,n)*xdp+(1.0-xdp)*wrp(i0,j0  ,n)
        var2=wrp(i0+1,j0+1,n)*xdp+(1.0-xdp)*wrp(i0,j0+1,n)
        sre(n)=var2*ydp+(1.0-ydp)*var1
        var1=wip(i0+1,j0  ,n)*xdp+(1.0-xdp)*wip(i0,j0  ,n)
        var2=wip(i0+1,j0+1,n)*xdp+(1.0-xdp)*wip(i0,j0+1,n)
        sim(n)=var2*ydp+(1.0-ydp)*var1
       enddo
#else
       ptot=0.0_rlg
!
       do i=i0,i0+1
         do j=j0,j0+1
            do n=1,nbfes
             if(wrp(i,j,n)== valmanq .or.wip(i,j,n)== valmanq) goto 200
            end do
            pds=(dgfes-mod(abs(long-(g0fes+(i-1)*dgfes)),dgfes)) &
               *(dphifes-abs(latit-(phi0fes+(j-1)*dphifes)))
            ptot=ptot+pds
            do n=1,nbfes
             sre(n)=sre(n)+wrp(i,j,n)*pds
             sim(n)=sim(n)+wip(i,j,n)*pds
            enddo
 200      continue
        enddo
      enddo
#endif
!
       do n=1,nbfes
#if ! defined key_tide_shom_bilinear
        if(ptot.ne.0.0_rlg) then
         sre(n)=sre(n)/ptot
         sim(n)=sim(n)/ptot
#endif
         a0(nu(n))=sqrt(sre(n)*sre(n)+sim(n)*sim(n))
         if(sre(n)==0.0_rsh .and. sim(n)==0.0_rsh) then
          ph0(nu(n))=0.0_rsh
         else
          ph0(nu(n))=atan2(sim(n),sre(n))*180.0_rsh/pi
         end if
         if(ph0(nu(n)) < 0.0_rsh) ph0(nu(n))=ph0(nu(n))+360.0_rsh
#if ! defined key_tide_shom_bilinear
        else
         a0(nu(n))=valmanq
         ph0(nu(n))=valmanq
        endif
#endif
       enddo   
!
      end
