!**************************************************
!
       subroutine tide_harmcompo
!                           
!***************************************************
!
!     objet    : calcul des niveaux de la surface libre a la
!                limite par composition harmonique (type Simon).
!                Composition harmonique basee sur 8 a 13 ondes :
!                M2, S2, N2, K2, K1, O1, P1, Q1, MF, MM, M4, MS4, MN4 
!                issues du modele numerique de maree mondiale TPX07.
!                On peut construire un modele de maree partout dans le monde
!                a l aide de ces donnees. Possibilite d enrichir le spectre
!                par la methode d admittance.
!                Cette option est activee a l aide de la cle key_tide_tpx07
!                Pour l interpolation spatiale sur la grille du domaine d etude,
!                seuls les points ou la composante M2 est renseignee sont 
!                pris en compte.
!                Rappel : la composante M2 doit imperativement etre enregistree
!                en premier indice d onde pour pouvoir n utiliser que l onde M2
!                (option l_tide_M2harm)
!
!     appelant         : main
!     appeles          : ionc4_openr, ionc4_corres, ionc4_flimax
!     appeles          : ionc4_fljmax, ionc4_read_lat, ionc4_read_lon
!     appeles          : ionc4_read_subxy, admittances, interpolfes, tide_predic
!     appeles          : tide_data
!     canal d ecriture :
!     canal de lecture : file_tide_harmcp
!                      :
!     cle cpp          :
!                      :
!     variables in     : t, h0, valmanq, l_tide_M2harm, fisac, gwac, dgac, diac
!     variables in     : nbouc
!                      :
!     variables out    : l_presence (-> admittance)
!                      :
!     variables inout  :
!                      :
!     dernieres modifs : 28/03/2011
   !&E ** Reference : Use the OSU TOPEX/Poseidon Global Inverse Solution TPX0 :
   !&E                Egbert, Bennett, and Foreman,1994
   !&E                Egbert and Erofeeva, 2002 
   !&E
   !&E
   !&E ** History :
   !&E       !          Original code
   !&E       !  2011-03 (P. Douillet) creation from tideharmcomp_fes2004
   !&E       !  2011-11 (V. Garnier) upgrade to use ionc4
   !&E
!                      :
!*********************************************************************
!
!
USE parameters
use comvars2d
use commar
use comaree
use ionc4, only : ionc4_openr,ionc4_corres,ionc4_flimax,ionc4_fljmax, &
                  ionc4_read_lat,ionc4_read_lon,ionc4_read_subxy
implicit none
!
integer                                      :: nbtxp
character(len=lchain),dimension(:),pointer   :: mar1,mar2
character(len=4),dimension(:),pointer        ::nom_const
integer                                      :: delta
integer                                      :: i,j,n,kount
integer                                      :: ixmar,jxmar,cdf_id
integer                                      :: imarmin,imarmax,jmarmin,jmarmax
real(kind=rsh)                               :: xt,g0fes,phi0fes,dgfes,dphifes
integer                                      :: nc_err,var_id,dim_id
integer                                      :: start(2),tcount(2)
integer                                      ::  nf_inq_varid,nf_get_vara_int,nf_get_vara_text,nf_inq_dimid
real(kind=riolg),dimension(:),pointer        :: lonmar,latmar
real(kind=rsh)                               :: long,latit
real(kind=riosh),dimension(:,:),pointer      :: wra,wrg
real(kind=rsh),dimension(:,:,:),pointer      :: wrp,wip
real(kind=riosh),parameter                   :: valmanq_tide = 999.
!
external  nf_inq_varid
!
common /fes99/ g0fes,phi0fes,dgfes,dphifes,wrp,wip
common /fes99/ imarmin,imarmax,jmarmin,jmarmax
!
!************************************************************
!     Executable statements
!************************************************************
!
!   initialisation
!---------------------
!
      if(nbouc==0) then
!
! ouverture du fichier des constantes harmoniques
! reperage des indices correspondant a la zone geographique
!--------------------------------------------------------------
!
       call ionc4_openr(file_tide_harmcp)
       call ionc4_corres(file_tide_harmcp,cdf_id)
!
! lecture du nombre de constituants
       nc_err = nf_inq_varid(cdf_id,'NCONST',var_id)
       nc_err = nf_get_vara_int(cdf_id, var_id, 1,1, nbtxp)
       allocate(nom_const(nbtxp))
       nc_err =  nf_inq_dimid(cdf_id,'level',dim_id)

       nc_err = nf_inq_varid(cdf_id,'CONST',var_id)
       do i=1,nbtxp
        start(1) = 1
        start(2) = i
        tcount(1)=4
        tcount(2)=1
        nc_err =nf_get_vara_text(cdf_id,var_id,start,tcount,nom_const(i))
        write(*,*) 'iii',i,nom_const(i)
       enddo
       

       allocate(mar1(nbtxp)) 
       allocate(mar2(nbtxp)) 
       do i=1,nbtxp
         mar1(i)= "amp_" // TRIM(nom_const(i))
         write(*,*) 'eeee',i, TRIM(mar1(i))
       enddo
       do i=1,nbtxp
         mar2(i)= "pha_" // TRIM(nom_const(i))
         write(*,*) 'ffff',i, TRIM(mar2(i))
       enddo
            
!
       tide_nbharmcp=nbtxp
       l_presence(:)=.false.
       nu(:)=int(valmanq)
       a0sud(:,:)=valmanq
       ph0sud(:,:)=valmanq
       a0nord(:,:)=valmanq
       ph0nord(:,:)=valmanq
       a0ouest(:,:)=valmanq
       ph0ouest(:,:)=valmanq
       a0est(:,:)=valmanq
       ph0est(:,:)=valmanq
!
! initialisation du tableau des constituants harmoniques
!--------------------------------------------------------
!
       call tide_data
!
       nu(1)=im2
       nu(2)=is2
       nu(3)=ik2
       nu(4)=in2
       nu(5)=ik1
       nu(6)=io1
       nu(7)=ip1
       nu(8)=iq1
       nu(9)=imf
       nu(10)=imm
       nu(11)=im4
       nu(12)=ims4
       nu(13)=imn4
       l_presence(nu(1:nbtxp))=.true.    ! pour l admittance

       ixmar=ionc4_flimax(cdf_id)
       jxmar=ionc4_fljmax(cdf_id)
       allocate(latmar(1:jxmar),lonmar(1:ixmar))
       call ionc4_read_lat(file_tide_harmcp,latmar,1,jxmar)
       call ionc4_read_lon(file_tide_harmcp,lonmar,1,ixmar)
       do i=1,ixmar
         if (lonmar(i).gt.180.) lonmar(i)=lonmar(i)-360.
       enddo
!
       do i=1,ixmar-1
        if(gwac >= lonmar(i) .and. gwac < lonmar(i+1)) imarmin=i
        if(gwac+real(imax-1)*dgac >= lonmar(i)-epsilon .and. &
           gwac+real(imax-1)*dgac <= lonmar(i+1)+epsilon) imarmax=i+1
       enddo
!
       do j=1,jxmar-1
        if(fisac >= latmar(j) .and. fisac < latmar(j+1)) jmarmin=j
        if(fisac+real(jmax-1)*dfiac >= latmar(j)-epsilon .and. &
           fisac+real(jmax-1)*dfiac <= latmar(j+1)+epsilon) jmarmax=j+1
       enddo
!
       delta=3
       imarmin=max(1,imarmin-delta)
       imarmax=min(ixmar,imarmax+delta)
       jmarmin=max(1,jmarmin-delta)
       jmarmax=min(jxmar,jmarmax+delta)
       allocate(wrp(imarmin:imarmax,jmarmin:jmarmax,nbtxp))
       allocate(wip(imarmin:imarmax,jmarmin:jmarmax,nbtxp))
       allocate(wra(imarmin:imarmax,jmarmin:jmarmax))
       allocate(wrg(imarmin:imarmax,jmarmin:jmarmax))
!
! lecture des constantes harmoniques (phase et amplitude)
!--------------------------------------------------------------
!
       do n=1,nbtxp
        call ionc4_read_subxy(file_tide_harmcp,mar1(n),wra,&
	                      imarmin,imarmax,jmarmin,jmarmax,1,1)
        
        call ionc4_read_subxy(file_tide_harmcp,mar2(n),wrg,&
	                      imarmin,imarmax,jmarmin,jmarmax,1,1)
        do j=jmarmin,jmarmax
         do i=imarmin,imarmax
          if (wra(i,j) /= valmanq_tide .and. wrg(i,j) /= valmanq_tide) then
           wrp(i,j,n)= wra(i,j)*cos(-wrg(i,j)*pi/180)
           wip(i,j,n)=-wra(i,j)*sin(-wrg(i,j)*pi/180)
          else
           wrp(i,j,n)=valmanq
           wip(i,j,n)=valmanq
          endif
         enddo
        enddo
       end do
!
       g0fes=lonmar(imarmin)
       phi0fes=latmar(jmarmin)
       dgfes=lonmar(2)-lonmar(1)
       dphifes=latmar(2)-latmar(1)
!
! interpolation des constantes harmoniques le
! long des frontieres ouest et est.
!----------------------------------------------
!
       do j=jmin,jmax
        long=gwac
        latit=fisac+(j-1)*dfiac
        call interpolfes(long,latit,nbtxp,a0ouest(1,j),ph0ouest(1,j),h0_g(1,j))
        if(h0_g(1,j)/=-valmanq .and. a0ouest(im2,j)==valmanq) print*,j,'ouest'
        long=gwac+(imax-1)*dgac
        latit=fisac+(j-1)*dfiac
        call interpolfes(long,latit,nbtxp,a0est(1,j),ph0est(1,j),h0_g(imax,j)) 
        if(h0_g(imax,j)/=-valmanq .and. a0est(im2,j)==valmanq) print*,j,'est'
       end do
!
! interpolation des constantes harmoniques le long des frontieres nord
! et sud.
!----------------------------------------------
!
       do i=imin,imax
        long=gwac+(i-1)*dgac
        latit=fisac
        call interpolfes(long,latit,nbtxp,a0sud(1,i),ph0sud(1,i),h0_g(i,1))
        if(h0_g(i,1)/=-valmanq .and. a0sud(im2,i)==valmanq) print*,i,'sud'
        long=gwac+(i-1)*dgac
        latit=fisac+(jmax-1)*dfiac
        call interpolfes(long,latit,nbtxp,a0nord(1,i),ph0nord(1,i),h0_g(i,jmax))
        if(h0_g(i,jmax)/=-valmanq .and. a0nord(im2,i)==valmanq) print*,i,'nord'
       end do 
       deallocate(wip,wrp,latmar,lonmar,mar1,mar2)
!
! On procede a un calcul d admittance pour ajouter les ondes secondaires
!-------------------------------------------------
!
!      if(l_tide_admittance.and..not.l_tide_M2harm) then
!       call tide_admittances(tide_nbharmcp2)
!       tide_nbharmcp=tide_nbharmcp2
!      end if
      if(l_tide_M2harm) tide_nbharmcp=1    ! on ne conserve que l onde M2
!
! on reordonne les ondes pour decrire le spectre de facon compatible
! avec la liste SIMON, c est a dire par nu croissant
!-------------------------------------------------
!
     kount=1
     do while(kount/=0)
      kount=0
      do n=1,tide_nbharmcp-1
       if(nu(n) > nu(n+1)) then
        kount=kount+1
        xt=nu(n)
        nu(n)=nu(n+1)
        nu(n+1)=xt
       end if
      end do
     end do
!
   end if
!
! deroulement normal : tide_prediction harmonique de SIMON
!----------------------------------------------------
!
   call tide_predic
!
  end

!****************************************************************************
!
       subroutine interpolfes(long,latit,nbtxp,a0,ph0,prof)
!                           
!***************************************************************************
!
!     objet    : interpolation spatiale des constantes harmoniques en
!                complexe.
!
!     appelant         : tide_harmcompo
!     appeles          : 
!     canal d ecriture :
!     canal de lecture : 
!                      :
!     cle cpp          :
!                      :
!     variables in     : 
!                      :
!     variables out    : a0, ph0
!                      :
!     variables inout  :
!
!*********************************************************************
!
!
USE parameters
use commar
implicit none
!
logical                                :: iflag
integer                                :: nbtxp,i0,j0,i,j,n,nb,is
integer                                :: imarmin,imarmax,jmarmin,jmarmax
real(kind=rsh)                         :: prof,g0fes,phi0fes,dgfes,dphifes
real(kind=rsh)                         :: long,latit,x,y,dx,dy,x1,x2
real(kind=rsh),dimension(nbtxp)        :: sre,sim
real(kind=rsh),dimension(nmarmax)      :: a0,ph0
real(kind=rsh),dimension(:,:,:),pointer:: wrp,wip
     
common /fes99/ g0fes,phi0fes,dgfes,dphifes,wrp,wip
common /fes99/ imarmin,imarmax,jmarmin,jmarmax
!
!************************************************************
!     Executable statements
!************************************************************
!
       sre(:)=0.0_rsh
       sim(:)=0.0_rsh
!
!      long=long+360.0_rsh
       i0=int((long-g0fes)/dgfes)+imarmin
       j0=int((latit-phi0fes)/dphifes)+jmarmin
       x=g0fes+(i0-imarmin)*dgfes
       y=phi0fes+(j0-jmarmin)*dphifes
       dx=(long-x)/dgfes
       dy=(latit-y)/dphifes
!
       iflag=.true.
       do i=i0,i0+1
        do j=j0,j0+1
         do n=1,nbtxp
          if(wrp(i,j,n)== valmanq .or.wip(i,j,n)== valmanq) iflag=.false.
         end do
        enddo
       enddo
!
      if(iflag) then
       do n=1,nbtxp
        x1=(1.0_rsh-dx)*wrp(i0,j0,n)+dx*wrp(i0+1,j0,n)
        x2=(1.0_rsh-dx)*wrp(i0,j0+1,n)+dx*wrp(i0+1,j0+1,n)
        sre(n)=(1.0_rsh-dy)*x1+dy*x2
        x1=(1.0_rsh-dx)*wip(i0,j0,n)+dx*wip(i0+1,j0,n)
        x2=(1.0_rsh-dx)*wip(i0,j0+1,n)+dx*wip(i0+1,j0+1,n)
        sim(n)=(1.0_rsh-dy)*x1+dy*x2
       enddo
      end if
!
      if(prof/=-valmanq .and. .not.iflag) then
       is=0
       do while(.not.iflag)
        is=is+1
        sre(:)=0.0_rsh
        sim(:)=0.0_rsh
        nb=0
        do j=max(j0-is+1,jmarmin),min(j0+is,jmarmax)
         do i=max(i0-is+1,imarmin),min(i0+is,imarmax)
          do n=1,nbtxp
           if(wrp(i,j,n)/= valmanq .or.wip(i,j,n)/= valmanq) then
            sre(n)=sre(n)+wrp(i,j,n)
            sim(n)=sim(n)+wip(i,j,n)
            if(n==1) then
               nb=nb+1
               iflag=.true.
            end if
           end if
          end do
         enddo
        enddo
       enddo 
       sre(:)=sre(:)/nb
       sim(:)=sim(:)/nb
      end if
!
      do n=1,nbtxp
       if(iflag) then
        a0(nu(n))=sqrt(sre(n)*sre(n)+sim(n)*sim(n))  !/100.0
        if(sim(n)/=0.0_rsh .or.sre(n)/=0.0_rsh) then
	   a0(nu(n))=sqrt(sre(n)*sre(n)+sim(n)*sim(n))
	   ph0(nu(n))=atan2(sim(n),sre(n))*180.0_rsh/pi
	else
	   a0(nu(n))=0.0_rsh
	   ph0(nu(n))=0.0_rsh
	end if
        if(ph0(nu(n)) < 0.0_rsh) ph0(nu(n))=ph0(nu(n))+360.0_rsh
       else
        a0(nu(n))=valmanq
        ph0(nu(n))=valmanq
       endif
      enddo   
!
      end subroutine interpolfes
