 MODULE paramdebugwrite

   !!======================================================================
   !!                   ***  MODULE paramdebugwrite   ***
   !! 
   !!======================================================================
   !! * Modules used
   USE parameters,ONLY :ljmin,ljmax,imin,imax,limin,limax
   IMPLICIT NONE

   !! * Private variables
   INTEGER                    :: npmax
   INTEGER,ALLOCATABLE,          DIMENSION(:) :: proc_i,proc_j,proc_imin,proc_imax
   INTEGER,ALLOCATABLE,          DIMENSION(:) :: proc_jmin,proc_jmax
   CHARACTER(len=30),ALLOCATABLE,DIMENSION(:) :: proc_TAG
   CHARACTER(len=25)                    :: debug_filename
   INTEGER                              :: ncall=0,debug_nbouc=1,nbouc_fin=4

 END MODULE paramdebugwrite

   !!======================================================================
 
  SUBROUTINE setdebugwrite

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE setdebugwrite  ***
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE paramdebugwrite
   IMPLICIT NONE

   !! * Local declarations
   INTEGER          :: iii,i,n,k,i3,i2,i1
   CHARACTER(len=3) :: name1

   !!----------------------------------------------------------------------
   !! * Executable part

   iii=1100
   OPEN(iii,file='mpidebugprepare')
     READ(iii,800)npmax
     npmax=npmax-1
     allocate(proc_i(0:npmax))
     allocate(proc_j(0:npmax))
     allocate(proc_imin(0:npmax))
     allocate(proc_imax(0:npmax))
     allocate(proc_jmin(0:npmax))
     allocate(proc_jmax(0:npmax))
     allocate(proc_TAG(0:npmax))
   DO n=0,npmax
     READ(iii,800)i,proc_i(n),proc_j(n),proc_imin(n),proc_imax(n),proc_jmin(n),proc_jmax(n)
     IF(i.NE.n)THEN
       WRITE(*,*)'problem in the file, mpidebugprepare'
       STOP
     ENDIF
     k=proc_i(n)
     i3=k/100+48
     i2=(k-(i3-48)*100)/10+48
     i1=k-(i3-48)*100-(i2-48)*10+48
     name1=CHAR(i3)//CHAR(i2)//CHAR(i1)
     proc_TAG(n)=name1
     k=proc_j(n)
     i3=k/100+48
     i2=(k-(i3-48)*100)/10+48
     i1=k-(i3-48)*100-(i2-48)*10+48
     name1=CHAR(i3)//CHAR(i2)//CHAR(i1)
     proc_TAG(n)=TRIM(proc_TAG(n))//'.'//name1
   ENDDO
   CLOSE(iii)
800 FORMAT(7(i10.10,' '))
   ncall=ncall+1

   RETURN

  END SUBROUTINE setdebugwrite

   !!======================================================================

  SUBROUTINE DEBUGWRITE_4D_rsh(filenamein,A,ndim,nbiv)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEBUGWRITE_rsh  ***
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE paramdebugwrite
   USE comvars2d, ONLY : nbouc
#if defined key_MPI || defined key_MPI_2D
   USE toolmpi,   ONLY : iproc
#endif
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=25),                                     INTENT(in) :: filenamein
   INTEGER,                                               INTENT(in) :: ndim,nbiv
   REAL(kind=rsh),DIMENSION(nbiv,ndim,limin:limax,ljmin:ljmax),INTENT(in) :: A

   !! * Local declarations
   INTEGER           :: i,j,k,iii,iv
   INTEGER           :: i3,i2,i1,np
   CHARACTER(len=50) :: filename
   CHARACTER(len=3)  :: name1

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (ncall==0) CALL setdebugwrite
   IF (nbouc.LT.debug_nbouc) RETURN
   IF (nbouc.GT.nbouc_fin) STOP

   k=nbouc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name1=CHAR(i3)//CHAR(i2)//CHAR(i1)

#if defined key_MPI || defined key_MPI_2D
   DO np=iproc,iproc
#else
   DO np=0,npmax
#endif
     filename=TRIM(filenamein)//'.'//'nbouc='//TRIM(name1)//'.'//TRIM(proc_TAG(NP))
     iii=999
     OPEN(iii,file=filename)
     DO j=proc_jmin(np),proc_jmax(np)
       DO i=proc_imin(np),proc_imax(np)
         DO k=1,ndim
           WRITE(iii,*) k,i,j,(A(iv,k,i,j),iv=1,nbiv)
         ENDDO
       ENDDO
     ENDDO
     CLOSE(iii)
   ENDDO !for np

  END SUBROUTINE debugwrite_4D_rsh
   !!======================================================================

  SUBROUTINE DEBUGWRITE_rsh(filenamein,A,ndim)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEBUGWRITE_rsh  ***
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE paramdebugwrite
   USE comvars2d, ONLY : nbouc
#if defined key_MPI || defined key_MPI_2D
   USE toolmpi,   ONLY : iproc
#endif
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=25),                                     INTENT(in) :: filenamein
   INTEGER,                                               INTENT(in) :: ndim
   REAL(kind=rsh),DIMENSION(ndim,limin:limax,ljmin:ljmax),INTENT(in) :: A

   !! * Local declarations
   INTEGER           :: i,j,k,iii
   INTEGER           :: i3,i2,i1,np
   CHARACTER(len=50) :: filename
   CHARACTER(len=3)  :: name1

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (ncall==0) CALL setdebugwrite
   IF (nbouc.LT.debug_nbouc) RETURN
   IF (nbouc.GT.nbouc_fin) STOP

   k=nbouc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name1=CHAR(i3)//CHAR(i2)//CHAR(i1)

#if defined key_MPI || defined key_MPI_2D
   DO np=iproc,iproc
#else
   DO np=0,npmax
#endif
     filename=TRIM(filenamein)//'.'//'nbouc='//TRIM(name1)//'.'//TRIM(proc_TAG(NP))
     iii=999
     OPEN(iii,file=filename)
     DO j=proc_jmin(np),proc_jmax(np)
       DO i=proc_imin(np),proc_imax(np)
         DO k=1,ndim
           WRITE(iii,*) k,i,j,A(k,i,j)
         ENDDO
       ENDDO
     ENDDO
     CLOSE(iii)
   ENDDO !for np

  END SUBROUTINE debugwrite_rsh

   !!======================================================================

  SUBROUTINE DEBUGedge_j_rsh(filenamein,ix,iy,jx,jy,A,ndim,aimin,aimax,ajmin,ajmax)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEBUGedge_j_rsh  ***
   !&E
   !&E---------------------------------------------------------------------
!! * Modules used
   USE parameters
   USE paramdebugwrite
   USE comvars2d, ONLY : nbouc
#if defined key_MPI || defined key_MPI_2D
   USE toolmpi,   ONLY : iproc
#endif
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=25),                                     INTENT(in) :: filenamein
   INTEGER,                                               INTENT(in) :: ix,iy,jx,jy
   INTEGER,                                               INTENT(in) :: ndim
   INTEGER,                                               INTENT(in) :: aimin,aimax,ajmin,ajmax
   REAL(kind=rsh),DIMENSION(ndim,aimin:aimax,ajmin:ajmax),INTENT(in) :: A

   !! * Local declarations
   INTEGER                          :: i,j,k,iii
   INTEGER                          :: i3,i2,i1,np
   INTEGER                          :: jjdim,pjmin,pjmax,pimin,pimax,jj
   INTEGER,ALLOCATABLE,DIMENSION(:) :: jnum 
   CHARACTER(len=50)                :: filename
   CHARACTER(len=3)                 :: name1

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (ncall==0)CALL setdebugwrite
   if (nbouc.lt.debug_nbouc)return
   IF (nbouc.GT.nbouc_fin) STOP

   k=nbouc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name1=CHAR(i3)//CHAR(i2)//CHAR(i1)
   IF(ix.GT.0)THEN
     WRITE(*,*)'ix must be minus or zero'
     STOP
   ENDIF
   IF(jx.GT.0)THEN
     WRITE(*,*)'jx must be minus or zero'
     STOP
   ENDIF
#if defined key_MPI || defined key_MPI_2D
   DO np=iproc,iproc
#else
   DO np=0,npmax
#endif
     filename=TRIM(filenamein)//'.'//'nbouc='//TRIM(name1)//'.'//TRIM(proc_TAG(NP))
     iii=999
     OPEN(iii,file=filename)
     pjmin=proc_jmin(np)
     pjmax=proc_jmax(np)
     pimin=proc_imin(np)
     pimax=proc_imax(np)
     jjdim=0
     IF(pjmin.NE.jmin)jjdim=jjdim-jx
     IF(pjmax.NE.jmax)jjdim=jjdim+jy
     ALLOCATE(jnum(jjdim))
     jj=0
     IF(pjmin.NE.jmin)THEN
       DO j=pjmin+jx,pjmin-1
         jj=jj+1
         jnum(jj)=j
       ENDDO
     ENDIF
     IF(pjmax.NE.jmax)THEN
       DO j=pjmax+1,pjmax+jy
         jj=jj+1
         jnum(jj)=j
       ENDDO
     ENDIF
     WRITE(iii,*) pjmin,pjmax,pimin,pimax,ix,iy,jx,jy
     WRITE(iii,*) ndim,aimin,aimax,ajmin,ajmax
     WRITE(iii,*) jjdim
     WRITE(iii,*) jnum

     DO jj=1,jjdim
       j=jnum(jj)
       DO i=pimin+ix,pimax+iy
         DO k=1,ndim
           WRITE(iii,*) k,i,j,A(k,i,j)
         ENDDO
       ENDDO
     ENDDO
     CLOSE(iii)
     DEALLOCATE(jnum)
   ENDDO !for np

  END SUBROUTINE debugedge_j_rsh

   !!======================================================================

    SUBROUTINE DEBUGedge_i_rsh(filenamein,ix,iy,A,ndim,aimin,aimax,ajmin,ajmax)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEBUGedge_i_rsh  ***
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE paramdebugwrite
   USE comvars2d, ONLY : nbouc
#if defined key_MPI || defined key_MPI_2D
   USE toolmpi,   ONLY : iproc
#endif
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=25),                                     INTENT(in) :: filenamein
   INTEGER,                                               INTENT(in) :: ix,iy
   INTEGER,                                               INTENT(in) :: aimin,aimax,ajmin,ajmax
   INTEGER,                                               INTENT(in) :: ndim
   REAL(kind=rsh),DIMENSION(ndim,aimin:aimax,ajmin:ajmax),INTENT(in) :: A

   !! * Local declarations
   INTEGER                          :: i,j,k,iii
   INTEGER                          :: i3,i2,i1,np
   INTEGER                          :: iidim,pjmin,pjmax,pimin,pimax,ii
   INTEGER,ALLOCATABLE,DIMENSION(:) :: inum 
   CHARACTER(len=50)                :: filename
   CHARACTER(len=3)                 :: name1

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (ncall==0) CALL setdebugwrite
   if (nbouc.lt.debug_nbouc)return
   IF (nbouc.GT.nbouc_fin) STOP

   k=nbouc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name1=CHAR(i3)//CHAR(i2)//CHAR(i1)
   IF(ix.GT.0)THEN
     WRITE(*,*)'ix must be minus or zero'
     STOP
   ENDIF
#if defined key_MPI || defined key_MPI_2D
   DO np=iproc,iproc
#else
   DO np=0,npmax
#endif
     pjmin=proc_jmin(np)
     pjmax=proc_jmax(np)
     pimin=proc_imin(np)
     pimax=proc_imax(np)
     iidim=0
     IF(pimin.NE.imin)iidim=iidim-ix
     IF(pimax.NE.imax)iidim=iidim+iy
     ALLOCATE(inum(iidim))
     ii=0
     IF(pimin.NE.imin)THEN
       DO i=pimin+ix,pimin-1
         ii=ii+1
         inum(ii)=i
       ENDDO
     ENDIF
     IF(pimax.NE.imax)THEN
       DO i=pimax+1,pimax+iy
         ii=ii+1
         inum(ii)=i
       ENDDO
     ENDIF

     filename=TRIM(filenamein)//'.'//'nbouc='//TRIM(name1)//'.'//TRIM(proc_TAG(NP))
     iii=999
     OPEN(iii,file=filename)
     WRITE(iii,*) pjmin,pjmax,pimin,pimax,ix,iy
     WRITE(iii,*) ndim,aimin,aimax,ajmin,ajmax
     WRITE(iii,*) iidim
     WRITE(iii,*) inum
     DO j=pjmin,pjmax
       DO ii=1,iidim
         i=inum(ii)
         DO k=1,ndim
           WRITE(iii,*) k,i,j,A(k,i,j)
         ENDDO
       ENDDO
     ENDDO
     CLOSE(iii)
     DEALLOCATE(inum)
   ENDDO !for np

  END SUBROUTINE debugedge_i_rsh

   !!======================================================================

  SUBROUTINE DEBUGWRITE_alli_rsh(filenamein,A,ndim)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEBUGWRITE_alli_rsh  ***
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE paramdebugwrite
   USE comvars2d, ONLY : nbouc
#if defined key_MPI || defined key_MPI_2D
   USE toolmpi,   ONLY : iproc
#endif
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=25),                                   INTENT(in) :: filenamein
   INTEGER,                                             INTENT(in) :: ndim
   REAL(kind=rsh),DIMENSION(ndim,imin:imax,ljmin:ljmax),INTENT(in) :: A

   !! * Local declarations
   INTEGER           :: i,j,k,iii
   INTEGER           :: i3,i2,i1,np
   CHARACTER(len=50) :: filename
   CHARACTER(len=3)  :: name1

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (ncall==0)CALL setdebugwrite
   IF (nbouc.LT.debug_nbouc)RETURN
   IF (nbouc.GT.nbouc_fin) STOP

   k=nbouc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name1=CHAR(i3)//CHAR(i2)//CHAR(i1)

#if defined key_MPI || defined key_MPI_2D
   DO np=iproc,iproc
#else
   DO np=0,npmax
#endif
     filename=TRIM(filenamein)//'.'//'nbouc='//TRIM(name1)//'.'//TRIM(proc_TAG(NP))
     iii=999
     OPEN(iii,file=filename)
     DO j=proc_jmin(np),proc_jmax(np)
     ! do i=proc_imin(np),proc_imax(np)
       DO i=imin,imax
         DO k=1,ndim
           WRITE(iii,*) k,i,j,A(k,i,j)
         ENDDO
       ENDDO
     ENDDO
     CLOSE(iii)
   ENDDO !for np

  END SUBROUTINE debugwrite_alli_rsh

   !!======================================================================

  SUBROUTINE DEBUGWRITE_rlg(filenamein,A,ndim)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEBUGWRITE_rlg  ***
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE paramdebugwrite
   USE comvars2d, ONLY : nbouc
#if defined key_MPI || defined key_MPI_2D
   USE toolmpi,   ONLY : iproc
#endif
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=25),                                     INTENT(in) :: filenamein
   INTEGER,                                               INTENT(in) :: ndim
   REAL(kind=rlg),DIMENSION(ndim,limin:limax,ljmin:ljmax),INTENT(in) :: A

   !! * Local declarations
   INTEGER           :: i,j,k,iii
   INTEGER           :: i3,i2,i1,np
   CHARACTER(len=50) :: filename
   CHARACTER(len=3)  :: name1

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (ncall==0) CALL setdebugwrite
   IF (nbouc.LT.debug_nbouc) RETURN
   IF (nbouc.GT.nbouc_fin) STOP

   k=nbouc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name1=CHAR(i3)//CHAR(i2)//CHAR(i1)

#if defined key_MPI || defined key_MPI_2D
   DO np=iproc,iproc
#else
   DO np=0,npmax
#endif
     filename=TRIM(filenamein)//'.'//'nbouc='//TRIM(name1)//'.'//TRIM(proc_TAG(NP))
     iii=999
     OPEN(iii,file=filename)
     DO j=proc_jmin(np),proc_jmax(np)
       DO i=proc_imin(np),proc_imax(np)
         DO k=1,ndim
           WRITE(iii,*) k,i,j,A(k,i,j)
         ENDDO
       ENDDO
     ENDDO
     CLOSE(iii)
   ENDDO !for np

  END SUBROUTINE debugwrite_rlg

   !!======================================================================

  SUBROUTINE DEBUGWRITE_int(filenamein,A,ndim)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEBUGWRITE_int  ***
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE paramdebugwrite
   USE comvars2d, ONLY : nbouc
#if defined key_MPI || defined key_MPI_2D
   USE toolmpi,   ONLY : iproc
#endif
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=25),                              INTENT(in) :: filenamein
   INTEGER,                                        INTENT(in) :: ndim
   INTEGER,DIMENSION(ndim,limin:limax,ljmin:ljmax),INTENT(in) :: A

   !! * Local declarations
   INTEGER i,j,k,iii
   INTEGER i3,i2,i1,np
   CHARACTER(len=50)filename
   CHARACTER(len=3)name1

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (ncall==0) CALL setdebugwrite
   IF (nbouc.LT.debug_nbouc) RETURN
   IF (nbouc.GT.nbouc_fin) STOP

   k=nbouc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name1=CHAR(i3)//CHAR(i2)//CHAR(i1)

#if defined key_MPI || defined key_MPI_2D
   DO np=iproc,iproc
#else
   DO np=0,npmax
#endif
     filename=TRIM(filenamein)//'.'//'nbouc='//TRIM(name1)//'.'//TRIM(proc_TAG(NP))
     iii=999
     OPEN(iii,file=filename)
     DO j=proc_jmin(np),proc_jmax(np)
       DO i=proc_imin(np),proc_imax(np)
         DO k=1,ndim
           WRITE(iii,*) k,i,j,A(k,i,j)
         ENDDO
       ENDDO
     ENDDO
     CLOSE(iii)
   ENDDO !for np

  END SUBROUTINE debugwrite_int

