#ifdef key_substance

  SUBROUTINE parsub_advdifx(cv_wat,niter,nb_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdifx  ***
   !&E
   !&E ** Purpose : call the good advection scheme depending of the cpp key
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : cv_wat
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (P. Douillet) substances with settling velocities
   !&E       !                         issued from tssub_advdifx
   !&E
   !&E---------------------------------------------------------------------
   !! Modules used
   USE parameters, ONLY : rsh,imin,imax,jmin,ljminm2,ljmaxp2,jmax,kmax,ljmin,ljmax, &
                          liminm1,limaxp2,limin,limax,ljminm1,ljmaxp1,nb_var
#ifdef key_tssub_adv_ultimatequickestmacho
   USE comvars2d,  ONLY : nbouc
#endif

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,nb_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: cv_wat

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_tssub_adv_ultimatequickest
   CALL parsub_advdifx_uq(cv_wat,niter,nb_ws)

#elif key_tssub_adv_ultimatequickestmacho
   IF (MOD(nbouc,2) == 1) THEN
     CALL parsub_advdifx_uq(cv_wat,niter,nb_ws)
   ELSE
     CALL parsub_advdifx_uqm(cv_wat,niter,nb_ws)
   END IF

#elif key_tssub_adv_fourthorder
   CALL parsub_advdifx_fourthorder(cv_wat,niter,nb_ws)

#else
   CALL parsub_advdifx_quickupwind(cv_wat,niter,nb_ws)

#endif


  END SUBROUTINE parsub_advdifx
#endif
