#ifdef key_substance

  SUBROUTINE parsub_advdify(cv_wat,niter,nb_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdify  ***
   !&E
   !&E ** Purpose : call the good advection scheme depending of the cpp key
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : cv_wat
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (P. Douillet) substances with settling velocities
   !&E       !                         issued from tssub_advdifx
   !&E
   !&E---------------------------------------------------------------------
   !! Modules used
   USE parameters, ONLY : rsh,imin,imax,jmin,jmax,kmax,nb_var, &
                          limin,limax,liminm1,limaxp1,limaxp2, &
                          ljmin,ljmax,ljminm1,ljminm2,ljmaxp2
#ifdef key_tssub_adv_ultimatequickestmacho
   USE comvars2d,  ONLY : nbouc
#endif

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,nb_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: cv_wat

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_tssub_adv_ultimatequickest
   CALL parsub_advdify_uq(cv_wat,niter,nb_ws)

#elif key_tssub_adv_ultimatequickestmacho
   IF (MOD(nbouc,2) == 0) THEN
     CALL parsub_advdify_uq(cv_wat,niter,nb_ws)
   ELSE
     CALL parsub_advdify_uqm(cv_wat,niter,nb_ws)
   END IF

#elif key_tssub_adv_fourthorder
   CALL parsub_advdify_fourthorder(cv_wat,niter,nb_ws)

#else
   CALL parsub_advdify_quickupwind(cv_wat,niter,nb_ws)

#endif

  END SUBROUTINE parsub_advdify

#endif
