 MODULE parsubobc3dapply

#ifdef key_substance

   !!======================================================================
   !!                   ***  MODULE  parsubobc3dapply  ***
   !! Ocean dynamics:  apply open boundary conditions of variables
   !!                  (tracers other than temperature or salinity)
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC parsub_obc3d_apply             ! routine called by stepu.F90 stepv.F90
#ifdef key_agrif
   PUBLIC parsub_obc3d_apply_agrif       ! routine called by stepu.F90 stepv.F90
#endif

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE parsub_obc3d_apply(u,v,cv_wat,dtm,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_obc3d_apply  ***
   !&E
   !&E ** Purpose : open boundary conditions for tracers other than temperature and salinity
   !&E              For incoming conditions, used values are stored in the 
   !&E              obc_cv_wat_s, obc_cv_wat_n, obc_cv_wat_w and obc_cv_wat_e arrays,
   !&E              filled in the "subreaddat" routine.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : cv_wat,u,obc_cv_wat_w,hx,dx_u,obc_cv_wat_e,dy_v,
   !&E                     hy,obc_cv_wat_s,v,obc_cv_wat_n
   !&E 
   !&E ** Modified variables : cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2004-06
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2009-04 (A. Mano) MPI2D
   !&E       !  2010-04 (T. Odaka) MPI2D cyclic
   !&E       !  2011-02 (B. Thouvenin) correction for key_subs_part_eqsubdt key
   !&E                  WARNING : cyclic obc may not work anymore with MPI
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : dt,ig,hx,h0fond,dx_u,id,jb,hy,dy_v,jh,          &
                            l_obc_cycl_x,l_obc_cycl_y
   USE comsubstance, ONLY : obc_cv_wat_w,nvp,obc_cv_wat_e,                  &
                            obc_cv_wat_s,obc_cv_wat_n,                      &
                            iv_ws_max,nv_ws_max
   !USE obcogcm,      ONLY : obc_coefrel
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh

   !! * Arguments
   INTEGER,                                                       INTENT(in) :: num_ws
   REAL(KIND=rsh),                                                INTENT(in) :: dtm
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: u,v
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER                                                   :: i,j,k,l,iv
   REAL(KIND=rsh)                                            :: uu,vv,cou
   !REAL(KIND=rlg)                                            :: tf

   !!----------------------------------------------------------------------
   !! * Executable part

   !dtm=0.5_rlg*dt
   !tf=1.0_rlg/obc_coefrel


!$OMP SINGLE
   ! periodic conditions along x-axis
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
     DO l=1,nv_ws_max(num_ws)
       iv=iv_ws_max(num_ws,l)
!cval not sure it will work
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
     END DO
#else
     DO j=ljmin,ljmax
       DO l=1,nv_ws_max(num_ws)
          iv=iv_ws_max(num_ws,l)
   ! western boundary
          cv_wat(iv,:,imin+1,j)=cv_wat(iv,:,imax-1,j)
   ! eastern boundary
          cv_wat(iv,:,imax  ,j)=cv_wat(iv,:,imin+2,j)
       ENDDO
     END DO
#endif
   END IF

   ! periodic conditions along y-axis
   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
     DO l=1,nv_ws_max(num_ws)
       iv=iv_ws_max(num_ws,l)
!cval not sure it will work
       CALL_MPI ex_peri_y_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
     END DO
#else
       DO l=1,nv_ws_max(num_ws)
          iv=iv_ws_max(num_ws,l)
        ! southern boundary
          cv_wat(iv,:,limin:limax,jmin+1)=cv_wat(iv,:,limin:limax,jmax-1)
        ! northern boundary
          cv_wat(iv,:,limin:limax,jmax  )=cv_wat(iv,:,limin:limax,jmin+2)
       ENDDO
#endif
   END IF

   !----------------------------------------------
   ! on traite l advection quand le courant sort
   ! sur les limites gauche et droite
   !----------------------------------------------

   IF (.NOT.l_obc_cycl_x) THEN
     DO j=ljminp2,ljmaxm1

   ! western boundary
       i=ig(j)
       IF_MPI (limin==i-1) THEN
       IF (hx(i,j) /= -valmanq-h0fond) THEN
         DO k=1,kmax
           uu=u(k,i,j)
           IF (uu > 0.0_rsh) THEN
             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=obc_cv_wat_w(iv,k,i,j)
               !cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0-dtm/tf)+obc_cv_wat_w(iv,k,1,j)*dtm/tf
             ENDDO
           ELSE
             cou=uu*dtm/dx_u(i,j)
             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh+cou)-cou*cv_wat(iv,k,i+1,j)
             ENDDO
           END IF
         END DO
       END IF
       ENDIF_MPI

   ! eastern boundary
       i=id(j)
       IF_MPI (limax==i) THEN
       IF (hx(i-1,j) /= -valmanq-h0fond) THEN
         DO k=1,kmax
           uu=u(k,i-1,j)
           IF (uu < 0.0_rsh) THEN
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=obc_cv_wat_e(iv,k,i,j)
               !cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0-dtm/tf)+obc_cv_wat_e(iv,k,1,j)*dtm/tf
             ENDDO
           ELSE
             cou=uu*dtm/dx_u(i-1,j)
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh-cou)+cou*cv_wat(iv,k,i-1,j)
             ENDDO
           END IF
         END DO
       END IF
       ENDIF_MPI

     END DO
   END IF

   !----------------------------------------------
   ! on traite l advection quand le courant sort
   ! sur les limites basse et haute
   !----------------------------------------------

   IF (.NOT.l_obc_cycl_y) THEN
     DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

   ! southern boundary
       IF (j == jb(i)) THEN
         IF (hy(i,j) /= -valmanq-h0fond) THEN
           DO k=1,kmax
             vv=v(k,i,j)
             IF (vv > 0.0_rsh) THEN
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=obc_cv_wat_s(iv,k,i,j)
               !cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0-dtm/tf)+obc_cv_wat_s(iv,k,1,j)*dtm/tf
             ENDDO
             ELSE
               cou=vv*dtm/dy_v(i,j)
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh+cou)-cou*cv_wat(iv,k,i,j+1)
             ENDDO
             END IF
           END DO
         END IF
       END IF

   ! northern boundary
       IF (j == jh(i)) THEN
         IF (hy(i,j-1) /= -valmanq-h0fond) THEN
           DO k=1,kmax
             vv=v(k,i,j-1)
             IF (vv < 0.0_rsh) THEN
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=obc_cv_wat_n(iv,k,i,j)
               !cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0-dtm/tf)+obc_cv_wat_n(iv,k,1,j)*dtm/tf
             ENDDO
             ELSE
               cou=vv*dtm/dy_v(i,j-1)
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh-cou)+cou*cv_wat(iv,k,i,j-1)
             ENDDO
             END IF
           END DO
         END IF
       END IF

     END DO
     END DO
   END IF
!$OMP END SINGLE

!cval trop d echange si nombre d iteration varie avec les variables
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(cv_wat)

  END SUBROUTINE parsub_obc3d_apply

   !!======================================================================

#ifdef key_agrif

  SUBROUTINE parsub_obc3d_apply_agrif(num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_obc3d_apply_agrif  ***
   !&E
   !&E ** Purpose : open boundary conditions for tracers other than temperature and salinity
   !&E              For incoming conditions, used values are stored in the 
   !&E              obc_cv_wat_s, obc_cv_wat_n, obc_cv_wat_w and obc_cv_wat_e arrays,
   !&E              filled in the "subreaddat" routine.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E    
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2005-10
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvarp2d
   USE toolmpi
   USE agrifcommonid
   USE comsubstance, ONLY : nvp,iv_ws_max,nv_ws_max

   !! * Arguments
   INTEGER,                                                       INTENT(in) :: num_ws

   !! * Local declarations
   INTEGER  :: l,iv
   REAL(kind=rsh), DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2) :: sub_temp
   EXTERNAL :: interp3dsub

   !!----------------------------------------------------------------------
   !! * Executable part

   CALL Agrif_Bc_Variable(sub_temp,cv_wat_id,procname=interp3dsub)
   IF_MPI(limin==imin) THEN
    DO l=1,nv_ws_max(num_ws)
      iv=iv_ws_max(num_ws,l)
      cv_wat(iv,:,imin+1,ljminm1:ljmaxp2)=sub_temp(iv,:,imin+1,ljminm1:ljmaxp2)
    ENDDO
   ENDIF_MPI
   IF_MPI(limax==imax) THEN
    DO l=1,nv_ws_max(num_ws)
      iv=iv_ws_max(num_ws,l)
      cv_wat(iv,:,imax,ljminm1:ljmaxp2)=sub_temp(iv,:,imax,ljminm1:ljmaxp2)
    ENDDO
   ENDIF_MPI
   IF_MPI(ljmax==jmax) THEN
    DO l=1,nv_ws_max(num_ws)
      iv=iv_ws_max(num_ws,l)
      cv_wat(iv,:,liminm1:limaxp2,jmax  )=sub_temp(iv,:,liminm1:limaxp2,jmax  )
    ENDDO
   ENDIF_MPI
   IF_MPI(ljmin==jmin) THEN
    DO l=1,nv_ws_max(num_ws)
      iv=iv_ws_max(num_ws,l)
      cv_wat(iv,:,liminm1:limaxp2,jmin+1)=sub_temp(iv,:,liminm1:limaxp2,jmin+1)
    ENDDO
   ENDIF_MPI
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(cv_wat)


  END SUBROUTINE parsub_obc3d_apply_agrif
#endif
#endif

 END MODULE parsubobc3dapply
