  SUBROUTINE ts_stateeq_linear(sal,temp,boz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ts_stateeq_linear  ***
   !&E
   !&E ** Purpose : buoyancy estimation from a linear state equation
   !&E
   !&E ** Description : This option is used if l_stateeq_lin=.true.
   !&E                  in the namelist specified for the rank.
   !&E                  
   !&E
   !&E ** Called by : main, stepu, stepv, zoom
   !&E
   !&E ** Used ij-arrays : h0,temp,sal
   !&E 
   !&E ** Modified variables : boz
   !&E
   !&E ** Reference : 
   !&E
   !&E ** History :
   !&E       !  2004-10-06
   !&E       !  2008-07  (T. Duhaut) different law for baroclinic vortex key_Vortex_Ktest
   !&E       !  2009-04  (A. Mano) MPI2D
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,ONLY:h0
   USE comvars3d,ONLY:saliref_lin,tetaref_lin
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in)  :: sal,temp
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(out) :: boz

   !! * Local declarations
   INTEGER           :: i,j
   REAL(kind=rsh)    :: alpha,beta

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP parallel PRIVATE(alpha,beta,i,j)
   alpha=0.00016_rsh
   beta=0.0008_rsh

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp1,ljmax
     DO i=liminp1,limax
#ifdef key_casbcvortex
       IF(h0(i,j) > -valmanq) &
         boz(:,i,j)= -grav*(1030.0_rsh-0.28_rsh*temp(:,i,j)-1024.40_rsh)/1024.40_rsh
#elif defined key_equeta_temp
         boz(:,i,j)= temp(:,i,j)
#elif key_tssub_casadvlense
         boz(:,i,j)= boz(:,i,j)
#else
       IF(h0(i,j) > -valmanq) &
         boz(:,i,j)= grav*(-beta*(sal(:,i,j)-saliref_lin)+alpha*(temp(:,i,j)-tetaref_lin))
#endif
     END DO
   END DO
!$OMP END DO
!$OMP END PARALLEL

   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz(:,liminm1:limaxp2,ljminm1:ljmaxp2))

   PRINT_DBG*, 'exit TS_STATEEQ_LINEAR routine'

  END SUBROUTINE ts_stateeq_linear
