  SUBROUTINE tssub_advdify_fourthorder(sal,temp &
#ifdef key_substance
                                             ,c &
#endif
                                        )

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE tssub_advdify_fourthorder  ***
   !&E
   !&E ** Purpose : Estimate of meridional fluxes of temperature and salinity
   !&E              and other substances (advection and diffusion)
   !&E              Advection uses a fourth order centered scheme equivalent 
   !&E              to QUICKEST 
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                 - case of cyclicity in y : correct for northward propagation,
   !&E                    uncorrect for southward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hey,hy,fyeau,sal,temp,c(cv_wat),hzey,dy_v
   !&E                     fsaly,ftempy,fy
   !&E
   !&E ** Modified variables : fsaly,ftempy,fy
   !&E
   !&E ** Reference : Schema centre d ordre 4 corespondant au quickest 
   !&E
   !&E ** History :
   !&E           !  2010-10 (L. Debreu, T. Duhaut) original code
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

#ifdef key_tssub_adv_fourthorder

   !! * Modules used
   USE parameters
   USE comvars2d,   ONLY : jb,jh,hey,hy,h0fond,ky,hminkxky,dy_v,halfdt,l_obc_cycl_y,dx_v
#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : hzey
#endif
   USE comfluxdo,   ONLY : fsaly,fyeau
   USE cometemp,    ONLY : ftempy
#ifdef key_substance
   USE comsubstance,ONLY : nv_adv,fy,nvpi
   USE comsubstance,ONLY : c5
#endif
   USE_MPI toolmpi, ONLY : ex_j_rsh,ex_j_rlg
#ifdef key_tssub_adv_fourthorder
   USE_MPI toolmpi, ONLY : ex_i_rsh
   USE comvarp2d, ONLY : sal5,temp5  !VALERIE PROVISOIRE
#endif

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: sal,temp
#ifdef key_substance
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c
#endif

   !! * Local declarations
   INTEGER                   :: i,j,k
!   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: vv,vvv,cfl,d1,d2,d3,d4,kyb,  &
                                yas,yat,ybs,ybt,ycs,yct,yds,ydt, &
                                deltat,deltas
#ifdef key_substance
   INTEGER                              :: iv
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,yd,delta
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter TSSUB_ADVDIFY_FOURTHORDER.F90 routine'

!$OMP SINGLE
   sal5(:,liminm1:limaxp2,ljminm1:ljmaxp2) = sal(:,liminm1:limaxp2,ljminm1:ljmaxp2)
   temp5(:,liminm1:limaxp2,ljminm1:ljmaxp2) = temp(:,liminm1:limaxp2,ljminm1:ljmaxp2)
#ifdef key_substance
   c5(nb_var,:,liminm1:limaxp2,ljminm1:ljmaxp2) = c(nb_var,:,liminm1:limaxp2,ljminm1:ljmaxp2)
#endif
!$OMP END SINGLE
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,3,kmax,liminm2,limaxp3,ljminm2,ljmaxp3,sal5(:,liminm2:limaxp3,ljminm2:ljmaxp3))
   CALL_MPI ex_j_rsh(-2,3,kmax,liminm2,limaxp3,ljminm2,ljmaxp3,sal5(:,liminm2:limaxp3,ljminm2:ljmaxp3))
   CALL_MPI ex_i_rsh(-2,3,kmax,liminm2,limaxp3,ljminm2,ljmaxp3,temp5(:,liminm2:limaxp3,ljminm2:ljmaxp3))
   CALL_MPI ex_j_rsh(-2,3,kmax,liminm2,limaxp3,ljminm2,ljmaxp3,temp5(:,liminm2:limaxp3,ljminm2:ljmaxp3))
#ifdef key_substance
   CALL_MPI ex_i_rsh(-2,3,nb_var*kmax,liminm2,limaxp3,ljminm2,ljmaxp3,c5(:,:,liminm2:limaxp3,ljminm2:ljmaxp3))
   CALL_MPI ex_j_rsh(-2,3,nb_var*kmax,liminm2,limaxp3,ljminm2,ljmaxp3,c5(:,:,liminm2:limaxp3,ljminm2:ljmaxp3))
#endif
OMPMPI end master
OMPMPI barrier
#if ! defined key_offline
OMPMPI flush(sal5,temp5)
#endif
#ifdef key_substance
OMPMPI flush(c5)
#endif

!$OMP DO schedule(runtime),private(i,j,k                  &
!$OMP&,kyb,yas,yat,ybs,ybt,ycs,yct,yds,ydt,deltat,deltas  &
#ifdef key_substance
!$OMP& ,iv,ya,yb,yc,yd,delta                              &
#endif
!$OMP&,d1,d2,d3,d4,vv,vvv,cfl)

   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

       IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
         DO k=1,kmax
           IF (hey(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
#if ! defined key_offline
             fsaly(k,i,j)=0.0_rlg
             ftempy(k,i,j)=0.0_rlg
#endif
#ifdef key_substance
             fy(nvpi+1:nv_adv,k,i,j)=0.0_rsh
#endif
           ELSE
             kyb=ky
             IF (hey(i,j) < hminkxky) kyb=0.0_rsh ! no diffusion for small water depth
             vv=fyeau(k,i,j)
#if defined key_siggen || defined key_gencoord
             vvv = vv/(halfdt*hzey(k,i,j))
#else
             vvv = vv/(halfdt*hey(i,j))
#endif
             cfl = vvv*halfdt/dy_v(i,j)
             d1 = (1.0_rsh/24.0_rsh)*(-2.0_rlg+cfl+2.0_rlg*cfl**2-cfl**3)
             d2 = (1.0_rsh/24.0_rsh)*(14.0_rlg-15.0_rlg*cfl-2.0_rlg*cfl**2+3.0_rlg*cfl**3)
             d3 = (1.0_rsh/24.0_rsh)*(14.0_rlg+15.0_rlg*cfl-2.0_rlg*cfl**2-3.0_rlg*cfl**3)
             d4 = (1.0_rsh/24.0_rsh)*(-2.0_rlg-cfl+2.0_rlg*cfl**2+cfl**3)

#if ! defined key_offline
             ybs=sal5(k,i,j+1)
             ycs=sal5(k,i,j)
             ybt=temp5(k,i,j+1)
             yct=temp5(k,i,j)
             deltas=ybs-ycs
             deltat=ybt-yct
#endif
#ifdef key_substance
             yb(nvpi+1:nv_adv)=c5(nvpi+1:nv_adv,k,i,j+1)
             yc(nvpi+1:nv_adv)=c5(nvpi+1:nv_adv,k,i,j)
             delta(nvpi+1:nv_adv)=yb(nvpi+1:nv_adv)-yc(nvpi+1:nv_adv)
#endif

             IF (hey(i,j+1) > 0.0_rsh .AND. j+1 < jh(i)) THEN
#if ! defined key_offline
               yas=sal5(k,i,j+2)
               yat=temp5(k,i,j+2)
#endif
#ifdef key_substance
               ya(nvpi+1:nv_adv)=c5(nvpi+1:nv_adv,k,i,j+2)
#endif
             ELSE
#if ! defined key_offline
               yas=ybs
               yat=ybt
#endif
#ifdef key_substance
               ya(nvpi+1:nv_adv)=yb(nvpi+1:nv_adv)
#endif
             END IF

             IF (hey(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
#if ! defined key_offline
               yds=sal5(k,i,j-1)
               ydt=temp5(k,i,j-1)
#endif
#ifdef key_substance
               ya(nvpi+1:nv_adv)=c5(nvpi+1:nv_adv,k,i,j-1)
#endif
             ELSE
#if ! defined key_offline
               yds=ycs
               ydt=yct
#endif
#ifdef key_substance
               yd(nvpi+1:nv_adv)=yc(nvpi+1:nv_adv)
#endif
             END IF


#if ! defined key_offline

   ! flux of salinity advection
   ! --------------------------

!             IF ((ABS(dyds).LE.dsmall).OR.(ABS(dygs).LE.dsmall)) THEN
!               ! no horizontal gradient
!             ELSE
!             END IF
             fsaly(k,i,j) = vv*(d1*yas+d2*ybs+d3*ycs+d4*yds)

   ! flux of temperature advection
   ! -----------------------------

!             IF ((ABS(dydt).LE.dsmall).OR.(ABS(dygt).LE.dsmall)) THEN
!               ! no horizontal gradient
!             ELSE
!             END IF
             ftempy(k,i,j) = vv*(d1*yat+d2*ybt+d3*yct+d4*ydt)

   ! key_offline
#endif

#ifdef key_substance

   ! advection flux of other substances
   ! ----------------------------------

             DO iv=nvpi+1,nv_adv
!               IF ((ABS(dyd(iv)).LE.dsmall).OR.(ABS(dyg(iv)).LE.dsmall)) THEN
!                 ! no horizontal gradient
!               ELSE
!               END IF
               fy(iv,k,i,j) = vv*(d1*ya(iv)+d2*yb(iv)+d3*yc(iv)+d4*yd(iv))
             END DO
#endif

#if ! defined key_offline
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term alone
   ! --------------------
             ftempy(k,i,j) = ftempy(k,i,j)*dx_v(i,j)
             fsaly(k,i,j)  = fsaly(k,i,j)*dx_v(i,j)
#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
             ftempy(k,i,j)=ftempy(k,i,j)-hzey(k,i,j)*kyb*deltat/dy_v(i,j)*halfdt
             fsaly(k,i,j)=fsaly(k,i,j)-hzey(k,i,j)*kyb*deltas/dy_v(i,j)*halfdt              
#else
             ftempy(k,i,j)=ftempy(k,i,j)-hey(i,j)*kyb*deltat/dy_v(i,j)*halfdt
             fsaly(k,i,j)=fsaly(k,i,j)-hey(i,j)*kyb*deltas/dy_v(i,j)*halfdt              
#endif
#endif
#endif

#ifdef key_substance
             DO iv=nvpi+1,nv_adv
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term alone
   ! --------------------
               fy(iv,k,i,j) = fy(iv,k,i,j)*dx_v(i,j)
#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
               fy(iv,k,i,j) = fy(iv,k,i,j) - hzey(k,i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt
#else
               fy(iv,k,i,j) = fy(iv,k,i,j) - hey(i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt
#endif
#endif
             END DO
#endif

           END IF ! ends test on water depth
         END DO ! ends loop on k
       END IF ! for selecting right j loop
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
#if ! defined key_offline
   CALL_MPI ex_j_rlg(-1,0,kmax,limin,limax,ljminm1,ljmax,fsaly(:,limin:limax,ljminm1:ljmax))
   CALL_MPI ex_j_rlg(-1,0,kmax,limin,limax,ljminm1,ljmax,ftempy(:,limin:limax,ljminm1:ljmax))
#endif
#ifdef key_substance
   CALL_MPI ex_j_rsh(-1,0,nv_adv*kmax,limin,limax,ljminm1,ljmax,fy(:,:,limin:limax,ljminm1:ljmax))
#endif
OMPMPI end master
OMPMPI barrier
#if ! defined key_offline
OMPMPI flush(fsaly,ftempy)
#endif
#ifdef key_substance
OMPMPI flush(fy)
#endif

   PRINT_DBG*, 'exit TSSUB_ADVDIFY_FOURTHORDER.F90 routine'

#endif

  END SUBROUTINE tssub_advdify_fourthorder
