!!==============================================================================

#ifdef key_sedim
      SUBROUTINE sed_output(nb,tect,x,fileresu1)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_output  ***
   !&E
   !&E ** Purpose :
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : output.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-07 (V. Garnier) Original code extracted from sedim
   !&E       !  01-2011 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   !!   ------------
   USE parameters,  ONLY : lchain,imin,imax,jmin,jmax,ksdmin,ksdmax,rsh,rlg,riosh,  &
                           limin,limax,ljmin,ljmax,liminp1,limaxm1,ljminp1,ljmaxm1, &
                           liminm1,ljminm1,limaxp1,ljmaxp1,limaxp2,ljmaxp2,         &
                           rg_valmanq_io,valmanq,kmax
   USE ionc4,       ONLY : ionc4_createfile,ionc4_gatt,ionc4_createvar,ionc4_write_sig,&
                           ionc4_vatt_missing,ionc4_write_time,ionc4_write_xyt,&
                           ionc4_write_zxyt,ionc4_write_xy,ionc4_sync,ionc4_close
   USE comsubstance, ONLY : nv_adv,name_var,irk_fil
   USE comvars2d,    ONLY : lon2d_g,lat2d_g,lon2d,lat2d,date_ref,h0,ig,id,jb,jh,&
                            h0fond,out_dir,cdate,suffix,nbouc,name_out_h0,name_out_xe,&
                            riog_valid_min_h0,riog_valid_max_h0
   USE_MPI toolmpi,  ONLY : ADD_TAG,IPROC,NPROCS,ADD_ALL_MPI_INT
   USE outtools,     ONLY : out_write_xytvar,out_write_zxytvar,out_grid
   USE output,       ONLY : iextmin,iextmax,jextmin,jextmax,     &
                            igextmin,igextmax,jgextmin,jgextmax, &
                            stepi,stepj,stepk,l_filebydate,tsort,&
                            tfinsor,l_out_nc4par,l_out_pack
   USE sedimento,    ONLY : dzs,cv_sed,tenfon,tenfonc,tenfonw, &
                            ksmi,ksma,l_morphocoupl,filoutsed,&
                            name_out_eptot,name_out_nbniv,name_out_dzs,name_out_tenfon,&
                            name_out_tenfonc,name_out_tenfonw,&
                            riog_valid_min_eptot,riog_valid_max_eptot,riog_valid_min_nbniv, &
                            riog_valid_max_nbniv,riog_valid_min_dzs,riog_valid_max_dzs,     &
                            riog_valid_min_tenfon,riog_valid_max_tenfon
#ifdef key_wave
   USE comvars2d,    ONLY : riog_valid_min_wave_hs,riog_valid_max_wave_hs, &
                            riog_valid_min_wave_tp,riog_valid_max_wave_tp, &
                            name_out_wave_hs,name_out_wave_tp
   USE comwave,      ONLY : name_wave_hs,name_wave_tp,wave_tp,wave_hs
#endif

   IMPLICIT NONE

#ifdef key_MPI_2D
   INCLUDE 'mpif.h'
#endif

   !! * Arguments
   INTEGER       ,       INTENT(in) :: nb
   REAL(KIND=rlg),       INTENT(in) :: tect
   REAL(KIND=rsh),DIMENSION(     limin:limax,ljmin:ljmax),INTENT(in)     :: x
   CHARACTER(LEN=lchain),INTENT(in) :: fileresu1

   !! * Local declarations
   !!   ------------------
   INTEGER               :: i,j,k,l,ifois,lnam
   INTEGER               :: grp_world,grp_zoom,comm_zoom,ncpu,ip,ierr_mpi
   INTEGER, DIMENSION(:), ALLOCATABLE  :: numrank,listrank
   CHARACTER(len=4)       :: nomc
   CHARACTER(len=12)      :: chaine
   CHARACTER(len=lchain)  :: name_out_cvsed
   CHARACTER(LEN=lchain) :: chain
   LOGICAL,         DIMENSION(limin:limax,ljmin:ljmax)    :: mask_h0
   REAL(KIND=rsh),  DIMENSION(limin:limax,ljmin:ljmax)    :: var2D
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)    :: h0_local
   REAL(kind=riosh)::        riog_valid_min_cvsed,riog_valid_max_cvsed
!valerie que faut il prendre la ? aterme mettre le ksdmax ou plutot une autre variable
! dans fichier resultat ecrire un nombre de couche variable selon config
   REAL(KIND=riosh),DIMENSION(2,limin:limax,ljmin:ljmax)  :: tecrit3D
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)    :: tecrit2D
!     REAL(KIND=rsh),dimension(ksdmin-1:ksdmax+1)            :: sig2
!cval compatibilite ionc_creer      REAL(KIND=rsh),DIMENSION(1:2)                          :: niv
   REAL(KIND=riosh),DIMENSION(0:3)                        :: niv
   DATA ifois /-1/
   
   !! * Executable part
   !!   ---------------

   PRINT_DBG*, 'enter sed_output'

   ! Masks
   ! -----

   h0_local(:,:)=h0(limin:limax,ljmin:ljmax)
   mask_h0(:,:)=.TRUE.
   DO j=ljmin,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       IF ( h0_local(i,j)+x(i,j) > h0fond ) mask_h0(i,j)=.FALSE.
     END DO
   END DO

   comm_zoom=0
#ifdef key_MPI_2D
   comm_zoom=MPI_COMM_VAR
   IF ( (nbouc == 0 .AND. .NOT.l_filebydate(nb)) .OR. (l_filebydate(nb) .AND. tect >= tsort(nb)) ) THEN
     ! what a code ! Not a clever code probably but not clear in my head at the moment
     ! Count the number of processors active over the area to be saved
     ncpu=0
     IF ( jextmin(nb) <= jextmax(nb) .AND. iextmin(nb) <= iextmax(nb) ) ncpu=ncpu+1
     CALL ADD_ALL_MPI_INT(ncpu)
     ! Save the number of active processors
     ALLOCATE (numrank(ncpu))
     ALLOCATE (listrank(nprocs))
     numrank(:)=0   ! must be initialized if the proc 0 is active on the saved zoom
     listrank(:)=0
     ! which cpu is active ?
     IF ( jextmin(nb) <= jextmax(nb) .AND. iextmin(nb) <= iextmax(nb) ) listrank(1)=iproc
     ! all cpus get the number of active cpus plus sleeping cpus (set to 0)
     CALL MPI_ALLGATHER(listrank(1),1,MPI_INTEGER,listrank,1,MPI_INTEGER,MPI_COMM_VAR,ierr_mpi)
     ! all cpus keep the active cpus only
     IF (ncpu/=nprocs) THEN
       ip=0
       DO i=1,nprocs
         IF_MPI ( listrank(i) /= 0 ) THEN
         ip=ip+1
         numrank(ip)=listrank(i)
         ENDIF
       END DO
       ! Group relative to the MPI_COMM_VAR
       CALL MPI_COMM_GROUP(MPI_COMM_VAR,grp_world,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_GROUP_INCL(grp_world,ncpu,numrank,grp_zoom,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_COMM_CREATE(MPI_COMM_VAR,grp_zoom,comm_zoom,ierr_mpi)
     ENDIF
   ENDIF
#endif
   
   IF ( jextmin(nb) <= jextmax(nb) .AND. iextmin(nb) <= iextmax(nb) ) THEN

      ifois=ifois+1

      IF ( (ifois == 0 .AND. .NOT.l_filebydate(nb)) .OR. (l_filebydate(nb) .AND. tect >= tsort(nb)) ) THEN
!
!       sig2(0)=-0.001
!       do k=ksdmin,ksdmax
!         sig2(k)=(k-1)/(ksdmax-ksdmin)
!       end do
!       sig2(ksdmax+1)=1.0
!
!
        niv(0)=0
        niv(1)=1
        niv(2)=2
        niv(3)=0
!       on ne veut que la couche de fond et de surface
!       donc pas=1, et kmax=2
!
        IF (l_filebydate(nb)) THEN
          filoutsed=TRIM(out_dir)//TRIM(fileresu1)//'_'//TRIM(suffix)//'_sed'
          filoutsed=TRIM(filoutsed)//'_'//cdate(7:10)//cdate(4:5)//cdate(1:2)//cdate(12:13)//cdate(15:16)//cdate(18:19)
          filoutsed=TRIM(filoutsed)//'.nc'
        ELSE
          lnam=index(fileresu1,'.nc')-1
          filoutsed=TRIM(fileresu1(1:lnam))//'_sed.nc'
        END IF
        IF_MPI( .NOT. l_out_nc4par) CALL ADD_TAG(filoutsed,LEN(filoutsed))
        IF (.NOT. l_out_nc4par) THEN
          CALL ionc4_createfile(filoutsed, &
                           lon2d(iextmin(nb):iextmax(nb):stepi(nb),jextmin(nb):jextmax(nb):stepj(nb)), &
                           lat2d(iextmin(nb):iextmax(nb):stepi(nb),jextmin(nb):jextmax(nb):stepj(nb)), &
                           iextmin(nb),iextmax(nb),stepi(nb),   &
                           jextmin(nb),jextmax(nb),stepj(nb),   &
                           1,2,1,2,0)
        ELSE
          CALL ionc4_createfile(filoutsed, &
                           lon2d_g(igextmin(nb):igextmax(nb):stepi(nb),jgextmin(nb):jgextmax(nb):stepj(nb)), &
                           lat2d_g(igextmin(nb):igextmax(nb):stepi(nb),jgextmin(nb):jgextmax(nb):stepj(nb)), &
                           igextmin(nb),igextmax(nb),stepi(nb), &
                           jgextmin(nb),jgextmax(nb),stepj(nb), &
                           1,2,1,2,0,l_out_nc4par = .TRUE.,comm_active=comm_zoom)
        ENDIF

   ! save attribute of the spatial extension of the domain
   ! -----------------------------------------------------
     CALL ionc4_gatt(filoutsed,'global_imin',igextmin(nb))
     CALL ionc4_gatt(filoutsed,'global_imax',igextmax(nb))
     CALL ionc4_gatt(filoutsed,'global_jmin',jgextmin(nb))
     CALL ionc4_gatt(filoutsed,'global_jmax',jgextmax(nb))

   ! Add attributes to follow OCO netCDF convention
     CALL out_netcdf_conv(filoutsed,nb,comm_zoom,igextmin(nb),igextmax(nb),jgextmin(nb),jgextmax(nb))

     IF_MPI (l_out_nc4par .AND. comm_zoom/=MPI_COMM_VAR) CALL MPI_COMM_FREE(comm_zoom,ierr_mpi)

   ! indispensable pour concatenation pas terrible a revoir
     CALL ionc4_createvar(filoutsed,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                          standard_name= 'ocean_s_variable',&
#else
                          standard_name= 'ocean_sigma_variable',&
#endif
                          l_auxcoordinate=.false.,&
                          valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io,&
                          dims= 'z', l_out_nc4par = l_out_nc4par)
     CALL ionc4_write_sig(filoutsed,'SIG',niv,2)

     IF (l_morphocoupl) THEN
        CALL ionc4_createvar(filoutsed,TRIM(name_out_h0),'m',                   &
                             long_name = 'bathymetrie par rapport au niv moyen',&
                             dims = 'xyt',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_h0,&
                             valid_max = riog_valid_max_h0,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)
     ELSE
        CALL ionc4_createvar(filoutsed,TRIM(name_out_h0),'m',                   &
                             long_name = 'bathymetrie par rapport au niv moyen',&
                             dims = 'xy',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_h0,&
                             valid_max = riog_valid_max_h0,&
                             l_pack = .FALSE. , &
                             l_out_nc4par = l_out_nc4par)
        CALL ionc4_write_xy(filoutsed,name_out_h0,h0_local,limin,limax,ljmin,ljmax,&
                            fill_value = -rg_valmanq_io, l_out_nc4par = l_out_nc4par)
     END IF

        CALL ionc4_createvar(filoutsed,name_out_eptot,'m',                           &
                             long_name = 'epaisseur totale du sediment',&
                             dims = 'xyt',&
                             fill_value = rg_valmanq_io,&
                             valid_min = riog_valid_min_eptot,&
                             valid_max = riog_valid_max_eptot,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)

        CALL ionc4_createvar(filoutsed,name_out_nbniv,'quantite',                           &
                             long_name = 'Nombre de niveaux',&
                             dims = 'xyt',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_nbniv,&
                             valid_max = riog_valid_max_nbniv,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)

        CALL ionc4_createvar(filoutsed,name_out_dzs,'m',                           &
                             long_name = 'epais de la couche',&
                             dims = 'zxyt',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_dzs,&
                             valid_max = riog_valid_max_dzs,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)
                    
        DO l = -1,nv_adv ! pour sauver T et S
          IF (l > 0) chaine=TRIM(name_var(irk_fil(l)))
          IF (l==-1) chaine='temperature'
          IF (l==0)  chaine='salinite'

          IF (l > 0) WRITE(nomc,10) l
          IF (l==-1) nomc='TEM'
          IF (l==0) nomc='SAL'
          name_out_cvsed=nomc
          CALL ionc4_createvar(filoutsed,name_out_cvsed,'u.s.i',                           &
                               long_name = chaine,&
                               dims = 'zxyt',&
                               fill_value = rg_valmanq_io,&
!cval                               valid_min = riog_valid_min_cvsed,&
!cval                               valid_max = riog_valid_max_cvsed,&
                               l_pack = .FALSE. ,&
                               l_out_nc4par = l_out_nc4par)
        ENDDO
        
        CALL ionc4_createvar(filoutsed,name_out_tenfon,'N.m-2',                           &
                             long_name = 'tension de frottement sur le fond pour l"erosion',&
                             dims = 'xyt',&
                             fill_value = rg_valmanq_io,&
                             valid_min = riog_valid_min_tenfon,&
                             valid_max = riog_valid_max_tenfon,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)

#ifdef key_wave
        CALL ionc4_createvar(filoutsed,name_out_tenfonc,'N.m-2',                           &
                             long_name = 'tension de frottement due au courant',&
                             dims = 'xyt',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_tenfon,&
                             valid_max = riog_valid_max_tenfon,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)
        
        CALL ionc4_createvar(filoutsed,name_out_tenfonw,'N.m-2',                           &
                             long_name = 'tension de frottement due aux vagues',&
                             dims = 'xyt',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_tenfon,&
                             valid_max = riog_valid_max_tenfon,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)
        
        CALL ionc4_createvar(filoutsed,name_out_wave_tp,'s',                           &
                             long_name = 'peak period',&
                             dims = 'xyt',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_wave_tp,&
                             valid_max = riog_valid_max_wave_tp,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)
        
        CALL ionc4_createvar(filoutsed,name_out_wave_hs,'m',                           &
                             long_name = 'hauteur significative',&
                             dims = 'xyt',&
                             fill_value = -rg_valmanq_io,&
                             valid_min = riog_valid_min_wave_hs,&
                             valid_max = riog_valid_max_wave_hs,&
                             l_pack = .FALSE. ,&
                             l_out_nc4par = l_out_nc4par)
#endif


     END IF

     IF ( (ifois > 0 .AND. .NOT.l_filebydate(nb)) .OR. (l_filebydate(nb) .AND. tect >= tsort(nb) .AND. tect <= tfinsor(nb)) ) THEN
     
       CALL ionc4_write_time(filoutsed,0,tect)
       IF (l_morphocoupl) THEN
         DO j = ljmin,ljmax
           DO i = limin,limax
             DO k = ksdmin,ksdmax
               h0_local(i,j)=h0(i,j)
             ENDDO
           ENDDO
         ENDDO
         CALL ionc4_write_xyt(filoutsed,name_out_h0,h0_local,limin,limax,ljmin,ljmax,0,&
                              fill_value=-rg_valmanq_io, l_out_nc4par = l_out_nc4par)
       END IF
    
       DO j = ljmin,ljmax
         DO i = limin,limax
           var2D(i,j)=0.0_rsh
           DO k = ksdmin,ksdmax
             var2D(i,j)=var2D(i,j)+dzs(k,i,j)
           END DO
         END DO
       END DO
       CALL out_write_xytvar(filoutsed,name_out_eptot,rg_valmanq_io,mask_h0, &
                             var2D(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

       DO j = ljmin,ljmax
         DO i = limin,limax
           var2D(i,j)=ksma(i,j)
         END DO
       END DO
       CALL out_write_xytvar(filoutsed,name_out_nbniv,rg_valmanq_io,mask_h0, &
                             var2D(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

       DO j = ljmin,ljmax
         DO i = limin,limax
           IF(mask_h0(i,j)) THEN
             tecrit3D(:,i,j)=rg_valmanq_io
           ELSE 
             IF(ksma(i,j) > 1 .AND. ksma(i,j) /= INT(rg_valmanq_io)) THEN
               tecrit3D(2,i,j)=dzs(ksma(i,j),i,j)   
             ELSE
               tecrit3D(2,i,j)=rg_valmanq_io
             END IF      
             tecrit3D(1,i,j)=dzs(1,i,j)
           ENDIF
         END DO 
       END DO
       IF (l_out_nc4par) THEN
         CALL ionc4_write_zxyt(filoutsed,TRIM(ADJUSTL(ADJUSTR(name_out_dzs))),tecrit3D,    &
                limin,limax,ljmin,ljmax,2,0,fill_value=rg_valmanq_io,l_out_nc4par = l_out_nc4par)
       ELSE
         CALL ionc4_write_zxyt(filoutsed,TRIM(ADJUSTL(ADJUSTR(name_out_dzs))),tecrit3D,    &
                limin,limax,ljmin,ljmax,2,0,fill_value=rg_valmanq_io)
       ENDIF
 
       DO l = -1,nv_adv
         DO j = ljmin,ljmax
           DO i = limin,limax
             IF(mask_h0(i,j)) THEN
               tecrit3D(:,i,j)=rg_valmanq_io
             ELSE 
               IF(ksma(i,j) > 1 .AND. ksma(i,j) /= INT(rg_valmanq_io)) THEN
                 tecrit3D(2,i,j) = cv_sed(l,ksma(i,j),i,j)
               ELSE
                 tecrit3D(2,i,j)=rg_valmanq_io
               END IF
               IF(ksmi(i,j) /= INT(rg_valmanq_io)) THEN
                 tecrit3D(1,i,j) = cv_sed(l,MAX(ksmi(i,j),1),i,j)
               ELSE
                 tecrit3D(1,i,j)=rg_valmanq_io
               END IF
             ENDIF
           ENDDO
         ENDDO
         IF (l > 0) WRITE(nomc,10) l
         IF (l==-1) nomc='TEM'
         IF (l==0) nomc='SAL'
         name_out_cvsed=nomc

         IF (l_out_nc4par) THEN
           CALL ionc4_write_zxyt(filoutsed,TRIM(ADJUSTL(ADJUSTR(name_out_cvsed))),tecrit3D,    &
                limin,limax,ljmin,ljmax,2,0,fill_value=rg_valmanq_io,l_out_nc4par = l_out_nc4par)
         ELSE
           CALL ionc4_write_zxyt(filoutsed,TRIM(ADJUSTL(ADJUSTR(name_out_cvsed))),tecrit3D,    &
                limin,limax,ljmin,ljmax,2,0,fill_value=rg_valmanq_io)
         ENDIF
       ENDDO

       CALL out_write_xytvar(filoutsed,name_out_tenfon,rg_valmanq_io,mask_h0, &
                             tenfon(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

#ifdef key_wave
       CALL out_write_xytvar(filoutsed,name_out_tenfonc,rg_valmanq_io,mask_h0,&
                          tenfonc(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(filoutsed,name_out_tenfonw,rg_valmanq_io,mask_h0,&
                          tenfonw(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(filoutsed,name_out_wave_tp,rg_valmanq_io,mask_h0,&
                          wave_tp(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(filoutsed,name_out_wave_hs,rg_valmanq_io,mask_h0,&
                          wave_hs(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#endif
     END IF

     IF (l_filebydate(nb)) THEN
       CALL ionc4_close(filoutsed)
     ELSE
       ! To write the data on the disk and not loose data in case of run crash
       CALL ionc4_sync(filoutsed)
     ENDIF

   END IF

   10 FORMAT('C',i2.2)

   PRINT_DBG*, 'exit sed_output'

#else
      SUBROUTINE sed_output
#endif

  END SUBROUTINE sed_output

   !!===========================================================================
