 MODULE comvars3d
   !!======================================================================
   !!                   ***  MODULE comvars3d  ***
   !! 
   !! ** Description :
   !! 
   !! ** Reference :
   !! 
   !! ** History :
   !!
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters

   !! * Shared or public module variables
   
   !-------------------------------------------------------------------
   ! l_equisig : logique positionne a .true. pour repartir uniformement les
   !             niveaux sigma sur la verticale.
   ! sig       : tableau contenant la valeur de sigma sur les differents
   !             niveau de calcul des points (uz, vz, sal, temp...). Cette
   !             valeur est interpretee comme un pourcentage de la colonne
   !             d eau sous le niveau considere (valeur comprise entre 0 et 1).
   ! dsigu     : epaisseur sigma de la couche encadrant le niveau de calcul uz.
   ! dsigw     : epaisseur sigma de la couche encadrant le niveau de calcul wz
   !             (ou nz, kz, ect).
   LOGICAL                                 :: l_equisig
   REAL(kind=rsh),DIMENSION(0:kmax+1)      :: sig
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:) :: dsigu,dsigw

   !-------------------------------------------------------------------
   ! l_stability : logique positionne a .true. pour melanger les colonnes d eau
   !             trop instables.
   ! turb_nbeq : nombre d equations dans le modele de fermeture turbulente (0,1 ou 2)
   ! turb_0eq_option : choix de l option pour le modele a zero equation
   !              1- Kz et Nz constants
   !              2- Csanady
   !              3- Quetin
   !              4- Pacanowski et Philander
   ! kzinit    : valeur initiale du coefficient vertical de melange en m2/s.
   ! nzinit    : valeur initiale du coefficient vertical de viscosite en m2/s.
   ! kz        : tableau du coefficient turbulent vertical de melange en m2/s
   !             estime au cours du calcul en 3d.
   ! kz        : tableau du coefficient turbulent vertical de viscosite en m2/s
   !             estime au cours du calcul en 3d.
   ! bgdiff    : background diffusivity : valeur en deca de laquelle ne descendent
   !             ni les kz ni les nz. en m2/s
   LOGICAL        :: l_stability
   INTEGER        :: turb_nbeq,turb_0eq_option,turb_2eq_option
   REAL(kind=rsh) :: kzinit,nzinit,bgdiff
   REAL(kind=rsh),DIMENSION(:,:,:),ALLOCATABLE        :: kz,nz

   !-------------------------------------------------------------------
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: wz
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:)     :: sigw
! S.Petton 09/2013 : Utilisation d un profil moyen
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: rho2,boz2,qp1,rho !MANGA  
#ifdef  key_diag_physwz
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: wz_phys
#endif
   !-------------------------------------------------------------------
   LOGICAL        :: l_stateeq_lin
   REAL(kind=rsh) :: saliref,tetaref,rhoref,saliref_lin,tetaref_lin

   !-------------------------------------------------------------------
   ! seuil pour choix schema centre ou upstream sur la verticale (key_tssub_wquickupwind uniquement)
   REAL(kind=rsh) :: qmaxz

 CONTAINS

   !!==============================================================================

  SUBROUTINE alloc_comvars3d

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE alloc_comvars3d  ***
   !&E  
   !&E--------------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !!----------------------------------------------------------------------
   !! * Executable part

   ALLOCATE(nz(kmax,limin  :limaxp1,ljmin  :ljmaxp1) )
   !ALLOCATE(kz(kmax,limin:limax  ,ljmin:ljmax  ) )
   ! larger extension because of traj_3d routine
   ! might be not sufficient because of the medium location
   ALLOCATE(kz(kmax,liminm1:limaxp1,ljminm1:ljmaxp1) )
   ALLOCATE(sigw(0:kmax+1))
   ALLOCATE(dsigu(kmax),dsigw(kmax))

  END SUBROUTINE alloc_comvars3d

   !!==============================================================================

  SUBROUTINE dealloc_comvars3d

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE dealloc_comvars3d  ***
   !&E  
   !&E--------------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !!----------------------------------------------------------------------
   !! * Executable part

   DEALLOCATE(kz,nz)
   DEALLOCATE(sigw,dsigu,dsigw)

  END SUBROUTINE dealloc_comvars3d

   !!==============================================================================


 END MODULE comvars3d
