!*********   including dyn3dx_ipg_cs.h   ***********************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_ipg_cs.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  The vertical interpolation with a cubic spline
   !&E                  is done inside dyn3dyx.F90
   !&E                  Here we estimate the pressure gradient itself
   !&E                  Here we estimate the pressure gradient itself
   !&E                  The algorithm is based on the reconstruction of
   !&E                  the density field and the physical z coordinate
   !&E                  as continuous functions of transformed coordinates
   !&E                  with subsequent analytical integration to compute
   !&E                  the pressure-radient force
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,zr,boz,pr,dx_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E         Shchepetkin and Mc Williams, JGR, 2003
   !&E
   !&E ** History :
   !&E       !  2006-01  (V. Garnier)  Original code
   !&E       !  2006-06  (V. Garnier)  inlining
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2014-10  (F. Vandermeirsch) formulation equation etat du code ROMS  
   !&E
   !&E---------------------------------------------------------------------

   !*******************************************
   !     Pressure gradient along component x
   !*******************************************

   DO k = kmax, 1, -1
#ifdef SPLIT_EOS
!     IF ( i==ig(j) .OR. h0(i-1,j)<=-valmanq ) THEN
     IF (h0(i-1,j)<=-valmanq ) THEN
       dxzr = 0.0_rsh       
       dxboz = 0.0_rsh
     ELSE IF ( i==ig(j)) THEN
       dxzr  = zr(k,i+1  ,j)-zr(k,i,j)
       dxboz =(boz2(k,i+1,j)-boz2(k,i  ,j)		  &
     		   +0.5*(qp1(k,i+1,j)-qp1(k,i,j))	  & ! Adiabatic
     		     *( xec(i+1,j)+xec(i,j)	          & ! elementary
     		       -zr(k,i+1,j)-zr(k,i,j)	 ))	    ! difference         
     ELSE 
       dxzr  = zr(k,i  ,j)-zr(k,i-1,j)    
       dxboz =(boz2(k,i  ,j)-boz2(k,i-1,j)                  &
                    +0.5*(qp1(k,i,j)-qp1(k,i-1,j))        & ! Adiabatic
                     *( xec(i,j)+xec(i-1,j)               & ! elementary
                       -zr(k,i,j)-zr(k,i-1,j)  ))         ! difference                                                    
     END IF
!     
       dxzrp1 = zr(k,i+1,j)-zr(k,i  ,j)
       dxbozp1 =(boz2(k,i+1,j)-boz2(k,i  ,j)		  &
     		   +0.5*(qp1(k,i+1,j)-qp1(k,i,j))	  & ! Adiabatic
     		     *( xec(i+1,j)+xec(i,j)	          & ! elementary
     		       -zr(k,i+1,j)-zr(k,i,j)	 ))	    ! difference      
!
     IF ( i==id(j)-1 ) THEN
       dxzrp2 = 0.0_rsh
       dxbozp2 = 0.0_rsh
     ELSE IF ( h0(i+2,j)<=-valmanq ) THEN
       dxzrp2 = 0.0_rsh
       dxbozp2 = 0.0_rsh
     ELSE
       dxzrp2 = zr(k,i+2,j)-zr(k,i+1,j)
       dxbozp2 =(boz2(k,i+2,j)-boz2(k,i+1,j)		  &
     		   +0.5*(qp1(k,i+2,j)-qp1(k,i+1,j))	  & ! Adiabatic
     		     *( xec(i+2,j)+xec(i+1,j)             & ! elementary
     		       -zr(k,i+2,j)-zr(k,i+1,j) ))	    ! difference      
     END IF
!-------------------------------------------------------------------     
#else     
!     IF ( i==ig(j) .OR. h0(i-1,j)<=-valmanq ) THEN
     IF ( h0(i-1,j)<=-valmanq ) THEN
       dxzr = 0.0_rsh
       dxboz = 0.0_rsh
     ELSE
       dxzr  = zr(k,i  ,j)-zr(k,i-1,j)
!       dxboz = boz(k,i  ,j)-boz(k,i-1,j)
       dxboz = rho(k,i,j)-rho(k,i-1,j)
     END IF

     dxzrp1 = zr(k,i+1,j)-zr(k,i  ,j)
!    dxbozp1 = boz(k,i+1,j)-boz(k,i  ,j)
     dxbozp1 = rho(k,i+1,j)-rho(k,i  ,j)

     IF ( i==id(j)-1 ) THEN
       dxzrp2 = 0.0_rsh
       dxbozp2 = 0.0_rsh
     ELSE IF ( h0(i+2,j)<=-valmanq ) THEN
       dxzrp2 = 0.0_rsh
       dxbozp2 = 0.0_rsh
     ELSE
       dxzrp2 = zr(k,i+2,j)-zr(k,i+1,j)
!      dxbozp2 = boz(k,i+2,j)-boz(k,i+1,j)
       dxbozp2 = rho(k,i+2,j)-rho(k,i+1,j)
     END IF              
# endif
!-------------------------------------------------------------------     
!     if ( k==400) then
!      if (i<=40 .and. j==2 .and. k==1) then     
!      if (            j==3 .and. k==30) then
!      if (  i==55 .and. j==19 ) then
!     print*,'---------------------------------'
!     print*,'MARS avant gpx',i,j,k
!     print*,h0(i-1,j),h0(i,j)
!     print*,boz(k,i,j),boz(k,i+1,j),boz(k,i+2,j)
!
!     print*,rho(k,i-1,j),rho(k,i,j)
!     print*,rho(k,i+1,j),rho(k,i+2,j)
!     
!     print*,boz2(k,i-1,j),boz2(k,i,j)
!     print*,qp1(k,i-1,j),qp1(k,i,j)
!     print*,xec(i-1,j),xec(i,j)
!     print*
!     print*,dxboz,dxbozp1,dxbozp2
!     print*,dxzr ,dxzrp1 ,dxzrp2      
!     print*,zr(k,i-1,j),zr(k,i,j),zr(k,i+1,j)
!      print*,zr(k,i,j),zr(k,i+1,j),zr(k,i+2,j)
!      read(*,*)
!     endif
!     endif       
!-------------------------------------------------------------------
     cff = 2.0_rsh*dxzr*dxzrp1
     IF (cff > 0) THEN
       dxzr = cff / (dxzr+dxzrp1)
     ELSE
       dxzr = 0.0_rsh
     END IF
     cff = 2.0_rsh*dxzrp1*dxzrp2
     IF (cff > 0) THEN
       dxzrp1 = cff / (dxzrp1+dxzrp2)
     ELSE
       dxzrp1 = 0.0_rsh
     END IF
!     
     cff = 2.0_rsh*dxboz*dxbozp1
     IF (cff > 0) THEN
       dxboz = cff / (dxboz+dxbozp1)
     ELSE
       dxboz = 0.0_rsh
     END IF
     cff = 2.0_rsh*dxbozp1*dxbozp2
     IF (cff > 0) THEN
       dxbozp1 = cff / (dxbozp1+dxbozp2)
     ELSE
       dxbozp1 = 0.0_rsh
     END IF
#ifdef SPLIT_EOS
       dxboz  = dxboz  -qp1(k,i,j)*dxzr
       dxbozp1= dxbozp1-qp1(k,i+1,j)*dxzrp1
#endif
!-------------------------------------------------------------------       
!     if ( k==400) then       
!      if (i<=40 .and. j==2 .and. k==1) then
!      if (            j==3 .and. k==30) then
!      if (  i==55 .and. j==19 ) then
!     print*,'MARS apres  gpx',i,j,k
!     print* 
!     print*,dxboz,dxbozp1
!     print*,dxzr,dxzrp1    
!
!     print*,(   demi*(  (boz(k,i,j)+boz(k,i-1,j))*(zr(k,i,j)-zr(k,i-1,j))     &
!           - uncinq*( (dxbozp1-dxboz)*(zr(k,i,j)-zr(k,i-1,j)   &
!                             -undouz*(dxzrp1+dxzr))            &
!                     -(dxzrp1-dxzr)*(boz(k,i,j)-boz(k,i-1,j)   &
!                             -undouz*(dxbozp1+dxboz)) )  )   )
!
!     print*,(   demi*(  (rho(k,i+1,j)+rho(k,i,j))*(zr(k,i+1,j)-zr(k,i,j))     &
!           - uncinq*( (dxbozp1-dxboz)7245725.service*(zr(k,i+1,j)-zr(k,i,j)   &
!                             -undouz*(dxzrp1+dxzr))            & 
!                     -(dxzrp1-dxzr)*(rho(k,i+1,j)-rho(k,i,j)   &
!                             -undouz*(dxbozp1+dxboz)) )  )   )*(-grav/rhoref)			     
!     print*,'pr(k,i,j),pr(k,i+1,j)',pr(k,i,j),pr(k,i+1,j) 
!    print*
!    print*,'1',(rho(k,i+1,j)+rho(k,i,j))*(zr(k,i+1,j)-zr(k,i,j))   
!     print*,'2',-undouz*(dxzrp1+dxzr)   			      
!     print*,'3',-undouz*(dxbozp1+dxboz)
!     read(*,*)
!      endif       
!      endif          
!-------------------------------------------------------------------
!     tbc = pr(k,i+1,j) - pr(k,i,j) +                                       &
!           (   demi*(  (boz(k,i+1,j)+boz(k,i,j))*(zr(k,i+1,j)-zr(k,i,j))     &
!           - uncinq*( (dxbozp1-dxboz)*(zr(k,i+1,j)-zr(k,i,j)   &
!                             -undouz*(dxzrp1+dxzr))            & 
!                     -(dxzrp1-dxzr)*(boz(k,i+1,j)-boz(k,i,j)   &
!                             -undouz*(dxbozp1+dxboz)) )  )   )
     tbc = -pr(k,i+1,j) + pr(k,i,j) +                                       &
           (  -demi*grav/rhoref*(  (rho(k,i+1,j)+rho(k,i,j))*(zr(k,i+1,j)-zr(k,i,j))     &
           - uncinq*( (dxbozp1-dxboz)*(zr(k,i+1,j)-zr(k,i,j)   &
                             -undouz*(dxzrp1+dxzr))            & 
                     -(dxzrp1-dxzr)*(rho(k,i+1,j)-rho(k,i,j)   &
                             -undouz*(dxbozp1+dxboz)) )  )   )			     

     yz_exp(k,i,j) = yz_exp(k,i,j)+tbc/dx_u(i,j)

   END DO    

!*********** end including dyn3dx_ipg_cs.h *********************************
