!*********   including dyn3dy_ipg_cs.h  ************************************ 

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_ipg_cs.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the meridional momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  The vertical interpolation with a cubic spline
   !&E                  is done inside dyn3dyx.F90
   !&E                  Here we estimate the pressure gradient itself 
   !&E                  The algorithm is based on the reconstruction of
   !&E                  the density field and the physical z coordinate
   !&E                  as continuous functions of transformed coordinates
   !&E                  with subsequent analytical integration to compute
   !&E                  the pressure-gradient force
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,zr,boz,pr,dx_v
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E         Shchepetkin and Mc Williams, JGR, 2003
   !&E
   !&E ** History :
   !&E       !  2006-01  (V. Garnier)  Original code
   !&E       !  2006-06  (V. Garnier)  inlining
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2014-10  (F. Vandermeirsch) formulation equation etat du code ROMS  
   !&E
   !&E---------------------------------------------------------------------

   !*******************************************
   !     Pressure gradient along component x
   !*******************************************

   DO k = kmax, 1, -1
#ifdef SPLIT_EOS
!   IF ( j==jb(i) .OR. h0(i,j-1)<=-valmanq ) THEN
     IF (  h0(i,j-1)<=-valmanq ) THEN
       dyzr  = 0.0_rsh     
       dyboz = 0.0_rsh
     ELSE IF ( j==jb(i) ) THEN
       dyzr  = zr(k,i,j+1)-zr(k,i,j)     
       dyboz =(boz2(k,i,j+1)-boz2(k,i  ,j)		  &
     		   +0.5*(qp1(k,i,j+1)-qp1(k,i,j))	  & ! Adiabatic
     		     *( xec(i,j+1)+xec(i,j)	          & ! elementary
     		      -zr(k,i,j+1)-zr(k,i,j)	 ))	    ! difference          
     ELSE  
       dyzr  = zr(k,i,j)-zr(k,i,j-1)
       dyboz =(boz2(k,i,j)-boz2(k,i,j-1)                  &
                    +0.5*(qp1(k,i,j)-qp1(k,i,j-1))        & ! Adiabatic
                     *( xec(i,j)+xec(i,j-1)               & ! elementary
                       -zr(k,i,j)-zr(k,i,j-1)  ))         ! difference                                                    
     END IF
!     
       dyzrp1  = zr(k,i,j+1)-zr(k,i,j)
       dybozp1 =(boz2(k,i,j+1)-boz2(k,i  ,j)		  &
     		   +0.5*(qp1(k,i,j+1)-qp1(k,i,j))	  & ! Adiabatic
     		     *( xec(i,j+1)+xec(i,j)	          & ! elementary
     		      -zr(k,i,j+1)-zr(k,i,j)	 ))	    ! difference      
!
     IF ( j==jh(i)-1 ) THEN
       dyzrp2  = 0.0_rsh
       dybozp2 = 0.0_rsh
     ELSE IF (h0(i,j+2)<=-valmanq ) THEN
       dyzrp2  = 0.0_rsh
       dybozp2 = 0.0_rsh
     ELSE
       dyzrp2  = zr(k,i,j+2)-zr(k,i,j+1)
       dybozp2 =(boz2(k,i,j+2)-boz2(k,i,j+1)		  &
     		   +0.5*(qp1(k,i,j+2)-qp1(k,i,j+1))	  & ! Adiabatic
     		     *( xec(i,j+2)+xec(i,j+1)             & ! elementary
     		       -zr(k,i,j+2)-zr(k,i,j+1) ))	    ! difference      
     END IF
!-------------------------------------------------------------------
#else     
!   IF ( j==jb(i) .OR. h0(i,j-1)<=-valmanq ) THEN
     IF ( h0(i,j-1)<=-valmanq ) THEN
       dyzr  = 0.0_rsh
       dyboz = 0.0_rsh
     ELSE  IF ( j==jb(i) ) THEN
       dyzr  = zr(k,i,j+1)-zr(k,i,j)
       dyboz = rho(k,i,j+1)-rho(k,i,j)
     ELSE
       dyzr  = zr(k,i,j)-zr(k,i,j-1)
!       dyboz = boz(k,i,j)-boz(k,i,j-1)
       dyboz = rho(k,i,j)-rho(k,i,j-1)
     END IF
!
     dyzrp1  = zr(k,i,j+1)-zr(k,i,j)
!    dybozp1 = boz(k,i,j+1)-boz(k,i,j)
     dybozp1 = rho(k,i,j+1)-rho(k,i,j)
!
     IF ( j==jh(i)-1 ) THEN
       dyzrp2  = 0.0_rsh
       dybozp2 = 0.0_rsh
     ELSE IF (h0(i,j+2)<=-valmanq ) THEN
       dyzrp2  = 0.0_rsh
       dybozp2 = 0.0_rsh
     ELSE
       dyzrp2  = zr(k,i,j+2)-zr(k,i,j+1)
!       dybozp2 = boz(k,i,j+2)-boz(k,i,j+1)
        dybozp2 = rho(k,i,j+2)-rho(k,i,j+1)
     END IF
# endif              
!------------------------------------------------------------------- 
!     if ( k==400) then           
!      if ( i==19 .and. j==55 ) then
!      if (i==2 .and. j<=4 .and. k==1) then     
!     print*,'---------------------------------'
!     print*,'MARS avant gpy',i,j,k
!     print*,h0(i-1,j),h0(i,j)
!     print*,rho(k,i,j-1),rho(k,i,j)
!     print*,rho(k,i,j+1),rho(k,i,j+2)
!     print*
!     print*,dyboz,dybozp1,dybozp2
!     print*,dyzr ,dyzrp1 ,dyzrp2      
!     print*,zr(k,i,j-1),zr(k,i,j),zr(k,i,j+1)
!     print*,zr(k,i,j),zr(k,i,j+1),zr(k,i,j+2)
!     read(*,*)
!      endif
!     endif
!-------------------------------------------------------------------    
     cff = 2.0_rsh*dyzr*dyzrp1
     IF (cff > 0) THEN
       dyzr = cff / (dyzr+dyzrp1)
     ELSE
       dyzr = 0.0_rsh
     ENDIF
     cff = 2.0_rsh*dyzrp1*dyzrp2
     IF (cff > 0) THEN
       dyzrp1 = cff / (dyzrp1+dyzrp2)
     ELSE
       dyzrp1 = 0.0_rsh
     END IF
!
     cff = 2.0_rsh*dyboz*dybozp1
     IF (cff > 0) THEN
       dyboz   = cff / (dyboz+dybozp1)
     ELSE
       dyboz   = 0.0_rsh
     ENDIF
     cff = 2.0_rsh*dybozp1*dybozp2
     IF (cff > 0) THEN
       dybozp1 = cff / (dybozp1+dybozp2)
     ELSE
       dybozp1 = 0.0_rsh
     ENDIF    
#ifdef SPLIT_EOS
       dyboz  = dyboz  -qp1(k,i,j)*dyzr
       dybozp1= dybozp1-qp1(k,i,j+1)*dyzrp1
#endif
!-------------------------------------------------------------------         
!        if ( k==400) then  
!      if (  i==19 .and. j==55 ) then
!      if (i==2 .and. j<=4 .and. k==1) then     
!     print*,'MARS apres  gpy',i,j,k
!     print* 
!     print*,dyboz,dybozp1
!     print*,dyzr,dyzrp1
!
!     print*,          (   demi*(  (boz(k,i,j)+boz(k,i,j-1))*(zr(k,i,j)-zr(k,i,j-1))     &
!           - uncinq*( (dybozp1-dyboz)*(zr(k,i,j)-zr(k,i,j-1)   &
!                             -undouz*(dyzrp1+dyzr))            & 
!                     -(dyzrp1-dyzr)*(boz(k,i,j)-boz(k,i,j-1)   &
!                             -undouz*(dybozp1+dyboz)) )  )   )
!      print*,          (   demi*(  (rho(k,i,j+1)+rho(k,i,j))*(zr(k,i,j+1)-zr(k,i,j))     &
!           - uncinq*( (dybozp1-dyboz)*(zr(k,i,j+1)-zr(k,i,j)   &
!                             -undouz*(dyzrp1+dyzr))            & 
!                     -(dyzrp1-dyzr)*(rho(k,i,j+1)-rho(k,i,j)   &
!                             -undouz*(dybozp1+dyboz)) )  )   )*(-grav/rhoref)
!     print*,'pr(k,i,j),pr(k,i,j+1)',pr(k,i,j),pr(k,i,j+1) 
!     print*
!     print*,'1',(rho(k,i,j+1)+rho(k,i,j))*(zr(k,i,j+1)-zr(k,i,j))   
!     print*,'2',-undouz*(dyzrp1+dyzr)   			      
!     print*,'3',-undouz*(dybozp1+dyboz)
!     read(*,*)
!      endif         
!       endif         
!-------------------------------------------------------------------
!     tbc = pr(k,i,j+1) - pr(k,i,j) +                                       &
!           (   demi*(  (boz(k,i,j+1)+boz(k,i,j))*(zr(k,i,j+1)-zr(k,i,j))     &
!           - uncinq*( (dybozp1-dyboz)*(zr(k,i,j+1)-zr(k,i,j)   &
!                             -undouz*(dyzrp1+dyzr))            & 
!                     -(dyzrp1-dyzr)*(boz(k,i,j+1)-boz(k,i,j)   &
!                             -undouz*(dybozp1+dyboz)) )  )   ) 
     tbc = -pr(k,i,j+1) + pr(k,i,j) +                                       &
           (  -demi*grav/rhoref*(  (rho(k,i,j+1)+rho(k,i,j))*(zr(k,i,j+1)-zr(k,i,j))     &
           - uncinq*( (dybozp1-dyboz)*(zr(k,i,j+1)-zr(k,i,j)   &
                             -undouz*(dyzrp1+dyzr))            & 
                     -(dyzrp1-dyzr)*(rho(k,i,j+1)-rho(k,i,j)   &
                             -undouz*(dybozp1+dyboz)) )  )   )			         

     yz_exp(k,i,j) = yz_exp(k,i,j)+tbc/dy_v(i,j)

   END DO
   
!*********** end including dyn3dy_ipg_cs.h *******************************
