   !!======================================================================

  SUBROUTINE init_viscoef

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE init_viscoef  ***
   !&E
   !&E ** Purpose : Initialize the coefficients for horizontal viscosity.
   !&E              Set up sponge layers.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables : vis2d, vish_xe, vish_phi
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        !  2004-08-16
   !&E        !  2008-08  (T. Odaka) MPI
   !&E        !  2009-07  (N. Bruneau, F. Dumas) coupling with SWAN
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

#include "toolcpp.h"

   USE parameters
! FVDM 2014 : utilisation de forwrka, forwrkb
   USE comvars2d, ONLY : fvisc, vish_xe, dy, vis2d, vish_phi,forwrka,forwrkb,        &
                         jb,jh, id, ig, h0_g,sponge_nbcell,vismul,   &
                         l_spongn, l_sponge, l_spongs, l_spongw,     &
                         l_modele2d
#ifdef key_waveforcing
   USE comvars3d,  ONLY : rhoref
   USE comvarp2d,  ONLY : ssh
   USE waveforcing,ONLY : M_battjes,Dissip
#endif
   USE_MPI toolmpi,   ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
! FVDM 2014 : nouvelle declaration de variable k, ibnd
   INTEGER        :: k,i,j,ibnd
   REAL(kind=rsh) :: nu0,vismuloc

   !!----------------------------------------------------------------------
   !! * Executable part
!FVDM 5/02/2014
! La viscosite est fonction d'un cosinus
! dans la couche eponge (tableau forwrka)
! et continue avec l'interieur du domaine 
!
! La relaxation dans la couche eponge est 
! fonction d'un tanh (tableau forwrkb)
! et continue avec l'interireur du domaine
 
    DO k=1,kmax
    do j=ljminp1,ljmax
      do i=liminp1,limax
	  ibnd=sponge_nbcell
!    IF(l_spongw .and. h0_g(imin+1,j) > -valmanq) THEN 
    IF(l_spongw) THEN 
          ibnd=min(ibnd,i)
    END IF
!    IF(l_sponge .and. h0_g(imax-1,j) > -valmanq) THEN 
    IF(l_sponge) THEN 
          ibnd=min(ibnd,imax+1-i)
    END IF
!    IF(l_spongs .and. h0_g(i,jmin+1) > -valmanq) THEN 
    IF(l_spongs) THEN 
          ibnd=min(ibnd,j)
    END IF
!    IF(l_spongn .and. h0_g(i,jmax-1) > -valmanq) THEN 
    IF(l_spongn) THEN 
          ibnd=min(ibnd,jmax+1-j)      
    END IF
       forwrka(i,j) = .5*(cos(pi*float(ibnd-1)/(sponge_nbcell-1))+1.)
!       forwrkb(i,j) = .5*(-tanh((float(ibnd)-sponge_nbcell/3.)/2.)+1. - (-tanh((sponge_nbcell-sponge_nbcell/3.)/2.)+1.))		      
!       forwrk(k,i,j)=.5*(cos(pi*float(ibnd-1)/(sponge_nbcell-1))+1.)*exp(-5.*float(kmax-k)/float(kmax))
     enddo
    enddo
    ENDDO
	 
   DO j=ljmin,ljmax
     DO i=limin,limax
!      DO k=1,kmax
!FVDM       nu0=fvisc*0.01_rsh*dy(i,j)**1.15_rsh
           nu0=fvisc
#ifdef key_waveforcing
       nu0 = nu0 + M_battjes * (h0_g(i,j)+ssh(i,j)) * (Dissip(i,j)/rhoref)**(1.0_rsh/3.0_rsh)
  !If (l_wave_brgm_roller.and. M_battjes /=0 ) nu0 = nu0 +
#endif
!   nu0=fvisc*0.01*dy(1,1)**1.15
! FVDM 2014
       vis2d(i,j)=nu0 + vismul*forwrka(i,j)
       IF(.NOT.l_modele2d) THEN
         vish_xe(:,i,j)=nu0  + vismul*forwrka(i,j)  
         vish_phi(:,i,j)=nu0 + vismul*forwrka(i,j)
       END IF
!      END DO
     END DO
   END DO

!$OMP parallel default(none) private(i,j,k,nu0,vismuloc) &
!$OMP& shared(fvisc,dy,vis2d,vish_xe,vish_phi,h0_g,             &
!$OMP& ljmin,ljmax,ljminp1,l_modele2d,jb,jh,id,ig,              &
!$OMP& limin,limax,liminp1,				        &
#ifdef key_waveforcing
!$OMP& rhoref,ssh,M_battjes,Dissip,                    &
#endif
!$OMP& l_spongw,sponge_nbcell,vismul,l_sponge,l_spongn,l_spongs)
!$OMP Do schedule(static)
!do j=ljminp1m1,ljmaxp1
   DO j=ljminp1,ljmax
     IF(l_spongw) THEN                        ! couche eponge a l ouest si ouverte
       IF(h0_g(ig(j),j).GT.-valmanq) THEN
         DO i=ig(j),MAX(MIN(ig(j)+sponge_nbcell,imax),imin)
           IF_MPI ((i >= limin).AND.(i <= limax)) THEN
!     vismuloc=(5.* exp(-real(i-1)/10.))
!     vismuloc=(i-(ig(j)+sponge_nbcell))*(1-vismul)/sponge_nbcell+1.0_rsh
           vismuloc=vismul
!FVDM           nu0=fvisc*0.01_rsh*dy(i,j)**1.15_rsh
           nu0=fvisc
#ifdef key_waveforcing
           nu0 = nu0 + M_battjes * (h0_g(i,j)+ssh(i,j)) * (Dissip(i,j)/rhoref)**(1.0_rsh/3.0_rsh)
#endif
!FVDM           vis2d(i,j)=MAX(nu0*vismuloc,vis2d(i,j))
           IF(.NOT.l_modele2d) THEN
!FVDM             vish_phi(:,i,j)= nu0 + vismul*forwrk(i,j)
!FVDM            vish_xe(:,i,j) = nu0 + vismul*forwrk(i,j)  
!FVDM             vish_phi(:,i,j)=MAX(nu0*vismuloc,vish_phi(:,i,j))
!FVDM             vish_xe(:,i,j) =MAX(nu0*vismuloc,vish_xe(:,i,j))
           END IF
           ENDIF_MPI
         ENDDO
       ENDIF
     ENDIF

     IF(l_sponge) THEN                         ! couche eponge a l est si ouverte
       IF(h0_g(id(j),j).GT.-valmanq) THEN
         DO i=MIN(MAX(id(j)-sponge_nbcell,imin),imax),id(j)
           IF_MPI ((i >= limin).AND.(i <= limax)) THEN
!     vismuloc=(5.* exp(-real(id(j)-i)/10.))
!     vismuloc=((id(j)-sponge_nbcell)-i)*(1-vismul)/sponge_nbcell+1.0_rsh
           vismuloc=vismul
           nu0=fvisc*0.01_rsh*dy(i,j)**1.15_rsh
#ifdef key_waveforcing
           nu0 = nu0 + M_battjes * (h0_g(i,j)+ssh(i,j)) * (Dissip(i,j)/rhoref)**(1.0_rsh/3.0_rsh)
#endif
           vis2d(i,j)=MAX(nu0*vismuloc,vis2d(i,j))
           IF (.NOT.l_modele2d) THEN
!FVDM             vish_phi(:,i,j)= nu0 + vismul*forwrk(i,j)
!FVDM             vish_xe(:,i,j) = nu0 + vismul*forwrk(i,j)
!FVDM   	vish_phi(:,i,j)=MAX(nu0*vismuloc,vish_phi(:,i,j))
!FVDM   	vish_xe(:,i,j) =MAX(nu0*vismuloc,vish_xe(:,i,j))	
	     
           END IF
           ENDIF_MPI
         END DO
       END IF
     END IF
   END DO
!$OMP end Do

!$OMP Do schedule(static)
   DO i=liminp1,limax
     IF (l_spongn) THEN                         ! couche eponge au nord si ouverte
       IF (h0_g(i,jh(i)).GT.-valmanq) THEN
         DO j=MIN(MAX(jh(i)-sponge_nbcell,jmin),jmax),jh(i)
           IF_MPI ((j.GE.ljmin).AND.(j.LE.ljmax)) THEN
!     vismuloc=(5.* exp(-real(jh(i)-j)/10.))
!     vismuloc=((jh(i)-sponge_nbcell)-j)*(1-vismul)/sponge_nbcell+1.0_rsh
             vismuloc=vismul
             nu0=fvisc*0.01_rsh*dy(i,j)**1.15_rsh
#ifdef key_waveforcing
             nu0 = nu0 + M_battjes * (h0_g(i,j)+ssh(i,j)) * (Dissip(i,j)/rhoref)**(1.0_rsh/3.0_rsh)
#endif
!FVDM             vis2d(i,j)=MAX(nu0*vismuloc,vis2d(i,j))
             IF (.NOT.l_modele2d) THEN
!FVDM               vish_phi(:,i,j)= nu0 + vismul*forwrk(i,j)
!FVDM               vish_xe(:,i,j) = nu0 + vismul*forwrk(i,j)
!FVDM   	vish_phi(:,i,j)=MAX(nu0*vismuloc,vish_phi(:,i,j))
!FVDM   	vish_xe(:,i,j) =MAX(nu0*vismuloc,vish_xe(:,i,j))
             END IF
           ENDIF_MPI
         END DO
       END IF
     END IF

     IF (l_spongs) THEN                         ! couche eponge au sud si ouverte
       IF (h0_g(i,jb(i)).GT.-valmanq) THEN
         DO j=jb(i),MAX(MIN(jb(i)+sponge_nbcell,jmax),jmin)
           IF_MPI ((j.GE.ljmin).AND.(j.LE.ljmax)) THEN
!     vismuloc=(5.* exp(-real(j-1)/10.))
!     vismuloc=(j-(jb(i)+sponge_nbcell))*(1-vismul)/sponge_nbcell+1.0_rsh
             vismuloc=vismul
             nu0=fvisc*0.01_rsh*dy(i,j)**1.15_rsh
#ifdef key_waveforcing
             nu0 = nu0 + M_battjes * (h0_g(i,j)+ssh(i,j)) * (Dissip(i,j)/rhoref)**(1.0_rsh/3.0_rsh)
#endif
!FVDM             vis2d(i,j)=MAX(nu0*vismuloc,vis2d(i,j))
             IF (.NOT.l_modele2d) THEN
!FVDM               vish_phi(:,i,j)= nu0 + vismul*forwrk(i,j)
!FVDM               vish_xe(:,i,j) = nu0 + vismul*forwrk(i,j)
!FVDM  	     vish_phi(:,i,j)=MAX(nu0*vismuloc,vish_phi(:,i,j))
!FVDM  	     vish_xe(:,i,j) =MAX(nu0*vismuloc,vish_xe(:,i,j))	
             END IF
           ENDIF_MPI
         END DO
       END IF
     END IF

   END DO
!$OMP end Do

!$OMP Do schedule(static) 
   DO j=ljmin,ljmax
     DO i=limin,limax
       IF (h0_g(i,j)==-valmanq) THEN
         vis2d(i,j)=valmanq
         vish_xe(:,i,j)=valmanq
         vish_phi(:,i,j)=valmanq
       END IF
     END DO
   END DO
!$OMP end do


!$OMP end parallel 

   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,vis2d(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,vis2d(liminm1:limaxp1,ljminm1:ljmaxp1))
   IF (.NOT.l_modele2d) THEN
     CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax,ljminm1,ljmax,vish_phi(:,liminm1:limax,ljminm1:ljmax))
     CALL_MPI ex_j_rsh(-1,0,kmax,liminm1,limax,ljminm1,ljmax,vish_phi(:,liminm1:limax,ljminm1:ljmax))
     CALL_MPI ex_i_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,vish_xe(:,limin:limaxp1,ljmin:ljmaxp1))
     CALL_MPI ex_j_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,vish_xe(:,limin:limaxp1,ljmin:ljmaxp1))
   END IF

END SUBROUTINE init_viscoef
