   !!======================================================================

  SUBROUTINE output_save(nb,tect,u,uz,x,v,vz,sal,temp,        &
                         vis2d,vish_xe,vish_phi,nz,kz,boz,    &
#ifdef key_substance
                                                      cv_wat, &
#endif
                                                             hexm,heym,xem,kzm)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE output_save  ***
   !&E
   !&E ** Purpose :
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : output_mng.F90, mpimod.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  11-2004
   !&E       !  01-2006 (V. Garnier introduction of different output types)
   !&E       !  2008-03 (C. Struski) introduction of parameter l_out_subs to save the substance in output files
   !&E       !  2008-10 (V. Garnier) output managing relative to displacement of init_fromfile from step to main
   !&E       !  2008-10 (M. Jouan, V. Garnier) save conservative (in flux meanning) variable uz and vz (for offline purpose)
   !&E       !  2008-09 (G. Galibert / ALTRAN) attributes of output files that respect NetCDF OCO v1.1 file format
   !&E       !  2009-03 (V. Garnier) attributes according to NetCDF OCO v1.2 file format
   !&E                               saving of generalized sigma parameters
   !&E       !  2009-05  (V. Garnier)  Save cumulative atmospherical fluxes
   !&E       !  2009-09  (P. Garreau)  Save viscosity and diffusivity variables
   !&E       !  2009-12  (S. Michel, HOCER)  Save buoyancy variable
   !&E       !  2010-02  (G. Simon) Save waves statistics: Hs, direction, foc
   !&E       !  2010-07 (R. Verney) Introduction of orbital velocity variable
   !&E       !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E       !  2012-02  (P. Garreau) Saving of physical vertical velocity (key_diag_physwz)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! Modules used
   USE netcdf
   USE ionc4,       ONLY : ionc4_createfile,ionc4_write_lon,ionc4_gatt,ionc4_createvar,&
                           ionc4_sync,ionc4_write,ionc4_write_sig,ionc4_write_xy,ionc4_write_zxy,&
                           ionc4_write_time,ionc4_write_xyt,ionc4_write_zxyt,ionc4_close,&
                           ionc4_vatt_chain
   USE parameters,  ONLY : rsh,rlg,riosh,kmax,imin,imax,limin,limax,ljmin,ljmax,  &
                           rg_valmanq_io,ljminp1,riolg,valmanq,lchain,grav
#if defined key_xios
   USE parameters,  ONLY : MPI_COMM_VAR
#endif
   USE output,      ONLY : fileresu,file_output,irlos_cum,latlos_cum,senslos_cum, &
                           rad_cum,evap_cum,rain_cum,tosx_cum,tosy_cum,dt_cum,dtn,&
#ifdef key_sflx_analyse
                           windx_cum,windy_cum,rh_cum,fcc_cum,sat_cum,sst_cum,pa_cum, &
#endif
                           l_filebydate,tsort,tfinsor,stepi,stepj,stepk,dt_ave,   &
                           iextmin,iextmax,jextmin,jextmax,kextmin,kextmax,nbave, &
                           igextmin,igextmax,jgextmin,jgextmax,nbatm,l_out_zaxis_w,&
                           l_out_ssh,l_out_sal,l_out_temp,l_out_ubt,l_out_u3d,    &
                           l_out_visc,l_out_turb,l_out_buoy,l_out_atmsflx,        &
                           l_out_timeave,                                         &
                           l_out_wind,l_out_off,l_out_wave,l_out_nc4par,l_out_pack
   USE output,      ONLY : ssh_ave,vis2d_ave,u_ave,v_ave,uz_ave,vz_ave, &
                           temp_ave,sal_ave,vish_xe_ave,vish_phi_ave,   &
                           nz_ave,kz_ave,boz_ave
#ifdef key_diag_physwz
   USE output,      ONLY : wz_phys_ave,wz_ave
#endif

   USE outtools,    ONLY : out_write_xytvar,out_write_zxytvar,out_grid
   USE comvars2d,   ONLY : lon2d,lon2d_g,lat2d_g,lat2d,fu,fv
   USE comvars2d,   ONLY : nbouc,date_ref,h0fond,date_startshift,iwarnlog,          &
                           h0,hx,hy,name_out_hx,name_out_hy,name_out_h0,name_out_vz,name_out_sal,name_out_temp, &
                           name_out_vis2d,name_out_vishxe,name_out_vishphi,name_out_kz,name_out_nz,name_out_bz, &
                           name_out_xe,name_out_u,name_out_v,name_out_uz,l_modele2d,l_initfromfile,             &
                           ig,id,jb,jh,name_out_tosx,name_out_tosy,out_dir,suffix,iscreen,                      &
                           name_out_mhex,name_out_mhey,name_out_mxe,name_out_mkz,                               &
                           name_out_mrad,name_out_mirlos,name_out_mlatlos,name_out_msenslos,                    &
                           name_out_mevap,name_out_mrain,name_out_mtosx,name_out_mtosy,                         &
                           riog_valid_min_h0,riog_valid_max_h0,riog_valid_min_u,riog_valid_max_u,               &
                           riog_valid_min_temp,riog_valid_max_temp,riog_valid_min_sal,riog_valid_max_sal,       &
                           riog_valid_min_xe,riog_valid_max_xe,riog_valid_min_wstress,riog_valid_max_wstress
   USE comvars2d,   ONLY : riog_valid_min_flux,riog_valid_max_flux,riog_valid_min_kz,riog_valid_max_kz,t,       &
                           riog_valid_min_visc,riog_valid_max_visc,riog_valid_min_nz,riog_valid_max_nz,         &
                           riog_valid_min_bz,riog_valid_max_bz
#ifdef key_gencoord_test
   USE toolgeom,    ONLY : f_zu,f_zw,f_zu_x,f_zu_y
#endif
#ifdef key_grid_rotated
   USE comvars2d,   ONLY : grid_angle
#endif
#ifdef key_sflx_analyse
   USE comvars2d,   ONLY : name_out_mwindx,name_out_mwindy,name_out_mrh,name_out_mfcc,name_out_msat,            &
                           name_out_msst,name_out_mpa
#endif
#ifdef key_wave
   USE comvars2d,   ONLY : name_out_wave_dir,riog_valid_min_wave_dir,riog_valid_max_wave_dir, &
                           name_out_wave_hs,riog_valid_min_wave_hs,riog_valid_max_wave_hs,    &
                           name_out_wave_foc,riog_valid_min_wave_foc,riog_valid_max_wave_foc, &
                           name_out_wave_ubot,riog_valid_min_wave_ubot,riog_valid_max_wave_ubot
#endif
   USE comvars3d,   ONLY : sig,sigw
#ifdef key_diag_physwz
   USE comvars3d,   ONLY : wz_phys,wz
#endif
   USE cometdyn,    ONLY : tosx,tosy
#ifdef key_wave
   USE comwave,     ONLY : wave_dir,wave_hs,wave_foc,wave_ubot,l_wave_rdir,l_wave_rhs,l_wave_rfoc,l_wave_rubot, &
                           l_wave_ruubot, l_wave_rvubot
#endif
   USE sflxatm,     ONLY : rad,irlos,latlos,senslos
   USE comfluxdo,   ONLY : evap,rain
#ifdef key_sflx_analyse
   USE sflxatm,     ONLY : windx,windy
   USE cometdyn,    ONLY : pa
   USE cometemp,    ONLY : rh,fcc,sat
#endif
#ifdef key_substance
   USE parameters,  ONLY : nb_var
   USE comsubstance,ONLY : irk_fil,name_var,unit_var,nv_state,l_out_log,l_out_subs,   &
                           long_name_var,standard_name_var,valid_min_var,valid_max_var
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : hc_sig,Csu_sig,Csw_sig,b_sig,theta_sig
#endif
   USE obccombine,  ONLY : l_obc_tide
   USE_MPI toolmpi, ONLY : ADD_TAG,IPROC,NPROCS,ADD_ALL_MPI_INT

   IMPLICIT NONE

#ifdef key_MPI_2D
   INCLUDE 'mpif.h'
#endif

   !! * Arguments
   INTEGER       ,INTENT(in) :: nb
   REAL(KIND=rlg),INTENT(in) :: tect
   REAL(KIND=rsh),DIMENSION(     limin:limax,ljmin:ljmax),INTENT(in)          :: vis2d
   REAL(KIND=rsh),DIMENSION(     limin:limax,ljmin:ljmax),INTENT(in)          :: u,x,v
   REAL(KIND=rsh),DIMENSION(kmax,limin:limax,ljmin:ljmax),INTENT(in)          :: uz,vz,sal,temp
   REAL(KIND=rsh),DIMENSION(kmax,limin:limax,ljmin:ljmax),INTENT(in)          :: vish_xe,vish_phi
   REAL(KIND=rsh),DIMENSION(kmax,limin:limax,ljmin:ljmax),INTENT(in)          :: nz,kz,boz
   REAL(KIND=rsh),DIMENSION(     limin:limax,ljmin:ljmax),INTENT(in),OPTIONAL :: hexm,heym,xem
   REAL(KIND=rsh),DIMENSION(kmax,limin:limax,ljmin:ljmax),INTENT(in),OPTIONAL :: kzm
#ifdef key_substance
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,limin:limax,ljmin:ljmax),INTENT(in) :: cv_wat
#endif

   !! * Local declarations
   INTEGER               :: i,j,k,l
   INTEGER               :: grp_world,grp_zoom,comm_zoom,ncpu,ip,ierr_mpi
   INTEGER, DIMENSION(:), ALLOCATABLE  :: numrank,listrank
   CHARACTER(LEN=lchain) :: fileres,gridfile,name_out
   CHARACTER(LEN=lchain) :: chain
   CHARACTER (len=19)    :: tool_sectodat,cdateout
   LOGICAL,         DIMENSION(limin:limax,ljmin:ljmax)      :: mask_h0,mask_hx,mask_hy
   REAL(KIND=rsh)        :: xx,yy
   REAL(KIND=riosh)      :: vecrit
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)      :: tecrit
   REAL(KIND=riosh),DIMENSION(kmax,limin:limax,ljmin:ljmax) :: tzecrit
   REAL(KIND=rsh),DIMENSION(kmax,limin:limax,ljmin:ljmax)   :: tmp3d
   ! toseelater virer variables tmp3du,tmp3dv,tmp3dw quand variables dz integrees globalement
   REAL(KIND=rsh),DIMENSION(kmax,limin:limax,ljmin:ljmax)   :: tmp3du,tmp3dv,tmp3dw
   REAL(KIND=riosh),DIMENSION(0:kmax+1)                     :: sigecrit
   REAL(KIND=riolg),DIMENSION(imin:imax)                    :: lon

#ifdef key_diag_physwz
   CHARACTER(LEN=lchain), PARAMETER :: name_out_wz_phys='w',name_out_wz='w_sig'
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter OUTPUT_SAVE routine'

   ! Masks
   ! -----

   mask_h0(:,:)=.TRUE.
   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       ! For heights smaller than h0fond, the cell is considered as wet
       IF ( h0(i,j)+x(i,j) > h0fond ) mask_h0(i,j)=.FALSE.
     END DO
   END DO

   mask_hx(:,:)=.TRUE.
   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)-1),MIN(limax,id(j))
       IF ( hx(i,j) /= -valmanq -h0fond ) mask_hx(i,j)=.FALSE.
     END DO
   END DO

   mask_hy(:,:)=.TRUE.
   DO j=ljmin,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       IF ( (j >= jb(i)-1 .AND. j <= jh(i)) .AND. (hy(i,j) /= -valmanq -h0fond) ) mask_hy(i,j)=.FALSE.
     END DO
   END DO

   comm_zoom=0
#ifdef key_MPI_2D
   comm_zoom=MPI_COMM_VAR
   !IF ( ( (nbouc == 0 .AND. .NOT.l_filebydate(nb)) .OR. (l_filebydate(nb) .AND. tect >= tsort(nb)) ) .AND. l_out_nc4par ) THEN
   IF ( (nbouc == 0 .AND. .NOT.l_filebydate(nb)) .OR. (l_filebydate(nb) .AND. tect >= tsort(nb)) ) THEN
     ! what a code ! Not a clever code probably but not clear in my head at the moment
     ! Count the number of processors active over the area to be saved
     ncpu=0
     IF ( jextmin(nb) <= jextmax(nb) .AND. iextmin(nb) <= iextmax(nb) ) ncpu=ncpu+1
     CALL ADD_ALL_MPI_INT(ncpu)
     ! Save the number of active processors
     ALLOCATE (numrank(ncpu))
     ALLOCATE (listrank(nprocs))
     numrank(:)=0   ! must be initialized if the proc 0 is active on the saved zoom
     listrank(:)=0
     ! which cpu is active ?
     IF ( jextmin(nb) <= jextmax(nb) .AND. iextmin(nb) <= iextmax(nb) ) listrank(1)=iproc
     ! all cpus get the number of active cpus plus sleeping cpus (set to 0)
     CALL MPI_ALLGATHER(listrank(1),1,MPI_INTEGER,listrank,1,MPI_INTEGER,MPI_COMM_VAR,ierr_mpi)
     ! all cpus keep the active cpus only
     IF (ncpu/=nprocs) THEN
       ip=0
       DO i=1,nprocs 
         IF_MPI ( listrank(i) /= 0 ) THEN
         ip=ip+1
         numrank(ip)=listrank(i)
         ENDIF
       END DO
       ! Group relative to the MPI_COMM_VAR
       CALL MPI_COMM_GROUP(MPI_COMM_VAR,grp_world,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_GROUP_INCL(grp_world,ncpu,numrank,grp_zoom,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_COMM_CREATE(MPI_COMM_VAR,grp_zoom,comm_zoom,ierr_mpi)
     ENDIF
   ENDIF
#endif


   IF ( jextmin(nb) <= jextmax(nb) .AND. iextmin(nb) <= iextmax(nb) ) THEN

   !-----------------------
   ! create the ouput files
   !-----------------------

   IF ( (nbouc == 0 .AND. .NOT.l_filebydate(nb)) .OR. (l_filebydate(nb) .AND. tect >= tsort(nb)) ) THEN
!   IF ( nbouc == 0 .OR. l_filebydate(nb) ) THEN

     IF (l_filebydate(nb)) THEN
       cdateout = tool_sectodat(tect)
       fileres=TRIM(out_dir)//TRIM(fileresu(nb))//'_'//TRIM(suffix)
       fileres=TRIM(fileres)//'_'//cdateout(7:10)//cdateout(4:5)//cdateout(1:2)//'T'
       fileres=TRIM(fileres)//cdateout(12:13)//cdateout(15:16)//cdateout(18:19)//'Z'
       fileres=TRIM(fileres)//'.nc'
       IF_MPI (.NOT. l_out_nc4par) CALL ADD_TAG(fileres,LEN(fileres))
     ELSE
       fileres=TRIM(fileresu(nb))
     END IF

!     DO i=imin,imax   ! to tina, yes it is correct, at least up to now.
!       lon(i)=gwac+(i-1)*dgac  !+360.0
!     END DO
     sigecrit(:)=sig(:)
     IF (l_out_nc4par) THEN
       ! To allow the different cpus to write at the same time into a global
       ! file, set l_out_nc4par= .TRUE.
       ! The dimension of the file must be global (igextmin)
       CALL ionc4_createfile(fileres,lon2d_g(igextmin(nb):igextmax(nb):stepi(nb),jgextmin(nb):jgextmax(nb):stepj(nb)), &
                                     lat2d_g(igextmin(nb):igextmax(nb):stepi(nb),jgextmin(nb):jgextmax(nb):stepj(nb)), &
                                     igextmin(nb),igextmax(nb),stepi(nb), &
                                     jgextmin(nb),jgextmax(nb),stepj(nb), &
                                     kextmin(nb),kextmax(nb),stepk(nb),   &
                                     kmax,0,l_out_nc4par= .TRUE.,comm_active=comm_zoom, &
                                     zaxis_var=sigecrit(1:kmax))
     ELSE
       ! If none l_out_nc4par argument, l_out_nc4par is considered as false
       ! Each cpu create its own output file inside the local domain (iextmin)
       CALL ionc4_createfile(fileres,lon2d(iextmin(nb):iextmax(nb):stepi(nb),jextmin(nb):jextmax(nb):stepj(nb)), &
                       lat2d(iextmin(nb):iextmax(nb):stepi(nb),jextmin(nb):jextmax(nb):stepj(nb)), &
                       iextmin(nb),iextmax(nb),stepi(nb),jextmin(nb),jextmax(nb),stepj(nb), &
                       kextmin(nb),kextmax(nb),stepk(nb),kmax,0,zaxis_var=sigecrit(1:kmax))
     ENDIF
!     CALL ionc4_write_lon(fileres,lon(iextmin(nb):iextmax(nb)),iextmin(nb),iextmax(nb))

!     CALL ionc4_torigine(fileres,date_ref)

   ! save attribute of the spatial extension of the domain
   ! -----------------------------------------------------
     CALL ionc4_gatt(fileres,'global_imin',igextmin(nb))
     CALL ionc4_gatt(fileres,'global_imax',igextmax(nb))
     CALL ionc4_gatt(fileres,'global_jmin',jgextmin(nb))
     CALL ionc4_gatt(fileres,'global_jmax',jgextmax(nb))

   ! save longitude, latitude fields and variables relative to the mesh grid
   ! -----------------------------------------------------------------------
     CALL out_grid(fileres)

   ! Add the vertical axis at w location
     IF (l_out_zaxis_w(nb)) CALL out_zaxis_w(fileres,kextmin(nb),kextmax(nb),stepk(nb),comm_zoom)

     CALL ionc4_createvar(fileres,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                          standard_name= 'ocean_s_variable',&
#else
                          standard_name= 'ocean_sigma_variable',&
#endif                          
                          l_auxcoordinate=.false.,&
                          dims='z',valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io)
     CALL ionc4_write_sig(fileres,'SIG',sigecrit,kmax)

     IF (l_out_zaxis_w(nb)) THEN
       CALL ionc4_createvar(fileres,'SIG_W',' ','sigma variable at the interface',&
#if defined key_siggen || defined key_gencoord
                          standard_name= 'ocean_s_variable_at_w_location',&
#else
                          standard_name= 'ocean_sigma_variable_at_w_location',&
#endif                          
                          l_auxcoordinate=.false.,&
                          dims='z',valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io)
       sigecrit(:)=sigw(:)
       CALL ionc4_write_sig(fileres,'SIG_W',sigecrit,kmax)
     END IF

#if defined key_siggen || defined key_gencoord
     
     sigecrit(:)=Csu_sig(:)
     CALL ionc4_createvar(fileres,'Csu_sig',' ','C(s) law at mid layer',dims='z', &
                           standard_name='ocean_s_coordinate_function_at_midlayer', &
                           valid_min=-2.0_riosh,valid_max=2.0_riosh,fill_value= rg_valmanq_io, &
                           l_auxcoordinate=.false.)
     CALL ionc4_write_sig(fileres,'Csu_sig',sigecrit,kmax)
     CALL ionc4_vatt_chain(fileres,'Csu_sig','formula_definition', &
          'C(k) = (1-b)*sinh(a*s(k))/sinh(a) + b*[tanh(a*(s(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')

     IF (l_out_zaxis_w(nb)) THEN
       sigecrit(0:kmax)=Csw_sig(0:kmax)
       sigecrit(kmax+1)=0.0_riosh
       CALL ionc4_createvar(fileres,'Csw_sig',' ','C(s) law at interface',dims='z',location_in_cell='w', &
                           standard_name='ocean_s_coordinate_function_at_interface', &
                           valid_min= -2.0_riosh,valid_max= 2.0_riosh,fill_value= rg_valmanq_io, &
                           l_auxcoordinate=.false.)
       CALL ionc4_write_sig(fileres,'Csw_sig',sigecrit,kmax)
       CALL ionc4_vatt_chain(fileres,'Csw_sig','formula_definition', &
          'C_w(k) = (1-b)*sinh(a*s_w(k))/sinh(a) + b*[tanh(a*(s_w(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')
     END IF
     
     CALL ionc4_createvar(fileres,'hc','m', 'depth over which the resolution is increased', &
                           standard_name='ocean_s_coordinate_thick_of_surface_resolution',dims='xy', &
                           valid_min= -20.0_riosh,valid_max= 500.0_riosh,fill_value=-rg_valmanq_io,&
                           l_out_nc4par=l_out_nc4par,l_pack=l_out_pack )
     tecrit(limin:limax,ljmin:ljmax)=hc_sig(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy(fileres,'hc',tecrit,limin,limax,ljmin,ljmax,fill_value=-rg_valmanq_io,l_out_nc4par=l_out_nc4par)
     
     CALL ionc4_createvar(fileres,'b',' ','bottom control parameter',  &
                           standard_name='ocean_s_coordinate_bottom_control', &
                           valid_min=0.0_riosh,valid_max=1.0_riosh,fill_value=rg_valmanq_io)
     vecrit=b_sig
     CALL ionc4_write(fileres,'b',vecrit)
     
     CALL ionc4_createvar(fileres,'theta',' ','surface control parameter', &
                           standard_name ='ocean_s_coordinate_surface_control', &
                           valid_min=0.0_riosh,valid_max=20.0_riosh,fill_value=rg_valmanq_io)
     vecrit=theta_sig
     CALL ionc4_write(fileres,'theta',vecrit)
 
#endif  

   ! Add attributes to follow OCO netCDF convention
     CALL out_netcdf_conv(fileres,nb,comm_zoom,igextmin(nb),igextmax(nb),jgextmin(nb),jgextmax(nb))

     IF_MPI (l_out_nc4par .AND. comm_zoom/=MPI_COMM_VAR) CALL MPI_COMM_FREE(comm_zoom,ierr_mpi)


   ! Save the bathymetry
   ! -------------------

   ! h0
   ! --
     IF (l_obc_tide) THEN
       chain='model_sea_floor_depth_below_sea_level'
     ELSE
       chain='model_sea_floor_depth_below_geoid'
     END IF
     CALL ionc4_createvar(fileres,name_out_h0,units='m',&
                           long_name='bathymetry relative to the mean level',&
                           standard_name = TRIM(chain),&
                           fill_value = -rg_valmanq_io,&
                           valid_min = riog_valid_min_h0,&
                           valid_max = riog_valid_max_h0,&
                           dims = 'xy',&
                           l_pack = l_out_pack,&
                           l_out_nc4par = l_out_nc4par) 
     tecrit(:,:)=h0(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy(fileres,TRIM(name_out_h0),tecrit,limin,limax,ljmin,ljmax,&
                          fill_value=-rg_valmanq_io,l_out_nc4par = l_out_nc4par )


   ! hx
   ! --
     IF (l_obc_tide) THEN
       chain='model_sea_floor_depth_below_sea_level_at_u_location'
     ELSE
       chain='model_sea_floor_depth_below_geoid_at_u_location'
     END IF
     CALL ionc4_createvar(fileres,name_out_hx,units='m',location_in_cell='u',&
                           long_name = 'bathymetry at u-location relative to the mean level',&
                           standard_name = TRIM(chain),&
                           fill_value = -REAL(rg_valmanq_io+h0fond,riosh),&
                           valid_min = riog_valid_min_h0,&
                           valid_max = riog_valid_max_h0,&
                           dims = 'xy',&
                           l_pack = l_out_pack,&
                           l_out_nc4par = l_out_nc4par)
     tecrit(:,:)=hx(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy(fileres,TRIM(name_out_hx),tecrit,limin,limax,ljmin,ljmax ,&
                         fill_value = -REAL(rg_valmanq_io+h0fond,riosh),l_out_nc4par = l_out_nc4par)


   ! hy
   ! --
     IF (l_obc_tide) THEN
       chain='model_sea_floor_depth_below_sea_level_at_v_location'
     ELSE
       chain='model_sea_floor_depth_below_geoid_at_v_location'
     END IF
     CALL ionc4_createvar(fileres,name_out_hy,units='m',location_in_cell='v',&
                           long_name = 'bathymetry at v-location relative to the mean level',&
                           standard_name = TRIM(chain),&
                           fill_value = -REAL(rg_valmanq_io+h0fond,riosh),&
                           valid_min = riog_valid_min_h0,&
                           valid_max = riog_valid_max_h0,&
                           dims = 'xy',&
                           l_pack = l_out_pack,&
                           l_out_nc4par = l_out_nc4par)
     tecrit(:,:)=hy(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy(fileres,TRIM(name_out_hy),tecrit,limin,limax,ljmin,ljmax,&
                         fill_value = -REAL(rg_valmanq_io+h0fond,riosh),l_out_nc4par = l_out_nc4par)


#if defined key_romstools || defined key_castest

   ! save the Coriolis parameter
   ! ---------------------------

     CALL ionc4_createvar(fileres,'corio_u',units='s-1',location_in_cell='u', &
                          long_name ='coriolis parameter', &
                          standard_name = 'coriolis_parameter_at_u_location', &
                          fill_value = REAL(rg_valmanq_io,riosh),&
                          valid_min = -0.001_riosh, &
                          valid_max = 0.001_riosh, &
                          dims= 'xy',l_out_nc4par = l_out_nc4par)
     tecrit(:,:)=fu(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy(fileres,'corio_u',tecrit,limin,limax,ljmin,ljmax,fill_value=rg_valmanq_io,l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(fileres,'corio_v',units='s-1',location_in_cell='v', &
                          long_name ='coriolis parameter', &
                          standard_name = 'coriolis_parameter_at_v_location', &
                          fill_value = REAL(rg_valmanq_io,riosh),&
                          valid_min = -0.001_riosh, &
                          valid_max = 0.001_riosh, &
                          dims= 'xy',l_out_nc4par = l_out_nc4par)
     tecrit(:,:)=fv(limin:limax,ljmin:ljmax)
     CALL ionc4_write_xy(fileres,'corio_v',tecrit,limin,limax,ljmin,ljmax,fill_value=rg_valmanq_io,l_out_nc4par = l_out_nc4par)
#endif


   ! Create time-depending variables
   !--------------------------------

    IF (.NOT. l_out_timeave(nb) ) THEN
     IF (l_out_ssh(nb)) THEN
       IF (l_obc_tide) THEN
         chain='sea_surface_height_above_sea_level'
       ELSE
         chain='sea_surface_height_above_geoid'
       END IF
       CALL ionc4_createvar(fileres,name_out_xe,units= 'm',&
                            long_name = 'sea surface height',&
                            standard_name = TRIM(chain),&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_xe,&
                            valid_max = riog_valid_max_xe,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)    

!       CALL ionc4_vatt_axis(fileres,TRIM(name_out_xe),'Z')
!       CALL ionc4_vatt_positive(fileres,TRIM(name_out_xe),'up')
     END IF

     IF (l_out_ubt(nb)) THEN
       CALL ionc4_createvar(fileres,name_out_u,units= 'm s-1',&
                            long_name = 'barotropic zonal velocity',&
                            standard_name = 'barotropic_sea_water_x_velocity_at_u_location',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       CALL ionc4_createvar(fileres,name_out_v,units= 'm s-1',&
                            long_name = 'barotropic meridional velocity',&
                            standard_name = 'barotropic_sea_water_y_velocity_at_v_location',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                              
     END IF

     IF (l_out_u3d(nb).AND..NOT.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_uz,units= 'm s-1',&
                            long_name = '3d zonal velocity',&
                            standard_name = 'sea_water_x_velocity_at_u_location',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       CALL ionc4_createvar(fileres,name_out_vz,units= 'm s-1',&
                            long_name = '3d meridional velocity',&
                            standard_name = 'sea_water_y_velocity_at_v_location',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
#ifdef key_diag_physwz
      CALL ionc4_createvar(fileres,name_out_wz,units= 's-1',&
                           long_name = '3d terrain following vertical velocity',&
                           standard_name = 'upward_sea_water_sigma_velocity_at_w_location',&
                           location_in_cell='w',&
                           fill_value = rg_valmanq_io,&
                           valid_min = riog_valid_min_u,&
                           valid_max = riog_valid_max_u,&
                           dims= 'zxyt',&
                           l_pack = l_out_pack,&
                           l_out_nc4par = l_out_nc4par)
      CALL ionc4_createvar(fileres,name_out_wz_phys,units= 'm s-1',&
                           long_name = '3d vertical velocity',&
                           standard_name = 'upward_sea_water_velocity_at_w_location',&
                           location_in_cell='w',&
                           fill_value = rg_valmanq_io,&
                           valid_min = riog_valid_min_u,&
                           valid_max = riog_valid_max_u,&
                           dims= 'zxyt',&
                           l_pack = l_out_pack,&
                           l_out_nc4par = l_out_nc4par)
#endif
     END IF

     IF (l_out_sal(nb).AND.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_sal,units= '1e-3',&
                            long_name = 'salinity',&
                            standard_name = 'sea_water_salinity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_sal,&
                            valid_max = riog_valid_max_sal,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     ELSE IF (l_out_sal(nb).AND..NOT.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_sal,units= '1e-3',&
                            long_name = 'salinity',&
                            standard_name = 'sea_water_salinity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_sal,&
                            valid_max = riog_valid_max_sal,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     END IF

     IF (l_out_temp(nb).AND.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_temp,units= 'degrees_Celsius',&
                            long_name = 'temperature',&
                            standard_name = 'sea_water_potential_temperature',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_temp,&
                            valid_max = riog_valid_max_temp,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     ELSE IF (l_out_temp(nb).AND..NOT.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_temp,units= 'degrees_Celsius',&
                            long_name = 'temperature',&
                            standard_name = 'sea_water_potential_temperature',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_temp,&
                            valid_max = riog_valid_max_temp,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     END IF

     IF (l_out_visc(nb).AND.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_vis2d,units= 'm2 s-1',&
                            long_name = 'viscosity',&
                            standard_name = 'ocean_tracer_xy_laplacian_diffusivity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_visc,&
                            valid_max = riog_valid_max_visc,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     ELSE IF (l_out_visc(nb).AND..NOT.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_vishxe,units= 'm2 s-1',&
                            long_name = 'horizontal viscosity',&
                            standard_name = 'ocean_momentum_xy_laplacian_diffusivity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_visc,&
                            valid_max = riog_valid_max_visc,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)      
       CALL ionc4_createvar(fileres,name_out_vishphi,units= 'm2 s-1',&
                            long_name = 'horizontal viscosity',&
                            standard_name = 'ocean_momentum_xy_laplacian_diffusivity_at_f_location',&
                            location_in_cell='f',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_visc,&
                            valid_max = riog_valid_max_visc,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     END IF

     IF (l_out_turb(nb).AND..NOT.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_nz,units= 'm2 s-1',&
                            long_name = 'vertical viscosity',&
                            standard_name = 'ocean_vertical_momentum_diffusivity_at_w_location',&
                            location_in_cell='w',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_nz,&
                            valid_max = riog_valid_max_nz,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
       CALL ionc4_createvar(fileres,name_out_kz,units= 'm2 s-1',&
                            long_name = 'vertical diffusivity',&
                            standard_name = 'ocean_vertical_tracer_diffusivity_at_w_location',&
                            location_in_cell='w',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_kz,&
                            valid_max = riog_valid_max_kz,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_buoy(nb).AND..NOT.l_modele2d) THEN
       CALL ionc4_createvar(fileres,name_out_bz,units= 'kg m-3',&
                            long_name = 'density of water at atmospheric pressure minus 1000 kg m-3',&
                            standard_name = 'sea_water_sigma_t',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_bz,&
                            valid_max = riog_valid_max_bz,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_wind(nb)) THEN
       CALL ionc4_createvar(fileres,name_out_tosx,units= 'N m-2',&
                            long_name = 'U-component of Surface Wind Stress',&
                            standard_name = 'surface_downward_x_stress_at_u_location',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wstress,&
                            valid_max = riog_valid_max_wstress,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       CALL ionc4_createvar(fileres,name_out_tosy,units= 'N m-2',&
                            long_name = 'V-component of Surface Wind Stress',&
                            standard_name = 'surface_downward_y_stress_at_v_location',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wstress,&
                            valid_max = riog_valid_max_wstress,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
     END IF

#ifdef key_gencoord_test
     name_out = 'depth_t'
     CALL ionc4_createvar(fileres,name_out,units= 'm',&
                            long_name = 'depth of the tracer',&
                            standard_name = 'ocean_layer_depth',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_h0,&
                            valid_max = riog_valid_max_h0,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     name_out = 'depth_u'
     CALL ionc4_createvar(fileres,name_out,units= 'm',&
                            location_in_cell='u',&
                            long_name = 'depth of the v variable (from 0 and not the SSH)',&
                            standard_name = 'ocean_layer_depth_at_u_location',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_h0,&
                            valid_max = riog_valid_max_h0,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     name_out = 'depth_v'
     CALL ionc4_createvar(fileres,name_out,units= 'm',&
                            location_in_cell='v',&
                            long_name = 'depth of the v variable (from 0 and not the SSH)',&
                            standard_name = 'ocean_layer_depth_at_v_location',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_h0,&
                            valid_max = riog_valid_max_h0,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
     name_out = 'depth_w'
     CALL ionc4_createvar(fileres,name_out,units= 'm',&
                            location_in_cell='w',&
                            long_name = 'depth of the interface (from 0 and not the SSH)',&
                            standard_name = 'ocean_interface_depth_at_w_location',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_h0,&
                            valid_max = riog_valid_max_h0,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par) 
#endif

#ifdef key_wave
     IF (l_out_wave(nb)) THEN
       IF (l_wave_rdir) &
       CALL ionc4_createvar(fileres,name_out_wave_dir,units= 'degrees',&
                            long_name = 'Direction of the combined wind waves and swell',&
                            !standard_name = 'sea_surface_wave_to_direction_due_to_wind_and_swell_waves',&
                            standard_name = 'sea_surface_wave_to_direction',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wave_dir,&
                            valid_max = riog_valid_max_wave_dir,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       IF (l_wave_rhs) &
       CALL ionc4_createvar(fileres,name_out_wave_hs,units= 'm',&
                            long_name = 'Significant height of combined wind waves and swell',&
                            !standard_name = 'sea_surface_wave_significant_height_due_to_wind_and_swell_waves',&
                            standard_name = 'sea_surface_wave_significant_height',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wave_hs,&
                            valid_max = riog_valid_max_wave_hs,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       IF (l_wave_rfoc) &
       CALL ionc4_createvar(fileres,name_out_wave_foc,units= 'W m-2',&
                            long_name = 'Wave mixing energy',&
                            standard_name = 'wind_mixing_energy_flux_into_sea_water_due_to_wave',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wave_foc,&
                            valid_max = riog_valid_max_wave_foc,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)       
       IF (l_wave_rubot .OR. (l_wave_ruubot .AND. l_wave_rvubot)) &
       CALL ionc4_createvar(fileres,name_out_wave_ubot,units= 'm s-1',&
                            long_name = 'magnitude of the velocity due to waves at the sea floor',&
                            standard_name = 'magnitude_of_sea_floor_due_to_wave',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wave_ubot,&
                            valid_max = riog_valid_max_wave_ubot,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)       
     END IF

#endif
    END IF   ! end test on .NOT.l_out_timeave(nb)

     IF (nbave /= 0 .AND. l_out_timeave(nb) ) THEN
       IF (l_out_ssh(nb)) THEN
         chain='average_of_sea_surface_height_above_geoid'
         CALL ionc4_createvar(fileres,name_out_xe,units= 'm',&
                            long_name = 'time averaged sea surface height',&
                            standard_name = TRIM(chain),&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_xe,&
                            valid_max = riog_valid_max_xe,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)

!         CALL ionc4_vatt_axis(fileres,TRIM(name_out_xe),'Z')
!         CALL ionc4_vatt_positive(fileres,TRIM(name_out_xe),'up')
       END IF
       IF (l_out_ubt(nb)) THEN
         CALL ionc4_createvar(fileres,name_out_u,units= 'm s-1',&
                            long_name = 'time averaged barotropic zonal velocity',&
                            standard_name = 'average_of_barotropic_sea_water_x_velocity_at_u_location',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
         CALL ionc4_createvar(fileres,name_out_v,units= 'm s-1',&
                            long_name = 'time averaged barotropic meridional velocity',&
                            standard_name = 'average_of_barotropic_sea_water_y_velocity_at_v_location',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                         
       END IF
       IF (l_out_u3d(nb).AND..NOT.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_uz,units= 'm s-1',&
                            long_name = 'time averaged 3d zonal velocity',&
                            standard_name = 'average_of_sea_water_x_velocity_at_u_location',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
         CALL ionc4_createvar(fileres,name_out_vz,units= 'm s-1',&
                            long_name = 'time averaged 3d meridional velocity',&
                            standard_name = 'average_of_sea_water_y_velocity_at_v_location',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_u,&
                            valid_max = riog_valid_max_u,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
#ifdef key_diag_physwz
        CALL ionc4_createvar(fileres,name_out_wz,units= 's-1',&
                           long_name = 'time averaged 3d terrain following vertical velocity',&
                           standard_name = 'upward_sea_water_sigma_velocity_at_w_location',&
                           location_in_cell='w',&
                           fill_value = rg_valmanq_io,&
                           valid_min = riog_valid_min_u,&
                           valid_max = riog_valid_max_u,&
                           dims= 'zxyt',&
                           l_pack = l_out_pack,&
                           l_out_nc4par = l_out_nc4par)
        CALL ionc4_createvar(fileres,name_out_wz_phys,units= 'm s-1',&
                           long_name = 'time averaged 3d vertical velocity',&
                           standard_name = 'upward_sea_water_velocity_at_w_location',&
                           location_in_cell='w',&
                           fill_value = rg_valmanq_io,&
                           valid_min = riog_valid_min_u,&
                           valid_max = riog_valid_max_u,&
                           dims= 'zxyt',&
                           l_pack = l_out_pack,&
                           l_out_nc4par = l_out_nc4par)
#endif
       END IF
       IF (l_out_sal(nb).AND.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_sal,units= '1e-3',&
                            long_name = 'time averaged salinity',&
                            standard_name = 'average_of_sea_water_salinity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_sal,&
                            valid_max = riog_valid_max_sal,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       ELSE IF (l_out_sal(nb).AND..NOT.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_sal,units= '1e-3',&
                            long_name = 'time averaged salinity',&
                            standard_name = 'average_of_sea_water_salinity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_sal,&
                            valid_max = riog_valid_max_sal,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       END IF
       IF (l_out_temp(nb).AND.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_temp,units= 'degrees_Celsius',&
                            long_name = 'time averaged temperature',&
                            standard_name = 'average_of_sea_water_potential_temperature',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_temp,&
                            valid_max = riog_valid_max_temp,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       ELSE IF (l_out_temp(nb).AND..NOT.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_temp,units= 'degrees_Celsius',&
                            long_name = 'time averaged temperature',&
                            standard_name = 'average_of_sea_water_potential_temperature',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_temp,&
                            valid_max = riog_valid_max_temp,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       END IF
       IF (l_out_visc(nb).AND.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_vis2d,units= 'm2 s-1',&
                            long_name = 'time averaged viscosity',&
                            standard_name = 'average_of_ocean_tracer_xy_laplacian_diffusivity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_visc,&
                            valid_max = riog_valid_max_visc,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       ELSE IF (l_out_visc(nb).AND..NOT.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_vishxe,units= 'm2 s-1',&
                            long_name = 'time averaged horizontal viscosity',&
                            standard_name = 'average_of_ocean_momentum_xy_laplacian_diffusivity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_visc,&
                            valid_max = riog_valid_max_visc,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
         CALL ionc4_createvar(fileres,name_out_vishphi,units= 'm2 s-1',&
                            long_name = 'time average horizontal viscosity',&
                            standard_name = 'average_of_ocean_momentum_xy_laplacian_diffusivity_at_f_location',&
                            location_in_cell='f',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_visc,&
                            valid_max = riog_valid_max_visc,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       END IF
       IF (l_out_turb(nb).AND..NOT.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_nz,units= 'm2 s-1',&
                            long_name = 'time averaged vertical viscosity',&
                            standard_name = 'average_of_ocean_vertical_momentum_diffusivity_at_w_location',&
                            location_in_cell='w',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_nz,&
                            valid_max = riog_valid_max_nz,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
         CALL ionc4_createvar(fileres,name_out_kz,units= 'm2 s-1',&
                            long_name = 'time averaged vertical diffusivity',&
                            standard_name = 'average_of_ocean_vertical_tracer_diffusivity_at_w_location',&
                            location_in_cell='w',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_kz,&
                            valid_max = riog_valid_max_kz,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       END IF
       IF (l_out_buoy(nb).AND..NOT.l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_out_bz,units= 'kg m-3',&
                            long_name = 'time averaged density of water at atmospheric pressure minus 1000 kg m-3',&
                            standard_name = 'average_of_sea_water_sigma_t',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_bz,&
                            valid_max = riog_valid_max_bz,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       END IF


     END IF


     IF (nbatm /= 0 .AND. l_out_atmsflx(nb) ) THEN
       CALL ionc4_createvar(fileres,name_out_mrad,units= 'W m-2',&
                            long_name = 'radiative flux',&
                            standard_name = 'average_of_surface_net_downward_shortwave_flux',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_flux,&
                            valid_max = riog_valid_max_flux,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)      
       CALL ionc4_createvar(fileres,name_out_mirlos,units= 'W m-2',&
                            long_name = 'thermal flux',&
                            standard_name = 'average_of_surface_net_downward_longwave_flux',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_flux,&
                            valid_max = riog_valid_max_flux,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                               
       CALL ionc4_createvar(fileres,name_out_mlatlos,units= 'W m-2',&
                            long_name = 'latent flux',&
                            standard_name = 'average_of_surface_downward_latent_heat_flux',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_flux,&
                            valid_max = riog_valid_max_flux,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                               
       CALL ionc4_createvar(fileres,name_out_msenslos,units= 'W m-2',&
                            long_name = 'sensible flux',&
                            standard_name = 'average_of_surface_downward_sensible_heat_flux',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_flux,&
                            valid_max = riog_valid_max_flux,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                               
       CALL ionc4_createvar(fileres,name_out_mevap,units= 'm',&
                            long_name = 'Evaporation',&
                            standard_name = 'lwe_thickness_of_water_evaporation_amount',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_flux,&
                            valid_max = riog_valid_max_flux,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                               
       CALL ionc4_createvar(fileres,name_out_mrain,units= 'm',&
                            long_name = 'rain',&
                            standard_name = 'lwe_thickness_of_precipitation_amount',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_flux,&
                            valid_max = riog_valid_max_flux,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                               
       CALL ionc4_createvar(fileres,name_out_mtosx,units= 'N m-2',&
                            long_name = 'Averaged U-component of Surface Wind Stress',&
                            standard_name = 'average_of_surface_downward_x_stress_at_u_location',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wstress,&
                            valid_max = riog_valid_max_wstress,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                               
       CALL ionc4_createvar(fileres,name_out_mtosy,units= 'N m-2',&
                            long_name = 'Averaged V-component of Surface Wind Stress',&
                            standard_name = 'average_of_surface_downward_y_stress_at_v_location',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wstress,&
                            valid_max = riog_valid_max_wstress,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                               
#ifdef key_sflx_analyse
       name_out_mwindx='uwind'
       CALL ionc4_createvar(fileres,name_out_mwindx,units= 'm s-1',&
                            long_name = 'Averaged U-wind at 10 m',&
                            standard_name = 'average_of_x_wind_at_10m',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wstress,&
                            valid_max = riog_valid_max_wstress,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       name_out_mwindy='vwind'
       CALL ionc4_createvar(fileres,name_out_mwindy,units= 'm s-1',&
                            long_name = 'Averaged V-wind at 10 m',&
                            standard_name = 'average_of_y_wind_at_10m',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_wstress,&
                            valid_max = riog_valid_max_wstress,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)        
       name_out_mrh='rh'
       CALL ionc4_createvar(fileres,name_out_mrh,units= '1',&
                            long_name = 'Relative Humidity',&
                            standard_name = 'average_of_relative_humidity_at_2m',&
                            fill_value = rg_valmanq_io,&
                            valid_min = -1.0_riosh,&
                            valid_max = 2.0_riosh,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       name_out_mfcc='fcc'
       CALL ionc4_createvar(fileres,name_out_mfcc,units= '1',&
                            long_name = 'Total Cloud Cover',&
                            standard_name = 'average_of_cloud_area_fraction',&
                            fill_value = rg_valmanq_io,&
                            valid_min = -1.0_riosh,&
                            valid_max = 2.0_riosh,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       name_out_msat='sat'
       CALL ionc4_createvar(fileres,name_out_msat,units= 'K',&
                            long_name = 'air temperature at 2m',&
                            standard_name = 'average_of_air_temperature_at_2m',&
                            fill_value = rg_valmanq_io,&
                            valid_min = -riog_valid_max_temp,&
                            valid_max = riog_valid_max_temp,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       name_out_msst='sst'
       CALL ionc4_createvar(fileres,name_out_msst,units= 'degrees_Celsius',&
                            long_name = 'sea surface temperature',&
                            standard_name = 'average_of_sea_surface_temperature',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_temp,&
                            valid_max = riog_valid_max_temp,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)
       name_out_mpa='pa'
       CALL ionc4_createvar(fileres,name_out_mpa,units= 'Pa',&
                            long_name = 'air pressure',&
                            standard_name = 'average_of_air_pressure_at_sea_level',&
                            fill_value = rg_valmanq_io,&
                            valid_min = 0.0_riosh,&
                            valid_max = 1500.0_riosh,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)

#endif

     END IF

#ifdef key_offline_outuvflux
     IF ( l_out_off(nb) ) THEN
       IF (l_obc_tide) THEN
         chain='sea_surface_height_above_sea_level'
       ELSE
         chain='sea_surface_height_above_geoid'
       END IF
       CALL ionc4_createvar(fileres,name_out_mxe,units= 'm',&
                            long_name = 'mean sea surface height',&
                            standard_name = TRIM(chain),&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_xe,&
                            valid_max = riog_valid_max_xe,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)       

       CALL ionc4_createvar(fileres,name_out_mhex,units= 'm',&
                            long_name = 'mean water depth at u location',&
                            standard_name = 'average_of_water_depth_below_geoid_u_location',&
                            location_in_cell='u',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_h0,&
                            valid_max = riog_valid_max_h0,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)

       CALL ionc4_createvar(fileres,name_out_mhey,units= 'm',&
                            long_name = 'mean water depth at v location',&
                            standard_name = 'average_of_water_depth_below_geoid_v_location',&
                            location_in_cell='v',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_h0,&
                            valid_max = riog_valid_max_h0,&
                            dims= 'xyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)

       CALL ionc4_createvar(fileres,name_out_mkz,units= 'm2 s-1',&
                            long_name = 'mean turbulent diffusion',&
                            standard_name = 'average_of_ocean_vertical_tracer_diffusivity',&
                            fill_value = rg_valmanq_io,&
                            valid_min = riog_valid_min_kz,&
                            valid_max = riog_valid_max_kz,&
                            dims= 'zxyt',&
                            l_pack = l_out_pack,&
                            l_out_nc4par = l_out_nc4par)                            
     END IF
#endif

#ifdef key_substance

     DO l = 1,nv_state
      IF (l_out_subs(l)) THEN

       IF (l_out_log) THEN
         unit_var(irk_fil(l))= 'log('//TRIM(unit_var(irk_fil(l)))//')' 
       ELSE
         unit_var(irk_fil(l))=TRIM(unit_var(irk_fil(l)))
       ENDIF
       IF (l_modele2d) THEN
         CALL ionc4_createvar(fileres,name_var(irk_fil(l)),units= TRIM(unit_var(irk_fil(l))),&
                              long_name = TRIM(long_name_var(irk_fil(l))),&
                              standard_name = TRIM(standard_name_var(irk_fil(l))),&
                              fill_value = rg_valmanq_io,&
                              valid_min = REAL(valid_min_var(irk_fil(l)),riosh),&
                              valid_max = REAL(valid_max_var(irk_fil(l)),riosh),&
                              dims= 'xyt',&
                              l_pack = .false.,&
                              l_out_nc4par = l_out_nc4par) 
       ELSE
         CALL ionc4_createvar(fileres,name_var(irk_fil(l)),units= TRIM(unit_var(irk_fil(l))),&
                              long_name = TRIM(long_name_var(irk_fil(l))),&
                              standard_name = TRIM(standard_name_var(irk_fil(l))),&
                              fill_value = rg_valmanq_io,&
                              valid_min = REAL(valid_min_var(irk_fil(l)),riosh),&
                              valid_max = REAL(valid_max_var(irk_fil(l)),riosh),&
                              dims= 'zxyt',&
                              l_pack = .false.,&
                              l_out_nc4par = l_out_nc4par)         
       ENDIF
      ENDIF
     ENDDO
#endif

!------------------------------------
! sauvegarde de la condition initiale
!------------------------------------

!     IF ( (.not.l_initfromfile .and. date_startshift == 0.0_rlg) .OR. (l_filebydate(nb) .AND. nbouc /= 0) .OR. (l_initfromfile .AND. nbouc== 0) ) THEN
     IF (tect >= tsort(nb).AND.tect <= tfinsor(nb)) THEN
       CALL ionc4_write_time(fileres,0,tect)
       
      IF (.NOT. l_out_timeave(nb) ) THEN   ! instantaneous outputs

       IF (l_out_ssh(nb)) CALL out_write_xytvar(fileres,name_out_xe,rg_valmanq_io,mask_h0, &
                                                x(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

       IF (l_out_ubt(nb)) THEN
         CALL out_write_xytvar(fileres,name_out_u,rg_valmanq_io,mask_hx, &
                               u(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_v,rg_valmanq_io,mask_hy, &
                               v(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF


       IF (l_out_u3d(nb) .AND. .NOT.l_modele2d) THEN
         CALL out_write_zxytvar(fileres,name_out_uz,rg_valmanq_io,mask_hx, &
                                uz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_zxytvar(fileres,name_out_vz,rg_valmanq_io,mask_hy, &
                                vz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#ifdef key_diag_physwz
         CALL out_write_zxytvar(fileres,name_out_wz,rg_valmanq_io,mask_h0, & 
!                                wz(0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
                                wz(0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_zxytvar(fileres,name_out_wz_phys,rg_valmanq_io,mask_h0, & 
                                wz_phys(0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
!                                wz_phys(0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
#endif
       END IF

       IF (l_out_sal(nb) .AND. l_modele2d) THEN
         CALL out_write_xytvar(fileres,name_out_sal,rg_valmanq_io,mask_h0, &
                               sal(1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       ELSE IF (l_out_sal(nb).AND..NOT.l_modele2d) THEN
         CALL out_write_zxytvar(fileres,name_out_sal,rg_valmanq_io,mask_h0, &
                                sal(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF

       IF (l_out_temp(nb) .AND. l_modele2d) THEN
         CALL out_write_xytvar(fileres,name_out_temp,rg_valmanq_io,mask_h0, &
                               temp(1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       ELSE IF (l_out_temp(nb) .AND. .NOT.l_modele2d) THEN
         CALL out_write_zxytvar(fileres,name_out_temp,rg_valmanq_io,mask_h0, &
                                temp(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF

       IF (l_out_visc(nb) .AND. l_modele2d) THEN
         CALL out_write_xytvar(fileres,name_out_vis2d,rg_valmanq_io,mask_h0, &
                               vis2d(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       ELSE IF (l_out_visc(nb) .AND. .NOT.l_modele2d) THEN
         CALL out_write_zxytvar(fileres,name_out_vishxe,rg_valmanq_io,mask_h0, &
                                vish_xe(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_zxytvar(fileres,name_out_vishphi,rg_valmanq_io,mask_h0, &
                                vish_phi(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF

       IF (l_out_turb(nb) .AND. .NOT.l_modele2d) THEN
         CALL out_write_zxytvar(fileres,name_out_nz,rg_valmanq_io,mask_h0, &
                                nz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_zxytvar(fileres,name_out_kz,rg_valmanq_io,mask_h0, &
                                kz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF

       IF (l_out_buoy(nb) .AND. .NOT.l_modele2d) THEN
         tmp3d(:,:,:) = 1000.0_rsh - 1000.0_rsh*boz(:,limin:limax,ljmin:ljmax)/grav
         CALL out_write_zxytvar(fileres,name_out_bz,rg_valmanq_io,mask_h0, &
                                tmp3d,0,l_out_nc4par = l_out_nc4par)
       END IF

       IF (l_out_wind(nb)) THEN
         CALL out_write_xytvar(fileres,name_out_tosx,rg_valmanq_io,mask_h0, &
                               tosx(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_tosy,rg_valmanq_io,mask_h0, &
                               tosy(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF

#ifdef key_gencoord_test
     tmp3d(:,:,:)=rg_valmanq_io
     tmp3du(:,:,:)=rg_valmanq_io
     tmp3dv(:,:,:)=rg_valmanq_io
     tmp3dw(:,:,:)=rg_valmanq_io
     DO j=ljmin,ljmax
       DO i=MAX(limin,ig(j)),MIN(limax,id(j))
         IF ((j.GE.jb(i)).AND.(j.LE.jh(i))) THEN
           IF (h0(i,j)+x(i,j) > h0fond) THEN
             DO k=1,kmax
               tmp3d(k,i,j) = f_zu(h0(i,j),x(i,j),k,i,j)
               tmp3dw(k,i,j) = f_zw(h0(i,j),x(i,j),k,i,j)
             END DO
           END IF
           ! toseelater test et estimation ci dessus pas nickel
           IF (hx(i,j)+x(i,j) > h0fond) THEN
             DO k=1,kmax
               tmp3du(k,i,j) = f_zu_x(hx(i,j),x(i,j),k,i,j)
             END DO
           END IF
           IF (hy(i,j)+x(i,j) > h0fond) THEN
             DO k=1,kmax
               tmp3dv(k,i,j) = f_zu_y(hy(i,j),x(i,j),k,i,j)
             END DO
           END IF
         END IF
       END DO
     END DO
     name_out = 'depth_t'
     CALL out_write_zxytvar(fileres,name_out,rg_valmanq_io,mask_h0, &
                            tmp3d,0,l_out_nc4par = l_out_nc4par)
     name_out = 'depth_u'
     CALL out_write_zxytvar(fileres,name_out,rg_valmanq_io,mask_hx, &
                           tmp3du,0,l_out_nc4par = l_out_nc4par)
     name_out = 'depth_v'
     CALL out_write_zxytvar(fileres,name_out,rg_valmanq_io,mask_hy, &
                           tmp3dv,0,l_out_nc4par = l_out_nc4par)
     name_out = 'depth_w'
     CALL out_write_zxytvar(fileres,name_out,rg_valmanq_io,mask_h0, &
                           tmp3dw,0,l_out_nc4par = l_out_nc4par)
#endif


#ifdef key_wave
       IF (l_out_wave(nb)) THEN
         IF (l_wave_rdir) &
         CALL out_write_xytvar(fileres,name_out_wave_dir,rg_valmanq_io,mask_h0, &
                               wave_dir(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         IF (l_wave_rhs) &
         CALL out_write_xytvar(fileres,name_out_wave_hs,rg_valmanq_io,mask_h0, &
                               wave_hs(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         IF (l_wave_rfoc) &
         CALL out_write_xytvar(fileres,name_out_wave_foc,rg_valmanq_io,mask_h0, &
                               wave_foc(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         IF (l_wave_rubot .OR. (l_wave_ruubot .AND. l_wave_rvubot)) &
         CALL out_write_xytvar(fileres,name_out_wave_ubot,rg_valmanq_io,mask_h0, &
                               wave_ubot(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF
#endif

    END IF   ! end test on .NOT.l_out_timeave(nb)

       IF (nbave /= 0 .AND. l_out_timeave(nb) ) THEN
         IF (l_out_ssh(nb)) CALL out_write_xytvar(fileres,name_out_xe,rg_valmanq_io,mask_h0, &
                                                ssh_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         IF (l_out_ubt(nb)) THEN
           CALL out_write_xytvar(fileres,name_out_u,rg_valmanq_io,mask_hx, &
                               u_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_xytvar(fileres,name_out_v,rg_valmanq_io,mask_hy, &
                               v_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF
         IF (l_out_u3d(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileres,name_out_uz,rg_valmanq_io,mask_hx, &
                                uz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileres,name_out_vz,rg_valmanq_io,mask_hy, &
                                vz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#ifdef key_diag_physwz
           CALL out_write_zxytvar(fileres,name_out_wz,rg_valmanq_io,mask_h0, &
!                                wz_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
                                wz_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileres,name_out_wz_phys,rg_valmanq_io,mask_h0, &
!                                wz_phys_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
                                wz_phys_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#endif
         END IF

         IF (l_out_sal(nb) .AND. l_modele2d) THEN
           CALL out_write_xytvar(fileres,name_out_sal,rg_valmanq_io,mask_h0, &
                               sal_ave(nb,1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         ELSE IF (l_out_sal(nb).AND..NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileres,name_out_sal,rg_valmanq_io,mask_h0, &
                                sal_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF
         IF (l_out_temp(nb) .AND. l_modele2d) THEN
           CALL out_write_xytvar(fileres,name_out_temp,rg_valmanq_io,mask_h0, &
                               temp_ave(nb,1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         ELSE IF (l_out_temp(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileres,name_out_temp,rg_valmanq_io,mask_h0, &
                                temp_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF

         IF (l_out_visc(nb) .AND. l_modele2d) THEN
           CALL out_write_xytvar(fileres,name_out_vis2d,rg_valmanq_io,mask_h0, &
                               vis2d_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         ELSE IF (l_out_visc(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileres,name_out_vishxe,rg_valmanq_io,mask_h0, &
                                vish_xe_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileres,name_out_vishphi,rg_valmanq_io,mask_h0, &
                                vish_phi_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF
         IF (l_out_turb(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileres,name_out_nz,rg_valmanq_io,mask_h0, &
                                nz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileres,name_out_kz,rg_valmanq_io,mask_h0, &
                                kz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF

         IF (l_out_buoy(nb) .AND. .NOT.l_modele2d) THEN
           tmp3d(:,:,:) = 1000.0_rsh - 1000.0_rsh*boz_ave(nb,:,limin:limax,ljmin:ljmax)/grav
           CALL out_write_zxytvar(fileres,name_out_bz,rg_valmanq_io,mask_h0, &
                                tmp3d,0,l_out_nc4par = l_out_nc4par)
         END IF

       ! initialize averaged dynamical and thermodynamical variables
       ! -----------------------------------------------------------

         dtn = t - tsort(nb)
         IF (l_out_ssh(nb)) &
         ssh_ave(nb,:,:) = x(limin:limax,ljmin:ljmax)*dtn
         IF (l_out_ubt(nb)) THEN
           u_ave(nb,:,:) = u(limin:limax,ljmin:ljmax)*dtn
           v_ave(nb,:,:) = v(limin:limax,ljmin:ljmax)*dtn
         END IF
         IF (l_out_u3d(nb).AND..NOT.l_modele2d) THEN
           uz_ave(nb,:,:,:) = dtn * uz(:,limin:limax,ljmin:ljmax)
           vz_ave(nb,:,:,:) = dtn * vz(:,limin:limax,ljmin:ljmax)
         END IF
         IF (l_out_temp(nb)) &
           temp_ave(nb,:,:,:) = dtn * temp(:,limin:limax,ljmin:ljmax)
         IF (l_out_sal(nb)) &
           sal_ave(nb,:,:,:) = dtn * sal(:,limin:limax,ljmin:ljmax)
         IF (l_out_visc(nb).AND.l_modele2d) THEN
           vis2d_ave(nb,:,:) = dtn * vis2d(limin:limax,ljmin:ljmax)
         ELSE IF (l_out_visc(nb).AND..NOT.l_modele2d) THEN
           vish_xe_ave(nb,:,:,:) = dtn * vish_xe(:,limin:limax,ljmin:ljmax)
           vish_phi_ave(nb,:,:,:) = dtn * vish_phi(:,limin:limax,ljmin:ljmax)
         END IF
         IF (l_out_turb(nb).AND..NOT.l_modele2d) THEN
           nz_ave(nb,:,:,:) = dtn * nz(:,limin:limax,ljmin:ljmax)
           kz_ave(nb,:,:,:) = dtn * kz(:,limin:limax,ljmin:ljmax)
         END IF
         IF (l_out_buoy(nb).AND..NOT.l_modele2d) &
           boz_ave(nb,:,:,:) = dtn * boz(:,limin:limax,ljmin:ljmax)
         dt_ave(nb) =  dtn
       END IF

       IF (nbatm /= 0 .AND. l_out_atmsflx(nb) ) THEN
         CALL out_write_xytvar(fileres,name_out_mrad,rg_valmanq_io,mask_h0,     &
                               rad_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_mirlos,rg_valmanq_io,mask_h0,   &
                               irlos_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_mlatlos,rg_valmanq_io,mask_h0,  &
                               latlos_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_msenslos,rg_valmanq_io,mask_h0, &
                               senslos_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_mevap,rg_valmanq_io,mask_h0, &
                               evap_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_mrain,rg_valmanq_io,mask_h0, &
                               rain_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_mtosx,rg_valmanq_io,mask_h0, &
                               tosx_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         CALL out_write_xytvar(fileres,name_out_mtosy,rg_valmanq_io,mask_h0, &
                               tosy_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#ifdef key_sflx_analyse
         name_out_mwindx='uwind'
         CALL out_write_xytvar(fileres,name_out_mwindx,rg_valmanq_io,mask_h0, &
                               windx_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         name_out_mwindy='vwind'
         CALL out_write_xytvar(fileres,name_out_mwindy,rg_valmanq_io,mask_h0, &
                               windy_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         name_out_mrh='rh'
         CALL out_write_xytvar(fileres,name_out_mrh,rg_valmanq_io,mask_h0, &
                               rh_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         name_out_mfcc='fcc'
         CALL out_write_xytvar(fileres,name_out_mfcc,rg_valmanq_io,mask_h0, &
                               fcc_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         name_out_msat='sat'
         CALL out_write_xytvar(fileres,name_out_msat,rg_valmanq_io,mask_h0, &
                               sat_cum(nb,limin:limax,ljmin:ljmax)+273.15_rsh,0,l_out_nc4par = l_out_nc4par)
         name_out_msst='sst'
         CALL out_write_xytvar(fileres,name_out_msst,rg_valmanq_io,mask_h0, &
                               sst_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         name_out_mpa='pa'
         CALL out_write_xytvar(fileres,name_out_mpa,rg_valmanq_io,mask_h0, &
                               pa_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#endif

       ! initialize cumulative atmospherical surface fluxes
       ! --------------------------------------------------

         dtn = t - tsort(nb)
         rad_cum(nb,:,:)     = rad(limin:limax,ljmin:ljmax)    *dtn
         irlos_cum(nb,:,:)   = irlos(limin:limax,ljmin:ljmax)  *dtn
         latlos_cum(nb,:,:)  = latlos(limin:limax,ljmin:ljmax) *dtn
         senslos_cum(nb,:,:) = senslos(limin:limax,ljmin:ljmax)*dtn
         evap_cum(nb,:,:)    = evap(limin:limax,ljmin:ljmax)   *dtn
         rain_cum(nb,:,:)    = rain(limin:limax,ljmin:ljmax)   *dtn
         tosx_cum(nb,:,:)    = tosx(limin:limax,ljmin:ljmax)   *dtn
         tosy_cum(nb,:,:)    = tosy(limin:limax,ljmin:ljmax)   *dtn
#ifdef key_sflx_analyse
         windx_cum(nb,:,:)   = windx(limin:limax,ljmin:ljmax)  *dtn
         windy_cum(nb,:,:)   = windy(limin:limax,ljmin:ljmax)  *dtn
         rh_cum(nb,:,:)      = rh(limin:limax,ljmin:ljmax)     *dtn
         fcc_cum(nb,:,:)     = fcc(limin:limax,ljmin:ljmax)    *dtn
         sat_cum(nb,:,:)     = sat(limin:limax,ljmin:ljmax)    *dtn
         sst_cum(nb,:,:)     = temp(kmax,limin:limax,ljmin:ljmax)*dtn
         pa_cum(nb,:,:)      = pa(limin:limax,ljmin:ljmax)     *dtn
#endif
         dt_cum(nb) =  dtn

       END IF

#ifdef key_offline_outuvflux

       IF ( l_out_off(nb) ) THEN

       ! WARNING : wetting and drying are not taken into account yet !
       ! doit on vraiment tourner sur i=ig(j)-1,id(j) pour hexm ???? (r363 et plus)

         CALL out_write_xytvar(fileres,name_out_mxe,rg_valmanq_io,mask_h0, &
                               xem(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         CALL out_write_xytvar(fileres,name_out_mhex,rg_valmanq_io,mask_h0, &
                               hexm(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         CALL out_write_xytvar(fileres,name_out_mhey,rg_valmanq_io,mask_h0, &
                               heym(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         CALL out_write_zxytvar(fileres,name_out_mkz,rg_valmanq_io,mask_h0, &
                                kzm(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

       END IF
#endif

#ifdef key_substance
       DO l = 1,nv_state
        IF (l_out_subs(l)) THEN
         IF (l_modele2d) THEN
           tecrit(:,:)=rg_valmanq_io
           DO j=ljmin,ljmax
             DO i=MAX(limin,ig(j)),MIN(limax,id(j))
               IF ((j.GE.jb(i)).AND.(j.LE.jh(i))) THEN
                 IF (l_out_log) THEN
                   tecrit(i,j) = LOG(MAX(cv_wat(l,1,i,j),1.0e-30))/LOG(10.0_rsh)
                 ELSE
                   !tecrit(i,j) =MAX(cv_wat(l,1,i,j),1.0e-30)
                   tecrit(i,j) =cv_wat(l,1,i,j)
                 END IF
                 IF (h0(i,j)+x(i,j) <= h0fond) tecrit(i,j)=rg_valmanq_io
               END IF
             END DO
           END DO
           CALL ionc4_write_xyt(fileres,TRIM(name_var(irk_fil(l))),tecrit,limin,limax,ljmin,ljmax,0,l_out_nc4par = l_out_nc4par)
         ELSE
           tzecrit(:,:,:)=rg_valmanq_io
           DO j=ljmin,ljmax
             DO i=MAX(limin,ig(j)),MIN(limax,id(j))
               IF ((j.GE.jb(i)).AND.(j.LE.jh(i))) THEN
                 IF (l_out_log) THEN
                   tzecrit(:,i,j) = LOG(MAX(cv_wat(l,:,i,j),1.0e-30))/LOG(10.0_rsh)
                 ELSE
                   !tzecrit(:,i,j) = MAX(cv_wat(l,:,i,j),1.0e-30)
                   tzecrit(:,i,j) = cv_wat(l,:,i,j)
                 END IF
                 IF (h0(i,j)+x(i,j) <= h0fond) tzecrit(:,i,j)=rg_valmanq_io
               END IF
             END DO
           END DO
           CALL ionc4_write_zxyt(fileres,TRIM(name_var(irk_fil(l))),tzecrit,limin,limax,ljmin,ljmax,kmax,0, &
                                         l_out_nc4par = l_out_nc4par)
         END IF
        END IF
       END DO
#endif

       IF (l_filebydate(nb)) THEN
         CALL ionc4_close(fileres)
       ELSE
         ! To write the data on the disk and not loose data in case of run crash
         CALL ionc4_sync(fileres)
       ENDIF

     END IF


   ELSE IF ( .NOT. l_filebydate(nb) ) THEN


     CALL ionc4_write_time(fileresu(nb),0,tect)

    IF (.NOT. l_out_timeave(nb) ) THEN  ! instantaneous outputs

     IF (l_out_ssh(nb)) CALL out_write_xytvar(fileresu(nb),name_out_xe,rg_valmanq_io,mask_h0, &
                                              x(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     IF (l_out_ubt(nb)) THEN
       CALL out_write_xytvar(fileresu(nb),name_out_u,rg_valmanq_io,mask_hx, &
                             u(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_v,rg_valmanq_io,mask_hy, &
                             v(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_u3d(nb) .AND. .NOT.l_modele2d) THEN
       CALL out_write_zxytvar(fileresu(nb),name_out_uz,rg_valmanq_io,mask_hx, &
                              uz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_zxytvar(fileresu(nb),name_out_vz,rg_valmanq_io,mask_hy, &
                              vz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#ifdef key_diag_physwz
       CALL out_write_zxytvar(fileresu(nb),name_out_wz,rg_valmanq_io,mask_h0, &
!                              wz(0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
                              wz(0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_zxytvar(fileresu(nb),name_out_wz_phys,rg_valmanq_io,mask_h0, &
!                              wz_phys(0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
                              wz_phys(0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#endif
     END IF

     IF (l_out_sal(nb) .AND. l_modele2d) THEN
       CALL out_write_xytvar(fileresu(nb),name_out_sal,rg_valmanq_io,mask_h0, &
                             sal(1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     ELSE IF (l_out_sal(nb) .AND. .NOT.l_modele2d) THEN
       CALL out_write_zxytvar(fileresu(nb),name_out_sal,rg_valmanq_io,mask_h0, &
                              sal(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_temp(nb) .AND. l_modele2d) THEN
       CALL out_write_xytvar(fileresu(nb),name_out_temp,rg_valmanq_io,mask_h0, &
                             temp(1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     ELSE IF (l_out_temp(nb) .AND. .NOT.l_modele2d) THEN
       CALL out_write_zxytvar(fileresu(nb),name_out_temp,rg_valmanq_io,mask_h0, &
                              temp(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_visc(nb) .AND. l_modele2d) THEN
       CALL out_write_xytvar(fileresu(nb),name_out_vis2d,rg_valmanq_io,mask_h0, &
                             vis2d(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     ELSE IF (l_out_visc(nb) .AND. .NOT.l_modele2d) THEN
       CALL out_write_zxytvar(fileresu(nb),name_out_vishxe,rg_valmanq_io,mask_h0, &
                              vish_xe(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_zxytvar(fileresu(nb),name_out_vishphi,rg_valmanq_io,mask_h0, &
                              vish_phi(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_turb(nb) .AND. .NOT.l_modele2d) THEN
       CALL out_write_zxytvar(fileresu(nb),name_out_nz,rg_valmanq_io,mask_h0, &
                              nz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_zxytvar(fileresu(nb),name_out_kz,rg_valmanq_io,mask_h0, &
                              kz(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_buoy(nb) .AND. .NOT.l_modele2d) THEN
       tmp3d(:,:,:) = 1000.0_rsh - 1000.0_rsh*boz(:,limin:limax,ljmin:ljmax)/grav
       CALL out_write_zxytvar(fileresu(nb),name_out_bz,rg_valmanq_io,mask_h0, &
                              tmp3d,0,l_out_nc4par = l_out_nc4par)
     END IF

     IF (l_out_wind(nb)) THEN
       CALL out_write_xytvar(fileresu(nb),name_out_tosx,rg_valmanq_io,mask_h0, &
                             tosx(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_tosy,rg_valmanq_io,mask_h0, &
                             tosy(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
     END IF

#ifdef key_gencoord_test
     tmp3d(:,:,:)=rg_valmanq_io
     tmp3du(:,:,:)=rg_valmanq_io
     tmp3dv(:,:,:)=rg_valmanq_io
     tmp3dw(:,:,:)=rg_valmanq_io
     DO j=ljmin,ljmax
       DO i=MAX(limin,ig(j)),MIN(limax,id(j))
         IF ((j.GE.jb(i)).AND.(j.LE.jh(i))) THEN
           IF (h0(i,j)+x(i,j) > h0fond) THEN
             DO k=1,kmax
               tmp3d(k,i,j) = f_zu(h0(i,j),x(i,j),k,i,j)
               tmp3dw(k,i,j) = f_zw(h0(i,j),x(i,j),k,i,j)
             END DO
           END IF
           ! toseelater test et estimation ci dessus pas nickel
           IF (hx(i,j)+x(i,j) > h0fond) THEN
             DO k=1,kmax
               tmp3du(k,i,j) = f_zu_x(hx(i,j),x(i,j),k,i,j)
             END DO
           END IF
           IF (hy(i,j)+x(i,j) > h0fond) THEN
             DO k=1,kmax
               tmp3dv(k,i,j) = f_zu_y(hy(i,j),x(i,j),k,i,j)
             END DO
           END IF
         END IF
       END DO
     END DO
     name_out = 'depth_t'
     CALL out_write_zxytvar(fileresu(nb),name_out,rg_valmanq_io,mask_h0, &
                            tmp3d,0,l_out_nc4par = l_out_nc4par)
     name_out = 'depth_u'
     CALL out_write_zxytvar(fileresu(nb),name_out,rg_valmanq_io,mask_hx, &
                           tmp3du,0,l_out_nc4par = l_out_nc4par)
     name_out = 'depth_v'
     CALL out_write_zxytvar(fileresu(nb),name_out,rg_valmanq_io,mask_hy, &
                           tmp3dv,0,l_out_nc4par = l_out_nc4par)
     name_out = 'depth_w'
     CALL out_write_zxytvar(fileresu(nb),name_out,rg_valmanq_io,mask_h0, &
                           tmp3dw,0,l_out_nc4par = l_out_nc4par)
#endif


#ifdef key_wave
       IF (l_out_wave(nb)) THEN
         IF (l_wave_rdir) &
         CALL out_write_xytvar(fileresu(nb),name_out_wave_dir,rg_valmanq_io,mask_h0, &
                               wave_dir(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         IF (l_wave_rhs) &
         CALL out_write_xytvar(fileresu(nb),name_out_wave_hs,rg_valmanq_io,mask_h0, &
                               wave_hs(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         IF (l_wave_rfoc) &
         CALL out_write_xytvar(fileresu(nb),name_out_wave_foc,rg_valmanq_io,mask_h0, &
                               wave_foc(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         IF (l_wave_rubot .OR. (l_wave_ruubot .AND. l_wave_rvubot)) &
         CALL out_write_xytvar(fileresu(nb),name_out_wave_ubot,rg_valmanq_io,mask_h0, &
                               wave_ubot(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       END IF
#endif

    END IF   ! end test on .NOT.l_out_timeave(nb)

     IF (nbave /= 0 .AND. l_out_timeave(nb) ) THEN
         IF (l_out_ssh(nb)) CALL out_write_xytvar(fileresu(nb),name_out_xe,rg_valmanq_io,mask_h0, &
                                                ssh_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         IF (l_out_ubt(nb)) THEN
           CALL out_write_xytvar(fileresu(nb),name_out_u,rg_valmanq_io,mask_hx, &
                               u_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_xytvar(fileresu(nb),name_out_v,rg_valmanq_io,mask_hy, &
                               v_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF
         IF (l_out_u3d(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileresu(nb),name_out_uz,rg_valmanq_io,mask_hx, &
                                uz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileresu(nb),name_out_vz,rg_valmanq_io,mask_hy, &
                                vz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#ifdef key_diag_physwz
           CALL out_write_zxytvar(fileresu(nb),name_out_wz,rg_valmanq_io,mask_h0, &
!                                wz_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
                                wz_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileresu(nb),name_out_wz_phys,rg_valmanq_io,mask_h0, &
!                                wz_phys_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,0,l_out_nc4par = l_out_nc4par)
                                wz_phys_ave(nb,0:kmax,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#endif
         END IF

         IF (l_out_sal(nb) .AND. l_modele2d) THEN
           CALL out_write_xytvar(fileresu(nb),name_out_sal,rg_valmanq_io,mask_h0, &
                               sal_ave(nb,1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         ELSE IF (l_out_sal(nb).AND..NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileresu(nb),name_out_sal,rg_valmanq_io,mask_h0, &
                                sal_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF
         IF (l_out_temp(nb) .AND. l_modele2d) THEN
           CALL out_write_xytvar(fileresu(nb),name_out_temp,rg_valmanq_io,mask_h0, &
                               temp_ave(nb,1,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         ELSE IF (l_out_temp(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileresu(nb),name_out_temp,rg_valmanq_io,mask_h0, &
                                temp_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF

         IF (l_out_visc(nb) .AND. l_modele2d) THEN
           CALL out_write_xytvar(fileresu(nb),name_out_vis2d,rg_valmanq_io,mask_h0, &
                               vis2d_ave(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         ELSE IF (l_out_visc(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileresu(nb),name_out_vishxe,rg_valmanq_io,mask_h0, &
                                vish_xe_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileresu(nb),name_out_vishphi,rg_valmanq_io,mask_h0, &
                                vish_phi_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF
         IF (l_out_turb(nb) .AND. .NOT.l_modele2d) THEN
           CALL out_write_zxytvar(fileresu(nb),name_out_nz,rg_valmanq_io,mask_h0, &
                                nz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
           CALL out_write_zxytvar(fileresu(nb),name_out_kz,rg_valmanq_io,mask_h0, &
                                kz_ave(nb,:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
         END IF

         IF (l_out_buoy(nb) .AND. .NOT.l_modele2d) THEN
           tmp3d(:,:,:) = 1000.0_rsh - 1000.0_rsh*boz_ave(nb,:,limin:limax,ljmin:ljmax)/grav
           CALL out_write_zxytvar(fileresu(nb),name_out_bz,rg_valmanq_io,mask_h0, &
                                tmp3d,0,l_out_nc4par = l_out_nc4par)
         END IF

       ! initialize averaged dynamical and thermodynamical variables
       ! -----------------------------------------------------------

         dtn = t - tsort(nb)
         IF (l_out_ssh(nb)) &
         ssh_ave(nb,:,:) = x(limin:limax,ljmin:ljmax)*dtn
         IF (l_out_ubt(nb)) THEN
           u_ave(nb,:,:) = u(limin:limax,ljmin:ljmax)*dtn
           v_ave(nb,:,:) = v(limin:limax,ljmin:ljmax)*dtn
         END IF
         IF (l_out_u3d(nb).AND..NOT.l_modele2d) THEN
           uz_ave(nb,:,:,:) = dtn * uz(:,limin:limax,ljmin:ljmax)
           vz_ave(nb,:,:,:) = dtn * vz(:,limin:limax,ljmin:ljmax)
         END IF
         IF (l_out_temp(nb)) &
           temp_ave(nb,:,:,:) = dtn * temp(:,limin:limax,ljmin:ljmax)
         IF (l_out_sal(nb)) &
           sal_ave(nb,:,:,:) = dtn * sal(:,limin:limax,ljmin:ljmax)
         IF (l_out_visc(nb).AND.l_modele2d) THEN
           vis2d_ave(nb,:,:) = dtn * vis2d(limin:limax,ljmin:ljmax)
         ELSE IF (l_out_visc(nb).AND..NOT.l_modele2d) THEN
           vish_xe_ave(nb,:,:,:) = dtn * vish_xe(:,limin:limax,ljmin:ljmax)
           vish_phi_ave(nb,:,:,:) = dtn * vish_phi(:,limin:limax,ljmin:ljmax)
         END IF
         IF (l_out_turb(nb).AND..NOT.l_modele2d) THEN
           nz_ave(nb,:,:,:) = dtn * nz(:,limin:limax,ljmin:ljmax)
           kz_ave(nb,:,:,:) = dtn * kz(:,limin:limax,ljmin:ljmax)
         END IF
         IF (l_out_buoy(nb).AND..NOT.l_modele2d) &
           boz_ave(nb,:,:,:) = dtn * boz(:,limin:limax,ljmin:ljmax)
         dt_ave(nb) =  dtn
     END IF

     IF (nbatm /= 0 .AND. l_out_atmsflx(nb) ) THEN
       CALL out_write_xytvar(fileresu(nb),name_out_mrad,rg_valmanq_io,mask_h0,     &
                             rad_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_mirlos,rg_valmanq_io,mask_h0,   &
                             irlos_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_mlatlos,rg_valmanq_io,mask_h0,  &
                             latlos_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_msenslos,rg_valmanq_io,mask_h0, &
                             senslos_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_mevap,rg_valmanq_io,mask_h0, &
                             evap_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_mrain,rg_valmanq_io,mask_h0, &
                             rain_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_mtosx,rg_valmanq_io,mask_h0, &
                             tosx_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       CALL out_write_xytvar(fileresu(nb),name_out_mtosy,rg_valmanq_io,mask_h0, &
                             tosy_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#ifdef key_sflx_analyse
       name_out_mwindx='uwind'
       CALL out_write_xytvar(fileresu(nb),name_out_mwindx,rg_valmanq_io,mask_h0, &
                             windx_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       name_out_mwindy='vwind'
       CALL out_write_xytvar(fileresu(nb),name_out_mwindy,rg_valmanq_io,mask_h0, &
                             windy_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       name_out_mrh='rh'
       CALL out_write_xytvar(fileresu(nb),name_out_mrh,rg_valmanq_io,mask_h0, &
                             rh_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       name_out_mfcc='fcc'
       CALL out_write_xytvar(fileresu(nb),name_out_mfcc,rg_valmanq_io,mask_h0, &
                             fcc_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       name_out_msat='sat'
       CALL out_write_xytvar(fileresu(nb),name_out_msat,rg_valmanq_io,mask_h0, &
                             sat_cum(nb,limin:limax,ljmin:ljmax)+273.15_rsh,0,l_out_nc4par = l_out_nc4par)
       name_out_msst='sst'
       CALL out_write_xytvar(fileresu(nb),name_out_msst,rg_valmanq_io,mask_h0, &
                             sst_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
       name_out_mpa='pa'
       CALL out_write_xytvar(fileresu(nb),name_out_mpa,rg_valmanq_io,mask_h0, &
                             pa_cum(nb,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)
#endif

       ! initialize cumulative atmospherical surface fluxes
       ! --------------------------------------------------
 
       dtn = t - tsort(nb)
       rad_cum(nb,:,:)     = rad(limin:limax,ljmin:ljmax)    *dtn
       irlos_cum(nb,:,:)   = irlos(limin:limax,ljmin:ljmax)  *dtn
       latlos_cum(nb,:,:)  = latlos(limin:limax,ljmin:ljmax) *dtn
       senslos_cum(nb,:,:) = senslos(limin:limax,ljmin:ljmax)*dtn
       evap_cum(nb,:,:)    = evap(limin:limax,ljmin:ljmax)   *dtn
       rain_cum(nb,:,:)    = rain(limin:limax,ljmin:ljmax)   *dtn
       tosx_cum(nb,:,:)    = tosx(limin:limax,ljmin:ljmax)   *dtn
       tosy_cum(nb,:,:)    = tosy(limin:limax,ljmin:ljmax)   *dtn
#ifdef key_sflx_analyse
       windx_cum(nb,:,:)   = windx(limin:limax,ljmin:ljmax)  *dtn
       windy_cum(nb,:,:)   = windy(limin:limax,ljmin:ljmax)  *dtn
       rh_cum(nb,:,:)      = rh(limin:limax,ljmin:ljmax)     *dtn
       fcc_cum(nb,:,:)     = fcc(limin:limax,ljmin:ljmax)    *dtn
       sat_cum(nb,:,:)     = sat(limin:limax,ljmin:ljmax)    *dtn
       sst_cum(nb,:,:)     = temp(kmax,limin:limax,ljmin:ljmax)*dtn
       pa_cum(nb,:,:)      = pa(limin:limax,ljmin:ljmax)     *dtn
#endif
       dt_cum(nb) =  dtn

     END IF

#ifdef key_offline_outuvflux

     IF ( l_out_off(nb) ) THEN

       ! WARNING : wetting and drying are not taken into account yet !
       ! doit on vraiment tourner sur i=ig(j)-1,id(j) pour hexm ???? (r363 et plus)

         CALL out_write_xytvar(fileresu(nb),name_out_mxe,rg_valmanq_io,mask_h0, &
                               xem(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         CALL out_write_xytvar(fileresu(nb),name_out_mhex,rg_valmanq_io,mask_h0, &
                               hexm(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         CALL out_write_xytvar(fileresu(nb),name_out_mhey,rg_valmanq_io,mask_h0, &
                               heym(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

         CALL out_write_zxytvar(fileresu(nb),name_out_mkz,rg_valmanq_io,mask_h0, &
                                kzm(:,limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

     END IF
#endif

#ifdef key_substance
     DO l = 1,nv_state
      IF (l_out_subs(l)) THEN
       IF (l_modele2d) THEN
         tecrit(:,:)=rg_valmanq_io
         DO j=ljmin,ljmax
           DO i=MAX(limin,ig(j)),MIN(limax,id(j))
             IF ((j.GE.jb(i)).AND.(j.LE.jh(i))) THEN
               IF (l_out_log) THEN
                 tecrit(i,j) = LOG(MAX(cv_wat(l,1,i,j),1.0e-30))/LOG(10.0_rsh)
               ELSE
                 !tecrit(i,j) =MAX(cv_wat(l,1,i,j),1.0e-30)
                 tecrit(i,j) =cv_wat(l,1,i,j)
               END IF
               IF (h0(i,j)+x(i,j) <= h0fond) tecrit(i,j)=rg_valmanq_io
             END IF
           END DO
         END DO
         CALL ionc4_write_xyt(fileresu(nb),TRIM(name_var(irk_fil(l))),tecrit,limin,limax,ljmin,ljmax,0,l_out_nc4par = l_out_nc4par)
       ELSE
         tzecrit(:,:,:)=rg_valmanq_io
         DO j=ljmin,ljmax
           DO i=MAX(limin,ig(j)),MIN(limax,id(j))
             IF ((j.GE.jb(i)).AND.(j.LE.jh(i))) THEN
               IF (l_out_log) THEN
                 tzecrit(:,i,j) = LOG(MAX(cv_wat(l,:,i,j),1.0e-30))/LOG(10.0_rsh)
               ELSE
                 !tzecrit(:,i,j) = MAX(cv_wat(l,:,i,j),1.0e-30)
                 tzecrit(:,i,j) = cv_wat(l,:,i,j)
               END IF
               IF (h0(i,j)+x(i,j) <= h0fond) tzecrit(:,i,j)=rg_valmanq_io
             END IF
           END DO
         END DO
         CALL ionc4_write_zxyt(fileresu(nb),TRIM(name_var(irk_fil(l))),tzecrit,limin,limax,ljmin,ljmax,kmax,0, &
                                            l_out_nc4par = l_out_nc4par)
       END IF
      END IF
     END DO
#endif

     ! To write the data on the disk and not loose data in case of run crash
     CALL ionc4_sync(fileresu(nb))

   END IF

   END IF

   PRINT_DBG*, 'exit OUTPUT_SAVE routine',TRIM(fileresu(nb))

  END SUBROUTINE output_save

   !!======================================================================

