   !!======================================================================
 
  SUBROUTINE out_saverestart(u,uz,v,vz,xev,sal,temp,        &
                                           vstar,igdv_old,  &
#ifdef key_substance
                                           cv_wat,          &
#endif
                                           ect)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE out_saverestart  ***
   !&E
   !&E ** Purpose : save model variables in a NetCDF file for further restart
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main, step
   !&E
   !&E ** External calls : ionc4_createfile, ionc4_torigine, 
   !&E                     ionc4_createvar, ionc4_write_time,ionc4_write_t,
   !&E                     ionc4_write_xy, ionc4_write_xyt,ionc4_write_zxyt
   !&E                     ionc4_write
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (F. Dumas)  Original code
   !&E       !  2006-04  (V. Garnier) Management of simple and double precision
   !&E       !  2009-03  (V. Garnier A. Mano) MPI2D
   !&E       !  2009-10  (V. Garnier) reproductibility when using restart file
   !&E       !  2010-12 (R. Ramel) genericity and use of ionc4
   !&E       !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE ionc4,     ONLY : ionc4_createfile,ionc4_createvar,ionc4_write_sig,  &
                         ionc4_vatt_standard_name,ionc4_vatt_fill,ionc4_vatt_valid_min,&
                         ionc4_vatt_valid_max,ionc4_write_xy,ionc4_write,ionc4_gatt,&
                         ionc4_torigine,ionc4_write_time,ionc4_write,ionc4_write_t,&
                         ionc4_write_xy,ionc4_write_xyt,ionc4_write_zxyt,ionc4_close,&
                         ionc4_sync,ionc4_vatt_chain
   USE output,    ONLY : l_out_nc4par,l_out_zaxis_w
   USE comvars2d, ONLY : name_in_dt,name_in_xe,name_in_xev,                 &
                         name_bathy_h0,name_bathy_hx,name_bathy_hy,         &
                         name_in_u,name_in_v,name_in_uz,name_in_vz,         &
                         name_in_wz,name_in_temp,name_in_sal,               &
                         name_in_ect,name_in_psi,name_in_kz,name_in_nz,     &
                         name_in_vis2d,name_in_vis3d_xe,name_in_vis3d_phi,  &
                         l_saverestart_bydate,l_boolsauv,file_saverestart,  &
                         date_ref,tsauv,cdate,t,nbouc,dt,saverestart_step,  &
                         lon2d_g,lat2d_g,lon2d,lat2d,cflumaxmax,cflvmaxmax, &
                         l_modele2d,vish_xe,vish_phi,vis2d,h0,hx,hy,tdebobs,&
                         name_out_h0,name_out_xe,                           &
! S.P. 092013 Pour le restartabilite ...
                         tchrono,indchrono,&
! S.T. 05/03/2014 pour enregistrer restart juste avant la fin du run.
                         tfin,dtmax
   USE comvars3d, ONLY : sig,sigw,wz,kz,nz
   USE comvarp3d, ONLY : psi
   USE comfluxdo, ONLY : phieau
#ifdef key_substance
   USE comsubstance, ONLY : nv_tot,name_var,irk_fil,unit_var
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : b_sig,theta_sig,hc_sig,Csu_sig,Csw_sig
#endif
#ifdef key_MPI_2D
   USE toolmpi,   ONLY : ADD_TAG
#endif
   USE obccombine,ONLY : l_obc_tide
   IMPLICIT NONE

#ifdef key_MPI_2D
   INCLUDE 'mpif.h'
#endif

   !! * Arguments
   INTEGER,INTENT(in),DIMENSION(limin:limax,ljmin:ljmax)             :: igdv_old
   REAL(KIND=rsh),INTENT(in),DIMENSION(limin:limax,ljmin:ljmax)      :: xev
   REAL(KIND=rsh),INTENT(in),DIMENSION(     liminm2:limaxp2,ljminm2:ljmaxp2) :: u,v,vstar
   REAL(KIND=rsh),INTENT(in),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2) :: uz,vz
   REAL(KIND=rsh),INTENT(in),DIMENSION(kmax,limin:limax,ljmin:ljmax)         :: ect
   REAL(KIND=rsh),INTENT(in),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2) :: sal,temp
#ifdef key_substance
   REAL(KIND=rsh),INTENT(in),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2) :: cv_wat
#endif

   !! * Local declarations
   INTEGER                                                       :: l,comm_zoom
   CHARACTER(LEN=lchain) :: chain
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)           :: tmpigd
   REAL(KIND=rsh),  DIMENSION(limin:limax,ljmin:ljmax)           :: tmp2d
   REAL(KIND=rsh),  DIMENSION(kmax,limin:limax,ljmin:ljmax)      :: tmp3d
   REAL(KIND=rlg),  DIMENSION(kmax,limin:limax,ljmin:ljmax)      :: tmp3drlg
   REAL(KIND=riosh),DIMENSION(0:kmax+1)                          :: sigecrit
   REAL(KIND=riosh)                                              :: vecrit
   REAL(KIND=rsh)                                                :: fillval,iozero

   !!----------------------------------------------------------------------
   !! * Executable part

   ! ST.On force a ecrire le save.nc recombine via ecriture //
   ! en remplacant :
   ! l_out_nc4par=l_out_nc4par
   ! par :
   ! l_out_nc4par=.true.
   ! j aurais voulu utiliser l'attribut SAVE lors de la declaration de 
   ! l_out_nc4par dans init.F90 et valoriser une seule fois l_out_nc4par
   ! au debut de cette sous-routine mais echec ...


   PRINT_DBG*, 'enter OUT_SAVERESTART routine'

   IF_AGRIF (Agrif_root()) l_boolsauv=.FALSE.
   IF_AGRIF (.NOT. Agrif_root()) THEN
     IF_AGRIF (Agrif_NbStepint()/=0) l_boolsauv=.FALSE.
     IF_AGRIF (Agrif_NbStepint()==0) l_boolsauv=Agrif_parent(l_boolsauv)
   ENDIF_AGRIF

   !! Management of simple and double precision
   IF(rsh==4) THEN
      fillval = rg_valmanq_io
      iozero  = 0.0_riosh
   ELSEIF(rsh==8) THEN
      fillval = dg_valmanq_io
      iozero  = 0.0_riolg
   ENDIF
      
   
     
     IF(nbouc == 0 .AND. .NOT.(l_saverestart_bydate)) THEN ! creation dans le cas non-sequentiel
                                                !------------------------------------

#ifdef key_MPI_2D
       IF (.NOT. l_out_nc4par) CALL ADD_TAG( file_saverestart, LEN(file_saverestart) )
#endif

       sigecrit(:)=sig(:)
       IF (l_out_nc4par) THEN
         CALL ionc4_createfile(file_saverestart,lon2d_g,lat2d_g,imin, &
                           imax,1,jmin,jmax,1,1,kmax,1,kmax,0,l_out_nc4par = .TRUE.,zaxis_var=sigecrit(1:kmax))
       ELSE
         CALL ionc4_createfile(file_saverestart,lon2d,lat2d,limin, &
                           limax,1,ljmin,ljmax,1,1,kmax,1,kmax,0,zaxis_var=sigecrit(1:kmax))
       ENDIF

       comm_zoom=0
#ifdef key_MPI_2D
       comm_zoom=MPI_COMM_WORLD
#endif

       ! Add the vertical axis at w-location
       CALL out_zaxis_w(file_saverestart,1,kmax,1,comm_zoom)

       ! save the attributes of the spatial extension of the domain
       ! ----------------------------------------------------------

       CALL ionc4_gatt(file_saverestart,'global_imin',imin)
       CALL ionc4_gatt(file_saverestart,'global_imax',imax)
       CALL ionc4_gatt(file_saverestart,'global_jmin',jmin)
       CALL ionc4_gatt(file_saverestart,'global_jmax',jmax)

       ! save some time variables to allow a perfect continuity of the run despite
       ! the interruption of the run
       ! -------------------------------------------------------------------------

       CALL ionc4_torigine(file_saverestart,date_ref)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_dt),'s','dt',fill_value=-dg_valmanq_io)
       IF (.NOT. l_modele2d) THEN
         CALL ionc4_createvar(file_saverestart,'tdebobs','s','tdebobs',fill_value=-dg_valmanq_io)
         CALL ionc4_createvar(file_saverestart,'cflumaxmax',' ','cfl of max u',dims='t',fill_value=fillval)
         CALL ionc4_createvar(file_saverestart,'cflvmaxmax',' ','cfl of max v',dims='t',fill_value=fillval)
       END IF
       CALL ionc4_createvar(file_saverestart,'tchrono_deb','s', &
            'tchrono starting time for river-meteo interpolation',fill_value=-dg_valmanq_io)

       CALL ionc4_createvar(file_saverestart,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                            standard_name= 'ocean_s_variable',&
#else
                            standard_name= 'ocean_sigma_variable',&
#endif
                            l_auxcoordinate=.false.,&
                            valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io,&
!                            dims= 'z', l_out_nc4par=.true.)
                            dims= 'z', l_out_nc4par = l_out_nc4par)
       CALL ionc4_write_sig(file_saverestart,'SIG',sigecrit,kmax)

       CALL ionc4_createvar(file_saverestart,'SIG_W',' ','sigma variable at the interface',&
#if defined key_siggen || defined key_gencoord
                          standard_name= 'ocean_s_variable_at_w_location',&
#else
                          standard_name= 'ocean_sigma_variable_at_w_location',&
#endif                          
                          l_auxcoordinate=.false.,&
                          dims='z',valid_min= -1.0_riosh,valid_max=0.0_riosh,fill_value= rg_valmanq_io)
       sigecrit(:)=sigw(:)
       CALL ionc4_write_sig(file_saverestart,'SIG_W',sigecrit,kmax)

#if defined key_siggen || defined key_gencoord
       sigecrit(:)=Csu_sig(:)
       CALL ionc4_createvar(file_saverestart,'Csu_sig',' ','C(s) law at mid layer',dims='z',&
                                     standard_name='ocean_s_coordinate_function_at_midlayer',&
                                     valid_min=-2.0_riosh,valid_max=2.0_riosh,fill_value=rg_valmanq_io,&
                                     l_auxcoordinate=.false.)
       CALL ionc4_write_sig(file_saverestart,'Csu_sig',sigecrit,kmax)
       CALL ionc4_vatt_chain(file_saverestart,'Csu_sig','formula_definition', &
          'C(k) = (1-b)*sinh(a*s(k))/sinh(a) + b*[tanh(a*(s(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')
       
       sigecrit(0:kmax)=Csw_sig(0:kmax)
       sigecrit(kmax+1)=0.0_riosh
       CALL ionc4_createvar(file_saverestart,'Csw_sig',' ','C(s) law at interface',dims='z',&
                                     standard_name='ocean_s_coordinate_function_at_interface',&
!                                     location_in_cell='w',l_auxcoordinate=.false.,&
                                     l_auxcoordinate=.false.,&
                                     valid_min=-2.0_riosh,valid_max=2.0_riosh,fill_value=rg_valmanq_io)
       CALL ionc4_write_sig(file_saverestart,'Csw_sig',sigecrit,kmax)
       CALL ionc4_vatt_chain(file_saverestart,'Csw_sig','formula_definition', &
          'C_w(k) = (1-b)*sinh(a*s_w(k))/sinh(a) + b*[tanh(a*(s_w(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')

       tmp2d(:,:)=hc_sig(limin:limax,ljmin:ljmax)
       CALL ionc4_createvar(file_saverestart,'hc',units='m',dims='xy',                              &
                                     long_name='depth over which the resolution is increased',      &
                                     standard_name='ocean_s_coordinate_thick_of_surface_resolution',&
                                     valid_min=-20.0_riosh,valid_max=500.0_riosh,fill_value=-rg_valmanq_io,&
                                     l_out_nc4par=l_out_nc4par)
!                                     l_out_nc4par=.true.)
       CALL ionc4_write_xy(file_saverestart,'hc',tmp2d,limin,limax,ljmin,ljmax,l_out_nc4par=.true.)
       
       vecrit=b_sig
       CALL ionc4_createvar(file_saverestart,'b',' ','bottom control parameter',&
                                     standard_name='ocean_s_coordinate_bottom_control',&
                                     valid_min=0.0_riosh,valid_max=1.0_riosh,fill_value=rg_valmanq_io)
       CALL ionc4_write(file_saverestart,'b',vecrit)
      
       vecrit=theta_sig
       CALL ionc4_createvar(file_saverestart,'theta',' ','surface control parameter',&
                                     standard_name='ocean_s_coordinate_surface_control',&
                                     valid_min=0.0_riosh,valid_max=20.0_riosh,fill_value=rg_valmanq_io)
       CALL ionc4_write(file_saverestart,'theta',vecrit)
#endif

       ! Add attributes to follow OCO netCDF convention
       CALL out_netcdf_conv(file_saverestart,1,comm_zoom,imin,imax,jmin,jmax)

       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level'
       ELSE
         chain='model_sea_floor_depth_below_geoid'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_bathy_h0),units= 'm',dims='xy',fill_value=-fillval, &
                           long_name='bathymetry relative to the mean level',                            &
                           standard_name = TRIM(chain),                                                  &
                           l_out_nc4par=l_out_nc4par)
!                           l_out_nc4par=.true.)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_u_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_u_location'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_bathy_hx),units= 'm',dims='xy',fill_value=-fillval, &
                           long_name = 'bathymetry at u-location relative to the mean level',            &
                           standard_name = TRIM(chain),                                                  &
!                           location_in_cell='u',l_out_nc4par=.true.)
                           l_out_nc4par=l_out_nc4par)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_v_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_v_location'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_bathy_hy),units= 'm',dims='xy',fill_value=-fillval, &
                           long_name = 'bathymetry at v-location relative to the mean level',            &
                           standard_name = TRIM(chain),                                                  &
!                           location_in_cell='v',l_out_nc4par=.true.)
                           l_out_nc4par=l_out_nc4par)
       IF (l_obc_tide) THEN
         chain='sea_surface_height_above_sea_level'
       ELSE
         chain='sea_surface_height_above_geoid'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_xev),units= 'm',dims='xyt',fill_value=fillval, &
                            long_name = 'sea surface height',                                            &
                            standard_name = TRIM(chain),                                                 &
                            l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_u),units= 'm s-1',dims='xyt',fill_value=iozero,&
                            long_name = 'barotropic zonal velocity',                                     &
                            standard_name = 'barotropic_sea_water_x_velocity_at_u_location',                    &
                            location_in_cell='u',                                                        &
                            l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_v),units= 'm s-1',dims='xyt',fill_value=iozero,&
                            long_name = 'barotropic meridional velocity',                                &
                            standard_name = 'barotropic_sea_water_y_velocity_at_v_location',                   &
                            location_in_cell='v',&
                            l_out_nc4par=l_out_nc4par)
       IF (.NOT. l_modele2d) THEN
         CALL ionc4_createvar(file_saverestart,'vstar',units= 'm s-1',dims='xyt',fill_value=iozero,  &
                              long_name = 'first guess of the barotropic meridional velocity',       &
!                              location_in_cell='v',l_out_nc4par=.true.)
                              l_out_nc4par=l_out_nc4par)
         CALL ionc4_createvar(file_saverestart,'igdv_old','0-4','igdv_old',dims='xyt', &
                                               fill_value=real(fillval,riosh),l_out_nc4par=l_out_nc4par)
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_vis2d),'m2 s-1','vis2d',dims='xyt', &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_uz),units= 'm s-1',dims='zxyt',   &
                                             long_name = '3d zonal velocity',               &
                                             standard_name = 'sea_water_x_velocity_at_u_location', &
!                                             location_in_cell='u',                          &
                                             fill_value=iozero,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_vz),units= 'm s-1',dims='zxyt',     &
                                             long_name = '3d meridional velocity',          &
                                             standard_name = 'sea_water_y_velocity_at_v_location',&
                                             location_in_cell='v',                          &
                                             fill_value=iozero,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_wz),'m s-1','wz',dims='zxyt', &
                                             location_in_cell='w',                          &
                                             fill_value=iozero,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_sal),units= '1e-3',dims='zxyt', &
                                             long_name = 'salinity',                         &
                                             standard_name = 'sea_water_salinity',           &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_temp),units= 'degree C',dims='zxyt', &
                                             long_name = 'temperature',                        &
                                             standard_name = 'sea_water_potential_temperature',          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_ect),'m2 s-2','ect',dims='zxyt', &
                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_psi),'m2 s-2','psi',dims='zxyt', &
                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_kz),'m2 s-1','vertical diffusivity',dims='zxyt', &
                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_nz),'m2 s-1','vertical viscosity',dims='zxyt', &
                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       IF (.NOT.l_modele2d) THEN
         CALL ionc4_createvar(file_saverestart,TRIM(name_in_vis3d_xe),'m2 s-1','vish3d_xe',dims='zxyt', &
                                               fill_value=fillval,l_out_nc4par=l_out_nc4par)
         CALL ionc4_createvar(file_saverestart,TRIM(name_in_vis3d_phi),units= 'm2 s-1',long_name ='vish3d_phi',dims='zxyt', &
!                                               location_in_cell='f',fill_value=fillval,l_out_nc4par=.true.)
                                               fill_value=fillval,l_out_nc4par=l_out_nc4par)
       END IF
       CALL ionc4_createvar(file_saverestart,'phieau','m3.s-1','water mass flux',dims='zxyt', &
                                             fill_value= 0.0_riolg,l_out_nc4par=l_out_nc4par)
!
#ifdef key_substance
       DO l = 1,nv_tot
         CALL ionc4_createvar(file_saverestart,TRIM(name_var(irk_fil(l))),unit_var(irk_fil(l)), &
                               TRIM(name_var(irk_fil(l))),dims='zxyt',fill_value=fillval,l_out_nc4par=l_out_nc4par)
       END DO
#endif
     END IF    ! fin creation dans le cas non-sequentiel
               !----------------------------------------
!
     !IF(l_saverestart_bydate .AND. (t >= tsauv.OR.l_boolsauv) ) THEN  ! creation dans le cas sequentiel
     IF(l_saverestart_bydate .AND. (t >= tsauv.OR.l_boolsauv.OR.(t >= (tfin-0.5*dtmax) ))) THEN  ! creation dans le cas sequentiel
                                        !--------------------------------
!
! cdate = DD-MM-YYYY-hh-mm-ss
       file_saverestart='./save_'//cdate(7:10)//cdate(4:5)//cdate(1:2)//cdate(12:13)// &
                    cdate(15:16)//cdate(18:19)
       IF_AGRIF (.NOT. Agrif_Root()) file_saverestart='./sauv_agrif'//TRIM(Agrif_Cfixed())// &
       INST_AGRIF  '_'//cdate(7:10)//cdate(4:5)//cdate(1:2)//cdate(12:13)//cdate(15:16)//cdate(18:19)
       file_saverestart=TRIM(file_saverestart)//'.nc'
#ifdef key_MPI_2D
       IF (.NOT. l_out_nc4par) CALL ADD_TAG( file_saverestart, LEN(file_saverestart) )
#endif

       sigecrit(:)=sig(:)
       IF (l_out_nc4par) THEN
         CALL ionc4_createfile(file_saverestart,lon2d_g,lat2d_g,imin, &
                           imax,1,jmin,jmax,1,1,kmax,1,kmax,0,l_out_nc4par = l_out_nc4par,zaxis_var=sigecrit(1:kmax))
                           !imax,1,jmin,jmax,1,1,kmax,1,kmax,0,l_out_nc4par = l_out_nc4par,zaxis_var=sigecrit(1:kmax))
       ELSE
         CALL ionc4_createfile(file_saverestart,lon2d,lat2d,limin, &
                           limax,1,ljmin,ljmax,1,1,kmax,1,kmax,0,l_out_nc4par = l_out_nc4par,zaxis_var=sigecrit(1:kmax))
       ENDIF  

       comm_zoom=0
#ifdef key_MPI_2D
       comm_zoom=MPI_COMM_WORLD
#endif

       ! Add the vertical axis at w-location
       CALL out_zaxis_w(file_saverestart,1,kmax,1,comm_zoom)

       ! save attribute of the spatial extension of the domain
       ! -----------------------------------------------------
       CALL ionc4_gatt(file_saverestart,'global_imin',imin)
       CALL ionc4_gatt(file_saverestart,'global_imax',imax)
       CALL ionc4_gatt(file_saverestart,'global_jmin',jmin)
       CALL ionc4_gatt(file_saverestart,'global_jmax',jmax)

       ! save some time variables to allow a perfect continuity of the run despite
       ! the interruption of the run
       ! -------------------------------------------------------------------------

       CALL ionc4_torigine(file_saverestart,date_ref)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_dt),'s','dt',fill_value=-dg_valmanq_io)
       IF (.NOT. l_modele2d) THEN
         CALL ionc4_createvar(file_saverestart,'tdebobs','s','tdebobs',fill_value=-dg_valmanq_io)
         CALL ionc4_createvar(file_saverestart,'cflumaxmax',' ','cfl of max u',dims='t',fill_value=fillval)
         CALL ionc4_createvar(file_saverestart,'cflvmaxmax',' ','cfl of max v',dims='t',fill_value=fillval)
       END IF
       CALL ionc4_createvar(file_saverestart,'tchrono_deb','s',&
                     'tchrono starting time for river-meteo interpolation',fill_value=-dg_valmanq_io)

       CALL ionc4_createvar(file_saverestart,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                            standard_name= 'ocean_s_variable',&
#else
                            standard_name= 'ocean_sigma_variable',&
#endif
                            l_auxcoordinate=.false.,&
                            valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io,&
      !                      dims= 'z', l_out_nc4par=.true.)
                            dims= 'z', l_out_nc4par = l_out_nc4par)
       CALL ionc4_write_sig(file_saverestart,'SIG',sigecrit,kmax)
       
       CALL ionc4_createvar(file_saverestart,'SIG_W',' ','sigma variable at the interface',&
#if defined key_siggen || defined key_gencoord
                          standard_name= 'ocean_s_variable_at_w_location',&
#else
                          standard_name= 'ocean_sigma_variable_at_w_location',&
#endif                          
                          l_auxcoordinate=.false.,&
                          dims='z',valid_min= -1.0_riosh,valid_max=0.0_riosh,fill_value= rg_valmanq_io)
       sigecrit(:)=sigw(:)
       CALL ionc4_write_sig(file_saverestart,'SIG_W',sigecrit,kmax)

#if defined key_siggen || defined key_gencoord
       sigecrit(:)=Csu_sig(:)
       CALL ionc4_createvar(file_saverestart,'Csu_sig',' ','C(s) law at mid layer',dims='z',&
                                     standard_name='ocean_s_coordinate_function_at_midlayer',&
                                     valid_min=-2.0_riosh,valid_max=2.0_riosh,fill_value=rg_valmanq_io,&
                                     l_auxcoordinate=.false.)
       CALL ionc4_write_sig(file_saverestart,'Csu_sig',sigecrit,kmax)
       
       sigecrit(0:kmax)=Csw_sig(0:kmax)
       sigecrit(kmax+1)=0.0_riosh
       CALL ionc4_createvar(file_saverestart,'Csw_sig',' ','C(s) law at interface',dims='z',&
                                     standard_name='ocean_s_coordinate_function_at_interface',&
!                                     location_in_cell='w',l_auxcoordinate=.false.,&
                                     l_auxcoordinate=.false.,&
                                     valid_min=-2.0_riosh,valid_max=2.0_riosh,fill_value=rg_valmanq_io)
       CALL ionc4_write_sig(file_saverestart,'Csw_sig',sigecrit,kmax)

       tmp2d(:,:)=hc_sig(limin:limax,ljmin:ljmax)
       CALL ionc4_createvar(file_saverestart,'hc',units='m',dims='xy',                              &
                                     long_name='depth over which the resolution is increased',      &
                                     standard_name='ocean_s_coordinate_thick_of_surface_resolution',&
                                     valid_min=-20.0_riosh,valid_max=500.0_riosh,fill_value=rg_valmanq_io,&
                                     l_out_nc4par=l_out_nc4par)
       CALL ionc4_write_xy(file_saverestart,'hc',tmp2d,limin,limax,ljmin,ljmax,l_out_nc4par=l_out_nc4par)
       
       vecrit=b_sig
       CALL ionc4_createvar(file_saverestart,'b',' ','bottom control parameter',&
                                     standard_name='ocean_s_coordinate_bottom_control',&
                                     valid_min=0.0_riosh,valid_max=1.0_riosh,fill_value=rg_valmanq_io)
       CALL ionc4_write(file_saverestart,'b',vecrit)
      
       vecrit=theta_sig
       CALL ionc4_createvar(file_saverestart,'theta',' ','surface control parameter',&
                                     standard_name='ocean_s_coordinate_surface_control',&
                                     valid_min=0.0_riosh,valid_max=20.0_riosh,fill_value=rg_valmanq_io)
       CALL ionc4_write(file_saverestart,'theta',vecrit)
#endif

       ! Add attributes to follow OCO netCDF convention
       CALL out_netcdf_conv(file_saverestart,1,comm_zoom,imin,imax,jmin,jmax)

       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level'
       ELSE
         chain='model_sea_floor_depth_below_geoid'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_bathy_h0),units='m',dims='xy',fill_value=-fillval, &
                            long_name='bathymetry relative to the mean level',                            &
                            standard_name = TRIM(chain),                                                  &
                            l_out_nc4par=l_out_nc4par)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_u_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_u_location'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_bathy_hx),units='m',dims='xy',fill_value=-fillval, &
                            long_name = 'bathymetry at u-location relative to the mean level',            &
                            standard_name = TRIM(chain),                                                  &
!                            location_in_cell='u',l_out_nc4par=.true.)
                            l_out_nc4par=l_out_nc4par)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_v_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_v_location'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_bathy_hy),units='m',dims='xy',fill_value=-fillval, &
                            long_name = 'bathymetry at v-location relative to the mean level',            &
                            standard_name = TRIM(chain),                                                  &
!                            location_in_cell='v',l_out_nc4par=.true.)
                            l_out_nc4par=l_out_nc4par)
       IF (l_obc_tide) THEN
         chain='sea_surface_height_above_sea_level'
       ELSE
         chain='sea_surface_height_above_geoid'
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_xev),units='m',dims='xyt',fill_value=fillval,  &
                            long_name = 'sea surface height',                                            &
                            standard_name = TRIM(chain),                                                 &
                            l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_u),units='m s-1',dims='xyt',fill_value=iozero, &
                            long_name = 'barotropic zonal velocity',                                     &
                            standard_name = 'barotropic_sea_water_x_velocity_at_u_location',                    &
!                            location_in_cell='u',                                                        &
                            l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_v),units='m s-1',dims='xyt',fill_value=iozero, &
                            long_name = 'barotropic meridional velocity',                                &
                            standard_name = 'barotropic_sea_water_y_velocity_at_v_location',                   &
!                            location_in_cell='v',&
                            l_out_nc4par=l_out_nc4par)
       IF (.NOT. l_modele2d) THEN
         CALL ionc4_createvar(file_saverestart,'vstar',units='m s-1',dims='xyt',fill_value=iozero, &
                              long_name = 'first guess of the barotropic meridional velocity',     &
!                              location_in_cell='v',l_out_nc4par=.true.)
                              l_out_nc4par=l_out_nc4par)
         CALL ionc4_createvar(file_saverestart,'igdv_old','0-4','igdv_old',dims='xyt', &
                                               fill_value=real(fillval,riosh),l_out_nc4par=l_out_nc4par)
       END IF
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_vis2d),'m2 s-1','vis2d',dims='xyt', &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_uz),units='m s-1',dims='zxyt',    &
                                             long_name = '3d zonal velocity',               &
                                             standard_name = 'sea_water_x_velocity_at_u_location', &
!                                             location_in_cell='u',                          &
                                             fill_value=iozero,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_vz),units='m s-1',dims='zxyt',    &
                                             long_name = '3d meridional velocity',          &
                                             standard_name = 'sea_water_y_velocity_at_v_location',&
!                                             location_in_cell='v',                          &
                                             fill_value=iozero,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_wz),'m s-1','wz',dims='zxyt',fill_value=iozero, &
!                                             location_in_cell='w',l_out_nc4par=.true.)
                                             l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_sal),units='1e-3',dims='zxyt',     &
                                             long_name = 'salinity',                        &
                                             standard_name = 'sea_water_salinity',          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_temp),units='degree C',dims='zxyt', &
                                             long_name = 'temperature',                       &
                                             standard_name = 'sea_water_potential_temperature',         &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_ect),'m2 s-2','ect',dims='zxyt', &
!                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_psi),'m2 s-2','psi',dims='zxyt', &
!                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_kz),'m2 s-1','vertical diffusivity',dims='zxyt', &
!                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       CALL ionc4_createvar(file_saverestart,TRIM(name_in_nz),'m2 s-1','vertical viscosity',dims='zxyt', &
!                                             location_in_cell='w',                          &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
       IF (.NOT.l_modele2d) THEN
         CALL ionc4_createvar(file_saverestart,TRIM(name_in_vis3d_xe),'m2 s-1','vish3d_xe',dims='zxyt', &
                                             fill_value=fillval,l_out_nc4par=l_out_nc4par)
         CALL ionc4_createvar(file_saverestart,TRIM(name_in_vis3d_phi),units= 'm2 s-1',long_name ='vish3d_phi',dims='zxyt', &
!                                               location_in_cell='f',fill_value=fillval,l_out_nc4par=.true.)
                                              fill_value=fillval,l_out_nc4par=l_out_nc4par)
       END IF
       CALL ionc4_createvar(file_saverestart,'phieau','m3.s-1','water mass flux',dims='zxyt', &
                                             fill_value= 0.0_riolg,l_out_nc4par=l_out_nc4par)
!
#ifdef key_substance
       DO l = 1,nv_tot
         CALL ionc4_createvar(file_saverestart,TRIM(name_var(irk_fil(l))),unit_var(irk_fil(l)), &
                               TRIM(name_var(irk_fil(l))),dims='zxyt',fill_value=fillval,l_out_nc4par=l_out_nc4par)
       END DO
#endif
     ENDIF                    ! fin de la creation dans le cas sequentiel
                              !------------------------------------------
!
     !IF(t >= tsauv.OR.l_boolsauv) THEN           ! ecriture
     IF(t >= tsauv.OR.l_boolsauv.OR.(t >= (tfin-0.5*dtmax))) THEN           ! ecriture
                                                 !---------
       IF_AGRIF (agrif_root()) l_boolsauv = .TRUE.

       ! new data overlay old data
       CALL ionc4_write_time(file_saverestart,1,t)
       CALL ionc4_write(file_saverestart,TRIM(name_in_dt),dt)
       IF (.NOT. l_modele2d) THEN
         CALL ionc4_write(file_saverestart,'tdebobs',tdebobs)
         CALL ionc4_write_t(file_saverestart,'cflumaxmax',cflumaxmax,1)
         CALL ionc4_write_t(file_saverestart,'cflvmaxmax',cflvmaxmax,1)
       END IF
       CALL ionc4_write(file_saverestart,'tchrono_deb',tchrono(indchrono-1))

       tmp2d(:,:)=h0(limin:limax,ljmin:ljmax)
       CALL ionc4_write_xy(file_saverestart,TRIM(name_bathy_h0),tmp2d,limin,limax,ljmin,ljmax,l_out_nc4par=l_out_nc4par)
       tmp2d(:,:)=hx(limin:limax,ljmin:ljmax)
       CALL ionc4_write_xy(file_saverestart,TRIM(name_bathy_hx),tmp2d,limin,limax,ljmin,ljmax,l_out_nc4par=l_out_nc4par)
       tmp2d(:,:)=hy(limin:limax,ljmin:ljmax)
       CALL ionc4_write_xy(file_saverestart,TRIM(name_bathy_hy),tmp2d,limin,limax,ljmin,ljmax,l_out_nc4par=l_out_nc4par)
       CALL ionc4_write_xyt(file_saverestart,TRIM(name_in_xev),xev,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)
       tmp2d(:,:)=u(limin:limax,ljmin:ljmax)
       CALL ionc4_write_xyt(file_saverestart,TRIM(name_in_u),tmp2d,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)
       tmp2d(:,:)=v(limin:limax,ljmin:ljmax)
       CALL ionc4_write_xyt(file_saverestart,TRIM(name_in_v),tmp2d,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)
       IF (.NOT. l_modele2d) THEN
         tmp2d(:,:)=vstar(limin:limax,ljmin:ljmax)
         CALL ionc4_write_xyt(file_saverestart,'vstar',tmp2d,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)
         tmpigd=REAL(igdv_old,riosh)
         CALL ionc4_write_xyt(file_saverestart,'igdv_old',tmpigd,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)
       END IF
       tmp2d(:,:)=vis2d(limin:limax,ljmin:ljmax)
       CALL ionc4_write_xyt(file_saverestart,TRIM(name_in_vis2d),tmp2d,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(1:kmax,:,:)=uz(1:kmax,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_uz),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(1:kmax,:,:)=vz(1:kmax,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_vz),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(1:kmax,:,:)=wz(1:kmax,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_wz),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(1:kmax,:,:)=sal(1:kmax,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_sal),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(1:kmax,:,:)=temp(1:kmax,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_temp),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_ect),ect,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_psi),psi,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(:,:,:)=kz(:,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_kz),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(:,:,:)=nz(:,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_nz),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       IF (.NOT.l_modele2d) THEN
       tmp3d(:,:,:)=vish_xe(:,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_vis3d_xe),tmp3d,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)
       tmp3d(:,:,:)=vish_phi(:,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,TRIM(name_in_vis3d_phi),tmp3d,limin,limax,ljmin,ljmax,kmax,1, &
                                              l_out_nc4par=l_out_nc4par)
       END IF
       tmp3drlg(:,:,:)=phieau(:,limin:limax,ljmin:ljmax)
       CALL ionc4_write_zxyt(file_saverestart,'phieau',tmp3drlg,limin,limax,ljmin,ljmax,kmax,1,l_out_nc4par=l_out_nc4par)

#ifdef key_substance
       DO l = 1,nv_tot
         tmp3d(:,:,:) = cv_wat(l,:,limin:limax,ljmin:ljmax)
         CALL ionc4_write_zxyt(file_saverestart,TRIM(name_var(irk_fil(l))),tmp3d,limin,limax,ljmin,ljmax,kmax,1, &
                                                l_out_nc4par=l_out_nc4par)
       END DO
#endif
       tsauv=t+saverestart_step
#if defined key_sedim && defined key_substance
       tsauv=t-saverestart_step
#endif

       IF (l_saverestart_bydate) THEN
         CALL ionc4_close(file_saverestart)
       ELSE
         CALL ionc4_sync(file_saverestart)
       ENDIF

     END IF

   PRINT_DBG*, 'exit OUT_SAVERESTART routine'

  END SUBROUTINE out_saverestart
