  SUBROUTINE tool_interp2d_sflx_1(i1,j1,xdp,ydp,tabin_1,tabout_1)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE tool_interp2d_sflx  ***
   !&E
   !&E ** Purpose : spatial interpolation of meteorological data
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : paramet
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : tabin_1, i1, j1, xdp, ydp
   !&E
   !&E ** Modified variables : tabout_1
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2013 S.T. Les variables meteo ne sont pas sur les memes grille
   !&E                    donc 2 interpolations differentes sont necessaires
   !&E       
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   !USE parameters ,only:rsh,riosh,imin,imax,jmin,jmax
   USE comvars2d,ONLY : jb,jh,ig,id
   USE cometdyn, ONLY : imetmin_1,imetmax_1,jmetmin_1,jmetmax_1
   IMPLICIT NONE

   !! * Arguments
   INTEGER         ,DIMENSION(limin:limax,ljmin:ljmax)        ,INTENT(in)  :: i1,j1
   REAL(kind=rsh)  ,DIMENSION(limin:limax,ljmin:ljmax)        ,INTENT(in)  :: xdp,ydp
   REAL(kind=riosh),DIMENSION(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1),INTENT(in)  :: tabin_1
   REAL(kind=rsh)  ,DIMENSION(limin:limax,ljmin:ljmax)        ,INTENT(out) :: tabout_1

   !! * Local declarations
   INTEGER        :: i,j,ii1,ii2,jj1,jj2
   REAL(kind=rsh) :: var1,var2

   !!----------------------------------------------------------------------
   !! * Executable part

   !do j = jmin+1,jmax
!$OMP do schedule(runtime),private(i,j,ii1,ii2,jj1,jj2,var1,var2)
   DO j = ljminp1,ljmax
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
       ii1=MAX(i1(i,j),imetmin_1)
       ii2=MIN(i1(i,j)+1,imetmax_1)
       jj1=MAX(j1(i,j),jmetmin_1)
       jj2=MIN(j1(i,j)+1,jmetmax_1)
       var1=xdp(i,j)*tabin_1(ii2,jj1)+(1.0_rsh-xdp(i,j))*tabin_1(ii1,jj1)
       var2=xdp(i,j)*tabin_1(ii2,jj2)+(1.0_rsh-xdp(i,j))*tabin_1(ii1,jj2)
       tabout_1(i,j)=(ydp(i,j)*var2+(1.0_rsh-ydp(i,j))*var1)
     END DO
   END DO
!$OMP end do

   ! fill borders

!$OMP do private(i,j,ii1,ii2,jj1,jj2,var1,var2)
   DO i = liminp1,limax

     ! bottom
     j=jb(i)
     IF ((j.LE.ljmax).AND.(j.GE.ljmin))THEN
       ii1=MAX(i1(i,j),imetmin_1)
       ii2=MIN(i1(i,j)+1,imetmax_1)
       jj1=MAX(j1(i,j),jmetmin_1)
       jj2=MIN(j1(i,j)+1,jmetmax_1)
       var1=xdp(i,j)*tabin_1(ii2,jj1)+(1.0_rsh-xdp(i,j))*tabin_1(ii1,jj1)
       var2=xdp(i,j)*tabin_1(ii2,jj2)+(1.0_rsh-xdp(i,j))*tabin_1(ii1,jj2)
       tabout_1(i,j)=(ydp(i,j)*var2+(1.0_rsh-ydp(i,j))*var1)
     ENDIF

     ! top
     j=jh(i)
     IF ((j.LE.ljmax).AND.(j.GE.ljmin))THEN
       ii1=MAX(i1(i,j),imetmin_1)
       ii2=MIN(i1(i,j)+1,imetmax_1)
       jj1=MAX(j1(i,j),jmetmin_1)
       jj2=MIN(j1(i,j)+1,jmetmax_1)
       var1=xdp(i,j)*tabin_1(ii2,jj1)+(1.0_rsh-xdp(i,j))*tabin_1(ii1,jj1)
       var2=xdp(i,j)*tabin_1(ii2,jj2)+(1.0_rsh-xdp(i,j))*tabin_1(ii1,jj2)
       tabout_1(i,j)=(ydp(i,j)*var2+(1.0_rsh-ydp(i,j))*var1)
     ENDIF
   END DO
!$OMP end do

  END SUBROUTINE tool_interp2d_sflx_1
